#ifndef GL_ES
#	define precision
#	define lowp
#	define mediump
#	define highp
#else
precision mediump float;
#endif

uniform   lowp sampler2D texture0;

uniform highp float SourceR0;
uniform highp float SourceR1;
uniform highp vec2 SourcePos0;
uniform	highp vec2 SourcePos1;
uniform lowp vec4 SourceCol0;
uniform lowp vec4 SourceCol1;

varying        vec2      out_uv0;
varying   lowp vec4      out_color;


void main()
{
	highp vec2 rv0 = out_uv0 - SourcePos0;
	highp vec2 rv1 = out_uv0 - SourcePos1;
	highp float dist_norm_inv0 = clamp (1.0 - (rv0.x * rv0.x + rv0.y * rv0.y) / (SourceR0 * SourceR0), 0.0, 1.0);
	highp float dist_norm_inv1 = clamp (1.0 - (rv1.x * rv1.x + rv1.y * rv1.y) / (SourceR1 * SourceR1), 0.0, 1.0);

	lowp vec4 light_color0 = SourceCol0;
	light_color0.a *= dist_norm_inv0;

	lowp vec4 light_color1 = SourceCol1; 
	light_color1.a *= dist_norm_inv1;

	lowp vec4 base = out_color * texture2D (texture0, out_uv0) * 0.5;
	lowp vec3 overlay = light_color0.rgb * light_color0.a + light_color1.rgb * light_color1.a;
	
	if (overlay.r < 0.5)
		overlay.r = 2.0 * overlay.r * overlay.r;
	else
		overlay.r = 1.0 - 2.0 * (1.0 - overlay.r) * (1.0 - overlay.r);

	if (overlay.g < 0.5)
		overlay.g = 2.0 * overlay.g * overlay.g;
	else
		overlay.g = 1.0 - 2.0 * (1.0 - overlay.g) * (1.0 - overlay.g);

	if (overlay.b < 0.5)
		overlay.b = 2.0 * overlay.b * overlay.b;
	else
		overlay.b = 1.0 - 2.0 * (1.0 - overlay.b) * (1.0 - overlay.b);

	gl_FragColor = vec4 (base.rgb + overlay, 1.0);
}
