#ifndef GL_ES
#	define precision
#	define lowp
#	define mediump
#	define highp
#else
precision highp float;
#endif

uniform   lowp sampler2D texture0;
uniform   lowp float     SaturationScale;

varying        vec2      out_uv0;
varying   lowp vec4      out_color;

const lowp float Epsilon = 1e-10;

lowp vec3 RGBtoHCV(lowp vec3 RGB)
{
    // Based on work by Sam Hocevar and Emil Persson
    lowp vec4 P = (RGB.g < RGB.b) ? vec4(RGB.bg, -1.0, 2.0/3.0) : vec4(RGB.gb, 0.0, -1.0/3.0);
    lowp vec4 Q = (RGB.r < P.x) ? vec4(P.xyw, RGB.r) : vec4(RGB.r, P.yzx);
    lowp float C = Q.x - min(Q.w, Q.y);
    lowp float H = abs((Q.w - Q.y) / (6.0 * C + Epsilon) + Q.z);
    return vec3(H, C, Q.x);
}

lowp vec3 HUEtoRGB(lowp float H)
{
    lowp float R = abs(H * 6.0 - 3.0) - 1.0;
    lowp float G = 2.0 - abs(H * 6.0 - 2.0);
    lowp float B = 2.0 - abs(H * 6.0 - 4.0);
    return clamp(vec3(R,G,B), 0.0, 1.0);
}

lowp vec3 RGBtoHSL(lowp vec3 RGB)
{
	lowp vec3 HCV = RGBtoHCV(RGB);
  	lowp float L = HCV.z - HCV.y * 0.5;
    lowp float S = HCV.y / (1.0 - abs(L * 2.0 - 1.0) + Epsilon);
    return vec3(HCV.x, S, L);
}

lowp vec3 HSLtoRGB(lowp vec3 HSL)
{
    lowp vec3 RGB = HUEtoRGB(HSL.x);
    lowp float C = (1.0 - abs(2.0 * HSL.z - 1.0)) * HSL.y;
    return (RGB - 0.5) * C + HSL.z;
}

void main()
{
	lowp vec4 color = out_color * texture2D( texture0, out_uv0 );
	lowp vec3 hsl   = RGBtoHSL(color.rgb);
	hsl.g *= SaturationScale;
	lowp vec3 rgb   = HSLtoRGB(hsl);
	gl_FragColor = vec4(rgb, color.a);
}
