local l10n = l10n or {}
local L = L or function( n, ... ) print( n, ...); return n end

function l10n.plural_class( n )
	return i == 1 and "one" or "other"
end

function l10n.time_span( time ) 
	local seconds = time % 60
	time = (time - seconds) / 60
	local minutes = time % 60
	time = (time - minutes) / 60
	local hours = time % 24
	time = (time - hours) / 24
	local days = time % 7
	time = (time - days) / 7
	local weeks = time

	if weeks > 0  then
		return L( "TIME_SPAN_WEEKS", weeks, days )
	elseif days > 0 then
		return L( "TIME_SPAN_DAYS", days, hours )
	elseif hours > 0 then
		return L( "TIME_SPAN_HOURS", hours, minutes )
	elseif minutes > 0 then
		return L( "TIME_SPAN_MINUTES", minutes, seconds )
	end

	return L( "TIME_SPAN_SECONDS", seconds )
end

local _day = 24 * 60 * 60
local _hour = 60 * 60

function l10n.time_passed( time )
	if  time >= 8 * 24 * 60 * 60 then
		return L "TIME_MORE_THAN_WEEK_AGO"
	elseif time >= 7 * 24 * 60 * 60  then
		return L "TIME_A_WEEK_AGO"
	elseif time >= _day then
		local days = math.floor( time / _day )
		return L( "TIME_DAYS_AGO", days )
	elseif time >= _hour then
		local hours = math.floor( time / _hour )
		return L( "TIME_HOURS_AGO", hours )
	elseif time >= 60 then
		local minutes = math.floor( time / 60 )
		return L( "TIME_MINUTES_AGO", minutes )
	end

	return L( "TIME_SECONDS_AGO", time )
end

--l10n.time_passed( 65 )