-- Lua port of JS port of webkit CSS cubic-bezier (https://gist.github.com/mckamey/3783009)

-- Duration only controls the precision of the solver

local default_duration = 0.400

local function solve_epsilon(duration)
	return 1.0 / (200.0 * duration * 1000.0)
end

anim = anim or {}

function anim.bezier(p1x,p1y,p2x,p2y)
	local cx = 3.0 * p1x
	local bx = 3.0 * (p2x - p1x) - cx
	local ax = 1.0 - cx - bx

	local cy = 3.0 * p1y
	local by = 3.0 * (p2y - p1y) - cy
	local ay = 1.0 - cy - by

	local function sample_curve_x(t)
		return ((ax * t + bx) * t + cx) * t
	end

	local function sample_curve_y(t)
		return ((ay * t + by) * t + cy) * t
	end

	local function sample_curve_dx(t)
		return (3.0 * ax * t + 2.0 * bx) * t + cx
	end

	local function solve_curve_x(x, epsilon)
		local t2 = x
		for i=1,8 do
			local x2 = sample_curve_x(t2) - x
			if math.abs(x2) < epsilon then
				return t2
			end

			local d2 = sample_curve_dx(t2)

			if math.abs(d2) < 1e-6 then
				break
			end

			t2 = t2 - x2 / d2
		end

		local t0 = 0
		local t1 = 1
		t2 = x

		if t2 < t0 then return t0 end
		if t2 > t1 then return t1 end

		while t0 < t1 do
			local x2 = sample_curve_x(t2)
			if math.abs(x2 - x) < epsilon then
				return t2
			end
			if x > x2 then t0 = t2 else t1 = t2 end
			t2 = (t1 - t0) * 0.5 + t0
		end

		return t2
	end

	local function solve(x, epsilon)
		return sample_curve_y(solve_curve_x(x, epsilon))
	end

	return function(x, duration)
		return solve(x, solve_epsilon(duration or default_duration))
	end
end

anim.easing = anim.easing or {}

anim.easing.linear      = anim.bezier(0,0,1,1)
anim.easing.ease        = anim.bezier(0.25, 0.1, 0.25, 1.0)
anim.easing.ease_in     = anim.bezier(0.42, 0, 1.0, 1.0)
anim.easing.ease_out    = anim.bezier(0, 0, 0.58, 1.0)
anim.easing.ease_in_out = anim.bezier(0.42, 0, 0.58, 1.0)
anim.easing.cubic_bezier= anim.bezier
