--module (..., package.builder)

require "anim/base.lua"
local base = anim.base

local function empty_update() end

local function update(self, dt)
	self.t = self.t + dt / self.time

	if self.t >= 1 then
		base.set_property_value(self, self.to)
		self.update = empty_update
		base.run_callback(self, "on_finished")
	else
		base.set_property_value(self, self.from + (self.to - self.from) * self.map(self.t))
	end
end

local function id(x) return x end

local function build(context)
	local map = context.map
	local time = assert(context.time or context.duration)

	if not map and context.easing then
		map = function(x) return context.easing(x, time) end
	end

	return {
		id   = context.id,

		time = time,
		from = assert(context.from),
		to   = assert(context.to),

		object = assert(context.object),
		property = assert(context.property),

		t = 0,

		map  = map or id,

		on_finished = context.on_finished,

		update = update,
		reset = function(self)
			self.t = 0
			self.update = update
		end
	}
end


anim.lerp = build

return build