--module (..., package.builder)

require "anim/base.lua"
local base = anim.base

local function empty_update() end

local function update(self, dt)
	local child = self.children[self.idx]

	if not child then
		self:reset()
		child = self.children[1]
	end

	child:update(dt)
end

local function build(context)
	local anim =  {
		id   = context.id,

		idx = 1,

		on_finished = context.on_finished,

		update = update,
		reset = function(self)
			self.idx = 1
			for i,v in ipairs(self.children) do
				v:reset()
			end
			self.update = update
		end
	}

	local children = {}

	for i,v in ipairs(context) do
		base.add_callback(v, "on_finished", function() anim.idx = anim.idx + 1 end )
		children[i] = v
	end

	anim.children = children

	return anim
end

anim.loop = build

return build