-- Application setting holder logics

local meta = {}

function meta.__index( table, key )
	local mt_l = rawget( meta, key )

	local function make_key( )
		local  ret = setmetatable( {}, meta )
		rawset( table, key, ret )
		return  ret
	end

	return mt_l or make_key()
end

function meta.__newindex( table, key, value )
	if type(value) == "table" then
		-- Chain metatables
		local old_mt = getmetatable( table )

		function mt_index( table, key )
			return rawget(meta,key) or (old_mt and old_mt[key]) or meta.__index( table, key )
		end

		value = setmetatable( value, { __index = mt_index } )
	end

	rawset(table, key, value)
end

function meta:flush()
	local path = "/home/settings.sv"
	local data = table.dump(self, nil, true)
	file.write_string(path, data)
end
-- Key must be a string value
function meta:get( key, default_value )
	local keys = string.split(key, ".")
	local value
	local tbl = self

	for i, v in ipairs( keys ) do
		value = rawget(tbl, v)
		tbl = value
		if value == nil then return default_value end
	end
	return value
end

function meta:set( key, value )
	local keys = string.split(key, ".")
	local temp
	local tbl = self

	key = table.remove( keys, #keys )

	for i, v in ipairs( keys ) do
		temp = tbl[v]
		tbl = temp
	end
	tbl[key] = value
end

function meta.new()
	local settings
	local path = "/home/settings.sv"
	if file.exists(path) then
		local data = file.read_string(path)
		settings = runtime.reload(path)
	else
		-- clean start
		settings = { version = VERSION }
	end

	-- Recursively set metatables on all tables
	local function update_meta( table )
		setmetatable( table, meta )
		for key, value in pairs( table ) do
			if type(value) == "table" then
				update_meta( value )
			end
		end
	end

	update_meta( settings )

	return settings
end

underworld2.settings = meta.new()
