android = android or {}
android.play = android.play or {}
android.play.cache = {}

local play_cache = android.play.cache

local cache_path = "/home/billing/play_cache.cache"

function play_cache.save()
	if not play_cache.state then return end

	local data = table.dump( play_cache.state )

	file.write_string( cache_path, data)
end

function play_cache.load()
	if file.is_file( cache_path ) and not file.is_empty( cache_path ) then
		play_cache.state = dofile( cache_path )
		print( "Store cache is loaded. ".. #play_cache.state .. " items")

		for i,tx in ipairs(play_cache.state) do
			if not tx.order_id then
				tx.order_id = ""
			end
		end
	else
		print( "Loading empty store cache")
		play_cache.state = {}
	end
end

function play_cache.has_unchecked_tx()
	return #play_cache.state > 0
end

--- ... contains pairs order, status
function play_cache.add_tx( data, signature, ... )
	local tx = {
		data = data,
		signature = signature,
		orders = {}
	}
	local orders = {...}
	for i=1,#orders,3 do 
		table.insert( tx.orders, { id = orders[ i ], order_id = orders[ i + 1 ], state = orders[ i + 2 ] } )
	end
	tx.orders_count = #tx.orders
	table.insert( play_cache.state, tx )
	print( "Adding transaction ".. table.dump( tx, nil, true ) )
	print("Cache size ", #play_cache.state)
	play_cache.save()
end

function play_cache.tx_checked( ... )
	-- try to match tx now
	print( "Searching for transaction: " .. table.dump( {...}, nil, true ))
	local match = { ... }
	local function matches_tx( tx )
		-- build a set from tx
		local set = {}
		for i, v in ipairs( tx.orders ) do
			local key = v.id .. v.state
			set[key] = true
		end
		local matched = 0
		for i=1,#match,2 do
			local key = match[ i ] .. match[ i + 1 ]
			if not set[key] then return false end
			matched = matched + 1
		end
		return matched == tx.orders_count
	end

	for i,v in ipairs( play_cache.state ) do
		if matches_tx( v ) then
			print("Transaction removed: ".. i .. " "..table.dump( v, nil, true ))
			print("Unchecked transactions left ", #play_cache.state - 1)
			table.remove( play_cache.state, i )
			play_cache.save()
			return 
		end
	end
end

function play_cache.add_order( order_id )
	if not play_cache.state.orders then
		play_cache.state.orders = {}
	end

	play_cache.state.orders[ order_id ] = true;
	play_cache.save()
end

function play_cache.has_order( order_id )
	if not play_cache.state.orders then return end
	return play_cache.state.orders[ order_id ]
end

-- Return the first unchecked tx
function play_cache.get_unchecked_tx()
	return play_cache.state[1]
end

play_cache.load()
--play_cache.save()