-- Underworld2 game initialization script

require "ui/core/view_controller.lua"
require "ui/core/view.lua"

require "underworld2/tutorial.lua"
require "underworld2/game_events.lua"

require "ui/config.lua"
require "ui/main_view.lua"
require "ui/status_panel.lua"
require "ui/enter_name_view.lua"
require "ui/event_view.lua"
require "ui/modal/error.lua"
require "ui/modal/game_update.lua"

require "ui/widgets/dealer_details.lua"

require "ui/equipment_shop_view.lua"
require "ui/crew_view.lua"

require "social/jabber.lua"

require "tutorial/tutorial.lua"

require "underworld2/upgrades_tracker.lua"
require "underworld2/inbox.lua"
require "underworld2/shared.lua"
require "underworld2/equipment_view_state.lua"

require "ui/modal/server_error.lua"
require "ui/modal/news_modal.lua"
require "ui/modal/fame_drop_view.lua"
require "ui/modal/custom_error_view.lua"

require "network/online_controller.lua"
require "network/game_online_controller.lua"

-- setup default online controller implementation
network.online_controller.default = network.game_online_controller.create()

-- sound assets -------------------------------------------

get_sound_pool():load ("ui/UI_Click_05.wav")
get_sound_pool():load ("ui/UI_Scroll_09.wav")
get_sound_pool():load ("ui/SFX_Menu_Close.wav")
get_sound_pool():load ("ui/SFX_Menu_Woosh_Open.wav")

get_sound_pool():load ("ui/SFX_Message_Get.wav")
get_sound_pool():load ("ui/SFX_Message_Send.wav")

local function play_ambient_sfx()
	local amb_snd_timer = 0.0 
	local amb_snd_map = 
	{
		"ambient/SFX_Atmos_01.wav",
		"ambient/SFX_Atmos_04.wav",
	}

	local amb_snd_cnt = 0
	local snd_state = {
		background = false,
		ambient = false
	}

	local function poll_background_track()
		local enabled = underworld2.settings:get ("background_music_enabled", true)
		get_music_player():setLoopMode (true)
		get_music_player():setGain (0.9)
		get_music_player():queue ("music/Underworld_Music_WIP")

		if enabled then
			get_music_player():play()
		end

		snd_state.background = true
	end

	local function poll_ambient_sfx()
		local enabled = underworld2.settings:get ("ambient_sfx_enabled", true)
		--get_music_player():setLoopMode (true)
		--get_music_player():queue ("music/Underworld_Music_WIP")

		if enabled then
			get_sound_pool():play (amb_snd_map [math.dice (#amb_snd_map)])
		end

		snd_state.ambient = true
	end

	events.connect (events.global_update(), function (dt)
		amb_snd_timer = amb_snd_timer + dt

		if (amb_snd_timer > 10) and (not snd_state.background) then
			poll_background_track ()
		end

		if (amb_snd_timer > 20) or (not snd_state.ambient) then
			poll_ambient_sfx ()
		end

		if (amb_snd_timer > 20) then
			amb_snd_timer = 0
		end
	end)

	events.connect (ui.core.view_controller.instance:getMovedToBackgroundSignalProxy(), function()
		if snd_state.background then
			get_music_player():pause()
		end
	end)
	events.connect (ui.core.view_controller.instance:getMovedToForegroundSignalProxy(), function()
		if snd_state.background then
			get_music_player():resume()
		end
	end)
end

local ignore_location_errors = false
underworld2.first_run = true

local loading_view
local main_view 	

function underworld2.show_main_screen()
	underworld2.ui.status_panel	= ui.core.view.builder (ui.status_panel).build()
	main_view= ui.core.view.builder (ui.main_view).build()

	underworld2.ui.stack 		= ui.core.stacked_window.builder (main_view).build()
	underworld2.ui.window 		= ui.core.side_menu.builder (underworld2.ui.stack).build()
	underworld2.ui.main_view	= main_view
	
	underworld2.ui.status_panel:on_window_stack_changed (underworld2.ui.stack)

	ui.core.view.delegate_builder (
		{
			on_back = function (view)
				if underworld2.tutorial:current_stage() ~= nil then
					if not underworld2.tutorial:on_back() then return end
				end

				if view.side_menu then
					underworld2.ui.status_panel:hide_profile_info()
				elseif underworld2.ui.stack:size() > 0 or not main_view:try_handle_back() then
					view.window:back()
				end
			end
		}).build (underworld2.ui.window)

	ui.core.view.delegate_builder (
		{
			on_back = function (view)
				if view:size() > 0 then
					local top_window = view:top()
					if top_window and top_window.on_back then
						top_window:on_back()
					end
					view:pop_window()
				else
					runtime.terminate_runloop()
				end
				--end
			end,

			on_window_pushed = function (view)
				underworld2.ui.status_panel:on_window_stack_changed (view)
			end,

			on_window_poped = function (view)
				underworld2.ui.status_panel:on_window_stack_changed (view)
			end
		}).build (underworld2.ui.stack)

	ui.core.view_controller.instance:setRootView (underworld2.ui.window._pdata)

	underworld2.ui.stack:add_subview (underworld2.ui.status_panel)

	play_ambient_sfx()
end

function underworld2.ui.show_message (text, handler)
	get_sound_pool():play ("ui/SFX_Notice.wav")

	local info_view = ui.information_view.create (text)
	ui.core.view_controller:show_modal_view (info_view)
	info_view.delegate_builder (
	{
		finished = function()
			ui.core.view_controller:close_modal_view (info_view)
			if handler then handler() end
		end
	}).build(info_view)
end

local function _load_game_data (handler)
	underworld2.network.get_items_description()
	underworld2.network.levels_description()
	underworld2.network.load_skill_groups (handler)
end

local _dev_events_setup = nil

local function _init_device_events()
	if _dev_events_setup then return end
	_dev_events_setup = { }
	_dev_events_setup.background_conn_id = events.connect (ui.core.view_controller.instance:getMovedToBackgroundSignalProxy(), function()		
		ignore_location_errors = false
		net.location_provider():lockUpdates()

		local player = underworld2.player
		
		local now 		= underworld2.server_time()
		
		pushLocalNotification ("com.asteroids.underworld2.intent.COME_BACK_1", 
			"Underworld", 
			L ("KEY_COME_BACK_1_NOTIFICATION"),
			os.timestamp.time() + 24 * 60 * 60)

		pushLocalNotification ("com.asteroids.underworld2.intent.COME_BACK_2",
			"Underworld",
			L ("KEY_COME_BACK_2_NOTIFICATION"),
			os.timestamp.time() + 3 * 24 * 60 * 60)

		pushLocalNotification ("com.asteroids.underworld2.intent.COME_BACK_3",
			"Underworld",
			L ("KEY_COME_BACK_3_NOTIFICATION"),
			os.timestamp.time() + 7 * 24 * 60 * 60)

		-- Item delivery notification
		local pending_items = player:getPendingItems()
		if pending_items:size() > 0 then
			local closest_delivery = now
			local equipment_id
			for i = 0, pending_items:size() - 1 do
				local id = pending_items[i]
				local when = player:getItemDeliveryTime (id)
				if when > closest_delivery then
					closest_delivery = when
					equipment_id = id
				end
			end
			if equipment_id then
				local item = underworld2.ItemsDatabase():getItem (equipment_id)
				pushLocalNotification ("com.asteroids.underworld2.intent.EQUIPMENT_DELIVERY",
					"Underworld",
					L ("KEY_EQUIPMENT_DELIVERY_NOTIFICATION", underworld2.StringsDatabase():get (item:getNameKey())),
					os.timestamp.time() + (closest_delivery - now))
			end
		end

		-- Crew member(skill) delivery notification
		local pending_skills = player:getPendingSkills()
		if pending_skills:size() > 0 then
			local closest_delivery = now
			local skill_id
			for i = 0, pending_skills:size() - 1 do
				local id = pending_skills[i]:getSkillId()
				local when = pending_skills[i]:getDeliveryEndTime()
				if when > closest_delivery then
					closest_delivery = when
					skill_id = id
				end
			end
			if skill_id then
				local skill_group = underworld2.SkillsDatabase():findGroupBySkillId (skill_id)

				pushLocalNotification ("com.asteroids.underworld2.intent.SKILL_DELIVERY",
					"Underworld",
					L ("KEY_SKILL_DELIVERY_NOTIFICATION", underworld2.StringsDatabase():get (skill_group:getTitleKey())),
					os.timestamp.time() + (closest_delivery - now))
			end
		end

		local exposure 		= player:getPoliceExposure()
		local cooldown_time = exposure:getCooldownTime (now)

		if cooldown_time > 0 then
			pushLocalNotification ("com.asteroids.underworld2.intent.HEAT_DROP",
				"Underworld",
				L ("KEY_POLICE_SAFE_NOTIFICATION"),
				os.timestamp.time() + cooldown_time)
		end
		
		underworld2.settings:set ("last_app_suspend", now)
		underworld2.settings:flush()

		local couriers_count = player:getCouriersCount()
		local closest_courier
		for i = 0, couriers_count - 1 do
			local courier = player:getCourierAt (i)
			if (courier:getState() ~= underworld2.CourierStateNotOccupied) and 
				(courier:getEndTime() > now) then
				if not closest_courier then 
					closest_courier = courier
				elseif closest_courier:getEndTime() > courier:getEndTime() then
					closest_courier = courier
				end
			end
		end

		if closest_courier then
			local time_left = closest_courier:getEndTime() - now

			local text = nil
			if closest_courier:getState() == underworld2.CourierStatePurchasing then
				local commodity = underworld2.CommoditiesDatabase():get (closest_courier:getCommodityId())
				local commodity_name = underworld2.StringsDatabase():get (commodity:getNameKey())
				text = L("KEY_ITEMS_DELIVERED_NOTIFICATION", closest_courier:getItemsAmount(), commodity_name)
			else
				text = L("KEY_MONEY_DELIVERED_NOTIFICATION", ui.format.currency_string (closest_courier:getMoney()))
			end

			pushLocalNotification ("com.asteroids.underworld2.intent.DEAL_FINISHED",
					"Underworld",
					text,
					os.timestamp.time() + time_left)
		end
	end)

	_dev_events_setup.foreground_conn_id = events.connect (ui.core.view_controller.instance:getMovedToForegroundSignalProxy(), function()
		cancelAllLocalNotifications()

		if underworld2.tutorial:current_stage() then
			return
		end
		
		local min_update_interval = 10 * 60
		local now = underworld2.server_time()
		local last_app_suspend = underworld2.settings:get ("last_app_suspend", 0)
		if not underworld2.network.has_running_request()
			and (((now - last_app_suspend) > min_update_interval) or (now < last_app_suspend)) then
			ui.event_view.show()
			local location = net.Location (net.location_provider():getLocation())
			underworld2.network.reset_session()
			underworld2.network.login (location, function() ui.event_view.close() end)
			--underworld2.ui.main_view:location_updated (net.location_provider():getLocation(), true)
		end
		net.location_provider():unlockUpdates()
	end)
end

local _messaging_initialized = false

local function _messaging_init()
	if _messaging_initialized then return end
	_messaging_initialized = true

	social.jabber.load_ext ("xep_0082")
	social.jabber.load_ext ("xep_0045")
	social.jabber.load_ext ("archive")
end

-- fwd decl
underworld2.show_enter_name_dialog = nil

function underworld2.show_latest_news(news)
	if #news == 0 then return end

	-- update badge
	underworld2.ui.status_panel:add_pending_notifications(#news)

	-- top, or the latest
	local top = news[1]

	-- event to show the modal
	local event = underworld2.news_event.create(function(e)
		if ui.core.view_controller:has_modal_views() then
			return false
		elseif not underworld2.player then
			return false
		elseif (#underworld2.player:getActiveTrigger() > 0) or (underworld2.tutorial:current_stage() ~= nil) then
			return false
		end

		ui.news_modal.show (top.title, top.description, top.image_url, top.url, function()
			e:finalize()
			underworld2.ui.status_panel:add_pending_notifications(-1)
			underworld2.archives.news.mark(1)
		end)

		return true
	end)

	underworld2.game_events:queue(event)
end

function underworld2.show_enter_name_dialog (default, handler)
	local enter_name_view = ui.enter_name_view.create(default)
	enter_name_view.delegate_builder (
	{
		finished = function()
			ui.core.view_controller:close_modal_view (enter_name_view)
			if handler then handler() end
		end
	}).build (enter_name_view)
	ui.core.view_controller:show_modal_view (enter_name_view)
end

local initialized = false
local billing_initialized = false

local function _init (location)
	if initialized then
		return
	end

	initialized = true
		
	underworld2.player = underworld2.Player_GetInstance()

	_messaging_init()

	social.jabber.connect (underworld2.player:getNickname(), underworld2.player:id(), "password")
	underworld2.inbox.init()
	
	_init_device_events()

	if not billing_initialized then
		billing.setup {
			purchase_state_updated = function( store_item_id, state, order_id ) 
				if state == billing.Purchased then
					underworld2.ui.show_message( L ("BILLING_ITEM_PURCHASED", store_item_id) )
					local purchase = underworld2.databases.in_app_purchases.get(store_item_id)

					print ( "Logging purchase to analytics providers ", store_item_id, " ", order_id )

					local price = purchase.price / 100
					social.facebook.log_purchase( price, "USD", billing.provider, "GOLD", store_item_id )

					analytics.mat.log_purchase {
						id             = store_item_id,
						price          = price,
						transaction_id = order_id or "unkonwn",
						currency       = "USD"
					}
				elseif state ~= billing.PurchaseDeclined then
					underworld2.ui.show_message( L ("BILLING_ITEM_PURCHASE_FAILED", store_item_id, billing.last_error()) )
				end
			end,
			purchases_data_updated = function( purchases ) 

			end
		}
		billing_initialized = true
	end

	underworld2.ui.main_view:init (location)
	underworld2.ui.status_panel:init()
	underworld2.ui.main_view:moved_to_foreground()

	ui.equipment_shop_view.update_item_delivery_queue()
	ui.crew_view.update_upgrades_delivery_queue()
end

local _process_login = function (location)
	
	ui.event_view.show()
	underworld2.network.login (location, function()
		ui.event_view.close()

		if underworld2.Player_IsCreated() then
			_init (location)
		end

		underworld2.upgrades_tracker.init()
		underworld2.ui.status_panel:on_player_stats_changed()
	end)
end

function underworld2.notifications_received (ns)
	for i, notif in ipairs (ns) do
		if notif.type == "APP_UPDATE_AVAILABLE" then
			local e = underworld2.game_update_event.create (function (e)
				local v = ui.game_update.create (notif.body, notif.external_url, { ui.game_update.BTN_LATER, ui.game_update.BTN_UPDATE_NOW }, notif.image_url)
				ui.core.view.delegate_builder(
				{
					finished = function (view)
						ui.core.view_controller:close_modal_view (view)
						e:finalize()
					end
				}).build (v)
				
				get_sound_pool():play ("ui/SFX_Notice.wav")
				ui.core.view_controller:show_modal_view (v)

				return true
			end)
			underworld2.game_events:queue (e)
		elseif notif.type == "FORCE_APP_UPDATE_AVAILABLE" then
			local e = underworld2.game_update_event.create (function (e)
				local v = ui.game_update.create (notif.body, notif.external_url, { ui.game_update.BTN_UPDATE_NOW }, notif.image_url)
				ui.core.view.delegate_builder(
				{
					finished = function (view)
						ui.core.view_controller:close_modal_view (view)
					end
				}).build (v)

				get_sound_pool():play ("ui/SFX_Notice.wav")
				ui.core.view_controller:show_modal_view (v)

				return true
			end)
			underworld2.game_events:queue (e)
		end

		if notif.type == "FAME_DROP" then
			local e = underworld2.fame_drop_event.create (function (e)
				local v = ui.fame_drop_view.show (notif.fame, notif.time_passed, function() e:finalize() end)
				get_sound_pool():play ("ui/SFX_Notice.wav")
				return true
			end)
			underworld2.game_events:queue (e)
		end

	end
end

local activity_report_view

function underworld2.show_activity_log (passive_deals, rp_earned, fame_earned)
	if not underworld2.player then return end

	if activity_report_view then
		activity_report_view:finalize()
		activity_report_view = nil
	end
	
	if #passive_deals > 0 then
		local e = underworld2.activity_report_event.create (function(e)
			local now = underworld2.server_time()
			activity_report_view = ui.core.view.builder (ui.activity_log_view).build (passive_deals, rp_earned, fame_earned, now - underworld2.settings:get ("last_app_suspend", now))
			activity_report_view.delegate_builder (
			{
				finished = function()
					ui.core.view_controller:close_modal_view (activity_report_view)
					activity_report_view = nil
					collectgarbage ("collect")
					
					underworld2.player:lockUpdates()
					underworld2.network.collect_passive_deals (function()
						underworld2.player:unlockUpdates()
					end)

					local prev_level_id = underworld2.player:getLevel()
					underworld2.player:addRespectPoints (rp_earned)
					local new_level_id 	= underworld2.player:getLevel()
					if new_level_id ~= prev_level_id then
						underworld2.ui.main_view:show_levelup (new_level_id)
					end
					--TODO:
					e:finalize()
				end
			}).build (activity_report_view)
			ui.core.view_controller:show_modal_view (activity_report_view)

			return true
		end)
		underworld2.game_events:queue (e)
	end
end

function underworld2.network_state_error()
	local info_view = ui.information_view.create (L ("KEY_NETWORK_STATE_ERROR"), { ui.information_view.BUTTON_TRY_AGAIN })
	ui.core.view_controller:show_modal_view (info_view)
	net.location_provider():lockUpdates()
	info_view.delegate_builder (
	{
		finished = function()
			net.location_provider():unlockUpdates()
			ui.core.view_controller:close_modal_view (info_view)
			underworld2.network.repeat_last_request()
		end
	}).build(info_view)

	get_sound_pool():play ("ui/UI_Error.wav")
end

function underworld2.facebook_token_expired()
	ui.event_view.force_close()

	local info_view = ui.information_view.create (L ("KEY_FACEBOOK_TOKEN_EXPIRED"), { ui.information_view.BUTTON_RELOGIN })
	ui.core.view_controller:show_modal_view (info_view)
	info_view.delegate_builder (
	{
		finished = function()
			ui.core.view_controller:close_modal_view (info_view)
			underworld2.openid_show_relogin (true)
			underworld2.full_reset()
		end
	}).build(info_view)
end

function underworld2.start_game()
	initialized = false
	underworld2.show_main_screen()
	underworld2.login()
end

function underworld2.login()
	-- show updating if we do not have a valid location at this point
	-- TODO: wake lock
	local location = net.location_provider():getLocation()
	if not location:isValid() then
		ui.event_view.show (L ("KEY_LOCATION_RENEW"))
		local upd_conn_id
		upd_conn_id = events.connect (net.location_provider():getLocationUpdateSignalProxy(), function (location)
			ui.event_view.close()
			events.disconnect (net.location_provider():getLocationUpdateSignalProxy(), upd_conn_id)
			_process_login (net.Location (location))
		end)
	else
		_process_login (location)
	end
end

--[[ 
 In general this case is possible only if response was delivered after game suspended on more 
 than session duration time (or similar cases), game will attempt to relogin if client does not have
 more requests in the queue, and the running one, that lead to this error, allows silent relogin. 
 If that not the case then user will be asked to relogin manually (basically to notify him that we will not
 execute queued requests, because it's almost pointless)
]]
function underworld2.session_expired (try_silent)
	if not try_silent or underworld2.network.has_running_request() then
		net.location_provider():lockUpdates()
		underworld2.ui.show_message (L "KEY_FORCE_RELOGIN", function()
			underworld2.openid_show_relogin()
			underworld2.full_reset()
			net.location_provider():unlockUpdates()
		end)
	else
		ui.event_view.show()
		local location = net.Location (net.location_provider():getLocation())
		underworld2.network.reset_session()
		underworld2.network.login (location, function() ui.event_view.close() end)
	end
end

function underworld2.full_reset()
	social.jabber.disconnect()
	
	ui.event_view.force_close()

	local vc = ui.core.view_controller
	while vc:has_modal_views() do
		local v, i = next (vc.modal_views)
		vc:close_modal_view (v)
		v:finalize()
	end

	underworld2.inbox.finalize()

	if underworld2.ui.main_view then
		underworld2.ui.main_view:destroy()
	end

	underworld2.timed_events:reset()
	underworld2.game_events:reset()

	underworld2.Shops():getUpdatedSignalProxy():clear()
	underworld2.ShopsNetwork():getNetworkChangedSignalProxy():clear()

	if underworld2.Player_IsCreated() then
		underworld2.Player_Destroy()
		underworld2.player = nil
	end
	
	if _dev_events_setup then
		events.disconnect (ui.core.view_controller.instance:getMovedToBackgroundSignalProxy(), _dev_events_setup.background_conn_id)
		events.disconnect (ui.core.view_controller.instance:getMovedToForegroundSignalProxy(), _dev_events_setup.foreground_conn_id)
		_dev_events_setup = nil
	end

	underworld2.ui.status_panel	= nil
	underworld2.ui.stack 		= nil
	underworld2.ui.window 		= nil
	underworld2.ui.main_view	= nil

	initialized = false

	underworld2.network.reset_session()
	underworld2.network.reset_queue()
	underworld2.network.set_silent_retry (false)

	collectgarbage ("collect")
end

function underworld2.critical_server_error( text )
	ui.core.view_controller.instance:resetTouchEventFilter()
	ui.core.view_controller.instance:setTouchEventFilterEnabled (true)
	underworld2.tutorial:reset()
	ui.event_view.force_close()
	local info_view = ui.server_error.create (false, text)
	ui.core.view_controller:show_modal_view (info_view)
	net.location_provider():lockUpdates()
	info_view.delegate_builder (
	{
		finished = function()
			ui.core.view_controller:close_modal_view (info_view)
			underworld2.openid_show_relogin()
			underworld2.full_reset()
			net.location_provider():unlockUpdates()
		end
	}).build(info_view)

	get_sound_pool():play ("ui/UI_Error.wav")
end

function underworld2.http_connection_error(code)
	local invalid_http_code = { [404] = true, [502] = true, [504] = true }
	ui.core.view_controller.instance:setTouchEventFilterEnabled (false)
	local info_view = ui.server_error.create (invalid_http_code[code] or false, "HTTP "..code, true)
	ui.core.view_controller:show_modal_view (info_view)
	net.location_provider():lockUpdates()
	info_view.delegate_builder (
	{
		finished = function()
			ui.core.view_controller.instance:setTouchEventFilterEnabled (true)
			ui.core.view_controller:close_modal_view (info_view)
			underworld2.network.repeat_last_request()
			net.location_provider():unlockUpdates()
		end
	}).build(info_view)

	get_sound_pool():play ("ui/UI_Error.wav")
end

function underworld2.server_not_recognized()
	local info_view
	local show_browser = { 
		tag = ui.information_view.BUTTON_USER0, 
		text_style = ui.text_styles.normal,
		texture = "ui/btn_share:filtered",
		text = L ("KEY_BTN_BROWSER"),
		handler = function (button)
			os.device:openURL ("http://a-steroids.com/")
		end
	}
	info_view = ui.custom_error_view.create (L ("KEY_NETWORK_SERVER_NOT_RECOGNIZED_ERROR"), 
		{ 
			show_browser, 
			ui.information_view.BUTTON_TRY_AGAIN
		})
	net.location_provider():lockUpdates()
	ui.core.view_controller:show_modal_view (info_view)
	info_view.delegate_builder (
	{
		finished = function()
			net.location_provider():unlockUpdates()
			ui.core.view_controller:close_modal_view (info_view)
			underworld2.network.repeat_last_request()
		end
	}).build(info_view)

	get_sound_pool():play ("ui/UI_Error.wav")
end

function underworld2.user_not_allowed()
	local error_view = ui.error.create (L("KEY_LOGIN_ERROR_USER_NOT_ALLOWED"), false)
	ui.core.view_controller:show_modal_view (error_view)

	get_sound_pool():play ("ui/UI_Error.wav")
end

events.connect (net.location_provider():getFailedSignalProxy(), function (err)
	if ignore_location_errors then
		return
	end

	-- on iOS it's shown by the OS anyway
	if (os.device:getDeviceOSString() == "iOS") and
		((err == net.LocationErrorDenied) or
	 	 (err == net.LocationErrorNotSupported)) then 
		--setup some location(may be invalid), will force LocationProvider to call location_updated signal
		local location = net.location_provider():getLocation()
		net.location_provider():setMockLocation (location)
		return 
	end

	net.location_provider():lockUpdates()

	get_sound_pool():play ("ui/SFX_Notice.wav")

	local text_map = { 
		[net.LocationErrorNotSupported] = "KEY_LOCATION_ERROR_NOT_SUPPORTED",
		[net.LocationErrorDenied] = "KEY_LOCATION_ERROR_DENIED",
		[net.LocationErrorTimeout] = "KEY_LOCATION_ERROR_TIMEOUT",
		[net.LocationErrorUnknown] = "KEY_LOCATION_ERROR_UNKNOWN"
	}

	local action_map = {
		[net.LocationErrorDenied] = "KEY_SERVER_ERROR_SETTINGS"
	}

	local handler_map = {
		[net.LocationErrorDenied] = function()
			os.device:openOSSettings(os.settings.location)
		end
	}

	local err_text_key = text_map[err] or text_map[net.LocationErrorUnknown]

	local info_view

	local contact_us = { 
		tag = ui.information_view.BUTTON_USER0, 
		text_style = ui.text_styles.button_regular,
		texture = "ui/btn_ditch:filtered",
		text = L (action_map[err] or "KEY_BTN_CONTACT_US"),
		handler = function (button)
			local handler = handler_map[err]
			if handler then
				handler()
			else
				underworld2.shared.send_feedback(L("KEY_FB_SHARING_APP_NAME") .. " support request: location issue")
			end
		end
	}

	local ok = {
		tag = ui.information_view.BUTTON_USER1, 
		text_style = ui.text_styles.button_regular,
		texture = "ui/btn_lower_normal:filtered",
		text = L ("KEY_BTN_GOT_IT"),
		handler = function (button)
			info_view:finalize()
		end
	}
	
	info_view = ui.information_view.create (L (err_text_key), { contact_us, ok })
	ignore_location_errors = true
	ui.core.view_controller:show_modal_view (info_view)
	info_view.delegate_builder (
	{
		finished = function()
			if not net.location_provider():getLocation():isValid() then
				net.location_provider():setMockLocation (net.Location())
			end

			ui.core.view_controller:close_modal_view (info_view)
			net.location_provider():unlockUpdates()
		end
	}).build(info_view)
end)

net.location_provider():unlockUpdates()
