-- Bootsrap the engine runtime
local debug = debug
local ipairs = ipairs	
local tostring = tostring
local runtime = runtime

require = runtime.include
dofile	= runtime.reload

runtime.include ("lib/math.lua")

-----------------------------------------------------------------------------------
if InfoPrint then
-- Print function results in printing message to the debug log
	
	function print(...)
		local args = {...}
		if #args == 0 then return end
		local s = ""
		for i,v in pairs(args) do 
			s = s .. tostring(v)
		end
		DebugPrint(s)
	end

	if debug then
		debug.STP = runtime.reload "lib/StackTracePlus.lua"
		debug.traceback = debug.STP.stacktrace
	end

	-- Error print to error log, additionally supplying information about function
	function error(msg, level)
		level = level or 2
		local s = ""
		if level ~= 0 and debug then
			local info = debug.getinfo(level, "Sln")
			if info then
				s = "'" .. info.short_src .. "':"..info.currentline..": "
			end
		end 
		if not msg then msg = "" end
		s = s..msg
	
		if debug then s = s .. "\n" .. debug.STP.stacktrace() end
	
		ErrorPrint(s)
	end

else -- Nope, we don't 
	-- Try to do our best with logging facilities
	function InfoPrint( msg ) print(msg)  end
	function DebugPrint( msg ) print(msg) end
	function ErrorPrint( msg ) error(msg) end 	
	function CriticalErrorPrint(msg) error(msg) end
end

function string:split(sep)
        local sep, fields = sep or ":", {}
        local pattern = string.format("([^%s]+)", sep)
        self:gsub(pattern, function(c) fields[#fields+1] = c end)
        return fields
end

local JSON = runtime.reload "lib/JSON.lua"
assert(JSON)

function table.to_json( tbl, pretty )
	if pretty then
		return JSON:encode_pretty( tbl )
	else
		return JSON:encode( tbl )
	end
end

function table.from_json( json )
	return JSON:decode( json )
end

function table.set (array)
	local retval = { }
	for i, v in ipairs (array) do
		retval[v] = true
	end
	return retval
end

function table.find (t, value)
	for i, v in pairs (t) do
		if v == value then
			return i
		end
	end
	return nil
end
