local signal_interface = { }

function signal_interface.connect (signal, object, method)
	local object_type = type (object)
	if object_type == "function" then
		signal.slots[object] = true
	elseif object_type == "table" then
		local methods = signal.slots[object] or { }
		methods[method] = true
		signal.slots[object] = methods
	else
		error ("invalid argument type passed to signal_interface.connect, function or table expected.")
	end
end

function signal_interface.disconnect (signal, object, method)
	local object_type = type (object)
	if object_type == "function" then
		signal.slots[object] = nil
	elseif object_type == "table" then
		if method then
			signal.slots[object][method] = nil
		else
			signal.slots[object] = nil
		end
	else
		error ("invalid argument type passed to signal_interface.disconnect, function or table expected.")
	end
end

function signal_interface:disconnect_all()
	self.slots = { }
end

function signal_interface:notify (...)
	for object, value in pairs (self.slots) do
		if type (object) == "function" then
			object (...)
		else
			for method in pairs (value) do
				method (object, ...)
			end
		end
	end
end

signal = function()
	return setmetatable ({ slots = { } }, { __index = signal_interface })
end
