require "ui/text_styles.lua"
require "ui/format.lua"
require "ui/config.lua"
require "ui/information_view.lua"
require "social_ids.lua"

local view = setmetatable ({ }, { __index = ui.core.window })

local game_started

local builder = ui.core.view.builder (view)

function view:create()
	local obj = builder.build()
	return obj
end

local function process_start_game()
	if game_started then return end
	require("game.lua")
	game_started = true
	underworld2.start_game()
end

local function process_login_failed (text, handler)
	--[[local info_view = ui.information_view.create (text, { ui.information_view.BUTTON_TRY_AGAIN })
	ui.core.view_controller:show_modal_view (info_view)
	info_view.delegate_builder (
	{
		finished = function()
			ui.core.view_controller:close_modal_view (info_view)
		end
	}).build(info_view)]]
	if os.device:getNetworkType() == 0 then
		local info_view = ui.information_view.create (L ("KEY_NETWORK_STATE_ERROR"), { ui.information_view.BUTTON_TRY_AGAIN })
		ui.core.view_controller:show_modal_view (info_view)
		info_view.delegate_builder (
		{
			finished = function()
				ui.core.view_controller:close_modal_view (info_view)
			end
		}).build(info_view)

		get_sound_pool():play ("ui/UI_Error.wav")
	end
end

-- Facebook related shit
local function _fb_handle_app_requests( json ) end

local _login_view_shown = false
local _fb_session = {}

local function _fb_session_updated( logged_in, token, err )
	local failed = err and #err > 0 or false

	_fb_session.logged_in = not failed and (logged_in == 1)
	_fb_session.token     = token
	_fb_session.error     = err

	print( "_fb_session_updated ", tostring(logged_in), " ", token, " ", tostring(err))

	if failed and _login_view_shown then
		process_login_failed ("Login failed.")
	elseif _fb_session.logged_in then
		process_start_game()
	end
end


social.gms.update_listener( function(status) 
	print( "GMS status "..status)
	if status == social.gms.status.Connected then
		process_start_game()
	elseif status == social.gms.status.Failed and _login_view_shown then
		process_login_failed( "Google+ login failed", function() social.gms.login() end )
	end
end )

local function _create_btn_layout (text, ico_rid, rid, handler)
	local btn_layout = ui.HorizontalLayout()
	btn_layout:setItemSpacing (ui.config.hspacing)

	local btn = ui.ContainerButton()
	btn:setTexture (rid)
	local height = btn:getPreferredSize().y
	btn:setHorizontalSizePolicy (ui.SizePolicyFixed)
	btn:setVerticalSizePolicy (ui.SizePolicyMinimum)
	events.connect (btn:getClickedSignalProxy(), handler)
	if ico_rid then 
		local ico = ui.Picture()
		ico:setTexture (ico_rid)
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		btn_layout:addWidget (ico)
	end

	local text_label = ui.TextLabel()
	text_label:setTextStyle(ui.text_styles.login_btn.build())
	text_label:setWordWrap (false)
	text_label:setText (text)
	text_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
	btn_layout:addWidget (text_label)

	btn:setLayout (btn_layout)

	return btn, height
end

function view:build_ui()
	local w = ui.Widget()
	w:setOrigin (0, 0)
	w:setSize (1, 1)

	do
		local img = ui.Picture()
		img:setTexture (bundle.resources.wallpapers.login)
		
		local original_size = img:getPreferredSize()
		
		local scale = 1.0 / original_size.x
		local size = math.vec2 (1.0, original_size.y * scale)

		img:setPlacingMode (ui.Picture_PlacingModeStretch)
		img:setSize (size)
		img:setOrigin (0, 0)
		w:addChild (img)
	end

	local facebook_login_btn, facebook_login_btn_height = _create_btn_layout (
		L "KEY_LOGIN_BTN_FACEBOOK", "ui/login/ico_facebook_login:filtered", 
		"ui/login/btn_facebook_signup:filtered", function()
			underworld2.social_ids.login("FACEBOOK", function(successful, err)
				if not successful then
					process_login_failed(err)
					return true
				else
					process_start_game()
				end
			end)
		end
	)

	local buttons_width = math.max( 0.7, facebook_login_btn:getPreferredSize().x )

	facebook_login_btn:setHidden( not social.facebook.available() )

	local google_play_login = _create_btn_layout (
		L "KEY_LOGIN_BTN_GOOGLE", "ui/login/ico_facebook_login:filtered", 
		"ui/login/btn_facebook_signup:filtered", function()
			if social.gms.available() and not social.gms.logged_in() then
				social.gms.login()
			else
				process_start_game()
			end
		end
	)

	buttons_width = math.max( buttons_width, google_play_login:getPreferredSize().x )

	google_play_login:setHidden( true or not social.gms.available() )	

	local skip_login = _create_btn_layout (
		L "KEY_LOGIN_BTN_SKIP", nil, 
		"ui/login/btn_skip:filtered", function()
				process_start_game()
		end
	)

	buttons_width = math.max( buttons_width, skip_login:getPreferredSize().x )

	do
		local container = ui.Frame()

		container:setBordersTexture ("ui/frame_modal_light:filtered")
		container:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
		
		local layout = ui.VerticalLayout()
		layout:setItemSpacing (ui.config.vspacing)

		layout:addWidget (ui.Widget())

		do
			local notice_label = ui.TextLabel()
			notice_label:setTextStyle (ui.text_styles.login_notice.build())
			notice_label:setTextAlignment (ui.AlignmentVerticalTop + ui.AlignmentHorizontalCenter)
			notice_label:setText (L "KEY_LOGIN_SOCIAL_LABEL")
			notice_label:setWordWrap(true)
			notice_label:setRowsCount(2)
			notice_label:setVerticalSizePolicy (ui.SizePolicyMinimum)
			notice_label:setWidth( buttons_width * 0.9 )
			notice_label:setHorizontalSizePolicy (ui.SizePolicyFixed)

			local hlayout = ui.HorizontalLayout()
			hlayout:addWidget (ui.Widget())
			hlayout:addWidget (notice_label)
			hlayout:addWidget (ui.Widget())
			layout:addItem (hlayout)
		end

		do
			local hlayout = ui.HorizontalLayout()
			hlayout:addWidget (ui.Widget())
			facebook_login_btn:setWidth(buttons_width)
			hlayout:addWidget (facebook_login_btn)
			hlayout:addWidget (ui.Widget())
			layout:addItem (hlayout)
		end

		do
			local hlayout = ui.HorizontalLayout()
			hlayout:addWidget (ui.Widget())
			google_play_login:setWidth(buttons_width)
			hlayout:addWidget (google_play_login)
			hlayout:addWidget (ui.Widget())
			layout:addItem (hlayout)
		end

		do
			local sep = ui.Picture()
			sep:setTexture ("ui/gold_line_large:filtered")
			local sep_height = sep:getPreferredSize().y
			sep:setHeight (sep_height)
			sep:setWidth(buttons_width * 0.9)
			sep:setPlacingMode (ui.Picture_PlacingModeStretch)
			sep:setVerticalSizePolicy (ui.SizePolicyFixed)
			sep:setHorizontalSizePolicy (ui.SizePolicyFixed)

			local hlayout = ui.HorizontalLayout()
			hlayout:addWidget (ui.Widget())
			hlayout:addWidget (sep)
			hlayout:addWidget (ui.Widget())
			layout:addItem (hlayout)
		end
		
		layout:addWidget (ui.Widget())

		do
			local hlayout = ui.HorizontalLayout()
			hlayout:addWidget (ui.Widget())
			skip_login:setWidth(buttons_width)
			hlayout:addWidget (skip_login)
			hlayout:addWidget (ui.Widget())
			layout:addItem (hlayout)
		end

		layout:addWidget (ui.Widget())

		container:setLayout (layout)

		local size = container:getPreferredSize()
		container:setSize (1 - 4 * ui.config.hspacing, size.y + 2 * ui.config.vspacing)
		container:setOrigin ( 2 * ui.config.hspacing, 1 - size.y - 4 * ui.config.vspacing)

		w:addChild (container)
	end

	return w
end

local function has_social_login()
	return underworld2.social_ids.supported() --[[ or social.gms.available() ]]
end

local function logged_in()
	return underworld2.social_ids.logged_in() --[[ or social.gms.logged_in()]]
end

-- we do not need view table after ui::View was create
if not logged_in() and has_social_login() then
	ui.core.view_controller.instance:setRootView (view.create()._pdata)
else
	process_start_game()
end

function underworld2.openid_show_relogin( force_logout )
	print("openid_show_relogin")
	game_started = false

	if force_logout then 
		underworld2.social_ids.logout( "FACEBOOK" )
	end

	ui.core.view_controller.instance:setRootView (view.create()._pdata)
end