require "network/online_controller.lua"
require "lib/signal.lua"

local controller = setmetatable ( { }, { __index = network.online_controller } )

local __get_child_for_name = network.child_for_name

local function __on_completed (request, result)
	local on_completed 	= request.on_completed
	local delegate 		= request.delegate
	if on_completed then
		if delegate then
			on_completed (delegate, result)
		else
			on_completed (result)
		end
	end
end

local __handlers =
{
	["/api/buy.do"] = function (self, request)
		local response = 
		{
			name = "method",
			children = 
			{
				{ name = "dealResult", text = "SUCCESS" },
				{ name = "payOffPrice" },
				{ name = "freeEscapeAllowed" },
				{ name = "courierWasChased" },
				{ name = "chaseRpReward" },
				{ name = "location" }
			}
		}

		__on_completed (request, response)
	end,

	["/api/sell.do"] = function (self, request)
		local response = 
		{
			name = "method",
			children = 
			{
				{ name = "dealResult", text = "SUCCESS" },
				{ name = "payOffPrice" },
				{ name = "freeEscapeAllowed" },
				{ name = "courierWasChased" },
				{ name = "chaseRpReward" },
				{ name = "location" }
			}
		}

		__on_completed (request, response)
	end,

	["/api/premiumCompleteDeal.do"] = function (self, request)
		__on_completed (request)
	end,

	["/api/completedeal.do"] = function (self, request)
		__on_completed (request)
	end,

	["/api/boost.do"] = function (self, request)
		__on_completed (request)
	end,
}

function controller.create()
	local self = setmetatable ({ }, { __index = controller })
	self.request_completed = signal()
	return self
end

function controller:send (request) 
	local handler = __handlers [request.url]
	underworld2.timed_events:queue (0.0, function() 
		handler (self, request)
		self.request_completed:notify (request.url)
	end)
end

network.tutorial_online_controller = controller
