require "ui/event_view.lua"

local facebook_sharing = {}
underworld2.facebook_sharing = facebook_sharing

local post_url = "http://underworld.zone"

local function base_url()
	return bundle.resources.social
end

local function get_key(key)
	return string.format("KEY_FB_SHARING_%s", key)
end

local function get_string(key)
	return L(get_key(key))
end

local data = {
	levelup = {
		titles = {
			get_string "LEVELUP_TITLE_1",
			get_string "LEVELUP_TITLE_2",
			get_string "LEVELUP_TITLE_3",
			get_string "LEVELUP_TITLE_4",
			get_string "LEVELUP_TITLE_5",
			get_string "LEVELUP_TITLE_6",
			get_string "LEVELUP_TITLE_7",
			get_string "LEVELUP_TITLE_8",
		},
		descriptions = {
			get_key "LEVELUP_DESCRIPTION_1",
			get_key "LEVELUP_DESCRIPTION_2",
			get_key "LEVELUP_DESCRIPTION_3",
			get_key "LEVELUP_DESCRIPTION_4",
			get_key "LEVELUP_DESCRIPTION_5",
			get_key "LEVELUP_DESCRIPTION_6",
			get_key "LEVELUP_DESCRIPTION_7",
			get_key "LEVELUP_DESCRIPTION_8"
		},
		icons = setmetatable ( { }, 
		{ 	__index = function( t, k ) 
				local url = string.format ("%s/level/share_rank_%i.png", base_url(), k-1)
				rawset (t, k, url)
				return url
			end
		}),
		levels = {
			{ 1,  100 },
			{ 25, 100 },
			{ 1,  10  },
			{ 10, 25  },
			{ 1,  100 },
			{ 1,  100 },
			{ 1,  25  },
			{ 1,  100 }
		}
	},
	crew = {
		COURIERS = {
			titles = {
				get_string "RUNNER_TITLE_1",
				get_string "RUNNER_TITLE_2",
				get_string "RUNNER_TITLE_3",
			},
			descriptions = {
				get_string "RUNNER_DESCRIPTION_1_2",
				get_string "RUNNER_DESCRIPTION_3",
				get_string "RUNNER_DESCRIPTION_4",
				get_string "RUNNER_DESCRIPTION_5",
				get_string "RUNNER_DESCRIPTION_6",
				get_string "RUNNER_DESCRIPTION_7",
				get_string "RUNNER_DESCRIPTION_8",
			},
			icons = {
				string.format("%s/crew/runners/share_runner_1.png", base_url()),
				string.format("%s/crew/runners/share_runner_1.png", base_url()),
				string.format("%s/crew/runners/share_runner_2.png", base_url()),
				string.format("%s/crew/runners/share_runner_3.png", base_url()),
				string.format("%s/crew/runners/share_runner_4.png", base_url()),
				string.format("%s/crew/runners/share_runner_5.png", base_url()),
				string.format("%s/crew/runners/share_runner_6.png", base_url()),
			}
		},
		ITEM = {
			titles = {
				get_string "SUPPLIER_TITLE_1",
				get_string "SUPPLIER_TITLE_2",
				get_string "SUPPLIER_TITLE_3",
			},
			descriptions = {
				get_string "SUPPLIER_DESCRIPTION_1",
				get_string "SUPPLIER_DESCRIPTION_2",
				get_string "SUPPLIER_DESCRIPTION_3",
				get_string "SUPPLIER_DESCRIPTION_4",
				get_string "SUPPLIER_DESCRIPTION_5",
				get_string "SUPPLIER_DESCRIPTION_6",
				get_string "SUPPLIER_DESCRIPTION_7",
			},
			icons = {
				string.format("%s/crew/supplier/share_supplier_1.png", base_url()),
				string.format("%s/crew/supplier/share_supplier_1.png", base_url()),
				string.format("%s/crew/supplier/share_supplier_2.png", base_url()),
				string.format("%s/crew/supplier/share_supplier_3.png", base_url()),
				string.format("%s/crew/supplier/share_supplier_4.png", base_url()),
				string.format("%s/crew/supplier/share_supplier_5.png", base_url()),
				string.format("%s/crew/supplier/share_supplier_6.png", base_url()),
			}
		},
		WALLET = {
			titles = {
				get_string "ACCOUNTANT_TITLE_1",
				get_string "ACCOUNTANT_TITLE_2",
				get_string "ACCOUNTANT_TITLE_3",
			},
			descriptions = {
				get_string "ACCOUNTANT_DESCRIPTION_1",
				get_string "ACCOUNTANT_DESCRIPTION_2",
				get_string "ACCOUNTANT_DESCRIPTION_3",
				get_string "ACCOUNTANT_DESCRIPTION_4",
				get_string "ACCOUNTANT_DESCRIPTION_5",
				get_string "ACCOUNTANT_DESCRIPTION_6",
				get_string "ACCOUNTANT_DESCRIPTION_7",
				get_string "ACCOUNTANT_DESCRIPTION_8",
				get_string "ACCOUNTANT_DESCRIPTION_9",
			},
			icons = {
				string.format("%s/crew/accountants/share_accountant_1.png", base_url()),
				string.format("%s/crew/accountants/share_accountant_1.png", base_url()),
				string.format("%s/crew/accountants/share_accountant_2.png", base_url()),
				string.format("%s/crew/accountants/share_accountant_3.png", base_url()),
				string.format("%s/crew/accountants/share_accountant_4.png", base_url()),
				string.format("%s/crew/accountants/share_accountant_5.png", base_url()),
				string.format("%s/crew/accountants/share_accountant_6.png", base_url()),
				string.format("%s/crew/accountants/share_accountant_7.png", base_url()),
				string.format("%s/crew/accountants/share_accountant_8.png", base_url()),
			}
		}
	}
}

function facebook_sharing.create_button ( text, reward, handler)
	local btn_layout = ui.HorizontalLayout()
	btn_layout:setItemSpacing (ui.config.hspacing)

	local btn = ui.ContainerButton()
	btn:setTexture ("ui/btn_share:filtered")
	local height = btn:getPreferredSize().y
	--btn:setHorizontalSizePolicy (ui.SizePolicyFixed)
	btn:setVerticalSizePolicy (ui.SizePolicyMinimum)
	events.connect (btn:getClickedSignalProxy(), handler)

	btn_layout:addWidget(ui.Widget())

	local ico = ui.Picture()
	ico:setTexture ("ui/ico_facebook_share:filtered")
	ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	btn_layout:addWidget (ico)


	local text_label = ui.TextLabel()
	text_label:setTextStyle (ui.text_styles.button_regular.build())
	text_label:setWordWrap (false)
	text_label:setText (text)
	text_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
	btn_layout:addWidget (text_label)

	if reward and reward > 0 then
		btn_layout:addWidget (ui.Widget())
		local ico = ui.Picture()
		ico:setTexture ("ui/ico_gold_small:filtered")
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		btn_layout:addWidget (ico)


		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.button_regular.build())
		text_label:setWordWrap (false)
		text_label:setText (tostring (reward))
		text_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
		btn_layout:addWidget (text_label)
	end

	btn_layout:addWidget(ui.Widget())

	btn:setLayout (btn_layout)

	return btn, height
end

function facebook_sharing.levelup_bonus( player )
	local current_level = underworld2.LevelsDatabase():get (player:getLevel())
	return current_level:getSharingReward()
end

function facebook_sharing.member_arrived_bonus( type, level )
	return underworld2.BalanceValuesDatabase():get ("crewShareReward", 5)
end

function facebook_sharing.levelup( player, handler )
	ui.event_view.show( get_string "SHARING" )

	local level = player:getLevel()
	local ids_list = {}

	for i,v in ipairs (data.levelup.levels) do 
		if v[1] <= level and v[2] >= level then
			table.insert (ids_list, i)
		end
	end

	local id = ids_list[math.dice(#ids_list)]

	local current_level = underworld2.LevelsDatabase():get (level)

	social.facebook.graph {
		method = "POST",
		path = "me/feed",
		write_permissions = { "publish_actions" },
		parameters = {
			name	    = data.levelup.titles[id],
			description	= L (data.levelup.descriptions[id], { rank = underworld2.StringsDatabase():get (current_level:getNameKey()) } ),
			picture	    = data.levelup.icons[level],
			caption     = get_string "APP_NAME",
			link	    = post_url
		},
		callback = function(operation)
			ui.event_view.close()
			if not operation:hasFailed() then
				ui.event_view.show()
				underworld2.network.share_reward {
					callback = function() 
						ui.event_view.close()
						handler(true) 
					end,
					type = "levelUp",
				}
			else
				handler( false )
			end
		end
	}
end

function facebook_sharing.member_arrived( id, type, level, handler )
	ui.event_view.show( get_string "SHARING" )

	social.facebook.graph {
		method = "POST",
		path = "me/feed",
		write_permissions = { "publish_actions" },
		parameters = {
			name	    = data.crew[type].titles[math.dice(#data.crew[type].titles)],
			description	= data.crew[type].descriptions[level],
			picture	    = data.crew[type].icons[level],
			caption     = get_string "APP_NAME",
			link	    = post_url
		},
		callback = function(operation)
			ui.event_view.close()
			if not operation:hasFailed() then
				ui.event_view.show()
				underworld2.network.share_reward {
					callback = function() 
						ui.event_view.close()
						handler(true) 
					end,
					type = "crew",
					upgradeId = id,
					level = level
				}
			else
				handler( false )
			end
		end
	}
end

function facebook_sharing.bust( item_id, count, courier_idx, geo_name, handler )
	ui.event_view.show( get_string "SHARING" )

	local courier_name   = underworld2.couriers.names[courier_idx]
	local courier_gender = underworld2.couriers.gender[courier_idx]
	local commodity_name = underworld2.StringsDatabase():get (underworld2.CommoditiesDatabase():get (item_id):getNameKey())

	local icon = string.format ("%s/busted/share_busted_%i.png", base_url(), item_id)

	social.facebook.graph {
		method = "POST",
		path = "me/feed",
		write_permissions = { "publish_actions" },
		parameters = {
			name	    = get_string "BUST_POST_TITLE",
			description	= L (string.format( get_key "BUST_POST_TEXT_%i", math.dice(5)), { 
				name     = courier_name, 
				gender   = courier_gender, 
				location = geo_name,
				item     = commodity_name 
			}),
			picture	    = icon,
			caption     = get_string "APP_NAME",
			link	    = post_url
		},
		callback = function(operation)
			ui.event_view.close()
			if not operation:hasFailed() then
				handler(true)
			else
				handler( false )
			end
		end
	}
end