require "util/delegate.lua"

local jabber = util.delegator ( {ext = { proto = { }, client = { } }} )

function jabber.get_stanza_child (stanza, name, ns)
	if stanza.children then
		for i, child in ipairs (stanza.children) do 
			if child.name and child.name == name then
				if ns == nil then
					return child
				elseif child.xmlns == ns then
					return child
				end
			end
		end
	end
	return nil
end

function jabber.jid_without_resource (jid)
	local resource = string.find (jid, "/", 1, true)
	if resource then
		return string.sub (jid, 1, resource - 1)
	end
	return jid
end

function jabber.resource_from_jid (jid)
	local resource = string.find (jid, "/", 1, true)
	if resource then
		return string.sub (jid, resource + 1, #jid)
	end
	return nil
end

function jabber.connect (username, uid, password)
	if listener then
		if DEBUG then
			ErrorPrint ("XMPP: connection already installed")
		end
		return
	end

	local xmppListener = social.xmppListener()

	jabber.domain_name = xmppListener:getDomainName()
	jabber.username = username
	jabber.uid = uid
	jabber.jid = string.format ("%s@%s", uid, jabber.domain_name)
	

	xmppListener:setJid (jabber.jid)
	xmppListener:setNickname (username)
	xmppListener:setPassword (password)
	xmppListener:onAuthStateChanged (function(auth_ok)
			jabber:notify ("auth_state_changed", auth_ok)
		end)
	xmppListener:onConnectionStarted (function()
		xmppListener:registerHandler ("", "", "", function (stanza)
			jabber:notify ("stanza_received", stanza)
		end)
	end)
	jabber.listener = xmppListener
	xmppListener:connect()
end

function jabber.disconnect ()
	if not listener then
		return
	end

	jabber:clear_delegates()
	jabber.listener:disconnect()

	jabber.domain_name = nil
	jabber.username = nil
	jabber.uid = nil
	jabber.jid = nil
	jabber.listener = nil
end

local _inc_id_gen = 0

function jabber.send (stanza, handler)
	assert (jabber.listener)
	if handler then
		stanza.id = tostring (_inc_id_gen)
		_inc_id_gen = _inc_id_gen + 1
	end
	jabber:notify ("stanza_sent", stanza)
	jabber.listener:send (stanza, handler or function() end)
end


function jabber.send_message (jid, text, user_data)
	local payload = user_data or { }
	table.insert (payload, { 
			name = "body",
			children = { { text = text } }
		})


	assert (jabber.listener)
	jabber.send (
	{
		name 	= "message",
		--from 	= jabber.listener:getJid(),
		to 		= jid,
		type	= "chat",
		children = payload
	})
end

function jabber.load_ext (name)
	print ("XMPP: loading ext "..name)
	local ext = require ("social/xmpp_ext/"..name..".lua")

	ext.init (jabber)

	jabber.ext[ext.name] = ext
	if ext.alias then
		if jabber[ext.alias] then
			ErrorPrint (string.format ("jabber: ext alias=%s conflict", ext.alias))
		end
		jabber[ext.alias] = ext
	end
end


social.jabber = social.jabber or jabber
