local archive = { 
	name = "client.archive",
	alias = "archive",
}

local j

local _cache = { }
local _do_cache_flush = false

local function _push (jid, sid, ts, text)

	local uid = j.jid_without_resource (jid)
	local conv = _cache[uid] or { }
	
	table.insert (conv, { sid=sid, ts=ts, text=text })

	_cache[uid] = conv
	_do_cache_flush = true
end

function _get_storage_path (uid)
	local hash = md5 (uid)
	local path = "/home/messaging/"..hash..".conv"
	return path
end

function archive.init (jabber)
	j = jabber
	j:add_delegate (archive)
end

function archive.auth_state_changed (_, online)
	if not online and _do_cache_flush then
		for uid, _ in pairs (_cache) do
			
			local conv = archive.get (uid)

			local dump = table.dump (conv)
			local path = _get_storage_path (uid)
			file.write_string (path, dump)
			
			_cache[uid] = nil
		end
		_do_cache_flush = false
	end
end

function archive.stanza_received (_, stanza)
	if tostring (stanza.name) == "message" and tostring (stanza.type) == "chat" then
		local text 	= j.get_stanza_child (stanza, "body").children[1].text 
		local delay = j.get_stanza_child (stanza, "delay")
		if delay then
			local ts = delay.stamp
			_push (stanza.from, stanza.from, ts, text)
		else
			local ts = j.date_time.delay_stamp()
			_push (stanza.from, stanza.from, ts, text)
		end
	end
end

function archive.stanza_sent (_, stanza)
	if tostring (stanza.name) == "message" and tostring (stanza.type) == "chat" then
		local text 	= j.get_stanza_child (stanza, "body").children[1].text
		_push (stanza.to, nil, j.date_time.delay_stamp(), text)
	end
end

function archive.get (jid)
	local uid = j.jid_without_resource (jid)
	local path = _get_storage_path (uid)
	local conv
	if file.exists (path) then
		conv = dofile (path)
		local conv_cache = _cache[uid]
		if conv_cache then
			local tinsert = table.insert
			for _, msg in ipairs (conv_cache) do
				tinsert (conv, msg)
			end
		end
		file.write_string (path, table.dump (conv))
		_cache[uid] = nil
	else
		conv = _cache[uid]
	end
	
	return conv
end

return archive

