
require "ui/event_view.lua"

local settings = underworld2.settings

local function __social_info ( network )
	return settings:get( string.format ("social.%s", network), nil )
end

local __known_networks = {
	"FACEBOOK"
}

local __network_supported = {
	FACEBOOK = social.facebook.available()
}

local __status = {}

underworld2.social_ids = {
	handlers = {
		FACEBOOK = {
			login = social.facebook.login,
			logout = social.facebook.logout,
			callbacks = {

			}
		}
	}
}

for i,v in ipairs(__known_networks) do
	__status[v] = __network_supported[v] and __social_info(v)

	if __status[v] then
		table.insert( underworld2.social_ids, __status[v] )
	end
end

function underworld2.social_ids.supported()
	for i,v in pairs(__network_supported) do
		if v then return true end
	end

	return false
end

function underworld2.social_ids.logged_in()
	for i,v in pairs(__network_supported) do
		if v and __social_info(i) then 
			return true
		end
	end

	return false
end

-- handler -> successful, error
function underworld2.social_ids.login( network, handler )
	if not __network_supported[network] then 
		handler( false, L ("KEY_ERROR_SOCIAL_NOT_AVAILABLE", network) )
		return false
	 end

	
	table.insert (underworld2.social_ids.handlers[network].callbacks, handler)

	if __social_info(network) then
		for i,handler in ipairs (underworld2.social_ids.handlers[network].callbacks) do
			handler (true, nil)
			underworld2.social_ids.handlers[network].callbacks = {}
		end
	else
		ui.event_view.show( L "KEY_LOGIN_PROCESSING" )
		table.insert( underworld2.social_ids.handlers[network].callbacks, ui.event_view.close )

		underworld2.social_ids.handlers[network].login()
	end

	return true
end

function underworld2.social_ids.logout( network )
	if __network_supported[network] then 
		 underworld2.social_ids.handlers[network].logout()
		 settings.social[network] = nil
		 settings:flush()

		for i,n in ipairs(underworld2.social_ids) do
			if n.network == network then
				table.remove( underworld2.social_ids, i )
				break
			end
		end
	end
end

local function __logged_in( network, successful, err, id, avatar_url, token )
	local data = successful and {
		id         = id,
		avatar_url = avatar_url,
		token      = token,
		network       = network
	} or nil

	settings.social[network] = data

	settings:flush()

	local entry_found = false

	for i,n in ipairs(underworld2.social_ids) do
		if n.network == network then
			if not successful then
				table.remove( underworld2.social_ids, i )
			else
				print( "__logged_in: updating token at ", i )
				underworld2.social_ids[i] = data
			end

			entry_found = true

			break
		end
	end

	if not entry_found and successful then
		print( "__logged_in: adding new entry " )
		table.insert (underworld2.social_ids, data)
	end

	entry_found = false

	for i,n in ipairs(underworld2.social_ids) do
		if n.network == network then
			assert(token == n.token, "Invalid token")
			entry_found = true
		end
	end

	assert(entry_found or not successful, "Entry not found")

	for i,handler in ipairs (underworld2.social_ids.handlers[network].callbacks) do
		handler (successful, err)
	end

	underworld2.social_ids.handlers[network].callbacks = {}

	if successful then
		if network == "FACEBOOK" then
			analytics.mat.set_facebook_id (id)
		end
	end
end
-- Facebook specific

function underworld2.social_ids.handlers.FACEBOOK.session_update (logged_in, token, err)
	local successful = not err or #err == 0 and #token > 0

	print( "FACEBOOK.session_update: ", logged_in, " token ", token, " error ", err )
 
	if not successful or logged_in == 0 then
		__logged_in( "FACEBOOK", false, err )
	else
		local entry_found

		for i,n in ipairs(underworld2.social_ids) do
			if n.network == "FACEBOOK" then
				entry_found = n
				break
			end
		end

		if not entry_found then
			social.facebook.graph {
				path = "me",
				parameters = {
					fields = "id,picture.type(square).height(160)"
				},
				callback = function (response)
					if response:hasFailed () then
						__logged_in ("FACEBOOK", false, response:getErrorMessage())
					else
						local resp     = response:getLuaResponse() 
						local previous = __social_info("FACEBOOK")

						if not previous or previous.id == resp.id then
							__logged_in ("FACEBOOK", true,"", resp.id, resp.picture.data.url, token)
						else
							__logged_in ("FACEBOOK", false, L "KEY_ERROR_SOCIAL_USER_HAS_CHANGED")
						end
					end
				end
			}
		else
			__logged_in ("FACEBOOK", true,"", entry_found.id, entry_found.avatar_url, token)
		end
	end
end

print( "_fb_start")
social.facebook.start { session_callback = underworld2.social_ids.handlers.FACEBOOK.session_update }