
local stages =
{
	-- [01] "I wouldn''t throw the word \"Trust\" around here, if I were you. It gets you 6 feet deeper or at least a new 9mm piercing in your back. No, never trust people."
	function (t)
			local skill_group = underworld2.SkillsDatabase():getGroup ("SMUGGLERSHIP")
			local skill = skill_group:getSkill ("ITEM")
			if underworld2.player:getSkillLevel (skill:getId()) >= 2 then
				t:skip_to_stage (10)
				t:next_stage()
				return
			else
				local pending_skills = underworld2.player:getPendingSkills()
	
				for i = 0, pending_skills:size() - 1 do
					local s = pending_skills[i]
					if s:getSkillId() == skill:getId() then
						t:skip_to_stage (9)
						t:next_stage()
						return
					end	
				end
			end

			underworld2.ui.status_panel:disable()
			underworld2.ui.main_view:show_map (false)
			underworld2.ui.main_view:lock_map()

			ui.preloadTexture ("ui/characters/ch_advisor_modal:filtered")
			ui.preloadTexture ("ui/tutorial/tutorial_finger_ill:filtered")
			ui.preloadTexture ("ui/modal_wide_window:filtered")

			local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_CREW_01"), "ui/characters/ch_advisor_modal:filtered")
			ui.core.view.delegate_builder (
			{
				finished = function (self)
					ui.core.view_controller:close_modal_view (self)
					t:next_stage()
				end
			}).build (view)

			ui.core.view_controller:show_modal_view (view)

			return nil, "message1"
	end,

	-- [02] "BUT: you need people you can rely on to get the job done. Guys and gals who know the streets and the biz. Or promising prospects as yourself."
	function (t)
		local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_CREW_02"), "ui/characters/ch_advisor_modal:filtered")
			ui.core.view.delegate_builder (
			{
				finished = function (self)
					ui.core.view_controller:close_modal_view (self)
					t:next_stage()
				end
			}).build (view)

			ui.core.view_controller:show_modal_view (view)
			return nil, "message2"
	end,

	-- [03] "I want you to check out some dossiers. Good people. Don't trust them, but trust their ability to help you run a business."
	function (t)
		local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_CREW_03"), "ui/characters/ch_advisor_modal:filtered")
			ui.core.view.delegate_builder (
			{
				finished = function (self)
					ui.core.view_controller:close_modal_view (self)
					t:next_stage()
				end
			}).build (view)

			ui.core.view_controller:show_modal_view (view)
			return nil, "message3"
	end,

	-- [04] Action: tap-finger is pointing at the Crew option in the men
	function (t)
		local btn = underworld2.ui.main_view.ui.btn_crew
		ui.core.view_controller.instance:setTouchEventFilter (btn:mapToRoot (btn:getOrigin()), btn:getSize())
		local pos = btn:mapToRoot (btn:getOrigin()) + btn:getSize() * 0.5
		local finger = underworld2.tutorial:create_tap_view (pos, math.pi)
		local conn_id 
		conn_id = events.connect (btn:getClickedSignalProxy(), function()
			ui.core.view_controller.instance:setTouchEventFilter(math.vec2 (0, 0), math.vec2 (0, 0))
			underworld2.ui.window:remove_subview (finger)
			finger:finalize()
			events.disconnect (btn:getClickedSignalProxy(), conn_id)
			underworld2.timed_events:queue (underworld2.server_time() + 1.5, function() t:next_stage() end)
		end)
		underworld2.ui.window:add_subview (finger)
		return function()
			ui.core.view_controller.instance:resetTouchEventFilter()
		end, "open_crew_view"
	end,

	-- [05] "Ah, yeah, supplies. A supplier is crucial. Having one around will open new sources of goods. Wanna tap into Weed deals? Here's your chance."
	function (t)
		local commodity = underworld2.CommoditiesDatabase():get (3)
		local localized_name = underworld2.StringsDatabase():get (commodity:getNameKey())
		local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_CREW_06", localized_name), "ui/characters/ch_advisor_modal:filtered")
		ui.core.view.delegate_builder (
		{
			finished = function (self)
				ui.core.view_controller:close_modal_view (self)
				t:next_stage()
			end
		}).build (view)

		ui.core.view_controller:show_modal_view (view)

		return nil, "message4"
	end,

	-- [06] Action: tap-finger is pointing at the Suppliers tab
	function (t)
		local window = underworld2.ui.stack:top()
		local btn = window.skills[2].button -- suppliets tab
		ui.core.view_controller.instance:setTouchEventFilter (btn:mapToRoot (btn:getOrigin()), btn:getSize())
		local pos = btn:mapToRoot (btn:getOrigin()) + btn:getSize() * 0.5
		local finger = underworld2.tutorial:create_tap_view (pos, 0.0)
		underworld2.ui.window:add_subview (finger)

		local conn_id
		conn_id = events.connect (btn:getClickedSignalProxy(), function()
			underworld2.ui.window:remove_subview (finger)
			events.disconnect (btn:getClickedSignalProxy(), conn_id)
			window:disable()
			underworld2.timed_events:queue (underworld2.server_time() + 0.5, function() 
				ui.core.view_controller.instance:resetTouchEventFilter()
				t:next_stage()
			end)
		end)

		return nil, "switch_to_suppliers"
	end,

	-- [07] Action: tap-finger is pointing at the "Hire" button for second supplier
	function (t)
		local window = underworld2.ui.stack:top()
		window.skills_tab:setEnabled (false)
		window.upgrades_scroll:setScrollEnabled (false)
		window:enable()
		local wrapper = window.rows[2]
		
		local btn = wrapper.buttons.buy
		local pos = btn:mapToRoot (btn:getOrigin()) + btn:getSize() * 0.5
		local finger = underworld2.tutorial:create_tap_view (pos, 0.0)
		underworld2.ui.window:add_subview (finger)

		local conn_id
		conn_id = events.connect (btn:getClickedSignalProxy(), function()
			underworld2.ui.window:remove_subview (finger)
			events.disconnect (btn:getClickedSignalProxy(), conn_id)
			ui.core.view_controller.instance:setTouchEventFilter (math.vec2 (0, 0), math.vec2 (0, 0))
			--t:next_stage()
		end)

		return nil, "hire"
	end,

	-- [08] Action: tap-finger is pointing at the "Now" button
	function (t)
		local window = underworld2.ui.stack:top()
		local wrapper = window.rows[2]

		local btn = wrapper.buttons.boost
		local pos = btn:mapToRoot (btn:getOrigin()) + btn:getSize() * 0.5
		local finger = underworld2.tutorial:create_tap_view (pos, 0.0)
		underworld2.ui.window:add_subview (finger)

		ui.core.view_controller.instance:setTouchEventFilter (btn:mapToRoot (btn:getOrigin()), btn:getSize())

		local conn_id
		local timed_event_id

		conn_id = events.connect (btn:getClickedSignalProxy(), function()
			underworld2.ui.window:remove_subview (finger)
			events.disconnect (btn:getClickedSignalProxy(), conn_id)
			underworld2.timed_events:unqueue (timed_event_id)
			ui.core.view_controller.instance:resetTouchEventFilter()
			t:next_stage()
		end)

		local pending_skill = underworld2.player:getPendingSkill (wrapper.skill:getId(), wrapper.level:getId())

		timed_event_id = underworld2.timed_events:queue (pending_skill:getDeliveryEndTime(), function()
			underworld2.ui.window:remove_subview (finger)
			events.disconnect (btn:getClickedSignalProxy(), conn_id)
			ui.core.view_controller.instance:resetTouchEventFilter()
			t:next_stage()
		end)

		return nil, "boost"
	end,

	-- [09] wait for add to crew
	function (t)
		return nil, "add_to_crew"
	end,

	-- [10]
	function (t)
		local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_CREW_11"), "ui/characters/ch_advisor_modal:filtered")
		ui.core.view.delegate_builder (
		{
			finished = function (self)
				ui.core.view_controller:close_modal_view (self)
				
				local window = underworld2.ui.stack:top()
				if window:get_type() == ui.crew_view then
					window.skills_tab:setEnabled (true)
					window.upgrades_scroll:setScrollEnabled (true)
				end
				
				underworld2.ui.status_panel:enable()
				underworld2.ui.main_view:unlock_map()

				ui.removeTextureFromPreloadList ("ui/characters/ch_advisor_modal:filtered")
				ui.removeTextureFromPreloadList ("ui/tutorial/tutorial_finger_ill:filtered")
				ui.removeTextureFromPreloadList ("ui/modal_wide_window:filtered")
				
				t:next_stage()
			end
		}).build (view)

		ui.core.view_controller:show_modal_view (view)

		return nil, "finished"
	end,
}

tutorial.crew = stages
