require "ui/modal/deal_completed_ribbon.lua"

local function _create_npc (data)
	local profile = underworld2.Profile (data.id, false)
	profile:setNickname (data.nickname)
	profile:setAvatarUrl (data.avatar_url)
	profile:setLocation (math.geopoint (0, 0))
	--profile:retain()
	profile_ptr = underworld2.ProfilePtr (runtime.disown (profile))
	underworld2.ProfileDatabase():add (profile_ptr)
	return profile_ptr
end

local activity_log = nil
local window = nil

local stages = 
{
	-- [1] Activity Report is shown.
	function (t)
		underworld2.ui.status_panel:disable()

		underworld2.ui.main_view:show_map (false)
		underworld2.ui.main_view:set_buying_mode()
		underworld2.ui.main_view:set_current_item (1)
		underworld2.ui.main_view:set_map_interactive (false)
		underworld2.ui.main_view.ui.commodities:lockPageMove()

		ui.preloadTexture ("ui/characters/ch_advisor_modal:filtered")
		ui.preloadTexture ("ui/tutorial/tutorial_finger_ill:filtered")
		ui.preloadTexture ("ui/modal_wide_window:filtered")

		local profile = _create_npc ({
			id = -2,
			nickname = L "KEY_TUTORIAL_INITIAL_NAME_2",
			avatar_url = "http://uw2.avatars.s3.amazonaws.com/38.png"
		})

		local passive_deals = 
		{
			{
				dealer = profile,
				deal = {
					item_id = 1,
					price = 13,
					count = 20,
					deal_type = "BUYING"
				}
			}
		}

		local rp_earned = 50
		local now = underworld2.server_time()
		activity_log = ui.core.view.builder (ui.activity_log_view).build (passive_deals, rp_earned, 0, now - underworld2.settings:get ("last_app_suspend", now))

		underworld2.player:addRespectPoints (rp_earned)
		underworld2.player:addItemsToStorage (passive_deals[1].deal.item_id, passive_deals[1].deal.count)

		window = ui.core.view.builder (setmetatable ({
			build_ui = function()
				return ui.Widget()
			end
			}, { __index = ui.core.window })).build()

		activity_log.delegate_builder (
		{
			finished = function()
				window:remove_subview (activity_log)
				activity_log = nil
				t:next_stage()
			end
		}).build (activity_log)
		window:add_subview (activity_log)

		ui.core.view_controller:show_modal_view (window)
		activity_log:disable()

		underworld2.timed_events:queue (underworld2.server_time() + 1.0, function() t:next_stage() end)

		return nil, "activity_report_purchase"
	end,

	-- [2] Message: "Let's face it, there ain't no rest for the wicked. And this biz is as wicked as it gets. Good will keep flowing even if you're offline."
	function (t)
		local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_PASSIVE_TRADING_02"), "ui/characters/ch_advisor_modal:filtered")
		ui.core.view.delegate_builder (
		{
			finished = function (self)
				ui.core.view_controller:close_modal_view (self)
				t:next_stage()
			end
		}).build (view)

		ui.core.view_controller:show_modal_view (view)

		return nil, "message1"
	end,

	-- [3] Message: "This is the recent Activity Report. What have we here? Someone has sold you some Niagra!"
	function (t)
		local commodity = underworld2.CommoditiesDatabase():get (1)
		local localized_name = underworld2.StringsDatabase():get (commodity:getNameKey())
		local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_PASSIVE_TRADING_03", localized_name), "ui/characters/ch_advisor_modal:filtered")
		ui.core.view.delegate_builder (
		{
			finished = function (self)
				ui.core.view_controller:close_modal_view (self)
				t:next_stage()
			end
		}).build (view)

		ui.core.view_controller:show_modal_view (view)

		return nil, "message2"
	end,

	-- [4] Action: "Niagra" green bar is highlighted, tap finger is pointing at "Collect" button
	function (t)
		activity_log:enable()

		local btn = activity_log.ui.collect_button
		ui.core.view_controller.instance:setTouchEventFilter (btn:mapToRoot (btn:getOrigin()), btn:getSize())
		local pos = btn:mapToRoot (btn:getOrigin()) + btn:getSize() * 0.5
		local finger = underworld2.tutorial:create_tap_view (pos, math.pi)
		window:add_subview (finger)
		return function()
			ui.core.view_controller.instance:resetTouchEventFilter()
			window:remove_subview (finger)
			ui.core.view_controller:close_modal_view (window)
			window = nil
		end, "action_collect"
		-- see stage 1
	end,

	-- [5] experience animation
	function (t)
		local deal_completed_ribbon = ui.deal_completed_ribbon.create (50)
		ui.core.view_controller:show_modal_view (deal_completed_ribbon, false)
		ui.core.view.delegate_builder (
		{
			finished = function (self)
				t:next_stage()
			end
		}).build (deal_completed_ribbon)
		return nil, "experience_animation"
	end,

	-- [6] Action: tap finger is pointing at "Home" button
	function (t)
		underworld2.ui.main_view.cards[1]:setOffer (1)
		underworld2.ui.main_view.cards[1]:setOrder (1)
		
		local btn = underworld2.ui.main_view.ui.btn_market
		ui.core.view_controller.instance:setTouchEventFilter (btn:mapToRoot (btn:getOrigin()), btn:getSize())
		local pos = btn:mapToRoot (btn:getOrigin()) + btn:getSize() * 0.5
		local finger = underworld2.tutorial:create_tap_view (pos, math.pi)
		local conn_id 
		conn_id = events.connect (btn:getClickedSignalProxy(), function()
			ui.core.view_controller.instance:setTouchEventFilter(math.vec2 (0, 0), math.vec2 (0, 0))
			underworld2.ui.main_view:lock_trade_gesture()
			underworld2.ui.window:remove_subview (finger)
			finger:finalize()
			events.disconnect (btn:getClickedSignalProxy(), conn_id)
			underworld2.timed_events:queue (underworld2.server_time() + 0.5, function() t:next_stage() end)
		end)
		underworld2.ui.window:add_subview (finger)

		return function (t) 
			ui.core.view_controller.instance:resetTouchEventFilter()
		end, "open_trade_menu"
	end,

	-- [7] "Alright, time for a some stock market 101: Buying and Selling Prices."
	function (t)
		underworld2.ui.main_view.ui.commodities:setEnabled (false)
		underworld2.timed_events:queue (underworld2.server_time() + 0.5, function() 
			local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_PASSIVE_TRADING_07"), "ui/characters/ch_advisor_modal:filtered")
			ui.core.view.delegate_builder (
			{
				finished = function (self)
					ui.core.view_controller:close_modal_view (self)
					t:next_stage()
				end
			}).build (view)

			ui.core.view_controller:show_modal_view (view)
		end)
		return nil, "message3"
	end,

	-- [8] Action: tap-finger is pointing at "Buying" tab
	function (t)
		underworld2.ui.main_view.ui.commodities:setEnabled (true)
		local finger = underworld2.ui.main_view:start_switch_to_buying_tutorial()
		underworld2.ui.window:add_subview (finger)

		return function (t)
			underworld2.ui.main_view.ui.commodities:setEnabled (false)
			underworld2.ui.window:remove_subview (finger)
		end, "open_order_editor"
	end,

	-- [9] "Use the barrel to set your Buying price. Let's say 13 for now. Trust me, I've been doing this for quite a while."
	function (t)
		ui.core.view_controller.instance:resetTouchEventFilter()
		underworld2.timed_events:queue (underworld2.server_time() + 0.5, function() 
			local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_PASSIVE_TRADING_09"), "ui/characters/ch_advisor_modal:filtered")
			ui.core.view.delegate_builder (
			{
				finished = function (self)
					ui.core.view_controller:close_modal_view (self)
					t:next_stage()
				end
			}).build (view)

			ui.core.view_controller:show_modal_view (view)
		end)
		return nil, "message4"
	end,

	-- [10] Action: nothing works until Player sets the price to 13 when barrel's at 14, it's locked at this value and won't move
	function (t)
		underworld2.ui.main_view.ui.commodities:setEnabled (true)
		local card = underworld2.ui.main_view.cards[1]
		card:startEditOrderTutorial (1, 13)
		return nil, "edit_order"
	end,

	-- [11] "The higher your Buying Price is, the faster somebody will sell you the stuff."
	function (t)
		local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_PASSIVE_TRADING_11"), "ui/characters/ch_advisor_modal:filtered")
		ui.core.view.delegate_builder (
		{
			finished = function (self)
				ui.core.view_controller:close_modal_view (self)
				t:next_stage()
			end
		}).build (view)

		ui.core.view_controller:show_modal_view (view)
		return nil, "message5"
	end,

	-- [12] Action: tap finger is pointing at "Selling" tab + Action: tap finger is pointing at "Selling" tab
	function (t)
		local finger = underworld2.ui.main_view:start_switch_to_selling_tutorial()
		underworld2.ui.window:add_subview (finger)

		return function (t)
			underworld2.ui.window:remove_subview (finger)
		end, "switch_to_offers"
	end,

	-- [13] "Let's see... I think 17 will do the trick... attractive selling price, good margin."
	function (t)
		ui.core.view_controller.instance:resetTouchEventFilter()
		underworld2.ui.main_view.ui.commodities:setEnabled (false)
		underworld2.timed_events:queue (underworld2.server_time() + 0.5, function() 
			local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_PASSIVE_TRADING_13"), "ui/characters/ch_advisor_modal:filtered")
			ui.core.view.delegate_builder (
			{
				finished = function (self)
					ui.core.view_controller:close_modal_view (self)
					t:next_stage()
				end
			}).build (view)

			ui.core.view_controller:show_modal_view (view)
		end)
		return nil, "message6"
	end,

	-- [14] Action: nothing works until Player sets the price to 17; when barrel's at 17, it's locked at this value and won't move
	function (t)
		underworld2.ui.main_view.ui.commodities:setEnabled (true)
		local card = underworld2.ui.main_view.cards[1]
		card:startEditOfferTutorial (1, 17)
		return nil, "edit_offer"
	end,

	-- [15] Message: "Always make sure that you've got a reasonable margin between your Buying and Selling prices, but don't be greedy..."
	function (t)
		local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_PASSIVE_TRADING_15"), "ui/characters/ch_advisor_modal:filtered")
		ui.core.view.delegate_builder (
		{
			finished = function (self)
				ui.core.view_controller:close_modal_view (self)
				t:next_stage()
			end
		}).build (view)

		ui.core.view_controller:show_modal_view (view)
		return nil, "message7"
	end,

	-- [16] Message: "Always see what the competition is up to. Check their prices. Be a player, not a piece. Give it a whirl, I'll check on you later, Ace"
	function (t)
		local view = t:create_brief_view (L ("KEY_ADVISOR"), L ("KEY_TUTORIAL_PASSIVE_TRADING_16"), "ui/characters/ch_advisor_modal:filtered")
		ui.core.view.delegate_builder (
		{
			finished = function (self)
				ui.core.view_controller:close_modal_view (self)
				t:next_stage()
			end
		}).build (view)

		ui.core.view_controller:show_modal_view (view)
		return nil, "message8"
	end,

	-- [17] Activity Report with all Niagra bought by "Mr. Rick" is shown.
	function (t)
		local profile = _create_npc ({
			id = -2,
			nickname = "John Deals",
			avatar_url = "http://uw2.avatars.s3.amazonaws.com/38.png"
		})

		local passive_deals = 
		{
			{
				dealer = profile,
				deal = {
					item_id = 1,
					price = underworld2.player:getOfferPrice (1),
					count = underworld2.player:getStorage (1):getUsage(),
					deal_type = "SELL"
				}
			}
		}

		local rp_earned = 50
		local now = underworld2.server_time()
		local activity_log = ui.core.view.builder (ui.activity_log_view).build (passive_deals, rp_earned, 0, now - underworld2.settings:get ("last_app_suspend", now))

		local deal = passive_deals[1].deal

		underworld2.player:addRespectPoints (rp_earned)
		underworld2.player:removeItemsFromStorage (deal.item_id, deal.count)
		underworld2.player:addMoney (deal.price * deal.count)

		activity_log.delegate_builder (
		{
			finished = function()
				ui.core.view_controller:close_modal_view (activity_log)
				t:next_stage()
			end
		}).build (activity_log)

		ui.core.view_controller:show_modal_view (activity_log)

		return nil, "activity_report_sale"
	end,

	-- [18] rp ribbon
	function (t)
		local deal_completed_ribbon = ui.deal_completed_ribbon.create (50)
		ui.core.view_controller:show_modal_view (deal_completed_ribbon, false)
		ui.core.view.delegate_builder (
		{
			finished = function (self)
				t:next_stage()

				underworld2.ui.status_panel:enable()

				ui.removeTextureFromPreloadList ("ui/characters/ch_advisor_modal:filtered")
				ui.removeTextureFromPreloadList ("ui/tutorial/tutorial_finger_ill:filtered")
				ui.removeTextureFromPreloadList ("ui/modal_wide_window:filtered")

				underworld2.ui.main_view:unlock_trade_gesture()
				underworld2.ui.main_view.ui.commodities:unlockPageMove()
			end
		}).build (deal_completed_ribbon)

		return nil, "finished"
	end,
}

tutorial.passive_trading = stages
