require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"

local view = setmetatable (
	{
	}, { __index = ui.core.view })
ui.activity_log_view = view

local builder = ui.core.view.builder(view)

function view:back()
	self:finalize()
end

local function _create_title ()
	local widget = ui.Widget()
	widget:setLeftMargin (ui.config.hspacing * 2.0)

	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing (ui.config.hspacing * 2.0)
	do
		local vlayout = ui.VerticalLayout()
		vlayout:addWidget (ui.Widget())
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_courier:filtered", "ui/vault/backlight_vault:filtered")
		vlayout:addWidget (ico)
		vlayout:addWidget (ui.Widget())
		hlayout:addItem (vlayout)
	end
	
	local vlayout = ui.VerticalLayout()

	local title = ui.TextLabel()
	title:setText (L ("KEY_ACTIVITY_REPORT_TITLE"))
	title:setTextStyle (ui.text_styles.large_hl.build())
	title:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
	vlayout:addWidget (title)

	local subtitle = ui.TextLabel()
	subtitle:setText (" ")
	subtitle:setTextStyle (ui.text_styles.small_hl.build())
	subtitle:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
	vlayout:addWidget (subtitle)

	hlayout:addItem (vlayout)

	widget:setLayout (hlayout)
	widget:setVerticalSizePolicy (ui.SizePolicyMinimum)

	return widget, subtitle

end

local function _create_income_row (profile, item_id, amount, income_amount)
	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing (ui.config.hspacing)
	hlayout:setVerticalPolicy (ui.SizePolicyMinimum)

	local avatar = ui.AvatarWidget()
	avatar:setProfile (profile)
	avatar:setAvatarSize (underworld2.Avatar_SizeNormal)
	hlayout:addWidget (avatar)

	do

		local vlayout = ui.VerticalLayout()
		
		local nickname = ui.TextLabel()
		nickname:setWordWrap (false)
		nickname:setTextStyle (ui.text_styles.small_hl.build())
		nickname:setText (profile:getNickname())
		vlayout:addWidget (nickname)

		do
			local item_layout = ui.HorizontalLayout()
			item_layout:setItemSpacing (ui.config.hspacing)

			local item_icon = ui.Picture()
			item_icon:setTexture (ui.config.commodity_small_icons[item_id])
			item_layout:addWidget (item_icon)

			local amount_label = ui.TextLabel()
			amount_label:setTextStyle (ui.text_styles.hint_grey_small.build())
			amount_label:setText (tostring (amount))
			amount_label:setWordWrap (false)
			item_layout:addWidget (amount_label)

			vlayout:addItem (item_layout)
		end

		hlayout:addItem (vlayout)
	end

	local spacer = ui.Widget()
	spacer:setHorizontalSizePolicy (ui.SizePolicyMaximum)
	hlayout:addWidget (spacer)

	local cash_ico = ui.Picture()
	cash_ico:setTexture ("ui/ico_cash_med:filtered")
	hlayout:addWidget (cash_ico)

	local income_label = ui.TextLabel()
	income_label:setWordWrap (false)
	income_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
	income_label:setTextStyle (ui.text_styles.large_hl.build())
	income_label:setText (tostring (income_amount))
	hlayout:addWidget (income_label)

	return hlayout
end

local function _create_commodity_row (profile, item_id, amount)
	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing (ui.config.hspacing)
	hlayout:setVerticalPolicy (ui.SizePolicyMinimum)

	local avatar = ui.AvatarWidget()
	avatar:setProfile (profile)
	avatar:setAvatarSize (underworld2.Avatar_SizeNormal)
	hlayout:addWidget (avatar)

	local nickname = ui.TextLabel()
	nickname:setTextAlignment (ui.AlignmentVerticalTop + ui.AlignmentHorizontalLeft)
	nickname:setTextStyle (ui.text_styles.small_hl.build())
	nickname:setText (profile:getNickname())
	nickname:setHorizontalSizePolicy (ui.SizePolicyMaximum)
	hlayout:addWidget (nickname)

	local item_ico = ui.Picture()
	item_ico:setTexture (ui.config.commodity_small_icons[item_id])
	hlayout:addWidget (item_ico)

	local income_label = ui.TextLabel()
	income_label:setWordWrap (false)
	income_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
	income_label:setTextStyle (ui.text_styles.large_hl.build())
	income_label:setText (tostring (amount))
	hlayout:addWidget (income_label)

	return hlayout
end

local group_widget = { }

function group_widget.create (bgid_closed, bgid_opened, icon_id)
	local obj = setmetatable ({  }, { __index = group_widget })

	local main_layout = ui.VerticalLayout()
	obj.main_layout = main_layout

	local btn = ui.ContainerButton()
	btn:setTexture (bgid_closed)

	do

		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)

		local ico = ui.Picture()
		ico:setTexture (icon_id)
		hlayout:addWidget (ico)

		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.button_regular.build())
		title_label:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)
		title_label:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		hlayout:addWidget (title_label)

		local amount_label = ui.TextLabel()
		amount_label:setTextStyle (ui.text_styles.button_regular.build())
		amount_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
		amount_label:setWordWrap (false)
		hlayout:addWidget (amount_label)

		local arrow = ui.Picture ()
		arrow:setTexture ("ui/activity_report/arrow_dropdown:filtered")
		arrow:setHorizontalSizePolicy (ui.SizePolicyFixed)
		arrow:setWidth (arrow:getImageSize().x)
		hlayout:addWidget (arrow)

		obj.title_label = title_label
		obj.amount_label = amount_label
		obj.arrow = arrow
		obj.value = 0
		obj.animation_dir = 0

		btn:setLayout (hlayout)

		main_layout:addWidget (btn)
	end

	local clip_widget = ui.ClipWidget()
	clip_widget:setVerticalSizePolicy (ui.SizePolicyFixed)
	clip_widget:setHeight (0)
	main_layout:addWidget (clip_widget)
	obj.clip_widget = clip_widget
	obj.body_max_height = 0

	events.connect (btn:getClickedSignalProxy(), function()
		if obj.value == 0 then
			obj.animation_dir = 1
			btn:setTexture (bgid_opened)
		elseif obj.value == 1 then
			obj.animation_dir = -1
		end
	end)

	events.connect (events.global_update(), function (dt)
		if obj.animation_dir ~= 0 then
			obj.value = obj.value + obj.animation_dir * dt * 6.0
			if (obj.animation_dir == -1) and (obj.value <= 0) then
				obj.value = 0
				obj.animation_dir = 0
				btn:setTexture (bgid_closed)
			elseif (obj.animation_dir == 1) and (obj.value >= 1) then
				obj.value = 1
				obj.animation_dir = 0
			end
			obj:update_geometry()
		end
	end)

	return obj
end

function group_widget:update_geometry()
	if self.body then
		local clip_height = self.body_max_height * self.value
		self.arrow:setRotation (-0.5 * math.pi * self.value)
		self.clip_widget:setHeight (clip_height)
		self.body:setOrigin (0, clip_height - self.body_max_height)
		self.body:setSize (self.clip_widget:getWidth(), self.body_max_height)
	end
end

function group_widget:set_title (title)
	self.title_label:setText (title)
end

function group_widget:set_amount (amount)
	self.amount_label:setText (tostring (amount))
end

function group_widget:set_body (body)
	self.body = body
	local body_size = body:getPreferredSize()
	self.body_max_height = body_size.y
	self.clip_widget:addChild (body)
	self:update_geometry()
end

local function _create_income_group (items)
	local group = group_widget.create ("ui/activity_report/income_dropdown_closed_top:filtered", "ui/activity_report/income_dropdown_open_top:filtered", "ui/ico_cash_med:filtered")
	group:set_title (L "KEY_ACTIVITY_INCOME_AMOUNT")
	group:set_amount (items.income_amount)

	local contents_layout = ui.VerticalLayout()
	
	local body = ui.Picture()
	body:setTexture ("ui/solid_background:filtered")
	body:setPlacingMode (ui.Picture_PlacingModeStretch)
	body:setColor (render.color (254, 254, 254, 31))
	body:setMargins (ui.config.hspacing, ui.config.hspacing, ui.config.vspacing, ui.config.vspacing)
	body_layout = ui.VerticalLayout()
	body_layout:setItemSpacing (ui.config.vspacing * 0.5)

	do
		local header_layout = ui.HorizontalLayout()
		header_layout:setVerticalPolicy (ui.SizePolicyMinimum)

		local column0 = ui.TextLabel()
		column0:setTextStyle (ui.text_styles.deal_hint.build())
		column0:setText (L "KEY_ACTIVITY_MERCH_SOLD_TO")
		column0:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)
		header_layout:addWidget (column0)

		local column1 = ui.TextLabel()
		column1:setTextStyle (ui.text_styles.deal_hint.build())
		column1:setText (L "KEY_ACTIVITY_INCOME_AMOUNT")
		column1:setTextAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalCenter)
		header_layout:addWidget (column1)

		body_layout:addItem (header_layout)
	end

	local line_sep = ui.Decorator()
	line_sep:setTexture ("ui/activity_report/line_grey:filtered", ui.Decorator_StyleHStretcherNoBorders)
	line_sep:setVerticalSizePolicy (ui.SizePolicyMinimum)
	body_layout:addWidget (line_sep)

	for i, v in ipairs (items.rows) do
		local deal = v.deal
		body_layout:addItem (_create_income_row (v.dealer, deal.item_id, deal.count, deal.count * deal.price))
	end

	local footer = ui.Decorator()
	footer:setTexture ("ui/activity_report/income_dropdown_bottom:filtered", ui.Decorator_StyleHStretcherWithBorders)

	body:setLayout (body_layout)

	contents_layout:addWidget (body)
	contents_layout:addWidget (footer)
	
	local content = ui.Widget()
	content:setLayout (contents_layout)
	
	group:set_body (content)
	return group
end

local function _create_items_group (items, item_id)
	local strings = underworld2.StringsDatabase()
	local group = group_widget.create ("ui/activity_report/items_dropdown_closed_top:filtered", "ui/activity_report/items_dropdown_open_top:filtered", ui.config.commodity_icons[item_id])
	group:set_title (strings:get (underworld2.CommoditiesDatabase():get (item_id):getNameKey()))
	group:set_amount (items.total_amount)

	local contents_layout = ui.VerticalLayout()
	
	local body = ui.Picture()
	body:setTexture ("ui/solid_background:filtered")
	body:setPlacingMode (ui.Picture_PlacingModeStretch)
	body:setColor (render.color (254, 254, 254, 31))
	body:setMargins (ui.config.hspacing, ui.config.hspacing, ui.config.vspacing, ui.config.vspacing)
	body_layout = ui.VerticalLayout()
	body_layout:setItemSpacing (ui.config.vspacing * 0.5)

	do
		local header_layout = ui.HorizontalLayout()
		header_layout:setVerticalPolicy (ui.SizePolicyMinimum)

		local column0 = ui.TextLabel()
		column0:setTextStyle (ui.text_styles.deal_hint.build())
		column0:setText (L "KEY_ACTIVITY_MERCH_BOUGHT_FROM")
		column0:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)
		header_layout:addWidget (column0)

		local column1 = ui.TextLabel()
		column1:setTextStyle (ui.text_styles.deal_hint.build())
		column1:setText (L "KEY_ACTIVITY_AMOUNT_BOUGHT")
		column1:setTextAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalCenter)
		header_layout:addWidget (column1)

		body_layout:addItem (header_layout)
	end

	local line_sep = ui.Decorator()
	line_sep:setTexture ("ui/activity_report/line_grey:filtered", ui.Decorator_StyleHStretcherNoBorders)
	line_sep:setVerticalSizePolicy (ui.SizePolicyMinimum)
	body_layout:addWidget (line_sep)

	for i, v in ipairs (items.rows) do
		body_layout:addItem (_create_commodity_row (v.dealer, item_id, v.deal.count))
	end

	local line_sep = ui.Decorator()
	line_sep:setTexture ("ui/activity_report/line_grey:filtered", ui.Decorator_StyleHStretcherNoBorders)
	line_sep:setVerticalSizePolicy (ui.SizePolicyMinimum)
	body_layout:addWidget (line_sep)

	do
		local summary_layout = ui.HorizontalLayout()
		summary_layout:setVerticalPolicy (ui.SizePolicyMinimum)

		summary_layout:addWidget (ui.Widget())
		summary_layout:addWidget (ui.Widget())

		local column0 = ui.TextLabel()
		column0:setTextStyle (ui.text_styles.deal_hint.build())
		column0:setText (L "KEY_ACTIVITY_TOTAL_SPENT")
		column0:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)
		summary_layout:addWidget (column0)

		local cash_ico = ui.Picture()
		cash_ico:setTexture ("ui/ico_cash_tiny:filtered")
		cash_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		summary_layout:addWidget (cash_ico)

		local column1 = ui.TextLabel()
		column1:setTextStyle (ui.text_styles.deal_hint.build())
		column1:setText (tostring (items.total_spent))
		column1:setTextAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalCenter)
		column1:setWordWrap (false)
		column1:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		summary_layout:addWidget (column1)

		body_layout:addItem (summary_layout)
	end

	local footer = ui.Decorator()
	footer:setTexture ("ui/activity_report/items_dropdown_bottom:filtered", ui.Decorator_StyleHStretcherWithBorders)

	body:setLayout (body_layout)

	contents_layout:addWidget (body)
	contents_layout:addWidget (footer)
	
	local content = ui.Widget()
	content:setLayout (contents_layout)
	
	group:set_body (content)
	return group
end

function view:build_ui (passive_deals, rp_earned, fame_earned, time_passed)
	self.ui = { }

	local widget = ui.Widget()
	widget:setOrigin (0, 0)
	widget:setSize (1, 1)

	local frame_size = math.vec2 (1.0, 0.9)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setOrigin (0.0, 0.05)
	frame:setSize (frame_size)
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing, ui.config.vspacing)
	widget:addChild (frame)

	local vlayout = ui.VerticalLayout()
	vlayout:setItemSpacing (ui.config.vspacing)

	local title, subtitle = _create_title ()
	vlayout:addWidget (title)
	self.subtitle = subtitle

	frame:setLeftMargin (0)
	frame:setRightMargin (0)

	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	vlayout:addWidget (sep)

	local spacing = ui.Widget()
	spacing:setHeight (ui.config.vspacing)
	spacing:setVerticalSizePolicy (ui.SizePolicyFixed)
	vlayout:addWidget (spacing)

	local scroll = ui.ScrollWidget()
	scroll:setScrollType (ui.ScrollTypeVertical)
	scroll:setScrollBarTexture ("ui/scroll:filtered")
	local scroll_layout = ui.VerticalLayout()
	scroll_layout:setItemSpacing (ui.config.vspacing)
	scroll:setLayout (scroll_layout)
	vlayout:addWidget (scroll)
	self.scroll_layout = scroll_layout

	ui.preloadTexture ("ui/activity_report/income_dropdown_closed_top:filtered")
	ui.preloadTexture ("ui/activity_report/income_dropdown_open_top:filtered")
	ui.preloadTexture ("ui/activity_report/items_dropdown_closed_top:filtered")
	ui.preloadTexture ("ui/activity_report/items_dropdown_open_top:filtered")

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)

	local spacing = ui.Widget()
	spacing:setHeight (ui.config.vspacing)
	spacing:setVerticalSizePolicy (ui.SizePolicyFixed)
	vlayout:addWidget (spacing)

	local btn_layout = ui.HorizontalLayout()
	btn_layout:setVerticalPolicy (ui.SizePolicyMinimum)

	local spacer = ui.Widget()
	spacer:setSize (0.25, 0.0)
	spacer:setHorizontalSizePolicy (ui.SizePolicyFixed)
	btn_layout:addWidget (spacer)

	local btn = ui.TextButton()
	btn:setTexture ("ui/btn_lower_normal:filtered")
	btn:setTextStyle (ui.text_styles.button_regular.build())
	btn:setText (L ("KEY_ACTIVITY_REPORT_COLLECT"))
	events.connect (btn:getClickedSignalProxy(), function()
		self:finalize()
	end)
	btn_layout:addWidget (btn)
	self.ui.collect_button = btn

	local spacer = ui.Widget()
	spacer:setSize (0.25, 0.0)
	spacer:setHorizontalSizePolicy (ui.SizePolicyFixed)
	btn_layout:addWidget (spacer)

	vlayout:addItem (btn_layout)

	frame:setLayout (vlayout)

	assert (type (passive_deals) == "table")
	print ("type="..type (time_passed))
	
	do
		local lui = ui.core.lib

		local frm_rp = lui.frame
		{
			texture = "ui/frame_modal_light:filtered",
			vsize_policy = ui.SizePolicyMinimum,
			layout = lui.horizontal_layout { spacing = ui.config.hspacing },
			children =
			{
				lui.picture 
				{
					texture = "ui/icons/ico_rp_sidemenu:filtered",
					hsize_policy = ui.SizePolicyMinimum,
				},
				lui.text_label
				{
					text_style = ui.text_styles.button_regular.build(),
					text = L "KEY_ACTIVITY_RESPECT"
				},
				lui.text_label
				{
					text_style = ui.text_styles.button_regular.build(),
					text = string.format ("+%d", rp_earned),
					word_wrap = false,
					hsize_policy = ui.SizePolicyMinimum,
				}
			}
		}
		scroll_layout:addWidget (frm_rp)

		if fame_earned and fame_earned > 0 then
			local frm_fame = lui.frame
			{
				texture = "ui/frame_modal_light:filtered",
				vsize_policy = ui.SizePolicyMinimum,
				layout = lui.horizontal_layout { spacing = ui.config.hspacing },
				children =
				{
					lui.picture 
					{
						texture = "ui/icons/ico_fame_56:filtered",
						hsize_policy = ui.SizePolicyMinimum,
					},
					lui.text_label
					{
						text_style = ui.text_styles.button_regular.build(),
						text = L "KEY_FAME"
					},
					lui.text_label
					{
						text_style = ui.text_styles.button_regular.build(),
						text = string.format ("+%d", fame_earned),
						word_wrap = false,
						hsize_policy = ui.SizePolicyMinimum,
					}
				}
			}
			scroll_layout:addWidget (frm_fame)
		end
		
		
	end
	
	self:fill (passive_deals)
	if time_passed > 0 then
		subtitle:setText (L ("KEY_ACTIVITY_REPORT_SUBTITLE", ui.format.duration_string (time_passed, true)))
	else
		subtitle:setText("")
	end
	
	return widget
end

local tinsert = table.insert

local function _add_income (income, deal_info)
	local deal = deal_info.deal
	local income_amount = deal.price * deal.count 
	income.income_amount = income.income_amount + income_amount
	local index = 1
	for i, v in ipairs (income.rows) do
		if income_amount > (v.deal.count * v.deal.price) then
			index = i
			break
		end
	end
	tinsert (income.rows, index, deal_info)
end

local function _add_items (items, deal_info)
	local deal = deal_info.deal
	local item = items[deal.item_id] or { rows = { }, total_spent = 0, total_amount = 0 }
	local index = 1
	local count = deal.count
	item.total_spent = item.total_spent + deal.price * deal.count
	item.total_amount = item.total_amount + deal.count
	for i, v in ipairs (item.rows) do
		if count > v.deal.count then
			index = i
			break
		end
	end
	tinsert (item.rows, index, deal_info)
	items[deal.item_id] = item
end

function view:fill (deals)

	local income = { rows = { }, income_amount = 0 }
	local items = {  }

	local tinsert = table.insert

	for i, v in ipairs (deals) do
		local deal = v.deal
		assert (v.dealer:get())
		if deal.deal_type == "SELL" then
			_add_income (income, v)
		elseif deal.deal_type == "BUYING" then
			_add_items (items, v)
		end
	end

	local scroll_layout = self.scroll_layout

	if #income.rows > 0 then
		scroll_layout:addItem (_create_income_group (income).main_layout)
	end

	for i, v in pairs (items) do
		scroll_layout:addItem (_create_items_group (v, i).main_layout)
	end

end

