require "anim.lua"

ui.animations = ui.animations or {}

--[[
	courier_car_position is relative to the car length  0 - road center
	police_car_offset is in size of cars 0 - cars are at the same level

	timer - is a time valur
	state = {
		courier_car_position     = 0,

		police_car_offset        = 0,

		camera_zoom              = 0,

		saturation               = 1,

		runner_avatar_alpha      = 0,
		runner_avatar_position   = 0,

		commodity_alpha          = 0,
		commodity_position       = 0,

		overlay_message_size     = 0,
		overlay_message_alpha    = 0,
		overlay_background_alpha = 0,

		global_alpha             = 0,
		fade_away_alpha          = 0,
	}
]]

local interface_transition_time = 0.35

local zoom = {
	initial = 0.5 * 1.5,
	chase_start = 0.65 * 1.5,
	chase_end = 0.75 * 1.5,
	out = 0.5 * 1.5
}

local function build_interface_in(state)
	return anim.parallel {
		anim.lerp {
			from = 0.8,
			to   = 1,
			time = interface_transition_time,
			object = state,
			property = "camera_rect_scale"
		},
		anim.lerp {
			from = zoom.initial,
			to   = zoom.chase_start,
			time = interface_transition_time,
			object = state,
			property = "camera_zoom"
		},
		anim.lerp {
			from = 0,
			to = 1,
			time = interface_transition_time,
			object = state,
			property = "text_position"
		},
		anim.lerp {
			from  = 0.5,
			to    = 1.0,
			time = interface_transition_time,
			object = state,
			property = "global_alpha"
		}
	}
end

local function build_interface_out(state)
	return anim.parallel {
		anim.lerp {
			from = 1,
			to   = 1.4,
			time = interface_transition_time,
			object = state,
			property = "camera_rect_scale"
		},
		anim.lerp {
			from = zoom.chase_end,
			to   = zoom.out,
			time = interface_transition_time,
			object = state,
			property = "camera_zoom"
		},
		anim.lerp {
			from = 1,
			to = 0,
			time = interface_transition_time,
			object = state,
			property = "text_position"
		},
		anim.lerp {
			from  = 1.0,
			to    = 0.5,
			time = interface_transition_time,
			object = state,
			property = "global_alpha"
		}
	}
end

-- Step 1. Courier car appears
local step10_timing  = 2.5
local step20_timing = 4 - 0.2 - step10_timing

local function set(object, property, value)
	return anim.lerp {
		from = value,
		to   = value,
		time = 0,
		object = object,
		property = property
	}
end

local function play_sound (object, id)
	return function() get_sound_pool():play (id) end
end

local escape_frame_keys = 
{
	courier = 
	{
		car_pos =
		{
			t =      { 0.0,  0.8, 1.1,  1.3, 5.0 },
			offset = { -1.5, 0.0, 0.05, 0.0, 2.0 },
		},
		engine = 
		{
			t =     {0.0, 1.3, 2.8, 4.0, 4.1, 4.2, 6.0 },
			pitch = { 1.0, 1.0, 0.96, 1.2, 0.96, 1.0, 1.35 }
		}
	},
	police = 
	{
		car_pos = 
		{
			t =      { 1.0,  2.3, 2.6,  2.8, 5.0, 6.0 },
			offset = { -1.5, 0.0, 0.05, 0.0, -0.2, -0.4 },
		},
	},
	camera_speed = 
	{
		t = { 0.0, 6.0 },
		values = { 10.0, 10.0 }
	}
}

local bust_frame_keys = 
{
	courier = 
	{
		car_pos =
		{
			t =      { 0.0,  0.8, 1.1,  1.3, 2.8, 4.0 },
			offset = { -1.5, 0.0, 0.05, 0.0, -0.05, -0.2 },
		},
		engine = 
		{
			t = {0.0, 1.3, 2.8, 6.0},
			pitch = { 1.0, 1.0, 0.9, 0.8 }
		}
	},
	police = 
	{
		car_pos = 
		{
			t =      { 1.0,  2.3, 2.6, 2.8, 5.0, 6.0 },
			offset = { -1.5, 0.0, 0.05, 0.0, 0.2, 0.15 },
		}
	},
	camera_speed = 
	{
		t = { 0.0, 2.8, 6.0 },
		values = { 10.0, 10.0, 3.0 }
	}
}

local function build_car_escape_anim (state, frames_set)
	local courier_fn = math.cspline (frames_set.courier.car_pos.t, frames_set.courier.car_pos.offset)
	local courier_snd_fn = math.pwlinear (frames_set.courier.engine.t, frames_set.courier.engine.pitch)
	local police_fn = math.cspline (frames_set.police.car_pos.t, frames_set.police.car_pos.offset)
	local camera_fn = math.pwlinear (frames_set.camera_speed.t, frames_set.camera_speed.values)

	return anim.parallel
	{
		anim.parallel
		{
			anim.lerp {
				from = 0.0, to = 6.0, time = 6.0,
				object = state,
				property = function (state, t) 
					local pos = courier_fn (t)
					state["courier_car_position"] = pos
					state["courier_engine_gain"] = math.clamp (1.0 - math.abs (0.0 + pos * 0.25), 0.0, 1.0)
					state["camera_speed"] = camera_fn (t)
				end 
			},
			anim.lerp {
				from = 0.0, to = 6.0, time = 6.0,
				object = state,
				property = function (state, t) state["courier_engine_pitch"] = courier_snd_fn (t) end 
			},
		},
		anim.lerp {
			from = 0.0, to = 6.0, time = 6.0,
			object = state,
			property = function (state, t) 
				local pos = police_fn (t)
				state["police_car_position"] = pos
				state["police_siren_gain"] = math.clamp (1.0 - math.abs (0.0 + pos * 0.5), 0.0, 1.0)
			end
		}
	}
end

local overlay_fade_in_time = 0.1

local range_max = 50
local range_scale = 1 / math.log(range_max + 1)

local function build_show_courier_anim (state)
	return anim.sequence {
		anim.parallel { 
			anim.lerp {
				from = 0.0,
				to   = 1.0,
				time = 0.3,
				object = state,
				property = "avatar_opacity"
			},
			anim.lerp {
				from = 0.0,
				to   = 1.0,
				time = 0.3,
				object = state,
				property = "avatar_position"
			},
		},
		anim.parallel {
			anim.lerp {
				from = 0.0,
				to = 1.0,
				time = overlay_fade_in_time,
				object = state,
				property = "commodity_position"
			},
			anim.lerp {
				from = 0.0,
				to = 1.0,
				time = overlay_fade_in_time,
				object = state,
				property = "commodity_opacity"
			}
		}
	}
end

local police_speed = 280 / 415 / step20_timing

local function build_step30_busted(state)
	return anim.parallel {
		anim.lerp {
			from  = 1.0,
			to    = 0.0,
			time  = 0.3,
			object = state,
			property = "saturation"
		},
		anim.sequence {
			anim.delay { time = 0.15 },
			anim.parallel {
				anim.lerp {
					from = 0.0,
					to = 1.0,
					time = overlay_fade_in_time,
					object = state,
					property = "overlay_opacity"
				},
				anim.lerp {
					from = 0.5,
					to = 1.0,
					time = overlay_fade_in_time,
					object = state,
					property = "overlay_scale"
				},
			},
			anim.delay { time = 0.3 },
			anim.parallel {
				anim.lerp {
					from = 0.0,
					to   = 0.9,
					time = 0.5,
					object = state,
					property = "fade_opacity"
				},
				build_interface_out(state)
			}
		}
	}
end

local function build_step30_escaped(state)
	return anim.sequence {
		anim.delay { time = 0.15 },
		anim.parallel {
			anim.lerp {
				from = 0.0,
				to = 1.0,
				time = 0.3,
				object = state,
				property = "overlay_opacity"
			},
			anim.lerp {
				from = 0.5,
				to = 1.0,
				time = 0.3,
				object = state,
				property = "overlay_scale"
			},
		},
		anim.delay { time = 0.3 },
		anim.parallel {
			anim.lerp {
				from = 0.0,
				to   = 0.9,
				time = 0.5,
				object = state,
				property = "fade_opacity"
			},
			build_interface_out(state)
		}
	}
end

local function build_step30(state, busted)
	if busted then
		return build_step30_busted(state)
	else
		return build_step30_escaped(state)
	end
end

local function build_chase_screen_animation(set_prop, finished_callback, busted)
	local state = setmetatable( {}, { __newindex = set_prop } )

	local animation = anim.sequence {
		on_finished = finished_callback,

		build_interface_in(state),
		anim.parallel {
			anim.lerp {
				from = 3.0,
				to   = 0.0,
				time = 3.0,
				object = state,
				property = "time"
			},
			anim.lerp {
				from = zoom.chase_start,
				to   = zoom.chase_end,
				time = 4.0,
				object = state,
				property = "camera_zoom"
			},
			build_car_escape_anim (state, busted and bust_frame_keys or escape_frame_keys),
			anim.sequence
			{
				anim.delay { time = 0.4 },
				build_show_courier_anim (state),
				anim.delay 
				{ 
					time = 1.8, 
					on_finished = function()
						state["avatar_opacity"] = 0
						state["commodity_opacity"] = 0
					end 
				},
				anim.delay 
				{ 
					time = 1.2,
					on_finished = function()
						get_sound_pool():play (busted and "chase/SFX_Fail.wav" or "chase/SFX_Getaway.wav")
					end
				},
				build_step30(state, busted),
			}
		}
	}

	return function(dt) animation:update(dt) end
end

ui.animations.chase_screen = build_chase_screen_animation