--module( ..., package.builder )
require "ui/animations/animation.lua"

ui = ui or {}
ui.animations = ui.animations or {}
ui.animations.factory = function( context )
	local state = {
		animations = {},
		idle = {}
	}

	local function lerp( from, to, t )
		return from + t * (to - from)
	end

	local function clamp( t, from, to )
		if t < from then return from end
		if t > to then return to end
		return t
	end

	function state.update( self, t, force )
		local anims = self.animations
		local idle = self.idle

		-- terrible hack due to the missing frame
		for key, frame in pairs( idle ) do
			if frame > 1 then
				self:cancel( key )
			end

			if frame > 0 then
				idle[key] = frame + 1
			end
		end

		-- active
		for key, anim in pairs( anims ) do
			if idle[key] == 0 then
				if (anim.delay > 0) and (not force) then
					anim.delay = anim.delay - t
				else
					if anim.step then
						anim:step( lerp( anim.from, anim.to, anim.subfunction( clamp( anim.value, 0, 1 ) ) ) )
					end

					if anim.value >= 1 then
						if (anim.iteration < anim.times) or (anim.times < 0) then -- forever
							anim.iteration = anim.iteration+1
							anim.value = 0
							if anim.oscillate then
								anim.from, anim.to = anim.to, anim.from -- all way back
							end
						else -- stop
							self.idle[key] = 1
						end
					else anim.value = anim.value + t/anim.duration end
				end
			end
		end
	end

	function state.bind( self, key, anim )
		self.animations[key] = anim
		self.idle[key] = 0
	end

	function state.cancel( self, key )
		local anim = self.animations[key]
		if not anim then return end
		self.animations[key] = nil
		self.idle[key] = 0
		if anim.stop then anim:stop() end
	end

	function state.reset( self )
		self.animations = {}
	end

	return state
end
