require "ui/config.lua"
require "ui/text_styles.lua"
require "ui/format.lua"

local completed_deal_widget =
{
}

function completed_deal_widget.create (params)
	local obj = setmetatable ( {ui = { }} , { __index = completed_deal_widget } )

	local buying = params.type == underworld2.deal.PURCHASE

	local root_widget = ui.ContainerButton()
	root_widget:setTexture ("ui/tab_side_menu:filtered")

	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing (ui.config.hspacing)
	
	local item_icon = ui.ItemIcon()
	item_icon:setBackgroundTexture ("ui/item_background:filtered")
	item_icon:setItemTexture (ui.config.commodity_icons[params.item.id])
	item_icon:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hlayout:addWidget (item_icon)
	obj.ui.item_icon = item_icon

	do
		local vlayout = ui.VerticalLayout()

		do 
			local hlayout = ui.HorizontalLayout()
			hlayout:setItemSpacing (ui.config.hspacing)

			local amount_ico = ui.Picture()
			amount_ico:setTexture ("ui/ico_box_small:filtered")
			amount_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)

			local amount = ui.TextLabel()
			amount:setTextStyle (ui.text_styles.deal_price.build())
			amount:setText (params.item.count)
			amount:setWordWrap (false)

			hlayout:addWidget (amount_ico)
			hlayout:addWidget (amount)
			
			vlayout:addItem (hlayout)
		end

		do
			local hlayout = ui.HorizontalLayout()
			hlayout:setItemSpacing (ui.config.hspacing)

			local arrow_ico = ui.Picture()
			arrow_ico:setTexture (buying and "ui/ico_arrow_green:filtered" or "ui/ico_arrow_orange:filtered")
			arrow_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)

			local hint = ui.TextLabel()
			if buying then
				hint:setText (L ("KEY_COMPLETE_DEAL_BOUGHT", underworld2.items[params.item.id].name))
			else
				hint:setText (L ("KEY_COMPLETE_DEAL_SOLD", underworld2.items[params.item.id].name))
			end
			hint:setWordWrap (false)
			hint:setTextStyle (ui.text_styles.deal_hint.build())

			hlayout:addWidget (arrow_ico)
			hlayout:addWidget (hint)

			vlayout:addItem (hlayout)
		end

		hlayout:addItem (vlayout)
	end

	local hlayout1 = ui.HorizontalLayout()
	hlayout1:setItemSpacing (ui.config.hspacing)

	do
		local vlayout = ui.VerticalLayout()

		do
			local hlayout = ui.HorizontalLayout()
			hlayout:setItemSpacing (ui.config.hspacing)

			local money_ico = ui.Picture()
			money_ico:setTexture ("ui/ico_cash_tiny:filtered")
			money_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)

			local money = ui.TextLabel()
			money:setText (params.item.price_per_unit)
			money:setWordWrap (false)
			money:setTextStyle (ui.text_styles.deal_price.build())

			hlayout:addWidget (money_ico)
			hlayout:addWidget (money)

			vlayout:addItem (hlayout)
		end

		local hint = ui.TextLabel()
		hint:setTextStyle (ui.text_styles.deal_hint.build())
		hint:setText (L ("KEY_COMPLETED_DEAL_HINT_PRICE"))
		hint:setWordWrap (false)
		vlayout:addWidget (hint)

		hlayout1:addItem (vlayout)
	end

	

	local time_label = ui.TextLabel()
	time_label:setTextStyle (ui.text_styles.deal_hint.build())
	
	local time_passed = os.difftime (underworld2.server_time(), params.ts or 0)
	local time_str = ui.format.time_passed (time_passed)

	time_label:setText (time_str)
	time_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
	hlayout1:addWidget (time_label)

	hlayout:addItem (hlayout1)
	
	local arrow_ico = ui.Picture()
	arrow_ico:setTexture("ui/arrow_off_ico:filtered")
	arrow_ico:setPlacingMode(ui.Picture_PlacingModeCenter)
	arrow_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hlayout:addWidget (arrow_ico)

	obj.ui.root_widget = root_widget

	root_widget:setLayout (hlayout)
	
	obj.params = params

	return obj
end

ui.completed_deal_widget = ui.completed_deal_widget or completed_deal_widget
