local lib = { }

local create = function (t, ctor, props)
	local object = ctor()
	for property, value in pairs (props) do
		assert (t[property], string.format ("object does not have '%s' property", tostring (property)))
		local setter = t[property]
		if type (setter) == "string" then
			object[setter] (object, value)
		else
			assert (type (setter) == "function")
			setter (object, value)
		end
	end
	return object
end

local function generate_builder_mt (ctor, parent)
	return {
		__index = parent,
		__call = function (t, props) return create (t, ctor, props) end
	}
end

local function build (t, ctor, cls, parent)
	return setmetatable (cls, generate_builder_mt (ctor, parent))
end

local builder = setmetatable ({ }, { __call = build })

lib.widget = builder (ui.Widget, {
	id				= "setWidgetID",
	origin			= "setOrigin",
	size 			= "setSize",
	offset			= "setOffset",
	width			= "setWidth",
	height			= "setHeight",
	enabled			= "setEnabled",
	left_margin		= "setLeftMargin",
	right_margin 	= "setRightMargin",
	top_margin		= "setTopMargin",
	bottom_margin	= "setBottomMargin",
	hsize_policy 	= "setHorizontalSizePolicy",
	vsize_policy	= "setVerticalSizePolicy",
	layout 			= "setLayout",
	scale 			= "setScale",
	hidden			= "setHidden",
	alignment 		= "setAlignment",
	self_anchor		= "setSelfAnchor",
	parent_anchor	= "setParentAnchor",
	pivot			= "setPivot",
	layout_alignment = "setLayoutAlignment",
	children 		= function (object, value)
		for i, child in ipairs (value) do
			object:addChild (child)
		end
	end,
})

lib.button = builder (nil, {
	on_clicked 	= function (object, value) events.connect (object:getClickedSignalProxy(), value) end
}, lib.widget)

lib.frame = builder (ui.Frame, {
	texture = "setBordersTexture",
	relative_size = "setRelativeSize"
}, lib.widget)

lib.decorator = builder (ui.Decorator, {
	texture = "setTexture",
	style   = "setStyle",
}, lib.widget)

lib.text_label = builder (ui.TextLabel, {
	text 			= "setText",
	text_style 		= "setTextStyle",
	text_alignment 	= "setTextAlignment",
	word_wrap		= "setWordWrap",
	elide			= "setElide",
	rows			= "setRowsCount",
	color			= "setColor",
	rotate			= "setRotation",
	zoom			= "setZoom",
}, lib.widget)

lib.rich_text = builder (ui.RichTextWidget, {
	text 		= "setText",
	text_style 	= "setDefaultTextStyle",
	horizontal_alignment  = "setDefaultHorizontalAlignment",
}, lib.widget)

lib.picture = builder (ui.Picture, {
	texture = "setTexture",
	mode 	= "setPlacingMode",
	color 	= "setColor",
	effect	= "setCustomEffect",
	scissors = "setScissors",
}, lib.widget)

lib.text_button = builder (ui.TextButton, {
	texture 	= "setTexture",
	text 		= "setText",
	text_style 	= "setTextStyle",
}, lib.button)

lib.price_button = builder (ui.PriceButton, {
	texture 	= "setStateTexture",
	currency 	= "setCurrencyIcon",
	price 		= "setPriceText",
	text 		= "setText",
	text_style 	= "setTextStyle",
}, lib.button)

lib.image_button = builder (ui.ImageButton, {
	texture 	= "setTexture",
}, lib.button)

lib.container_button = builder (ui.ContainerButton, {
	texture 	= "setTexture",
	animation	= "setStateAnimation"
}, lib.button)

lib.scroll_widget = builder (ui.ScrollWidget, {
	scroll_enabled      = "setScrollEnabled",
	scroll_type 		= "setScrollType",
	scroll_bar_texture	= "setScrollBarTexture",
}, lib.widget)

lib.clip_widget = builder (ui.ClipWidget, {
}, lib.widget)

lib.progress_bar = builder (ui.ProgressBar, {
	texture = "setTexture",
	value = "setValue",
	text_style = "setTextStyle",
	text = setText,
	left_to_right = "setLeftToRight",
}, lib.widget)

lib.cash_bar = builder (ui.CashBar, {
	texture = "setTexture",
	value   = "setValue",
	text_style = "setTextStyle",
	text 	= "setText",
	left_to_right = "setLeftToRight",
	color = "setColor",
	brightness = "setBrightness",
}, lib.widget)

lib.police_exposure_bar = builder (ui.PoliceExposureBar, {
	heat	= "setHeat",
	text 	= "setText",
	texture = "setTexture",
	text_style = "setTextStyle",
}, lib.widget)

lib.layout = builder (nil, {
	layout_alignment_flags = "setLayoutAlignmentFlags",
	items = function (object, value)
		for i, item in ipairs (value) do
			if item.layoutItem then object:addWidget (item)
			else object:addItem (item) end
		end
	end,
})

lib.vertical_layout = builder (ui.VerticalLayout, {
	spacing = "setItemSpacing",
	size_policy = "setHorizontalPolicy",
}, lib.layout)

lib.horizontal_layout = builder (ui.HorizontalLayout, {
	spacing = "setItemSpacing",
	size_policy = "setVerticalPolicy"	
}, lib.layout)

lib.layer_layout = builder (ui.LayerLayout, {
}, lib.layout)

ui.core.lib = lib
