local view_fallback_delegate =
{
	finished = function(view) end,
	destroy = function (self) end,
	on_back = function (view) end
}

local stack_fallback_delegate =
{
	finished = function (view) end,
	destroy = function (self) end,
	on_back = function (view) view:pop_window() end,
	on_window_pushed = function (view) end,
	on_window_poped = function (view) end,
}

ui.core.view = { _wrapper = ui.WidgetView, _fallback_delegate = view_fallback_delegate }
ui.core.window = setmetatable ({ _wrapper = ui.WidgetWindow, _fallback_delegate = view_fallback_delegate }, { __index = ui.core.view })
ui.core.stacked_window = setmetatable ({ _wrapper = ui.StackedWindow, _fallback_delegate = stack_fallback_delegate }, { __index = ui.core.window })
ui.core.side_menu = setmetatable ({ _wrapper = ui.BookmarksWindow, _fallback_delegate = view_fallback_delegate }, {__index = ui.core.window })
ui.core.move_gesture_view = setmetatable ({ _wrapper = ui.MoveGestureView, _fallback_delegate = view_fallback_delegate }, {__index = ui.core.view })

function ui.core.view.builder (view)
	return {
		build = function(...)
			local obj = setmetatable ({ _delegate = ui.core.view._fallback_delegate }, { __index = view })
			obj._pdata = view._wrapper (obj:build_ui (...))
			events.connect (obj._pdata:getUpdateSignalProxy(), function (dt) obj:update (dt) end)
			events.connect (obj._pdata:getBackSignalProxy(), function() obj:back() end)
			return obj
		end
	}
end

function ui.core.stacked_window.builder (first)
	return {
		build = function()
			local obj = setmetatable ({ _delegate = ui.core.stacked_window._fallback_delegate, _stack = { } }, { __index = ui.core.stacked_window })
			obj._pdata = ui.core.stacked_window._wrapper (first._pdata)
			obj.first = first
			events.connect (obj._pdata:getBackSignalProxy(), function() obj:back() end)
			return obj
		end
	}
end

function ui.core.side_menu.builder (window)
	return {
		build = function()
			local obj = setmetatable ({ _delegate = ui.core.side_menu._fallback_delegate }, { __index = ui.core.side_menu })
			obj._pdata = ui.core.side_menu._wrapper (window._pdata)
			obj.window = window
			events.connect (obj._pdata:getBackSignalProxy(), function() obj:back() end)
			return obj
		end
	}
end

function ui.core.view:get_type()
	return getmetatable (self).__index
end

function ui.core.view:enable()
	self._pdata:asWidget():setEnabled (true)
end

function ui.core.view:disable()
	self._pdata:asWidget():setEnabled (false)
end

function ui.core.view:set_origin (x, y)
	self._pdata:setOrigin (x, y)
end

function ui.core.view:get_origin()
	return self._pdata:getOrigin()
end

function ui.core.view:set_size (w, h)
	self._pdata:setSize (w, h)
end

function ui.core.view:get_size()
	return self._pdata:getSize()
end

function ui.core.view:destroy()
	self._delegate:destroy()
	self._delegate = nil
	self._pdata = nil
end

function ui.core.view:valid()
	return self._pdata ~= nil
end

function ui.core.view:build_ui() 
	return nil 
end

function ui.core.view.delegate_builder(delegate)
	return {
		build = function(view)
			local obj = setmetatable (delegate, { __index = view._fallback_delegate })
			view._delegate = obj 
		end
	}
end

function ui.core.view:finalize()
	self._delegate.finished(self)
	if self.on_destroy then self:on_destroy() end
	self:destroy ()
end

function ui.core.view:update() end
function ui.core.view:back()
	self._delegate.on_back (self)
end

function ui.core.window:add_subview (subview)
	self._pdata:addSubview (subview._pdata)
end

function ui.core.window:title()
	return "Window"
end

function ui.core.window:tag()
	return ""
end

function ui.core.window:icon()
	return nil
end

function ui.core.window:has_title_background()
	return true
end

function ui.core.window:moved_to_foreground(from_background)
end

function ui.core.window:moved_to_background()
end

function ui.core.window:remove_subview (subview)
	self._pdata:removeSubviewDeleteLater (subview._pdata)
end

function ui.core.stacked_window:push_window (window)
	self:top():moved_to_background()
	self._pdata:pushWindow (window._pdata)
	table.insert (self._stack, window)
	self._delegate.on_window_pushed (self)
	window:moved_to_foreground(false)
end

function ui.core.stacked_window:pop_window ()
	local sz = #self._stack
	if sz > 0 then
		self._pdata:popWindow ()
		self._stack[sz]:finalize()
		self._stack[sz] = nil
	end
	self._delegate.on_window_poped (self)
	self:top():moved_to_foreground(true)
end

function ui.core.stacked_window:top()
	local sz = #self._stack
	if sz > 0 then
		return self._stack[sz]
	else
		return self.first
	end
end

function ui.core.stacked_window:size()
	return #self._stack
end

function ui.core.side_menu:open (view)
	self.side_menu = true
	self._pdata:openBookmarks (view._pdata)
end

function ui.core.side_menu:close ()
	self.side_menu = false
	self._pdata:closeBookmarks()
end
