require "ui/core/view.lua"
require "ui/config.lua"
require "ui/animations/factory.lua"
require "ui/text_styles.lua"
require "ui/format.lua"
require "ui/widgets/gold_to_cash_widget.lua"
require "ui/widgets/picture_with_backlight.lua"
require "social/facebook_sharing.lua"
require "underworld2/couriers.lua"

local view = setmetatable (
	{
		RESULT_BUY = 0,
		RESULT_CANCEL = 1,
	}, { __index = ui.core.view })

local builder = ui.core.view.builder (view)
ui.deal_alert_view = view


local characters = {
	"ui/characters/ch_supplier_modal_1_2:filtered",
	"ui/characters/ch_supplier_modal_1_2:filtered",
	"ui/characters/ch_supplier_modal_3:filtered",
	"ui/characters/ch_supplier_modal_4:filtered",
	"ui/characters/ch_supplier_modal_5:filtered",
	"ui/characters/ch_supplier_modal_6:filtered",
	"ui/characters/ch_supplier_modal_7:filtered",
	"ui/characters/ch_supplier_modal_8:filtered"
}

function view.show (commodity_id, amount, duration, on_finished)
	local v = ui.deal_alert_view.create (commodity_id, amount, duration)
	ui.core.view_controller:show_modal_view (v)
	v.delegate_builder (
	{
		finished = function(v)
			ui.core.view_controller:close_modal_view (v)
			if on_finished then on_finished (v.result) end
		end,

		on_back = function(v)
			v.result = view.RESULT_CANCEL
			ui.core.view_controller:close_modal_view (v)
			v:finalize()
		end
	}).build (v)
end

function view.create (...)
	local obj = builder.build(...)
	return obj
end

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui (item_id, amount, duration)
	self.ui = {}

  	local strings = underworld2.StringsDatabase()
	local player = underworld2.player
	local max_money = player:getMaxMoney()
	local current_money = math.min (max_money, player:getMoney())

  	local skill = underworld2.SkillsDatabase():getGroup ("SMUGGLERSHIP"):getSkill ("ITEM")
	local supplier_key = math.max(item_id - 1, 1)
  	local supplier_name = strings:get (skill:getLevel (supplier_key):getTitleKey())
		
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)

	local character_image = ui.Picture()
	character_image:setTexture (characters[item_id])
	widget:addChild (character_image)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)

		local close = ui.ImageButton ()
		close:setTexture ("ui/btn_close:filtered")
		close:setSound ("ui/UI_Back.wav")
		close:setOrigin (0, 0)
		events.connect (close:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_warning_modal:filtered", "ui/backlight_busted:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_DEAL_ALERT_FULL")
		title_label:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		top_layout:addWidget (title_label)
		top_layout:addWidget (close)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText ( L("KEY_CREW_SUPPLIER") .. ", " .. supplier_name )
		text_layout:addWidget (title_label)
		
		--[[local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (strings:get(math.random_value(569, 573))) -- TO DO: guess, should not hardcode
		text_label:setRowsCount (3)
		text_layout:addWidget (text_label)]]
		local text_label = ui.RichTextWidget()
		text_label:setDefaultTextStyle (ui.text_styles.tutorial_text.build())
		text_label:setText (L "KEY_DEAL_ALERT_HINT")
		text_label:setMaxWidth (1.0 - frame:getLeftMargin() - frame:getRightMargin() - text_frame:getLeftMargin() - text_frame:getRightMargin())
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end

	do

		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")

		local text_layout = ui.HorizontalLayout()
		text_layout:setItemSpacing (ui.config.hspacing)
		text_frame:setLayout (text_layout)

		local ico = ui.Picture()
		ico:setTexture(ui.config.commodity_icons[item_id])
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)

		local commodity = underworld2.CommoditiesDatabase():get (item_id)
		commodity_name =  strings:get (commodity:getNameKey())

		local storage = underworld2.player:getStorage (item_id)
		local total_amount = storage:getUsage() + amount
		
		local material_amount = ui.RichTextWidget()
		material_amount:setDefaultTextStyle (ui.text_styles.large.build())
		material_amount:setText (string.format ("[c v='#FF0000']%d[/c]/%d", total_amount, storage:getCapacity()))
		--material_amount:setWordWrap (false)
		material_amount:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		--material_amount:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)


		local confiscated_lbl = ui.TextLabel()
		confiscated_lbl:setTextStyle (ui.text_styles.large.build())
		confiscated_lbl:setText (L ("KEY_DEAL_ALERT_STORAGE_HINT", commodity_name))
		confiscated_lbl:setWordWrap (false)
		confiscated_lbl:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		confiscated_lbl:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)

		text_layout:addWidget (confiscated_lbl)
		text_layout:addWidget (ico)
		text_layout:addWidget (material_amount)

		frame_layout:addWidget (text_frame)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)

		top_layout:addWidget (ui.Widget())

		local price_button = ui.TextButton()
		price_button:setTexture ("ui/btn_lower_normal:filtered")
		price_button:setTextStyle (ui.text_styles.button_regular.build())
		price_button:setText (L "KEY_DEAL_ALERT_BUY")
		price_button:setSize(0.5, 1.0)
		price_button:setHorizontalSizePolicy(ui.SizePolicyFixed)

		top_layout:addWidget (price_button)
		top_layout:addWidget (ui.Widget())

		events.connect (price_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_BUY
			self:finalize()
		end)

		frame_layout:addItem (top_layout)
	end

	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()

		local max_char_height = math.min (1.0 - frame_size.y - 0.05, character_size.y)
		
		character_size.x = character_size.x * max_char_height / character_size.y
		character_size.y = max_char_height
		
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, 1.0 - character_size.y - frame_size.y - 0.05)
		character_image:setSize (character_size)
		character_image:setVerticalSizePolicy (ui.SizePolicyFixed)
		character_image:setHorizontalSizePolicy (ui.SizePolicyFixed)
		character_image:setPlacingMode(ui.Picture_PlacingModeStretch)
		
		frame:setOrigin (0.0, 1.0 - frame_size.y - 0.05)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)

	return widget
end

function view:update (dt)
end
