require "ui/core/view.lua"
require "ui/text_styles.lua"
require "ui/event_view.lua"
require "ui/config.lua"

local deal_card = setmetatable ( 
{
	BTN_OK = 1,
	BTN_CONTACT = 2,
}, { __index = ui.core.view } )

local builder = ui.core.view.builder (deal_card)

function deal_card.create (params)
	local v = builder.build()
	v:set_params (params)
	return v
end

function deal_card:move_to (x, y)
	self.ui.root_widget:setOrigin (x, y)
end

function deal_card:back()
	self.result = deal_card.BTN_OK
	self:finalize()
end

function deal_card:build_ui ()
	self.ui = { }

	local root_widget = ui.Frame ()
	root_widget:setOrigin (0.25, 0.3)
	root_widget:setSize (0.5, 0.25)
	root_widget:setBordersTexture ("ui/frame_modal")

	local root_layout = ui.VerticalLayout()
	root_widget:setLayout (root_layout)

	local hlayout = ui.HorizontalLayout()
	root_layout:addItem (hlayout)

	local avatar = ui.Picture ()
	hlayout:addWidget (avatar)

	local vlayout = ui.VerticalLayout()
	hlayout:addItem (vlayout)

	local nickname = ui.TextLabel()
	vlayout:addWidget (nickname)

	local geoname = ui.TextLabel()
	vlayout:addWidget (geoname)

	self.ui.nickname	= nickname
	self.ui.avatar 		= avatar
	self.ui.geoname 	= geoname

	avatar:setTexture ("ui/taggert")
	avatar:setPlacingMode (ui.Picture_PlacingModeCenter)
	avatar:setVerticalSizePolicy (ui.SizePolicyMinimum)
	avatar:setHorizontalSizePolicy (ui.SizePolicyMinimum)

	nickname:setTextStyle (ui.text_styles.normal_hl.build())
	nickname:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)

	geoname:setTextStyle (ui.text_styles.small.build())
	geoname:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)

	do
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)

		local action = ui.TextLabel()
		action:setTextStyle (ui.text_styles.small.build())
		action:setTextAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalCenter)
		hlayout:addWidget (action)

		local item_icon = ui.Picture()
		item_icon:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		hlayout:addWidget (item_icon)

		local price = ui.TextLabel()
		price:setTextStyle (ui.text_styles.small.build())
		price:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
		hlayout:addWidget (price)

		self.ui.action 		= action
		self.ui.item_icon 	= item_icon
		self.ui.price 		= price

		root_layout:addItem (hlayout)
	end

	do
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)

		local contact = ui.TextButton()
		events.connect (contact:getClickedSignalProxy(), function()
			self.result = deal_card.BTN_CONTACT
			self:finalize()
		end)
		hlayout:addWidget (contact)

		local ok = ui.TextButton()
		hlayout:addWidget (ok)

		contact:setTexture ("ui/btn_neutral:filtered")
		contact:setTextStyle (ui.text_styles.normal.build())
		contact:setText (L "KEY_DEAL_CARD_CONTACT")

		ok:setTexture ("ui/btn_lower_normal:filtered")
		ok:setTextStyle (ui.text_styles.normal.build())
		ok:setText (L "KEY_DEAL_CARD_OK")

		events.connect (ok:getClickedSignalProxy(), function ()
			self.result = deal_card.BTN_OK
			self:finalize()
		end)

		self.ui.btn_contact = contact
		self.ui.btn_ok = ok

		root_layout:addItem (hlayout)
	end

	return root_widget
end

function deal_card:set_params (params)
	self.params = params
	if params.item and params.courier and params.owner then
		self.ui.nickname:setText (params.owner.nickname or (params.type == underworld2.deal.BUY and L "KEY_LAB" or L "KEY_JUNKIE"))
		self.ui.geoname:setText (L "KEY_DEAL_CARD_GEONAME_UNKNOWN")
		self.ui.action:setText (params.type == underworld2.deal.BUY and L "KEY_SOLD" or L "KEY_BOUGHT")
		self.ui.price:setText (string.format ("x%d = $%s", params.item.count, ui.format.currency_string (params.item.count * params.item.unit_price)))
		self.ui.item_icon:setTexture (ui.config.courier_item_icons[params.item.id])
	end
end

ui.deal_card_view = deal_card
