require "ui/core/view.lua"
require "ui/text_styles.lua"

local view = setmetatable ({ }, { __index = ui.core.view })
ui.event_view = view

local builder = ui.core.view.builder (view)

function view.create (text)
	local obj = builder.build()
	if text then
		obj:set_text (text)
	end
	return obj
end

local instance = nil
local locks = 0
local titles = {}

local function __push_title( title )
	titles[locks] = title
	if instance then
		instance:set_text( title )
	end
end

local function __pop_title()
	titles[locks] = nil
	if instance and titles[locks - 1] then
		instance:set_text( titles[locks - 1] )
	end
end

function view.show( title )
	if not instance then
		instance = view.create (L("KEY_PROFILE_RENEW"))
		ui.core.view_controller:show_modal_view (instance)
	end
	locks = locks + 1
	__push_title( title or L "KEY_PROFILE_RENEW" )
end

function view.close()
	__pop_title()

	locks = locks - 1
	if locks == 0 then
		ui.core.view_controller:close_modal_view (instance)
		instance = nil
	end
end

function view.force_close()
	locks = 0
	if instance then
		ui.core.view_controller:close_modal_view (instance)
		instance = nil
	end
end

function view:build_ui()
	self.ui = { }

	local root_widget = ui.Frame("root_widget")
	root_widget:setBordersTexture ("ui/modal_wide_window:filtered")
	--root_widget:setSize(0.7, 0.3)
	--root_widget:setOrigin(0.1, 0.45)

	local vlayout = ui.VerticalLayout()
	vlayout:setItemSpacing (ui.config.vspacing)
	
	local v_sep = ui.Widget()
	v_sep:setSize(1.0, 0.015)
	v_sep:setVerticalSizePolicy(ui.SizePolicyFixed)
	vlayout:addWidget(v_sep)

	do
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)
		
		hlayout:addWidget (ui.Widget())

		local waiting_widget = ui.WaitingWidget()
		waiting_widget:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		hlayout:addWidget (waiting_widget)

		local event_text = ui.TextLabel("event_text")
		event_text:setTextStyle(ui.text_styles.normal_hl.build())
		event_text:setTextAlignment(ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
		event_text:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		event_text:setWordWrap (false)
		hlayout:addWidget(event_text)

		hlayout:addWidget (ui.Widget())

		self.ui.event_text = event_text

		vlayout:addItem (hlayout)
	end



	local tips_label = ui.TextLabel()
	tips_label:setTextStyle (ui.text_styles.hint_grey_normal.build())
	tips_label:setText (L (string.format ("KEY_GAME_TIP_%d", math.random (18))))
	tips_label:setRowsCount (3)
	tips_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
	vlayout:addWidget (tips_label)

	root_widget:setLayout (vlayout)

	local frame_size = root_widget:getPreferredSize()
	frame_size.x = 1.0

	root_widget:setSize (frame_size)
	root_widget:setOrigin (0, (1.0 - frame_size.y) * 0.5)

	return root_widget
end

function view:set_text( text )
	self.text = text
	self.ui.event_text:setText(text)
end
