ui = ui or {}
ui.format = { }

local function __durarion_internal_short (d, h, m, s)
	if d > 0 and h > 0 then
		return L("KEY_DURATION_DAYS", d, h)
	elseif d > 0 then
		return L("KEY_DURATION_DAYS_SHORT", d)
	elseif h > 0 and m > 0 then
		return L("KEY_DURATION_HOURS", h, m)
	elseif h > 0 then
		return L("KEY_DURATION_HOURS_SHORT", h)
	elseif m > 0 and s > 0 then
		return L("KEY_DURATION_MINUTES", m, s)
	elseif m > 0 then
		return L("KEY_DURATION_MINUTES_SHORT", m)
	else
		return L("KEY_DURATION_SECONDS", s )
	end
end

function ui.format.modifier (value)
	local reminder = value - math.floor (value)
	local fmt = reminder > 0.05 and "x%.1f" or "x%.0f"
	return string.format (fmt, value)
end

function ui.format.duration_string (tm, deprecate_short)
	if deprecate_short then return L("{0,time_span}",tm) end

	local secs 	= math.floor (tm)
	local mins 	= math.floor (secs / 60)
	local hrs = math.floor (mins / 60)
	local days = math.floor (hrs / 24) 

	hrs = hrs % 24
	mins = mins % 60
	secs = secs % 60

	return __durarion_internal_short (days, hrs, mins, secs)
end

function ui.format.date_string (tm, fmt)
	fmt = fmt or "%x"
	return os.date (fmt, tm)
end

function ui.format.time_passed (a, b)
	local dt = b and os.difftime (a, b) or a
	local time_passed = dt / 60
	if time_passed < 1 then
		return L ("KEY_TIME_HINT_MOMENTS_AGO")
	elseif time_passed < 60 then
		return L("KEY_TIME_HINT_MINUTES_AGO", time_passed)
	elseif time_passed < 24 * 60 then
		return L("KEY_TIME_HINT_HOURS_AGO", time_passed / 60)
	elseif time_passed < 2 * 24 * 60 then
		return L ("KEY_TIME_HINT_YESTERDAY")
	end
	return os.date ("%x", b)
end

local function fraction(value)
	return value - math.floor(value)
end

function ui.format.currency_string (price)
	local val = math.abs (price)
	if val < 1000 then
		return string.format ("%d", math.floor (val))
	elseif price < 1000000 then
		local v = val / 1000
		local f = fraction(v)
		if f < 0.01 then
			return string.format ("%d k", v)
		else
			return string.format ("%.1f k", v)
		end
	else
		local v = val / 1000000
		local f = fraction(v)
		if f < 0.01 then
			return string.format ("%d mil", v)
		else
			return string.format ("%.1f mil", v)
		end
	end
end

function ui.format.digit (value, base)
    base = base or 1000
    
    local retval = ""
    local remainder = math.floor (value % base)
    value = math.floor (value / base)
    if value > 0 then
      retval = string.format ("%03d", remainder)
    else
      retval = tostring (remainder)
    end
      
    while (value > 0) do
      remainder = math.floor (value % base)
      value = math.floor (value / base)
      if value > 0 then
        retval = string.format ("%03d %s", remainder, retval)
      else
        retval = string.format ("%d %s", tostring (remainder), retval)
      end
    end
    return retval
  end
