require "ui/core/view.lua"
require "ui/text_styles.lua"

local view = setmetatable ({
	RESULT_CHOOSE_ANOTHER = 1,
	RESULT_DONE = 2
}, { __index = ui.core.view })

ui.image_customization_view = view

local builder = ui.core.view.builder (view)

function view.create (img)
	local obj = builder.build(img)
	return obj
end

function view:back()
	--self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:set_image(img)
	local picture = self.picture

	if img then
		picture:setImage(img)
	else -- TEST
		picture:setTexture(bundle.resources.wallpapers.login)
	end

	local pic_size = picture:getPreferredSize()

	picture:setViewSpan(true)
	picture:setViewBounds(math.rect(self.sel_center,self.sel_size))
	picture:setViewPosition(self.sel_center)

	local scale = 1.0

	local display = render.GetDisplay()
	local w = display:getWidth()
	local h = display:getHeight()

	if pic_size.x*w < pic_size.y*h then
		if pic_size.x > self.sel_size.x then
			scale = self.sel_size.x/pic_size.x
		end
	else
		if pic_size.y > self.sel_size.y then
			scale = self.sel_size.y/pic_size.y
		end
	end

	pic_size.x = pic_size.x * scale
	pic_size.y = pic_size.y * scale

	self.scale = scale

	picture:setSize(pic_size)
	picture:setZoomMax(1.0/scale)
	picture:setZoomValue(1.0)
end

function view:build_ui(img)
	self.ui = { }

	local root_widget = ui.Background()
	root_widget:setTexture ("ui/1px_Samples/bg_dark:filtered")
	root_widget:setSize(1.0, 1.0)
	root_widget:setOrigin(0, 0)
	root_widget:setColor(render.color(0,0,0,255))

	local picture = ui.PictureSelector()
	self.picture = picture

	--picture:setPlacingMode (ui.Picture_PlacingModeStretch)
	picture:setHorizontalSizePolicy(ui.SizePolicyFixed)
	picture:setVerticalSizePolicy(ui.SizePolicyFixed)
	picture:setOrigin (0, 0)

	local display = render.GetDisplay()
	local w = display:getWidth()
	local h = display:getHeight()

	local sel_size = math.vec2(0.4,(0.4*w)/h)
	local sel_center = math.vec2(0.5-sel_size.x/2, 0.5-sel_size.y/2)

	self.sel_size = sel_size
	self.sel_center = sel_center

	local selector = ui.Picture()
	selector:setTexture("ui/1px_Samples/white:filtered")
	selector:setPlacingMode (ui.Picture_PlacingModeStretch)
	selector:setHorizontalSizePolicy(ui.SizePolicyFixed)
	selector:setVerticalSizePolicy(ui.SizePolicyFixed)
	selector:setSize (1.0, 1.0)
	selector:setOrigin (0, 0)
	selector:setDiscard (math.rect(sel_center.x,sel_center.y,sel_size.x,sel_size.y))
	selector:setColor(render.color(0,0,0,100))

	root_widget:addChild (picture)
	root_widget:addChild (selector)

	do

		local function build_line(from, to)
			local line_width = 0.003
			local line_height = (line_width*w)/h

			local line = ui.Picture()
			line:setTexture("ui/icons/Map/Main/connection_line:filtered")
			line:setPlacingMode (ui.Picture_PlacingModeStretch)
			line:setHorizontalSizePolicy(ui.SizePolicyFixed)
			line:setVerticalSizePolicy(ui.SizePolicyFixed)
			
			local l_w = sel_size.x*(to.x-from.x)
			local l_h = sel_size.y*(to.y-from.y)

			local l_x = sel_center.x+from.x*sel_size.x
			local l_y = sel_center.y+from.y*sel_size.y

			if math.abs(l_w) < line_width then l_w=line_width; l_x=l_x-l_w*to.x end
			if math.abs(l_h) < line_height then l_h=line_height; l_y=l_y-l_h*to.y end

			line:setOrigin(l_x, l_y)
			line:setSize(l_w,l_h)

			return line
		end

		local l_1 = build_line(math.vec2(0.0, 0.0), math.vec2(1.0, 0.0))
		local l_2 = build_line(math.vec2(1.0, 0.0), math.vec2(1.0, 1.0))
		local l_3 = build_line(math.vec2(0.0, 1.0), math.vec2(1.0, 1.0))
		local l_4 = build_line(math.vec2(0.0, 0.0), math.vec2(0.0, 1.0))

		root_widget:addChild(l_1)
		root_widget:addChild(l_2)
		root_widget:addChild(l_3)
		root_widget:addChild(l_4)
	end

	do
		local hdr = ui.Picture()
		hdr:setTexture("ui/1px_Samples/white:filtered")
		hdr:setPlacingMode (ui.Picture_PlacingModeStretch)
		hdr:setHorizontalSizePolicy(ui.SizePolicyFixed)
		hdr:setVerticalSizePolicy(ui.SizePolicyFixed)
		hdr:setSize (1.0, 0.08)
		hdr:setOrigin (0, 0)
		hdr:setColor(render.color(120,120,120,255))

		local notice_label = ui.TextLabel()
		notice_label:setTextStyle (ui.text_styles.modal_window_title_big.build())
		notice_label:setTextAlignment (ui.AlignmentVerticalTop + ui.AlignmentHorizontalCenter)
		notice_label:setText (L "KEY_MOVE_AND_SCALE")
		notice_label:setWordWrap(false)

		local hlayout = ui.HorizontalLayout()
		hlayout:addWidget (ui.Widget())
		hlayout:addWidget (notice_label)
		hlayout:addWidget (ui.Widget())
		hdr:setLayout(hlayout)

		root_widget:addChild(hdr)
	end

	self:set_image(img)

	do
		local widget = ui.Widget()
		local vlayout = ui.VerticalLayout()
		vlayout:setItemSpacing(ui.config.vspacing*2)
		widget:setLayout(vlayout)

		local v_max = ui.Widget()
		v_max:setVerticalSizePolicy(ui.SizePolicyMaximum)
		vlayout:addWidget(v_max)

		do
			local button_layout = ui.HorizontalLayout()
			button_layout:setItemSpacing(ui.config.hspacing*2)
			vlayout:addItem(button_layout)

			local repeat_btn = ui.TextButton()
			repeat_btn:setTexture ("ui/Buttons/btn_sort:filtered")
			repeat_btn:setTextStyle (ui.text_styles.button_regular.build())
			repeat_btn:setText (L"KEY_AVATAR_CHOOSE_ANOTHER")
			repeat_btn:setHorizontalSizePolicy(ui.SizePolicyFixed)
			repeat_btn:setWidth(0.46)

			local action_btn = ui.TextButton()
			action_btn:setTexture ("ui/settings/btn_green_normal:filtered")
			action_btn:setTextStyle (ui.text_styles.button_regular.build())
			action_btn:setText (L"KEY_BUTTON_DONE")
			action_btn:setHorizontalSizePolicy(ui.SizePolicyFixed)
			action_btn:setWidth(0.46)

			button_layout:addWidget(ui.Widget())
			button_layout:addWidget(repeat_btn)
			button_layout:addWidget(action_btn)
			button_layout:addWidget(ui.Widget())

			events.connect (action_btn:getClickedSignalProxy(), function()
				self.result = view.RESULT_DONE

				local p_w = sel_size.x*w
				local p_h = sel_size.y*h

				local picture_origin = picture:getRemappedPosition()
				local origin = math.vec2(sel_center.x - picture_origin.x, sel_center.y - picture_origin.y)

				self.image_selected_area = math.rect(math.vec2(origin.x * w, origin.y * h), math.vec2(p_w, p_h))
				self.image_selected_scale = picture:getZoomValue()*self.scale
				self:finalize()
			end)

			events.connect (repeat_btn:getClickedSignalProxy(), function()
				self.result = view.RESULT_CHOOSE_ANOTHER
				if self.cb_choose_another then
					self.cb_choose_another()
				else
					self:finalize()
				end
			end)
		end

		vlayout:addWidget(ui.Widget())

		widget:setHorizontalSizePolicy(ui.SizePolicyFixed)
		widget:setVerticalSizePolicy(ui.SizePolicyFixed)

		widget:setSize(1.0, 1.0)

		root_widget:addChild(widget)
	end

	return root_widget
end

function view.show (img, cb_choose_another, cb_done)
	local v = view.create (img)

	ui.core.view.delegate_builder(
	{
		finished = function (view)
			ui.core.view_controller:close_modal_view (view)
			if view.result == view.RESULT_DONE and cb_done then
				cb_done(view.image_selected_area, view.image_selected_scale)
			end
		end
	}).build (v)

	v.cb_choose_another = cb_choose_another

	ui.core.view_controller:show_modal_view (v)
end