require "ui/core/view.lua"
require "ui/text_styles.lua"
require "ui/config.lua"

local view = setmetatable ({
	BUTTON_OK 		= 1,
	BUTTON_YES 		= 2,
	BUTTON_CANCEL 	= 3,
	BUTTON_NO 		= 4,
	BUTTON_CONTINUE = 5,
	BUTTON_TRY_AGAIN = 6,
	BUTTON_USER0    = 7,
	BUTTON_USER1	= 8,
	BUTTON_RELOGIN  = 9
 }, { __index = ui.core.view })

local builder = ui.core.view.builder (view)

local button_config =
{
	[view.BUTTON_OK] 		= { text_style = ui.text_styles.button_regular, texture = "ui/btn_lower_normal:filtered", text = L ("KEY_BTN_OK") },
	[view.BUTTON_YES] 		= { text_style = ui.text_styles.button_regular, texture = "ui/btn_lower_normal:filtered", text = L ("KEY_BTN_YES") },
	[view.BUTTON_CANCEL] 	= { text_style = ui.text_styles.button_regular, texture = "ui/btn_lower_normal:filtered", text = L ("KEY_BTN_CANCEL") },
	[view.BUTTON_NO] 		= { text_style = ui.text_styles.button_regular, texture = "ui/btn_lower_normal:filtered", text = L ("KEY_BTN_NO") },
	[view.BUTTON_CONTINUE] 	= { text_style = ui.text_styles.button_regular, texture = "ui/btn_lower_normal:filtered", text = L ("KEY_BTN_CONTINUE") },
	[view.BUTTON_TRY_AGAIN] = { text_style = ui.text_styles.button_regular, texture = "ui/btn_lower_normal:filtered", text = L ("KEY_BTN_TRY_AGAIN") },
	[view.BUTTON_RELOGIN] = { text_style = ui.text_styles.button_regular, texture = "ui/btn_lower_normal:filtered", text = L ("KEY_BTN_RELOGIN") },
}

view.button_config = button_config

function view.create (text, buttons)
	local v = builder.build()
	v.ui.info_label:setText (text)
	v:set_buttons (buttons or { view.BUTTON_OK })
	return v
end

function view:build_ui()
	local frame = ui.Frame()
	frame:setOrigin(math.vec2(0.1, 0.3))
	frame:setSize(math.vec2(0.8, 0.4))
	frame:setBordersTexture("ui/window_modal_large:filtered")
	
	local info = ui.TextLabel()
	info:setTextStyle(ui.text_styles.small.build())
	info:setTextAlignment(ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
	info:setVerticalSizePolicy (ui.SizePolicyMaximum)
	
	local vlayout = ui.VerticalLayout()
	vlayout:setItemSpacing (ui.config.vspacing)
	
	vlayout:addWidget (info)
	frame:setLayout(vlayout)
	
	self.ui = {
		info_label	= info,
		vlayout = vlayout
	}
	
	return frame
end

function view:set_buttons(buttons)
	local buttons_layout = ui.HorizontalLayout()
	buttons_layout:setItemSpacing (ui.config.hspacing)

	for i, v in ipairs (buttons) do
		local button = ui.TextButton()
		if type (v) == "table" then
			button:setTextStyle (v.text_style.build())
			button:setText (v.text)
			button:setTexture (v.texture)
			button:setTag (v.tag)
			events.connect (button:getClickedSignalProxy(), v.handler)
		else
			local cfg = button_config[v]
			button:setTextStyle (cfg.text_style.build())
			button:setText (cfg.text)
			button:setTexture (cfg.texture)
			button:setTag (v)
			events.connect(button:getClickedSignalProxy(), function(button) 
				self.result = button:getTag()
				self:finalize()
			end)
		end
		buttons_layout:addWidget (button)
	end
	self.ui.vlayout:addItem (buttons_layout)
end

ui.information_view = view
