require "ui/text_styles.lua"

local loan_shark = setmetatable ({ 

	MODE_CHOOSE_LOAN = 1,
	MODE_PAYMENT = 2

}, { __index = ui.core.window })

local __build_payment_part_widget = function (ico_sz, paid)
	local widget = ui.Frame()
	widget:setBordersTexture (paid and "ui/loan_shark/frame_payment_complete:filtered" or "ui/loan_shark/frame_payment:filtered")
	widget:setMargins (ui.config.hspacing, ui.config.hspacing, ui.config.vspacing, ui.config.vspacing)
	
	local ico = ui.Picture()
	ico:setTexture(paid and "ui/loan_shark/ico_tick_large:filtered" or "ui/loan_shark/ico_cash_large:filtered")
	ico:setSize (ico:getPreferredSize())
	ico:setOrigin ((ico_sz - ico:getPreferredSize()) * 0.5)

	widget:setHorizontalSizePolicy (ui.SizePolicyFixed)
	widget:setVerticalSizePolicy (ui.SizePolicyFixed)
	widget:setSize (math.vec2 (widget:getLeftMargin() + widget:getRightMargin(), widget:getTopMargin() + widget:getBottomMargin()) + ico_sz)
	widget:addChild (ico)

	return widget
end

local __build_payment_progress_ui = function (loan)
	local widget = ui.Widget()

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)

	local header_layout = ui.HorizontalLayout()
	header_layout:setItemSpacing (ui.config.hspacing)

	local header_label = ui.TextLabel()
	header_label:setTextStyle (ui.text_styles.large_hl:build())
	header_label:setText (L ("KEY_LOAN_SHARK_PAYMENT_HEADER"))
	header_layout:addWidget (header_label)

	local time_ico = ui.Picture()
	time_ico:setTexture ("ui/ico_timer:filtered")
	time_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	header_layout:addWidget (time_ico)

	local deadline_label = ui.TextLabel()
	deadline_label:setText ("3d 12h 8m")
	deadline_label:setTextStyle (ui.text_styles.large_hl.build())
	deadline_label:setWordWrap(false)
	deadline_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	header_layout:addWidget (deadline_label)

	local payments_layout = ui.HorizontalLayout()

	local ico = ui.Picture()
	ico:setTexture("ui/loan_shark/ico_cash_large:filtered")

	local ico_sz = ico:getPreferredSize()	

	for i = 1, 5 do
		local l = ui.VerticalLayout()
		l:setItemSpacing (ui.config.vspacing)
		
		local t = ui.TextLabel()
		t:setTextStyle (ui.text_styles.normal_hl.build())
		t:setText (L("KEY_LOAN_SHARK_PAYMENT_PART_HINT", i))
		t:setTextAlignment (ui.AlignmentVerticalTop + ui.AlignmentHorizontalCenter)

		local w = __build_payment_part_widget (ico_sz, (loan.total_loan - loan.debt) >= (loan.payment * i))
		
		l:addWidget (w)
		l:addWidget (t)

		payments_layout:addItem (l)
		if i ~= 5 then
			payments_layout:addWidget (ui.Widget())
		end
	end

	ui.destroy (ico)

	layout:addItem (header_layout)
	layout:addWidget (ui.Widget())
	layout:addItem (payments_layout)
	layout:addWidget (ui.Widget())

	
		local pay_btn = ui.PriceButton()
		pay_btn:setStateTexture ("ui/btn_lower_normal:filtered")
		pay_btn:setTextStyle (ui.text_styles.button_regular.build())
		pay_btn:setCurrencyIcon ("ui/ico_cash_small:filtered")
		pay_btn:setText (L ("KEY_LOAN_SHARK_PAYMENT_PAY"))
		pay_btn:setPrice (loan.payment)

		local pay_layout = ui.HorizontalLayout()
		pay_layout:addWidget (ui.Widget())
		pay_layout:addWidget (pay_btn)
		pay_layout:addWidget (ui.Widget())

		layout:addItem (pay_layout)
	

	widget:setLayout (layout)

	return widget, deadline_label, pay_btn 
end 

local __build_loan_button = function (type_text, amount_text)
	local btn = ui.UniversalButton()
	btn:setTexture ("ui/button_universal:filtered")
	btn:setStretch (ui.UniversalButton_StretchHorizontal + ui.UniversalButton_StretchVertical)
	btn:setPrimaryColor (ui.Button_ButtonStateNormal, render.color (145, 138, 133, 255))
	btn:setSecondaryColor (ui.Button_ButtonStateNormal, render.color (103, 96, 93, 255))
	btn:setPrimaryColor (ui.Button_ButtonStatePressed, render.color (36, 243, 127, 255))
	btn:setSecondaryColor (ui.Button_ButtonStatePressed, render.color (24, 203, 24, 255))

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)

	local type_label = ui.TextLabel()
	type_label:setTextStyle (ui.text_styles.button_regular.build())
	type_label:setText (type_text)
	type_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
	layout:addWidget (type_label)

	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing (ui.config.hspacing)

	hlayout:addWidget (ui.Widget())

	local ico = ui.Picture()
	ico:setTexture ("ui/ico_cash_small:filtered")
	ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hlayout:addWidget (ico)

	local price_label = ui.TextLabel()
	price_label:setTextStyle (ui.text_styles.button_regular.build())
	price_label:setText (amount_text)
	price_label:setWordWrap (false)
	price_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hlayout:addWidget (price_label)

	hlayout:addWidget (ui.Widget())

	layout:addItem (hlayout)

	btn:setLayout (layout)

	return btn
end

local __build_take_loan_ui = function (loans, loan_selected_handler)
	local widget = ui.Widget()
	widget:setVerticalSizePolicy (ui.SizePolicyMinimum)

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)

	local header_label = ui.TextLabel()
	header_label:setTextStyle (ui.text_styles.large_hl:build())
	header_label:setText (L ("KEY_LOAN_SHARK_CHOOSE_AMOUNT_HEADER"))
	layout:addWidget (header_label)

	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing (ui.config.hspacing * 2)

	local btn_group = ui.ButtonGroup("take_loan_group", true)

	local btn = __build_loan_button (L ("KEY_LOAN_SHARK_AMOUNT_SMALL"), tostring (loans[1]))
	btn:setTag (1)
	btn:setToggleButton (true)
	btn:setButtonGroup (btn_group)
	events.connect (btn:getClickedSignalProxy(), loan_selected_handler)
	hlayout:addWidget (btn)

	local btn = __build_loan_button (L ("KEY_LOAN_SHARK_AMOUNT_MEDIUM"), tostring (loans[2]))
	btn:setTag (2)
	btn:setToggleButton (true)
	btn:setButtonGroup (btn_group)
	events.connect (btn:getClickedSignalProxy(), loan_selected_handler)
	hlayout:addWidget (btn)

	local btn = __build_loan_button (L ("KEY_LOAN_SHARK_AMOUNT_LARGE"), tostring (loans[3]))
	btn:setTag (3)
	btn:setToggleButton (true)
	btn:setButtonGroup (btn_group)
	events.connect (btn:getClickedSignalProxy(), loan_selected_handler)
	hlayout:addWidget (btn)

	layout:addItem (hlayout)

	widget:setLayout (layout)
	return widget
end

local __build_confirm_widget_ui = function()
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/loan_shark/frame_main:filtered")
	frame:setVerticalSizePolicy (ui.SizePolicyMinimum)

	local header_layout = ui.HorizontalLayout()
	header_layout:setItemSpacing (ui.config.hspacing)

	local warn_ico = ui.Picture()
	warn_ico:setTexture ("ui/loan_shark/ico_warning:filtered")
	warn_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	header_layout:addWidget (warn_ico)

	local header_label = ui.TextLabel()
	header_label:setTextStyle (ui.text_styles.large_hl:build())
	header_label:setText (L ("KEY_LOAN_SHARK_LOAN_CONFIRM_HEADER"))
	header_layout:addWidget (header_label)

	local hint_layout = ui.HorizontalLayout()
	hint_layout:setItemSpacing (ui.config.hspacing)

	local payment_hint = ui.TextLabel()
	payment_hint:setTextStyle (ui.text_styles.large_hl:build())
	payment_hint:setText (L ("KEY_LOAN_SHARK_PAYMENT_HEADER"))
	hint_layout:addWidget (payment_hint)

	local ico_cash = ui.Picture()
	ico_cash:setTexture ("ui/ico_cash_small:filtered")
	ico_cash:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hint_layout:addWidget (ico_cash)

	local sum_label = ui.TextLabel()
	sum_label:setText ("1250 x 5 days")
	sum_label:setTextStyle (ui.text_styles.large_hl.build())
	sum_label:setWordWrap(false)
	sum_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hint_layout:addWidget (sum_label)

	local btns_layout = ui.HorizontalLayout()
	btns_layout:setItemSpacing (ui.config.hspacing * 2)

	--[[local btn_reject = ui.UniversalButton()
	btn_reject:setTextStyle (ui.text_styles.button_regular.build())
	btn_reject:setText (L ("KEY_LOAN_SHARK_LOAN_CONFIRM_REJECT"))
	btn_reject:setTexture ("ui/button_universal:filtered")
	btn_reject:setPrimaryColor (ui.Button_ButtonStateNormal, render.color (145, 138, 133, 255))
	btn_reject:setSecondaryColor (ui.Button_ButtonStateNormal, render.color (103, 96, 93, 255))
	btn_reject:setPrimaryColor (ui.Button_ButtonStatePressed, render.color (184, 178, 175, 255))
	btn_reject:setSecondaryColor (ui.Button_ButtonStatePressed, render.color (126, 118, 113, 255))
	btns_layout:addWidget (btn_reject)]]

	btns_layout:addWidget (ui.Widget())

	local btn_accept = ui.UniversalButton()
	btn_accept:setTextStyle (ui.text_styles.button_regular.build())
	btn_accept:setText (L ("KEY_LOAN_SHARK_LOAN_CONFIRM_ACCEPT"))
	btn_accept:setTexture ("ui/button_universal:filtered")
	btn_accept:setPrimaryColor (ui.Button_ButtonStateNormal, render.color (31, 222, 101, 255))
	btn_accept:setSecondaryColor (ui.Button_ButtonStateNormal, render.color (21, 167, 21, 255))
	btn_accept:setPrimaryColor (ui.Button_ButtonStatePressed, render.color (36, 243, 127, 255))
	btn_accept:setSecondaryColor (ui.Button_ButtonStatePressed, render.color (24, 203, 24, 255))
	btn_accept:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	btns_layout:addWidget (btn_accept)

	btns_layout:addWidget (ui.Widget())

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)
	layout:addItem (header_layout)
	layout:addItem (hint_layout)
	layout:addItem (btns_layout)

	frame:setLayout (layout)	

	return frame, sum_label, btn_accept
end

function loan_shark:build_ui( )
	local tinsert, get_skill_level = table.insert, underworld2.profile.get_skill_level
	
	_skill_map = { }

	self.ui = { }

	local W = ui.Background()
	W:setOrigin (0, 0)
	W:setSize (1, 1)
	W:setTexture("ui/bg_dark", false)

	local scroll = ui.ScrollWidget()
	scroll:setScrollType (ui.ScrollTypeVertical)
	scroll:setOrigin (0.0, underworld2.ui.status_panel:get_size().y)
	scroll:setSize (1.0, 1.0 - underworld2.ui.status_panel:get_size().y)
	scroll:setScrollBarTexture ("ui/scroll:filtered")
	W:addChild (scroll)

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/loan_shark/frame_main:filtered")
	frame:setHorizontalSizePolicy (ui.SizePolicyFixed)
	frame:setVerticalSizePolicy (ui.SizePolicyMinimum)

	local frame_layout = ui.VerticalLayout()

	local frame_origin = math.vec2 (0, underworld2.ui.status_panel:get_size().y + ui.config.vspacing)
	local frame_size = math.vec2 (1.0 - scroll:getLeftMargin() - scroll:getRightMargin(), 0)

	frame:setOrigin (frame_origin)
	frame:setSize (frame_size)
	
	do
		local header_widget = ui.Widget()

		local header_image = ui.Picture()
		local header_image_origin = math.vec2 (-frame:getLeftMargin() + ui.config.hspacing, -frame:getTopMargin() + ui.config.vspacing)
		header_image:setTexture ("ui/loan_shark/loan_shark_header:filtered")
		header_image:setOrigin (header_image_origin)
		local header_image_size = header_image:getPreferredSize()
		local header_image_scale = (frame_size.x - ui.config.hspacing * 2) / header_image_size.x
		header_image_size = header_image_size * header_image_scale
		header_image:setSize (header_image_size)
		header_image:setPlacingMode (ui.Picture_PlacingModeStretch)
		header_widget:addChild (header_image)

		local header_label = ui.TextLabel()
		header_label:setTextStyle (ui.text_styles.modal_window_title.build())
		header_label:setText (L ("KEY_LOAN_SHARK_GREETINGS_HEADER"))
		header_label:setOrigin (0, header_image_origin.y)
		header_label:setSize (header_image_size)
		header_widget:addChild (header_label)
		self.ui.header_label = header_label

		local char_image = ui.Picture()
		char_image:setTexture ("ui/loan_shark/ill_loan_shark:filtered")
		local char_image_size = char_image:getPreferredSize()
		char_image:setPlacingMode (ui.Picture_PlacingModeStretch)
		char_image:setSize (char_image_size)
		char_image:setOrigin (frame_size.x - frame:getLeftMargin() - frame:getRightMargin() - char_image_size.x, 0)
		header_widget:addChild (char_image)

		local sep_image = ui.Picture()
		sep_image:setTexture ("ui/golden_spacer:filtered")
		local sep_image_size = sep_image:getPreferredSize()
		sep_image_size.x = frame_size.x - frame:getLeftMargin() - frame:getRightMargin()
		sep_image:setSize (sep_image_size)
		sep_image:setOrigin (0, char_image_size.y)
		sep_image:setPlacingMode (ui.Picture_PlacingModeStretch)
		header_widget:addChild (sep_image)

		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small.build())
		text_label:setText (L ("KEY_LOAN_SHARK_GREETINGS_TEXT"))
		text_label:setOrigin (0, header_image_origin.x + header_image_size.y + frame:getTopMargin())
		text_label:setTextAlignment (ui.AlignmentVerticalTop + ui.AlignmentHorizontalLeft)
		text_label:setSize ((frame_size.x - frame:getLeftMargin() - frame:getRightMargin())* 0.7, char_image_size.y)
		header_widget:addChild (text_label)
		self.ui.text_label = text_label

		header_widget:setSize (frame_size.x - frame:getLeftMargin() - frame:getRightMargin(), sep_image_size.y + char_image_size.y)
		header_widget:setHorizontalSizePolicy (ui.SizePolicyFixed)
		header_widget:setVerticalSizePolicy (ui.SizePolicyFixed)

		frame_layout:addWidget (header_widget)

		--frame_layout:addWidget (__build_take_loan_ui())
		--[[local payment_widget = __build_take_loan_ui()
		local payment_widget_size = payment_widget:getPreferredSize()
		payment_widget:setOrigin (0, char_image_size.y + sep_image_size.y)
		payment_widget:setSize (frame_size.x - frame:getLeftMargin() - frame:getRightMargin(), payment_widget_size.y)
		frame:addChild (payment_widget)

		frame:setSize (frame_size.x, payment_widget_size.y + sep_image_size.y + char_image_size.y + frame:getTopMargin() + frame:getBottomMargin())]]
	end

	frame:setLayout (frame_layout)
	--[[local confirm_frame = __build_confirm_widget_ui()
	confirm_frame:setSize (frame_size.x, confirm_frame:getPreferredSize().y)
	confirm_frame:setHorizontalSizePolicy (ui.SizePolicyFixed)
	confirm_frame:setVerticalSizePolicy (ui.SizePolicyFixed)]]

	layout:addWidget (frame)
	--layout:addWidget (confirm_frame)
	scroll:setLayout (layout)

	self.ui.frame_layout = frame_layout
	self.ui.scroll_layout = layout

	underworld2.profile.add_delegate (self)
	self:profile_updated (underworld2.profile)

	return W
end

function loan_shark:destroy()
	underworld2.profile.remove_delegate (self)
end

function loan_shark:clear()
	self.ui.overdue_flag = false

	if self.ui.payment_widget then
		self.ui.frame_layout:removeWidget (self.ui.payment_widget)
	end

	if self.ui.choose_loan_widget then
		self.ui.frame_layout:removeWidget (self.ui.choose_loan_widget)
	end

	if self.ui.confirm_loan_widget then
		self.ui.scroll_layout:removeWidget (self.ui.confirm_loan_widget)
	end

	self.ui.payment_widget = nil
	self.ui.choose_loan_widget = nil
	self.ui.confirm_loan_widget = nil
end

function loan_shark:show_payments (loan)
	self:clear()

	self.ui.text_label:setText (L ("KEY_LOAN_SHARK_PAYMENT_TEXT"))

	self.ui.payment_widget, self.ui.deadline_label, pay_btn = __build_payment_progress_ui (loan)

	events.connect (pay_btn:getClickedSignalProxy(), function()
		if underworld2.profile.money >= underworld2.profile.loan.payment then
			communicating_view = ui.event_view.create (L("KEY_PROFILE_RENEW"), false)
			ui.core.view_controller:show_modal_view (communicating_view)
			underworld2.network.pay_loan (underworld2.profile.loan.payment, function (err)
				ui.core.view_controller:close_modal_view (communicating_view)
				if err == 30 then
					get_sound_pool():play ("ui/SFX_Cash_Out.wav")
					underworld2.ui.show_message (L ("KEY_LOAN_SHARK_PAYMENT_NOT_ENOUGH_MONEY"))
				end
			end)
		else
			get_sound_pool():play ("ui/SFX_Cash_Out.wav")
			underworld2.ui.show_message (L ("KEY_LOAN_SHARK_PAYMENT_NOT_ENOUGH_MONEY"))
		end
	end)
	
	self.ui.frame_layout:addWidget (self.ui.payment_widget)
end

function loan_shark:offer_loans (loans)
	self:clear()

	self.ui.text_label:setText (L ("KEY_LOAN_SHARK_GREETINGS_TEXT"))

	local on_loan_changed = function (index) 
		self.selected_loan = index
		payment_size:setText (L("KEY_LOAN_SHARK_LOAN_CONFIRM_PAYMENT_HINT", math.ceil (1.25 * loans[index])))
	end

	self.ui.confirm_loan_widget, payment_size, btn_accept = __build_confirm_widget_ui ()	
	self.ui.choose_loan_widget = __build_take_loan_ui (loans, function(btn) on_loan_changed (btn:getTag()) end)

	self.ui.frame_layout:addWidget (self.ui.choose_loan_widget)
	self.ui.scroll_layout:addWidget (self.ui.confirm_loan_widget)
	on_loan_changed (1)	

	events.connect (btn_accept:getClickedSignalProxy(), function()
		communicating_view = ui.event_view.create (L("KEY_PROFILE_RENEW"), false)
		ui.core.view_controller:show_modal_view (communicating_view)
		underworld2.network.take_loan (loans[self.selected_loan], function()
			ui.core.view_controller:close_modal_view (communicating_view)
		end)
	end)
end

function loan_shark:profile_updated (profile)
	if profile.loan.debt > 0 then
		self.ui.header_label:setText (L ("KEY_LOAN_SHARK_PAYMENT_HEADER"))
		self:show_payments (profile.loan)
	elseif #profile.loan.available_loans > 0 then
		self.ui.header_label:setText (L ("KEY_LOAN_SHARK_GREETINGS_HEADER"))
		self:offer_loans (profile.loan.available_loans)
	end
end

function loan_shark:update(dt)
	if self.ui.payment_widget and not self.overdue_flag then
		local now = underworld2.server_time()
		local time_left = underworld2.profile.loan.end_payments_ts - now
		if time_left > 0 then
			self.ui.deadline_label:setText (ui.format.duration_string (time_left))
		else
			self.overdue_flag = true
			self.ui.deadline_label:setText (L ("KEY_LOAN_SHARK_PAYMENT_OVERDUE"))
			--self.ui.deadline_label:setTextStyle (ui.text_styles.)
		end
	end
end

function loan_shark:title()
	return L ("KEY_LOAN_SHARK_WINDOW_TITLE")
end

ui.loan_shark = ui.loan_shark or loan_shark
