--require "ui/core/utils.lua"
require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"
require "ui/widgets/item_widget.lua"
require "ui/button_colors.lua"

local view = setmetatable (
	{
		RESULT_CANCEL = 0
	}, { __index = ui.core.view })
ui.avatar_updated = view

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui ()
	local strings = underworld2.StringsDatabase()

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
			
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title_big.build())
		title_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
		title_label:setText (L"KEY_AVATAR_HAS_BEEN_UPDATED")
		top_layout:addWidget (title_label)

		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing(ui.config.hspacing)

		do
			local vlayout = ui.VerticalLayout()
			vlayout:setItemSpacing(ui.config.vspacing)

			local choose_image_label = ui.TextLabel()
			choose_image_label:setTextStyle (ui.text_styles.small_hl.build())
			choose_image_label:setText (L"KEY_YOUR_NEW_AVATAR")
			choose_image_label:setWordWrap(false)
			choose_image_label:setTextAlignment(ui.AlignmentHorizontalCenter)
			vlayout:addWidget(choose_image_label)

			local player = underworld2.player
			player:retain()

			local tutor_1 = ui.AvatarWidget()
			tutor_1:setProfile (underworld2.ProfilePtr (runtime.cast_to (player:get(), "underworld2::Profile")))
			tutor_1:setAvatarSize (underworld2.Avatar_SizeLarge)
			vlayout:addWidget (tutor_1)

			hlayout:addItem(vlayout)
		end

		frame_layout:addItem(hlayout)
	end
	
	frame:setLayout (frame_layout)
	
	do
		local frame_size = frame:getPreferredSize()
		
		frame:setOrigin (0.0, 0.5 - frame_size.y * 0.5)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end

function view.create(...)
	local builder = ui.core.view.builder(view)
	local v = builder.build(...)
	return v
end

function view.show(...)
	local builder = ui.core.view.builder(view)
	local v = builder.build(...)

	ui.core.view.delegate_builder(
	{
		finished = function (view)
			ui.core.view_controller:close_modal_view (view)
		end
	}).build (v)

	ui.core.view_controller:show_modal_view (v)
end