require "ui/core/view.lua"
require "ui/config.lua"
require "ui/animations/factory.lua"
require "ui/text_styles.lua"
require "ui/format.lua"
require "ui/widgets/gold_to_cash_widget.lua"
require "ui/widgets/picture_with_backlight.lua"

require "ui/modal/ditch_confirm.lua"

require "social/facebook_sharing.lua"
require "underworld2/couriers.lua"

require "ui/animations/factory.lua"

local view = setmetatable (
	{
		RESULT_PAYOFF = 0,
		RESULT_CANCEL = 1,
		RESULT_SHARE  = 2
	}, { __index = ui.core.view })

local builder = ui.core.view.builder (view)
ui.busted_modal = view

function view.create (bust_details, allow_fb, text)
	local obj = builder.build (bust_details, allow_fb, text or nil)
	return obj
end

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function __show_ditch_confirm (self)
	local details = self.details
	local amount, icon
	if details.is_purchasing then
		amount = details.amount
		icon   = ui.config.commodity_icons[details.commodity_id]
	else
		amount = details.cash
		icon   = "ui/icons/ico_cash_small:filtered"
	end
	
	ui.ditch_confirm.show (amount, icon, function (confirmed)
		if confirmed then
			self.result = view.RESULT_CANCEL
			self:finalize()
		end
	end)
end

function view:build_ui (bust_details, allow_fb, text)
	local strings = underworld2.StringsDatabase()
	
	text = text or strings:get (math.random_value (569, 573))

	self.details = bust_details
	self.ui = {}

	local player = underworld2.player
	local max_money = player:getMaxMoney()
	local current_money = math.min (max_money, player:getMoney())
	
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)

	local backlights = {
		ui.Picture(),
		ui.Picture()
	}

	backlights[1]:setTexture ("ui/backlight_modal_large:filtered")
	backlights[1]:setColor(render.color(255, 11, 11, 255))
	backlights[1]:setSize(math.vec2(0.9, 0.5))
	backlights[1]:setPlacingMode (ui.Picture_PlacingModeStretch)

	backlights[2]:setTexture ("ui/backlight_modal_large:filtered")
	backlights[2]:setColor(render.color(11, 93, 255, 255))
	backlights[2]:setSize(math.vec2(0.9, 0.5))
	backlights[2]:setPlacingMode (ui.Picture_PlacingModeStretch)

	widget:addChild(backlights[1])
	widget:addChild(backlights[2])
	
	local character_clip = ui.ClipWidget()
	widget:addChild (character_clip)

	local character_image = ui.Picture()
	character_image:setTexture (string.format ("ui/characters/ch_cop_modal:filtered"))
	character_clip:addChild (character_image)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_busted:filtered", "ui/backlight_busted:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_BUSTED")
		top_layout:addWidget (title_label)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText ( L ("KEY_DETECTIVE") )
		text_layout:addWidget (title_label)
		
		local text_label = ui.RichTextWidget()
		text_label:setDefaultTextStyle (ui.text_styles.tutorial_text.build())
		text_label:setText (text)
		text_label:setMaxWidth (1.0 - frame:getLeftMargin() - frame:getRightMargin() - text_frame:getLeftMargin() - text_frame:getRightMargin())
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end

	do
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")

		local text_layout = ui.HorizontalLayout()
		text_layout:setItemSpacing (ui.config.hspacing)
		text_frame:setLayout (text_layout)

		local ico = ui.Picture()
		ico:setTexture(bust_details.is_purchasing and ui.config.commodity_icons[bust_details.commodity_id] or "ui/ico_cash:filtered")
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)

		local commodity_name = L "KEY_CASH"
		if bust_details.is_purchasing then
			local commodity = underworld2.CommoditiesDatabase():get (bust_details.commodity_id)
			commodity_name =  strings:get (commodity:getNameKey())
		end
		
		local material_amount = ui.TextLabel()
		material_amount:setTextStyle (ui.text_styles.large.build())
		material_amount:setText (string.format("%s %s", bust_details.is_purchasing and bust_details.amount or bust_details.cash, commodity_name))
		material_amount:setWordWrap (false)
		material_amount:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		material_amount:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)


		local confiscated_lbl = ui.TextLabel()
		confiscated_lbl:setTextStyle (ui.text_styles.large.build())
		confiscated_lbl:setText (L "KEY_BUSTED_TO_BE_CONFISCATED")
		confiscated_lbl:setWordWrap (false)
		confiscated_lbl:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		confiscated_lbl:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)

		text_layout:addWidget (confiscated_lbl)
		text_layout:addWidget (ico)
		text_layout:addWidget (material_amount)

		frame_layout:addWidget (text_frame)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)

		local ditch_button = ui.PriceButton()
		ditch_button:setStateTexture ("ui/btn_ditch:filtered")
		ditch_button:setTextStyle (ui.text_styles.button_regular.build())
		ditch_button:setPrice (bust_details.is_purchasing and bust_details.amount or bust_details.cash)
		ditch_button:setText (L "KEY_BUSTED_DITCH")

		ditch_button:setCurrencyIcon (bust_details.is_purchasing and ui.config.commodity_icons[bust_details.commodity_id] or "ui/ico_cash_small:filtered")
		top_layout:addWidget (ditch_button)

		events.connect (ditch_button:getClickedSignalProxy(), function()
			__show_ditch_confirm (self)
			--self.result = view.RESULT_CANCEL
			--self:finalize()
		end)

		local price_button = ui.PriceButton()
		price_button:setStateTexture ("ui/btn_premium:filtered")
		price_button:setTextStyle (ui.text_styles.button_regular.build())
		price_button:setPrice (bust_details.payoff_price)
		price_button:setText (L "KEY_BUSTED_BRIBE")
		price_button:setCurrencyIcon ("ui/ico_pbtn_gold:filtered")
		top_layout:addWidget (price_button)

		events.connect (price_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_PAYOFF
			self:finalize()
		end)
		self.ui.bribe_button = price_button
		self.ui.ditch_button = ditch_button

		frame_layout:addItem (top_layout)
	end

	if allow_fb then
		local courier_name   = underworld2.couriers.names[bust_details.courier_tag]
		local courier_gender = underworld2.couriers.gender[bust_details.courier_tag]

		local title_label = ui.TextLabel()

		title_label:setTextStyle (ui.text_styles.small_gold.build())
		title_label:setText ( L ("KEY_FB_SHARING_BUST_DESCRIPTION", { name = courier_name, gender = courier_gender } ) )
		title_label:setTextAlignment (ui.AlignmentHorizontalCenter)
		frame_layout:addWidget (title_label)

		local share_button = underworld2.facebook_sharing.create_button( L "KEY_FB_SHARING_BUST_CAPTION", nil, function()
			underworld2.facebook_sharing.bust( bust_details.commodity_id, bust_details.amount, bust_details.courier_tag, bust_details.location, function(result) 
				if result then
					self.result = view.RESULT_SHARE
					self:finalize()
				end
			end)
		end
		)

		local wrapper = ui.HorizontalLayout()
		wrapper:addWidget (ui.Widget())
		wrapper:addWidget (share_button)
		wrapper:addWidget (ui.Widget())
		frame_layout:addItem (wrapper)

		self.ui.share_button = share_button
	end
	
	frame:setLayout (frame_layout)

	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, 1.0 - character_size.y - frame_size.y - 0.05)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, 1.0 - frame_size.y - 0.05)
		frame:setSize (1.0, frame_size.y)

		local origin = character_image:getOrigin()
		backlights[1]:setOrigin(origin.x - 0.27, origin.y)
		backlights[2]:setOrigin(origin.x + 0.15, origin.y + 0.04)
	end
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()

		local max_char_height = math.min (1.0 - frame_size.y - 0.05, character_size.y)
		
		character_size.x = character_size.x * max_char_height / character_size.y
		character_size.y = max_char_height
		
		character_clip:setOrigin ((1.0 - character_size.x) * 0.5, 1.0 - character_size.y - frame_size.y - 0.05)
		character_clip:setSize (character_size)
		character_image:setSize (character_size)
		character_image:setPlacingMode(ui.Picture_PlacingModeStretch)
		
		frame:setOrigin (0.0, 1.0 - frame_size.y - 0.05)
		frame:setSize (1.0, frame_size.y)

		local set_button_scale = function (self, scale)
			self.ui.bribe_button:setScale (scale)
			self.ui.ditch_button:setScale (scale)
			if self.ui.share_button then 
				self.ui.share_button:setScale (scale) 
			end
		end

		self.character_anim = anim.sequence
		{
			anim.lerp {
				from = character_size.y * 0.1, to = 0.0, time = 0.7,
				object = self, 
				property = function (self, pos_y)
					character_image:setOrigin (0.0, pos_y)
				end,
				on_finished = function()
					self.ui.ditch_button:enableRendering()
					self.ui.bribe_button:enableRendering()
					self.ui.ditch_button:setScale (0.8)
					self.ui.bribe_button:setScale (0.8)
					if self.ui.share_button then
						self.ui.share_button:enableRendering()
						self.ui.share_button:setScale (0.8)
					end
				end
			},
			anim.sequence {
				anim.lerp
				{
					from = 0.8, to = 1.1, time = 0.1,
					object = self, property = set_button_scale
				},
				anim.lerp
				{
					from = 1.1, to = 1.0, time = 0.2,
					object = self, property = set_button_scale,
					on_finished = function() widget:setEnabled (true) end
				}
			}
		}

	end
	
	widget:addChild (frame)

	local red = render.color(255, 11, 11, 255)
	local blue = render.color(11, 93, 255, 255)

	local blink = ui.animations.build {
		duration = 0.5,
		from = 255,
		to = 0,
		times = -1,
		oscillate = true,

		step = function (self, value)
			local l = render.color(((self.iteration + 0) % 4 > 1) and blue or red)
			local r = render.color(((self.iteration + 1) % 4 > 1) and red or blue)

			backlights[1]:setColor(render.color(l.r, l.g, l.b, value))
			backlights[2]:setColor(render.color(r.r, r.g, r.b, 255-value))
		end
	}

	self.animations = ui.animations.factory()
  	self.animations:bind ("blink", blink)
	
	widget:setEnabled (false)
	self.ui.bribe_button:disableRendering()
	self.ui.ditch_button:disableRendering()
	if self.ui.share_button then 
		self.ui.share_button:disableRendering()
	end

	self:update(0)

	return widget
end

function view:update (dt)
	self.animations:update(dt)
	self.character_anim:update (dt)
end
