require "ui/core/view.lua"

local view = setmetatable ({ }, { __index = ui.core.view })

function view.show (courier_tag, rp, handler)
	local v = ui.core.view.builder (view).build (courier_tag, rp)
	ui.core.view.delegate_builder (
	{
		on_back = function (view)
			view:finalize()
		end,

		finished = function()
			ui.core.view_controller:close_modal_view (v)
			if handler then handler() end
		end
	}).build (v)
	ui.core.view_controller:show_modal_view (v)
end

function view:build_ui (courier_tag, rp)
	self.ui = { }

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_clip = ui.ClipWidget()
	widget:addChild (character_clip)

	local character_image = ui.Picture()
	character_image:setTexture (underworld2.couriers.image_large[courier_tag])
	character_clip:addChild (character_image)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_runner_modal:filtered", "ui/vault/backlight_vault:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_CHASE_RUNAWAY_MODAL_TITLE")
		top_layout:addWidget (title_label)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do
		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (L("KEY_CREW_COURIER") .. ", " .. underworld2.couriers.names[courier_tag])
		text_layout:addWidget (title_label)
		
		local texts = underworld2.couriers.chase_texts[courier_tag]

		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (texts[math.random_value (1, #texts)] or "")
		text_label:setRowsCount (2)
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end
	
	do
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")

		local text_layout = ui.HorizontalLayout()
		text_layout:setItemSpacing (ui.config.hspacing)
		text_frame:setLayout (text_layout)

		local ico = ui.Picture()
		ico:setTexture("ui/main/ico_topbar_rp:filtered")
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)

		local rp_amount = ui.TextLabel()
		rp_amount:setTextStyle (ui.text_styles.large_hl.build())
		rp_amount:setText (string.format ("%d RP", rp))
		rp_amount:setWordWrap (false)
		rp_amount:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		rp_amount:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)

		local reward_lbl = ui.TextLabel()
		reward_lbl:setTextStyle (ui.text_styles.large_hl.build())
		reward_lbl:setText (L "KEY_CHASE_RUNAWAY_MODAL_REWARD")
		reward_lbl:setWordWrap (false)
		reward_lbl:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		reward_lbl:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)

		text_layout:addWidget (reward_lbl)
		text_layout:addWidget (ico)
		text_layout:addWidget (rp_amount)

		frame_layout:addWidget (text_frame)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do

		local btn_layout = ui.HorizontalLayout()
		btn_layout:setVerticalPolicy (ui.SizePolicyMinimum)

		local spacer = ui.Widget()
		spacer:setSize (0.25, 0.0)
		spacer:setHorizontalSizePolicy (ui.SizePolicyFixed)
		btn_layout:addWidget (spacer)

		local btn = ui.TextButton()
		btn:setTexture ("ui/btn_lower_normal:filtered")
		btn:setTextStyle (ui.text_styles.button_regular.build())
		btn:setText (L "KEY_CHASE_RUNAWAY_OK")
		events.connect (btn:getClickedSignalProxy(), function()
			self:finalize()
		end)
		self.ui.btn_ok = btn
		btn_layout:addWidget (btn)

		local spacer = ui.Widget()
		spacer:setSize (0.25, 0.0)
		spacer:setHorizontalSizePolicy (ui.SizePolicyFixed)
		btn_layout:addWidget (spacer)

		frame_layout:addItem (btn_layout)

	end

	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_clip:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_clip:setSize (character_size)
		character_image:setSize (character_size)
		character_image:setPlacingMode(ui.Picture_PlacingModeStretch)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)

		local set_button_scale = function (self, scale)
			self.ui.btn_ok:setScale (scale)
		end

		self.anim = anim.sequence
		{
			anim.lerp {
				from = character_size.y * 0.1, to = 0.0, time = 0.7,
				object = self, 
				property = function (self, pos_y)
					character_image:setOrigin (0.0, pos_y)
				end,
				on_finished = function() 
					self.ui.btn_ok:enableRendering()
					self.ui.btn_ok:setScale (0.8)
				end
			},
			anim.lerp
			{
				from = 0.8, to = 1.1, time = 0.1,
				object = self, property = set_button_scale
			},
			anim.lerp
			{
				from = 1.1, to = 1.0, time = 0.2,
				object = self, property = set_button_scale,
				on_finished = function() widget:setEnabled (true) end
			}
		}

	end

	widget:addChild (frame)

	widget:setEnabled (false)
	self.ui.btn_ok:disableRendering()

	self:update (0.0)
	
	return widget
end

function view:update (dt)
	self.anim:update (dt)
end

ui.chase_runaway_view = view

