require "ui/core/view.lua"

local view = setmetatable ({ }, { __index = ui.core.view })

function view.show (city, courier_tag, commodity_id, amount, busted, on_finished)
	local v = ui.core.view.builder (view).build (city, courier_tag, commodity_id, amount, busted)
	ui.core.view.delegate_builder (
	{
		on_back = function (view)
			view:finalize()
		end,
		finished = function()
			ui.core.view_controller:close_modal_view (v)
			on_finished()
		end
	}).build (v)
	ui.core.view_controller:show_modal_view (v)
end

function view:build_ui (city, courier_tag, commodity_id, amount, busted)
	local root = ui.ChaseWidget (city or "NY", courier_tag, commodity_id, amount, busted)
	events.connect (root:getAnimationCompletedSignal(), function()
		self:finalize()
	end)

	local skip_btn = ui.TextButton()
	skip_btn:setTexture ("ui/top_hustler/btn_glossy_blue:filtered")
	skip_btn:setText (L "KEY_CHASE_SKIP")
	skip_btn:setTextStyle (ui.text_styles.button_regular.build())
	skip_btn:setSize (0.3, skip_btn:getPreferredSize().y)
	skip_btn:setOrigin ((1.0 - skip_btn:getSize().x) * 0.5, 1.0 - skip_btn:getHeight() * 2.0)
	events.connect (skip_btn:getClickedSignalProxy(), function() self:finalize() end)
	root:addChild (skip_btn)

	local set_button_scale = function (self, scale)
		skip_btn:setScale (scale)
	end

	self.anim = anim.sequence
	{
		anim.delay { 
			time = 1.0, 
			on_finished = function()
				skip_btn:enableRendering()
				skip_btn:setScale (0.8)
			end
		},
		anim.lerp
		{
			from = 0.8, to = 1.1, time = 0.1,
			object = self, property = set_button_scale,
		},
		anim.lerp
		{
			from = 1.1, to = 1.0, time = 0.2,
			object = self, property = set_button_scale,
			on_finished = function() skip_btn:setEnabled (true) end
		},
		anim.delay
		{
			time = 3.0,
			on_finished = function()
				skip_btn:setEnabled (false)
			end
		},
		anim.lerp
		{
			from = 1.0, to = 1.1, time = 0.2,
			object = self, property = set_button_scale,
		},
		anim.lerp
		{
			from = 1.1, to = 0.8, time = 0.1,
			object = self, property = set_button_scale,
			on_finished = function()
				skip_btn:disableRendering()
			end
		},
	}
	skip_btn:setEnabled (false)
	skip_btn:disableRendering()
	self:update (0.0)

	return root
end

function view:update (dt)
	self.anim:update (dt)
end

ui.chase_view = view

