--require "ui/core/utils.lua"
require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"
require "ui/widgets/item_widget.lua"
require "ui/button_colors.lua"
require "ui/modal/confirm_avatar.lua"

local view = setmetatable (
	{
		RESULT_CANCEL = 0,
		RESULT_CONFIRM = 1
	}, { __index = ui.core.view })
ui.choose_avatar = view

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui (cb)
	local strings = underworld2.StringsDatabase()

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
			
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title_big.build())
		title_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
		title_label:setText (L"KEY_AVATAR_CHOOSE_TITLE")
		top_layout:addWidget (title_label)

		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing(ui.config.hspacing)

		do
			local vlayout = ui.VerticalLayout()
			vlayout:setItemSpacing(ui.config.vspacing)

			local choose_image_label = ui.TextLabel()
			choose_image_label:setTextStyle (ui.text_styles.small_hl.build())
			choose_image_label:setText (L"KEY_AVATAR_CHOOSE_IMAGE")
			choose_image_label:setWordWrap(false)
			choose_image_label:setTextAlignment(ui.AlignmentHorizontalCenter)
			choose_image_label:setHorizontalSizePolicy(ui.SizePolicyMinimum)
			vlayout:addWidget(choose_image_label)

			local tutor_1 = ui.Picture()
			tutor_1:setTexture("ui/icons/ico_upload_picture:filtered")
			vlayout:addWidget (tutor_1)

			hlayout:addItem(vlayout)
		end

		local arrow_1 = ui.Picture()
		arrow_1:setTexture ("ui/icons/ico_arrow_transparent:filtered")
		hlayout:addWidget (arrow_1)

		do
			local vlayout = ui.VerticalLayout()
			vlayout:setItemSpacing(ui.config.vspacing)

			local choose_image_label = ui.TextLabel()
			choose_image_label:setTextStyle (ui.text_styles.small_hl.build())
			choose_image_label:setText (L"KEY_AVATAR_PREVIEW_AVATAR")
			choose_image_label:setWordWrap(false)
			choose_image_label:setTextAlignment(ui.AlignmentHorizontalCenter)
			choose_image_label:setHorizontalSizePolicy(ui.SizePolicyFixed)
			vlayout:addWidget(choose_image_label)

			local tutor_1 = ui.AvatarWidget()
			tutor_1:setUrl ("")
			tutor_1:setAvatarSize (underworld2.Avatar_SizeLarge)
			vlayout:addWidget (tutor_1)

			local sz = tutor_1:getPreferredSize()

			choose_image_label:setWidth(sz.x)

			hlayout:addItem(vlayout)
		end


		local arrow_2 = ui.Picture()
		arrow_2:setTexture ("ui/icons/ico_arrow_transparent:filtered")
		hlayout:addWidget (arrow_2)

		do
			local vlayout = ui.VerticalLayout()
			vlayout:setItemSpacing(ui.config.vspacing)

			local choose_image_label = ui.TextLabel()
			choose_image_label:setTextStyle (ui.text_styles.small_hl.build())
			choose_image_label:setText (L"KEY_AVATAR_CONFIRM_NEW_AVATAR")
			choose_image_label:setWordWrap(false)
			choose_image_label:setTextAlignment(ui.AlignmentHorizontalCenter)
			choose_image_label:setHorizontalSizePolicy(ui.SizePolicyFixed)
			vlayout:addWidget(choose_image_label)

			local tutor_1 = ui.AvatarWidget()
			tutor_1:setUrl ("")
			tutor_1:setAvatarSize (underworld2.Avatar_SizeLarge)
			vlayout:addWidget (tutor_1)

			local sz = tutor_1:getPreferredSize()

			choose_image_label:setWidth(sz.x)

			local tick = ui.Picture()
			tick:setTexture("ui/icons/ico_tick_51:filtered")
			tick:setHorizontalSizePolicy(ui.SizePolicyFixed)
			tick:setVerticalSizePolicy(ui.SizePolicyFixed)
			tick:setSize(tick:getPreferredSize())
			tick:setOrigin(math.vec2((sz.x-tick:getWidth()*1.3),(sz.y-tick:getHeight())*0.9))
			tutor_1:addChild (tick)

			hlayout:addItem(vlayout)
		end

		frame_layout:addItem(hlayout)
	end

	do
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (L"KEY_AVATAR_CHOOSE_EXPLANATION")
		text_label:setWidth(0.9)
		text_label:setRowsCount (text_label:getRowsForWidth(text_label:getWidth()))
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)

	frame_layout:addWidget (sep)

	do
		local h_layout = ui.HorizontalLayout()
		h_layout:setItemSpacing (ui.config.hspacing)

		local action_btn = ui.TextButton()
		action_btn:setTexture ("ui/settings/btn_green_normal:filtered")
		action_btn:setTextStyle (ui.text_styles.button_regular.build())
		action_btn:setText (L"KEY_AVATAR_CHOOSE_IMAGE_ACTION")

		h_layout:addWidget (ui.Widget())
		h_layout:addWidget (action_btn)
		h_layout:addWidget (ui.Widget())

		events.connect (action_btn:getClickedSignalProxy(), function() self.result = view.RESULT_CONFIRM; self:finalize() end)

		frame_layout:addItem(h_layout)
	end

	
	frame:setLayout (frame_layout)
	
	do
		local frame_size = frame:getPreferredSize()
		
		frame:setOrigin (0.0, 0.5 - frame_size.y * 0.5)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end

function view.create(...)
	local builder = ui.core.view.builder(view)
	local v = builder.build(...)
	return v
end

function view.show(cb)
	local builder = ui.core.view.builder(view)
	local v = builder.build(cb)

	ui.core.view.delegate_builder(
	{
		finished = function (v)
			ui.core.view_controller:close_modal_view (v)

			if v.result == view.RESULT_CONFIRM then
				ui.confirm_avatar.show(cb, true)
			end
		end
	}).build (v)

	ui.core.view_controller:show_modal_view (v)
end