--require "ui/core/utils.lua"
require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"
require "ui/widgets/item_widget.lua"
require "ui/button_colors.lua"
require "ui/image_customization_view.lua"

local view = setmetatable (
	{
		RESULT_CANCEL = 0,
		RESULT_CONFIRM = 1
	}, { __index = ui.core.view })
ui.confirm_avatar = view

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui ()
	local strings = underworld2.StringsDatabase()

	self.ui = {}

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
			
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title_big.build())
		title_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
		title_label:setText (L"KEY_AVATAR_CONFIRM_YOUR_NEW_AVATAR")
		top_layout:addWidget (title_label)

		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local vvlayout = ui.VerticalLayout()

		local hlayout = ui.HorizontalLayout()

		local avatar = ui.AvatarWidget()
		avatar:setUrl ("")
		avatar:setAvatarSize (underworld2.Avatar_SizeLarge)
		self.ui.avatar = avatar
		hlayout:addWidget (avatar)

		do
			local hhlayout = ui.HorizontalLayout()
			avatar:setLayout(hhlayout)

			local waiting_widget = ui.WaitingWidget()
			waiting_widget:setHidden(true)
			waiting_widget:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			self.ui.waiting_widget = waiting_widget

			hhlayout:addWidget(ui.Widget())
			hhlayout:addWidget(waiting_widget)
			hhlayout:addWidget(ui.Widget())
		end

		local arrow_2 = ui.Picture()
		arrow_2:setTexture ("ui/Stretcher_Frame/frame_white_tip:filtered")
		hlayout:addWidget (arrow_2)

		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		text_frame:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y*1.5, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (L"KEY_AVATAR_CONFIRM_WAITING_TEXT")
		text_layout:addWidget(text_label)
		self.ui.text_label = text_label

		local max_h = ui.Widget()
		max_h:setVerticalSizePolicy(ui.SizePolicyMaximum)
		text_layout:addWidget(max_h)
		
		text_frame:setLayout (text_layout)

		hlayout:addWidget (text_frame)
	
		local sep_1 = ui.Widget()
		sep_1:setVerticalSizePolicy(ui.SizePolicyFixed)
		sep_1:setHeight(0.05)

		local sep_2 = ui.Widget()
		sep_2:setVerticalSizePolicy(ui.SizePolicyFixed)
		sep_2:setHeight(0.05)

		vvlayout:addWidget(sep_1)
		vvlayout:addItem(hlayout)
		vvlayout:addWidget(sep_2)

		frame_layout:addItem (vvlayout)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)

	frame_layout:addWidget (sep)

	do
		local h_layout = ui.HorizontalLayout()
		h_layout:setItemSpacing (ui.config.hspacing)

		local repeat_btn = ui.TextButton()
		repeat_btn:setTexture ("ui/Buttons/btn_sort:filtered")
		repeat_btn:setTextStyle (ui.text_styles.button_regular.build())
		repeat_btn:setText (L"KEY_AVATAR_CHOOSE_ANOTHER")

		local action_btn = ui.TextButton()
		action_btn:setTexture ("ui/settings/btn_green_normal:filtered")
		action_btn:setTextStyle (ui.text_styles.button_regular.build())
		action_btn:setText (L"KEY_AVATAR_CONFIRM_NEW_AVATAR_AGAIN")

		h_layout:addWidget (repeat_btn)
		h_layout:addWidget (action_btn)

		events.connect (repeat_btn:getClickedSignalProxy(), function()
			if not self.uploading then
				self:pick ()
			end
		end)

		events.connect (action_btn:getClickedSignalProxy(), function()
			if not self.uploading then
				self.result = view.RESULT_CONFIRM
				self:finalize()
			end
		end)

		frame_layout:addItem(h_layout)
	end

	
	frame:setLayout (frame_layout)
	
	do
		local frame_size = frame:getPreferredSize()
		
		frame:setOrigin (0.0, 0.5 - frame_size.y * 0.5)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end

function view:pick(cb, image_customization_view)
	local picker = underworld2.AvatarPicker()
	local result = os.image_picker:show(picker)

	if result == 10 then
		if image_customization_view then
			image_customization_view:set_image(picker:getImage())
		else
			ui.image_customization_view.show(picker:getImage(),
				function(v)
					self:pick(cb, v)
				end,

				function(selected_area, selected_scale)
					picker:update(selected_scale, selected_area)
					--self.ui.avatar:setImage(picker:getImage())

					if cb then cb () end

					self.ui.waiting_widget:setHidden(false)
					self.uploading = true
					self.ui.text_label:setText(L"KEY_AVATAR_CONFIRM_WAITING_TEXT")


					picker:upload()

					events.connect (picker:getFallbackProxy(), function(generated_id)
						self.ui.text_label:setText(L"KEY_AVATAR_CONFIRM_TEXT")

						self.ui.waiting_widget:setHidden(true)
						self.ui.avatar:setImage(picker:getImage())
						self.generated_id = generated_id
						self.uploading = false
					end)
				end)
		end
	elseif result == 3 then
		self.ui.text_label:setText(L"KEY_AVATAR_OVERSIZED")
	end
end

function view.create(...)
	local builder = ui.core.view.builder(view)
	local v = builder.build(...)
	return v
end

function view.show(cb, from_native_gallery)
	local builder = ui.core.view.builder(view)
	local v = builder.build()

	ui.core.view.delegate_builder(
	{
		finished = function (v)
			ui.core.view_controller:close_modal_view (v)

			if v.result == view.RESULT_CONFIRM then
	 			if cb and v.generated_id then
					cb(v.generated_id)
				end
			end
		end
	}).build (v)

	if from_native_gallery then
		v:pick(function() ui.core.view_controller:show_modal_view (v) end)
	else
		ui.core.view_controller:show_modal_view (v)
	end
end