--require "ui/core/utils.lua"
require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"
require "ui/widgets/item_widget.lua"
require "ui/button_colors.lua"

local view = setmetatable (
	{
		RESULT_CANCEL = 0,
		RESULT_CONFIRM = 1
	}, { __index = ui.core.view })
ui.confirm_window = view

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui (title, text, confirm_btn_lbl, cancel_btn_lbl)
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_image = ui.Picture()
	character_image:setTexture ("ui/characters/ch_advisor_modal:filtered")
	widget:addChild (character_image)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_warning_modal:filtered", "ui/backlight_busted:filtered")
		top_layout:addWidget (ico)
			
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
		title_label:setText (title)
		top_layout:addWidget (title_label)

		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do
		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (L("KEY_ADVISOR_FULL"))
		text_layout:addWidget (title_label)
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (text)
		text_label:setRowsCount (4)
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)

	frame_layout:addWidget (sep)

	do
		local h_layout = ui.HorizontalLayout()
		h_layout:setItemSpacing (ui.config.hspacing)

		local reset_btn = ui.TextButton()
		reset_btn:setTexture ("ui/settings/btn_red_normal:filtered")
		reset_btn:setTextStyle (ui.text_styles.button_regular.build())
		reset_btn:setText (confirm_btn_lbl or L"KEY_CONFIRM")
		h_layout:addWidget (reset_btn)

		local cancel_btn = ui.TextButton()
		cancel_btn:setTexture ("ui/settings/btn_green_normal:filtered")
		cancel_btn:setTextStyle (ui.text_styles.button_regular.build())
		cancel_btn:setText (cancel_btn_lbl or L"KEY_CANCEL")
		h_layout:addWidget (cancel_btn)

		events.connect (reset_btn:getClickedSignalProxy(), function() self.result = view.RESULT_CONFIRM; self:finalize() end)
		events.connect (cancel_btn:getClickedSignalProxy(), function() self.result = view.RESULT_CANCEL; self:finalize() end)

		frame_layout:addItem(h_layout)
	end

	
	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end

function view.create(...)
	local builder = ui.core.view.builder(view)
	local v = builder.build(...)
	return v
end