require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"

require "social/facebook_sharing.lua"

require "ui/widgets/picture_with_backlight.lua"
require "ui/widgets/item_widget.lua"
require "ui/button_colors.lua"
require "underworld2/equipment_view_state.lua"

local view = setmetatable (
	{
		RESULT_GOTO_UPGRADES = 0,
		RESULT_CANCEL = 1
	}, { __index = ui.core.view })
ui.crew_member_confirmation = view

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

local builder = ui.core.view.builder(view)

-- guess, texts and types should be taken from DB
local texts = {
	["COURIERS"] = {
		637,
		638,
		639,
		640,
		641,
		642,
		712
	},
	["ITEM"] = {
		643,
		644,
		645,
		646,
		647,
		648
	},
	["WALLET"] = {
		649,
		650,
		651,
		652,
		653,
		654,
		655,
		656
	}
}

local types = {
	["COURIERS"] = L "KEY_CREW_COURIER",
	["ITEM"] = L "KEY_CREW_SUPPLIER",
	["WALLET"] = L "KEY_CREW_ACCOUNTANT"
}

local characters = {
	["COURIERS"] = {
		"ui/characters/ch_runner_modal_1:filtered",
		"ui/characters/ch_runner_modal_2:filtered",
		"ui/characters/ch_runner_modal_3:filtered",
		"ui/characters/ch_runner_modal_4:filtered",
		"ui/characters/ch_runner_modal_5:filtered",
		"ui/characters/ch_runner_modal_6:filtered",
		"ui/characters/ch_runner_modal_7:filtered"
	},

	["ITEM"] = {
		"ui/characters/ch_supplier_modal_1_2:filtered",
		"ui/characters/ch_supplier_modal_3:filtered",
		"ui/characters/ch_supplier_modal_4:filtered",
		"ui/characters/ch_supplier_modal_5:filtered",
		"ui/characters/ch_supplier_modal_6:filtered",
		"ui/characters/ch_supplier_modal_7:filtered",
		"ui/characters/ch_supplier_modal_8:filtered"
	},

	["WALLET"] = {
		"ui/characters/ch_accountant_modal_1:filtered",
		"ui/characters/ch_accountant_modal_2:filtered",
		"ui/characters/ch_accountant_modal_3:filtered",
		"ui/characters/ch_accountant_modal_4:filtered",
		"ui/characters/ch_accountant_modal_5:filtered",
		"ui/characters/ch_accountant_modal_6:filtered",
		"ui/characters/ch_accountant_modal_7:filtered",
		"ui/characters/ch_accountant_modal_8:filtered",
		"ui/characters/ch_accountant_modal_9:filtered"
	}
}

function view.create (...)
	local v = builder.build(...)
	return v
end

function view:build_ui (skill_id, level_id, show_add_to_crew)
	self.ui = {}

	local strings = underworld2.StringsDatabase()
	
	local skill = underworld2.SkillsDatabase():getGroup ("SMUGGLERSHIP"):getSkillById (skill_id)
	local level = skill:getLevel (level_id)
	local type = skill:getType()

	if type == "ITEM" then
		underworld2.equipment_view_state.session.unlock_skill_level(level:getValue())
	end

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_image = ui.Picture()
	character_image:setTexture (characters[type][level_id])
	widget:addChild (character_image)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_crew_modal:filtered", "ui/vault/backlight_vault:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_CREW_NEW_MEMBER")
		top_layout:addWidget (title_label)
		
		frame_layout:addItem (top_layout)

		if not show_add_to_crew then
			local close_button = ui.ImageButton()
			close_button:setTexture ("ui/btn_close:filtered")
			close_button:setSound ("ui/UI_Back.wav")
			close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			events.connect (close_button:getClickedSignalProxy(), function() 
				self.result = view.RESULT_CANCEL
				self:finalize()
			end)
			top_layout:addWidget (close_button)
		end
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")

		local scroll_layout = ui.VerticalLayout()
		scroll_layout:setItemSpacing (ui.config.vspacing)
		text_frame:setLayout (scroll_layout)

		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (types[type] .. ", " .. strings:get (level:getTitleKey()))
		scroll_layout:addWidget (title_label)
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())

		local range = texts[type]
		local text = ""
		if (range and range[level_id-1]) then
			text = strings:get(range[level_id-1]) -- guess, should not be hardcoded
		end

		text_label:setText (text)
		text_label:setRowsCount (5)
		scroll_layout:addWidget (text_label)

		frame_layout:addWidget (text_frame)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local v_sep = ui.Widget()
		v_sep:setSize(1.0, 0.002)
		v_sep:setVerticalSizePolicy (ui.SizePolicyFixed)
		frame_layout:addWidget (v_sep)

		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		top_layout:addWidget (ui.Widget())

		if show_add_to_crew then
			local btn = ui.TextButton()
			btn:setTexture ("ui/btn_lower_normal:filtered")
			btn:setTextStyle (ui.text_styles.button_regular.build())
			btn:setText (L "KEY_CREW_ADD_TO_CREW")
			btn:setTag (view.RESULT_GOTO_UPGRADES)
			top_layout:addWidget (btn)

			events.connect (btn:getClickedSignalProxy(), function()
				self.result = view.RESULT_GOTO_UPGRADES
				self:finalize()
			end)
		else
			local share_button = underworld2.facebook_sharing.create_button( 
			L"KEY_FB_SHARING_SHARE", 
			underworld2.facebook_sharing.member_arrived_bonus(type, level_id),
			function() 
				underworld2.facebook_sharing.member_arrived( skill_id, type, level_id, function( success ) 
					if success then 
						get_sound_pool():play ("ui/SFX_Premium_Purchase.wav") 
					end 
				end)
				self:finalize()
			end )
			top_layout:addWidget(share_button)
		end

		top_layout:addWidget (ui.Widget())
		frame_layout:addItem (top_layout)

		v_sep = ui.Widget()
		v_sep:setSize(1.0, 0.002)
		v_sep:setVerticalSizePolicy (ui.SizePolicyFixed)
		frame_layout:addWidget (v_sep)
	end
	
	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()

		local max_char_height = math.min (1.0 - frame_size.y - 0.05, character_size.y)
		
		character_size.x = character_size.x * max_char_height / character_size.y
		character_size.y = max_char_height
		
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, 1.0 - character_size.y - frame_size.y - 0.05)
		character_image:setSize (character_size)
		character_image:setVerticalSizePolicy (ui.SizePolicyFixed)
		character_image:setHorizontalSizePolicy (ui.SizePolicyFixed)
		character_image:setPlacingMode(ui.Picture_PlacingModeStretch)
		
		frame:setOrigin (0.0, 1.0 - frame_size.y - 0.05)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)

	return widget
end