local view = setmetatable ({ }, { __index = ui.core.view })

local builder = ui.core.view.builder(view)

function view.create (message, buttons)
	local obj = builder.build (message, buttons)
	return obj
end

function view:build_ui (message, buttons_cfg)
	self.ui = {}

	local strings = underworld2.StringsDatabase()

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_image = ui.Picture()
	character_image:setTexture ("ui/characters/ch_advisor_modal:filtered")
	widget:addChild (character_image)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local vl = ui.VerticalLayout()
		vl:addWidget(ui.Widget())
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_warning_modal:filtered", "ui/backlight_busted:filtered")
		vl:addWidget(ico)
		vl:addWidget(ui.Widget())
		top_layout:addItem (vl)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_SERVER_ERROR_MODAL_TITLE")
		title_label:setRowsCount (1)
		top_layout:addWidget (title_label)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (L "KEY_ADVISOR_FULL")
		text_layout:addWidget (title_label)

		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (message)
		text_label:setRowsCount (3)
		text_layout:addWidget (text_label)

		text_frame:setLayout (text_layout)
		frame_layout:addWidget (text_frame)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local v_sep = ui.Widget()
		v_sep:setSize(1.0, 0.002)
		v_sep:setVerticalSizePolicy (ui.SizePolicyFixed)
		frame_layout:addWidget (v_sep)

		local btn_layout = ui.HorizontalLayout()
		btn_layout:setItemSpacing (ui.config.hspacing)

		local buttons = { }
		for i, v in ipairs (buttons_cfg) do
			local button = ui.TextButton()
			if type (v) == "table" then
				button:setTextStyle (v.text_style.build())
				button:setText (v.text)
				button:setTexture (v.texture)
				button:setTag (v.tag)
				events.connect (button:getClickedSignalProxy(), v.handler)
			else
				local cfg = ui.information_view.button_config[v]
				button:setTextStyle (cfg.text_style.build())
				button:setText (cfg.text)
				button:setTexture (cfg.texture)
				button:setTag (v)
				events.connect(button:getClickedSignalProxy(), function(button) 
					self.result = button:getTag()
					self:finalize()
				end)
			end
			btn_layout:addWidget (button)
		end
		
		frame_layout:addItem (btn_layout)

		v_sep = ui.Widget()
		v_sep:setSize(1.0, 0.002)
		v_sep:setVerticalSizePolicy (ui.SizePolicyFixed)
		frame_layout:addWidget (v_sep)
	end
	
	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()

		local max_char_height = math.min (1.0 - frame_size.y - 0.05, character_size.y)
		
		character_size.x = character_size.x * max_char_height / character_size.y
		character_size.y = max_char_height
		
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, 1.0 - character_size.y - frame_size.y - 0.05)
		character_image:setSize (character_size)
		character_image:setVerticalSizePolicy (ui.SizePolicyFixed)
		character_image:setHorizontalSizePolicy (ui.SizePolicyFixed)
		character_image:setPlacingMode(ui.Picture_PlacingModeStretch)
		
		frame:setOrigin (0.0, 1.0 - frame_size.y - 0.05)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)

	return widget
end

ui.custom_error_view = view
