require "ui/core/view.lua"
require "ui/config.lua"
require "ui/animations/factory.lua"
require "ui/text_styles.lua"

ui = ui or {}
ui.deal_completed_ribbon = {}

local view = setmetatable (ui.deal_completed_ribbon, { __index = ui.core.view })
local builder = ui.core.view.builder (view)

function view.create (xp)
	local obj = builder.build(xp or 0)
	return obj
end

function view:back()
	--self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui (xp)
	local root_widget = ui.Frame("root_widget")
	root_widget:setBordersTexture ("ui/modal_wide_window_green:filtered")
	root_widget:setSize(1.0, 0.13)
	root_widget:setOrigin(0, 0.5 - 0.13 / 2)

	local layout = ui.VerticalLayout()
	local upper_layout = ui.HorizontalLayout()
	local bottom_layout = ui.HorizontalLayout()

	-- upper layout
	local upper_icon = ui.Picture()
	upper_icon:setTexture ("ui/ico_rp_sidemenu:filtered")
	upper_icon:setHorizontalSizePolicy (ui.SizePolicyMinimum)

	local upper_label = ui.TextLabel()
	upper_label:setTextStyle (ui.text_styles.large.build())
	upper_label:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)
	upper_label:setText ("+" .. xp)

	local upper_spacer = ui.Widget()
	upper_spacer:setSize (0.4, 0.0)
	upper_spacer:setHorizontalSizePolicy (ui.SizePolicyFixed)

	upper_layout:setItemSpacing (ui.config.hspacing)
	upper_layout:addWidget (upper_spacer)
	upper_layout:addWidget (upper_icon)
	upper_layout:addWidget (upper_label)

	-- bottom layout
	local bottom_bar = ui.ProgressBar ()
	bottom_bar:setTexture ("ui/bar_experience_large_white:filtered")
	bottom_bar:setTextStyle (ui.text_styles.small_hl.build())
	bottom_bar:setHorizontalSizePolicy (ui.SizePolicyFixed)
	bottom_bar:setSize (0.6, 0.0)

	local bottom_spacer = ui.Widget()
	bottom_spacer:setHorizontalSizePolicy (ui.SizePolicyMaximum)

	local bottom_icon = ui.Picture()
	bottom_icon:setTexture ("ui/ico_rp_sidemenu")

	local bottom_sublayout = ui.HorizontalLayout()
	bottom_sublayout:addWidget (bottom_spacer)
	bottom_sublayout:addWidget (bottom_icon)
	bottom_bar:setLayout (bottom_sublayout)

	local bottom_corner_spacer = ui.Widget()
	bottom_corner_spacer:setSize (0.2, 0.0)
	bottom_corner_spacer:setHorizontalSizePolicy (ui.SizePolicyFixed)

	bottom_layout:addWidget (bottom_corner_spacer)
	bottom_layout:addWidget (bottom_bar)

	-- layout
	layout:addItem (upper_layout)
	layout:addItem (bottom_layout)

	-- root widget
	root_widget:setLayout (layout)

	-- spawning animations
	local opacity = 40

	upper_icon:setColor (render.color (255, 255, 255, opacity))
	upper_label:setColor (render.color (255, 255, 255, opacity))
	bottom_bar:setColor (render.color (255, 255, 255, opacity))
	bottom_icon:setColor (render.color (255, 255, 255, opacity))
	root_widget:setColor (render.color (255, 255, 255, opacity))

	bottom_bar:setValue (0)

	local player = underworld2.player
	local current_level = underworld2.LevelsDatabase():get (player:getLevel())
	local next_level = underworld2.LevelsDatabase():get (player:getLevel() + 1)
	local current_progress = 0
	local text = ""
	if current_level:get() and next_level:get() then
		local rp = player:getRespectPoints() - current_level:getRespectPoints()
		local max = next_level:getRespectPoints() - current_level:getRespectPoints()
		if max > 0 then
			current_progress = rp / max
		end
		if rp >= max then
			text = L "KEY_RANKUP_CAPSED"
		end
	end

	bottom_bar:setText(text)

	local close = ui.animations.build {
		duration = 0.5,
		from = 255,
		to = opacity,
		parent = self,

		step = function (self, value)
			upper_icon:setColor (render.color (255, 255, 255, value))
			upper_label:setColor (render.color (255, 255, 255, value))
			bottom_bar:setColor (render.color (255, 255, 255, value))
			bottom_icon:setColor (render.color (255, 255, 255, value))
			root_widget:setColor (render.color (255, 255, 255, value))
		end,

		stop = function (self)
			ui.core.view_controller:close_modal_view (self.parent)
			self.parent:finalize ()
		end
	}

	local progress = ui.animations.build {
		duration = 0.5,
		from = 0,
		to = current_progress,
		parent = self,

		step = function (self, value)
			bottom_bar:setValue (value)
		end,

		stop = function (self)
			self.parent.animations:bind ("close", close)
		end
	}

	local spawn = ui.animations.build {
		duration = 0.5,
		from = opacity,
		to = 255,
		parent = self,

		step = function (self, value)
			upper_icon:setColor (render.color (255, 255, 255, value))
			upper_label:setColor (render.color (255, 255, 255, value))
			bottom_bar:setColor (render.color (255, 255, 255, value))
			bottom_icon:setColor (render.color (255, 255, 255, value))
			root_widget:setColor (render.color (255, 255, 255, value))
		end
	}

	self.animations = ui.animations.factory()
	self.animations:bind ("spawn", spawn)
	self.animations:bind ("progress", progress)

	return root_widget
end

function view:update (dt)
	self.animations:update (dt)
end