require "ui/core/view.lua"

local view = setmetatable ({ }, { __index = ui.core.view })

function view.show (fame, time_passed, handler)
	local v = ui.core.view.builder (view).build (fame, time_passed)
	ui.core.view.delegate_builder (
	{
		on_back = function (view)
			view:finalize()
		end,

		finished = function()
			ui.core.view_controller:close_modal_view (v)
			if handler then handler() end
		end
	}).build (v)
	ui.core.view_controller:show_modal_view (v)
end

function view:build_ui (fame, time_passed)
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_image = ui.Picture()
	character_image:setTexture ("ui/characters/ch_advisor_modal:filtered")
	widget:addChild (character_image)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/icons/ico_fame_hustler_header:filtered", "ui/backlight_busted:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_FAME_DROP_MODAL_TITLE")
		top_layout:addWidget (title_label)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do
		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (L "KEY_ADVISOR_FULL")
		text_layout:addWidget (title_label)
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (L "KEY_FAME_DROP_MODAL_TEXT")
		text_label:setRowsCount (3)
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end
	
	do
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")

		local text_layout = ui.HorizontalLayout()
		text_layout:setItemSpacing (ui.config.hspacing)
		text_frame:setLayout (text_layout)
		
		local time_title = ui.TextLabel()
		time_title:setTextStyle (ui.text_styles.large_hl.build())
		time_title:setText (L "KEY_FAME_DROP_MODAL_TIME")
		time_title:setWordWrap (false)
		time_title:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)

		local time_ico = ui.Picture()
		time_ico:setTexture ("ui/ico_timer:filtered")

		local time_value_label = ui.TextLabel()
		time_value_label:setTextStyle (ui.text_styles.large_hl.build())
		time_value_label:setText (ui.format.duration_string (time_passed, true))
		time_value_label:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		time_value_label:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)

		local fame_ico = ui.Picture()
		fame_ico:setTexture ("ui/icons/ico_fame_56:filtered")

		local fame_amount = ui.TextLabel()
		fame_amount:setTextStyle (ui.text_styles.large.build())
		fame_amount:setText (string.format ("-%d", fame))
		fame_amount:setWordWrap (false)
		fame_amount:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)

		text_layout:addWidget (time_title)
		text_layout:addWidget (time_ico)
		text_layout:addWidget (time_value_label)
		text_layout:addWidget (fame_ico)
		text_layout:addWidget (fame_amount)

		frame_layout:addWidget (text_frame)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do

		local btn_layout = ui.HorizontalLayout()
		btn_layout:setVerticalPolicy (ui.SizePolicyMinimum)

		local spacer = ui.Widget()
		spacer:setSize (0.25, 0.0)
		spacer:setHorizontalSizePolicy (ui.SizePolicyFixed)
		btn_layout:addWidget (spacer)

		local btn = ui.TextButton()
		btn:setTexture ("ui/btn_lower_normal:filtered")
		btn:setTextStyle (ui.text_styles.button_regular.build())
		btn:setText (L "KEY_FAME_DROP_BTN_OK_TEXT")
		events.connect (btn:getClickedSignalProxy(), function()
			self:finalize()
		end)
		btn_layout:addWidget (btn)

		local spacer = ui.Widget()
		spacer:setSize (0.25, 0.0)
		spacer:setHorizontalSizePolicy (ui.SizePolicyFixed)
		btn_layout:addWidget (spacer)

		frame_layout:addItem (btn_layout)

	end

	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end

ui.fame_drop_view = view

