require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/modal/news_modal.lua"

local view = setmetatable (
	{
		RESULT_CONFIRMED = 0,
		RESULT_POSTPONED = 1,
	}, { __index = ui.core.view })
ui.game_update = view

view.BTN_UPDATE_NOW = {
	texture = "ui/btn_lower_normal:filtered",
	text = "KEY_APP_UPDATE_UPDATE_NOW",
	tag = view.RESULT_CONFIRMED,
	handler = function (self) os.device:openURL (self.external_url) end
}

view.BTN_LATER = {
	texture = "ui/btn_ditch:filtered",
	text = "KEY_APP_UPDATE_POSTPONE",
	tag = view.RESULT_POSTPONED,
	handler = function (self) self:finalize() end
}

local builder = ui.core.view.builder(view)

function view.create (body, external_url, buttons, img_url)
	local v = builder.build(body, buttons, img_url)
	v.external_url = external_url
	return v
end

function view:back()

end

function view:build_ui(text, buttons, img_url)
	self.ui = {}

	local view = ui.news_modal.build_abstract_view(self, L"KEY_APP_UPDATE_TITLE", text, img_url, true)
	local widget = view.root
	local layout = view.layout

	do
		local h_layout = ui.HorizontalLayout()
		h_layout:setItemSpacing (ui.config.hspacing)
		--h_layout:addWidget(ui.Widget())

		for i, v in ipairs (buttons) do
			local btn = ui.TextButton()
			btn:setTexture (v.texture)
			btn:setTextStyle (ui.text_styles.button_regular.build())
			btn:setText (L (v.text))
			btn:setTag (v.tag)
			h_layout:addWidget (btn)

			events.connect (btn:getClickedSignalProxy(), function() v.handler (self) end)
		end

		--h_layout:addWidget(ui.Widget())
		layout:addItem(h_layout)
	end

	view.resize()

	return widget
end

function view:update (dt)
end

function view.test()
	local text = [[
		Lorem ipsum dolor sit amet, consectetur adipiscing elit. Suspendisse feugiat turpis dignissim turpis consequat blandit. Fusce quis vestibulum dolor. Vestibulum magna libero, ultrices id tristique et, vehicula a tellus. In hac habitasse platea dictumst. Nullam elementum tempus massa vitae cursus. Vivamus id tortor porttitor, commodo nisi et, ornare nisi. Mauris sagittis augue quis diam gravida viverra et sed lacus. Curabitur risus felis, tincidunt ut dui sit amet, hendrerit mattis enim. Ut consequat arcu ut quam tincidunt auctor.
	]]

	local v = ui.game_update.create (text, "http://a-steroids.com", { ui.game_update.BTN_LATER, ui.game_update.BTN_UPDATE_NOW })
	ui.core.view.delegate_builder(
	{
		finished = function (view)
			ui.core.view_controller:close_modal_view (view)
		end
	}).build (v)
	ui.core.view_controller:show_modal_view (v)
end
