require "ui/core/view.lua"
require "ui/config.lua"
require "ui/animations/factory.lua"
require "ui/text_styles.lua"
require "ui/format.lua"
require "ui/widgets/gold_to_cash_widget.lua"
require "ui/widgets/picture_with_backlight.lua"

local view = setmetatable (
	{
		RESULT_RESET = 0,
		RESULT_CANCEL = 1 
	}, { __index = ui.core.view })

ui.heat_modal = view

local texts = {
	"Hey chap,\nI can cover your ass most of the time. I need some extra cash for the boys in the department, especially when you screw up big time..."
}

function view.show(silent)
	if not silent then
		get_sound_pool():play ("ui/SFX_Busted_Loop.wav")
	end

	local instance = ui.core.view.builder (view).build()
	instance.delegate_builder (
	{
		finished = function (self)
			ui.core.view_controller:close_modal_view (self)
			if self.result == ui.heat_modal.RESULT_RESET then
				local player = underworld2.player
				local exposure = player:getPoliceExposure()
				local price = exposure:getResetPrice (underworld2.server_time())
				if price == 0 then
					get_sound_pool():play ("ui/UI_Click_05.wav")
				elseif price <= player:getGold() then
					get_sound_pool():play ("ui/SFX_Heat_Drop.wav")
					underworld2.network.reset_heat (price, function() end)
					player:removeGold (price)
					player:resetHeat()
				else
					get_sound_pool():play ("ui/SFX_Cash_Out.wav")
					ui.not_enough_gold.show(price - underworld2.player:getGold())
				end
			end
		end
	}).build (instance)
	ui.core.view_controller:show_modal_view (instance)
	return instance
end

local function build_attributes_widget(attributes)
	local widget = ui.Frame()
	widget:setBordersTexture ("ui/frame_modal_light:filtered")
	widget:setMargins (ui.config.hspacing * 2, ui.config.hspacing * 2, ui.config.vspacing, ui.config.vspacing)

	local vlayout = ui.VerticalLayout()
	vlayout:setItemSpacing(ui.config.vspacing)
	widget:setLayout(vlayout)

	local widgets = {}

	for name, data in pairs(attributes) do
		local attr_widget = ui.Widget()

		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing(ui.config.hspacing)

		local ico_cash = ui.Picture()
		ico_cash:setTexture ("ui/icons/ico_police_protection_51:filtered")
		hlayout:addWidget (ico_cash)

		local param = ui.TextLabel()
		param:setTextStyle (ui.text_styles.modal_character_name.build())
		param:setText ( data.text )
		param:setHorizontalSizePolicy(ui.SizePolicyMaximum)
		hlayout:addWidget (param)

		local ico = ui.Picture()
		ico:setTexture ("ui/ico_timer:filtered")
		hlayout:addWidget (ico)

		local param_value = ui.TextLabel()
		param_value:setTextStyle (ui.text_styles.tutorial_text.build())
		param_value:setText ( "" )
		param_value:setWordWrap (false)
		hlayout:addWidget (param_value)

		attr_widget:setLayout(hlayout)
		vlayout:addWidget(attr_widget)

		widgets[name] = {
			name_label = param,
			time_label = param_value,
			widget = attr_widget
		}
	end

	widgets.container = widget

	return widgets
end

local function current_timespan(timestamp)
	local timespan = timestamp - underworld2.server_time()
	if timespan < 0 then timespan = 0 end
	return timespan
end

function view:build_ui ()
	self.ui = {}

	local player = underworld2.player
	local max_money = player:getMaxMoney()
	local current_money = math.min (max_money, player:getMoney())
		
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)

	local backlights = {
		ui.Picture(),
		ui.Picture()
	}

	backlights[1]:setTexture ("ui/backlight_modal_large:filtered")
	backlights[1]:setColor(render.color(255, 11, 11, 255))
	backlights[1]:setSize(math.vec2(0.9, 0.5))
	backlights[1]:setPlacingMode (ui.Picture_PlacingModeStretch)

	backlights[2]:setTexture ("ui/backlight_modal_large:filtered")
	backlights[2]:setColor(render.color(11, 93, 255, 255))
	backlights[2]:setSize(math.vec2(0.9, 0.5))
	backlights[2]:setPlacingMode (ui.Picture_PlacingModeStretch)

	widget:addChild(backlights[1])
	widget:addChild(backlights[2])

	local character_image = ui.Picture()
	character_image:setTexture (string.format ("ui/characters/ch_cop_modal:filtered"))
	character_image:setSize (math.vec2(0.3, 0.3))
	widget:addChild (character_image)

	do
		local attributes = { }

		local bonus_ptr = underworld2.player:getBonus ("PoliceProtectionBonus")
		if bonus_ptr:get() then
			local bonus = runtime.cast_to (bonus_ptr:get(), "underworld2::PoliceProtectionBonus")
			local now = underworld2.server_time()
			local end_time = bonus:getEndTime()
			if now > bonus:getStartTime() and now < end_time then
				attributes.bust_protection = { text = L"KEY_BUST_PROTECTION", timestamp = end_time }
			end
		end

		local widgets = build_attributes_widget(attributes)
		widget:addChild(widgets.container)

		self.attributes = attributes
		self.ui.attributes = widgets
	end

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)

	local frame_origin = frame:getOrigin()
	local frame_size = frame:getSize()
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_dropheat:filtered", "ui/backlight_busted:filtered")
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_HEAT_HEAT")
		title_label:setHorizontalSizePolicy (ui.SizePolicyMaximum)

		local close = ui.ImageButton ()
		close:setTexture ("ui/btn_close:filtered")
		close:setSound ("ui/UI_Back.wav")
		close:setOrigin (0, 0)
		events.connect (close:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)

		top_layout:addWidget (ico)
		top_layout:addWidget (title_label)
		--top_layout:addWidget (ui.Widget())
		top_layout:addWidget (close)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText ( L ("KEY_DETECTIVE") )
		text_layout:addWidget (title_label)
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (texts[1]) -- TO DO: guess, should not hardcode
		text_label:setRowsCount (4)
		text_layout:addWidget (text_label)

		self.ui.text_label = text_label
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end


	local bottom_panel = ui.Widget()
	self.ui.bottom_panel = bottom_panel

	local bottom_layout = ui.VerticalLayout()
	bottom_panel:setLayout(bottom_layout)

	do
		local h_layout = ui.HorizontalLayout()
		h_layout:setItemSpacing (ui.config.hspacing)

		local heat_level = ui.TextLabel()
		heat_level:setTextStyle (ui.text_styles.normal_hl.build())
		heat_level:setText (L "KEY_HEAT_LEVEL")
		heat_level:setWordWrap (false)
		heat_level:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		heat_level:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)

		local heat = ui.TextLabel()
		heat:setTextStyle (ui.text_styles.normal_hl.build())
		heat:setText (L "KEY_STATUS_DANGER")
		heat:setWordWrap (false)
		heat:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		heat:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)
		self.ui.heat_level = heat
		
		h_layout:addWidget (ui.Widget())
		h_layout:addWidget (heat_level)
		h_layout:addWidget (heat)
		h_layout:addWidget (ui.Widget())

		bottom_layout:addItem (h_layout)
	end

	do
		local h_layout = ui.HorizontalLayout()
		h_layout:setItemSpacing (ui.config.hspacing)

		local bottom_bar = ui.ProgressBar ()
		bottom_bar:setTexture ("ui/bar_heat_large:filtered")
		bottom_bar:setTextStyle (ui.text_styles.small_hl.build())
		bottom_bar:setHorizontalSizePolicy (ui.SizePolicyFixed)
		bottom_bar:setSize (0.6, 0.0)
		self.ui.heat_bar = bottom_bar

		local bottom_spacer = ui.Widget()
		bottom_spacer:setHorizontalSizePolicy (ui.SizePolicyMaximum)

		local bottom_icon = ui.Picture()
		bottom_icon:setTexture ("ui/ico_heat:filtered")

		local bottom_sublayout = ui.HorizontalLayout()
		bottom_sublayout:addWidget (bottom_spacer)
		bottom_sublayout:addWidget (bottom_icon)
		bottom_bar:setLayout (bottom_sublayout)

		h_layout:addWidget (ui.Widget())
		h_layout:addWidget (bottom_bar)
		h_layout:addWidget (ui.Widget())

		bottom_layout:addItem (h_layout)
	end

	do
		local h_layout = ui.HorizontalLayout()
		h_layout:setItemSpacing (ui.config.hspacing)

		local heat_level = ui.TextLabel()
		heat_level:setTextStyle (ui.text_styles.normal_hl.build())
		heat_level:setText (L "KEY_HEAT_COOLDOWN_TIME")
		heat_level:setWordWrap (false)
		heat_level:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		heat_level:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)

		local ico = ui.Picture()
		ico:setTexture ("ui/ico_timer:filtered")
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)

		local cooldown_time = ui.TextLabel()
		cooldown_time:setTextStyle (ui.text_styles.normal_hl.build())
		cooldown_time:setWordWrap (false)
		cooldown_time:setText("1h 22m")
		cooldown_time:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		self.ui.cooldown_time = cooldown_time

		h_layout:addWidget (ui.Widget())
		h_layout:addWidget (heat_level)
		h_layout:addWidget (ico)
		h_layout:addWidget (cooldown_time)
		h_layout:addWidget (ui.Widget())

		bottom_layout:addItem (h_layout)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	bottom_layout:addWidget (sep)

	self.ui.dropped_heat = false

	do
		local v_sep = ui.Widget()
		v_sep:setSize(1.0, 0.002)
		v_sep:setVerticalSizePolicy (ui.SizePolicyFixed)
		bottom_layout:addWidget (v_sep)

		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)

		top_layout:addWidget (ui.Widget())

		local price_button = ui.PriceButton()
		price_button:setStateTexture ("ui/btn_premium:filtered")
		price_button:setTextStyle (ui.text_styles.button_regular.build())
		price_button:setPrice (10)
		price_button:setText (L "KEY_HEAT_MODAL_LOWER_BUTTON")
		price_button:setCurrencyIcon ("ui/ico_pbtn_gold:filtered")
		price_button:setSize(0.5, 1.0)
		price_button:setHorizontalSizePolicy(ui.SizePolicyFixed)
		self.ui.payoff = price_button

		top_layout:addWidget (price_button)
		top_layout:addWidget (ui.Widget())

		events.connect (price_button:getClickedSignalProxy(), function()

			local player = underworld2.player
			local exposure = player:getPoliceExposure()
			local price = exposure:getResetPrice (underworld2.server_time())

			if price == 0 then
				get_sound_pool():play ("ui/UI_Click_05.wav")
			elseif price <= player:getGold() then
				get_sound_pool():play ("ui/SFX_Heat_Drop.wav")
				underworld2.network.reset_heat (price, function() end)
				player:removeGold (price)
				player:resetHeat()
				self.ui.dropped_heat = true
			else
				self:finalize()
				get_sound_pool():play ("ui/SFX_Cash_Out.wav")
				ui.not_enough_gold.show(price - underworld2.player:getGold(), function() view.show(true) end)
			end
		end)

		bottom_layout:addItem (top_layout)

		v_sep = ui.Widget()
		v_sep:setSize(1.0, 0.002)
		v_sep:setVerticalSizePolicy (ui.SizePolicyFixed)
		bottom_layout:addWidget (v_sep)
	end

	frame_layout:addWidget(bottom_panel)

	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local max_char_height = math.min (1.0 - frame_size.y - 0.05, character_size.y)
		
		character_size.x = character_size.x * max_char_height / character_size.y
		character_size.y = max_char_height
		
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, 1.0 - character_size.y - frame_size.y - 0.05)
		character_image:setSize (character_size)
		character_image:setVerticalSizePolicy (ui.SizePolicyFixed)
		character_image:setHorizontalSizePolicy (ui.SizePolicyFixed)
		character_image:setPlacingMode(ui.Picture_PlacingModeStretch)
		
		frame:setOrigin (0.0, 1.0 - frame_size.y - 0.05)
		frame:setSize (1.0, frame_size.y)

		self.ui.frame = frame

		local origin = character_image:getOrigin()
		backlights[1]:setOrigin(origin.x - 0.27, origin.y)
		backlights[2]:setOrigin(origin.x + 0.15, origin.y + 0.04)

		-- attributes
		local attributes = self.ui.attributes.container
		attributes:setSize(1.0 - ui.config.hspacing * 4, attributes:getPreferredSize().y)
		attributes:setOrigin(ui.config.hspacing * 2.0, 1.0 - frame_size.y - attributes:getHeight() - ui.config.vspacing*2 - 0.05)
	end
	
	widget:addChild (frame)
	self.ui.frame = frame

	local red = render.color(255, 11, 11, 255)
	local blue = render.color(11, 93, 255, 255)

	local blink = ui.animations.build {
		duration = 0.5,
		from = 255,
		to = 0,
		times = -1,
		oscillate = true,

		step = function (self, value)
			local l = render.color(((self.iteration + 0) % 4 > 1) and blue or red)
			local r = render.color(((self.iteration + 1) % 4 > 1) and red or blue)

			backlights[1]:setColor(render.color(l.r, l.g, l.b, value))
			backlights[2]:setColor(render.color(r.r, r.g, r.b, 255-value))
		end
	}

	self.animations = ui.animations.factory()
  	self.animations:bind ("blink", blink)
	self:update(0)

	return widget
end

function view:update (dt)
	if not underworld2.player then 
		self.result = view.RESULT_CANCEL
		self:finalize()
		return 
	end
	
	local now = underworld2.server_time()
	local exposure = underworld2.player:getPoliceExposure()

	local cooldown_time = exposure:getCooldownTime (now)
	local heat_value 	= exposure:getHeatValue (now)
	local reset_price	= exposure:getResetPrice (now)
	local heat_level 	= exposure:getHeatLevel (heat_value)

	self.ui.heat_bar:setValue (heat_value)
	self.ui.cooldown_time:setText (ui.format.duration_string (cooldown_time))
	self.ui.payoff:setPrice (tostring (reset_price))

	self.animations:update(dt)

	-- boosts
	local attributes = self.attributes
	local timespan = 0

	for name, data in pairs(attributes) do
		local layout = self.ui.attributes[name]
		local duration = current_timespan(data.timestamp)
		if duration > timespan then timespan = duration end

		if duration > 0 then
			layout.time_label:setText(ui.format.duration_string(duration))
			layout.widget:setHidden(false)
		else
			layout.widget:setHidden(true)
		end
	end

	local height = self.ui.frame:getPreferredSize().y
	self.ui.frame:setHeight(height)

	local container = self.ui.attributes.container
	if timespan > 0 then
		self.ui.bottom_panel:setHidden(false)

		container:setHidden(false)
		container:setHeight(container:getPreferredSize().y)
		container:setOrigin(container:getOrigin().x, self.ui.frame:getOrigin().y - container:getHeight() - ui.config.vspacing*2)

		self.ui.text_label:setText (L "KEY_HEAT_PROTECTION_ACTIVE")
	else
		self.ui.bottom_panel:setHidden(true)
		container:setHidden(true)
		self:set_heat_hint (heat_level)
	end

	if timespan > 0 then
		self.ui.bottom_panel:setHidden(true)
		self.ui.attributes.container:setHidden(false)
	else
		self.ui.bottom_panel:setHidden(false)
		self.ui.attributes.container:setHidden(true)
	end
end

function view:set_heat_hint (level)
	local strings = underworld2.StringsDatabase()
	local map = { [1] = L "KEY_STATUS_SAFE", [2] = L "KEY_STATUS_SUSPICIOUS", [3] = L "KEY_STATUS_ALERT", [4] = L "KEY_STATUS_DANGER" }
	local texts = {
		[1] = 574,
		[2] = 575,
		[3] = 575,
		[4] = 575
	}
	local colors = {
		[1] = render.color(0, 153, 76),
		[2] = render.color(255, 153, 0),
		[3] = render.color(255, 102, 0),
		[4] = render.color(228, 53, 53)
	}

	self.ui.heat_level:setColor (colors[level])
	self.ui.heat_level:setText (map[level])

	if self.ui.dropped_heat then
		self.ui.text_label:setText (strings:get(576))
	else
		self.ui.text_label:setText (strings:get(texts[level]))
	end
end

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end