require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"

local view = setmetatable (
	{
		RESULT_GOTO_SHOP = 0,
		RESULT_CANCEL = 1 
	}, { __index = ui.core.view })
ui.item_delivery_confirmation = view

local builder = ui.core.view.builder(view)

function view.unschedule_all()
	scheduled_views = { }
end

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui (item)
	self.item = item
	self.ui = {}

	local strings = underworld2.StringsDatabase()

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_image = ui.Picture()
	character_image:setTexture ("ui/characters/ch_advisor_modal:filtered")
	widget:addChild (character_image)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_items_modal:filtered", "ui/vault/backlight_vault:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_ITEM_DELIVERED_HEADER")
		top_layout:addWidget (title_label)
		
		frame_layout:addItem (top_layout)

		do
			local close_button = ui.ImageButton()
			close_button:setTexture ("ui/btn_close:filtered")
			close_button:setSound ("ui/UI_Back.wav")
			close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			events.connect (close_button:getClickedSignalProxy(), function() 
				self.result = view.RESULT_CANCEL
				self:finalize()
			end)
			top_layout:addWidget (close_button)
		end
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local name_layout = ui.HorizontalLayout()

		name_layout:setItemSpacing (ui.config.hspacing)

		name_layout:addWidget (ui.Widget())

		local level_badge = ui.NotificationBadge()
		level_badge:setBadgeTexture ("ui/equipment/frame_item_level:filtered")
 		level_badge:setBadgeTextStyle (ui.text_styles.badge_gold.build())
 		level_badge:setHorizontalSizePolicy (ui.SizePolicyMinimum)
 		level_badge:setPendingNotifications (item:getLevelRequired())

 		name_layout:addWidget (level_badge)

 		local name_label = ui.TextLabel()

 		name_label:setTextStyle (ui.text_styles.normal_hl_green.build())
 		name_label:setText (underworld2.StringsDatabase():get (item:getNameKey()))
 		name_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
 		name_label:setWordWrap(false)

 		name_layout:addWidget (name_label)

		name_layout:addWidget (ui.Widget())

		frame_layout:addItem (name_layout)

		local item_layout = ui.HorizontalLayout()

		local item_widget = ui.ItemWidget()
		item_widget:setItem (item)
		item_widget:setLocked (false)
		
		item_layout:addWidget (ui.Widget())
		item_layout:addWidget (item_widget)
		item_layout:addWidget (ui.Widget())

		frame_layout:addItem (item_layout)

		do
			local hlayout = ui.HorizontalLayout()
			hlayout:addWidget (ui.Widget())

			ui.widgets.item_widget.add_item_stats ( setmetatable( { widgets = { stats = {} } }, { __index = ui.widgets.item_widget } ), hlayout, item)

			hlayout:addWidget (ui.Widget())
			frame_layout:addItem (hlayout)
		end
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local v_sep = ui.Widget()
		v_sep:setSize(1.0, 0.002)
		v_sep:setVerticalSizePolicy (ui.SizePolicyFixed)
		frame_layout:addWidget (v_sep)

		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		top_layout:addWidget (ui.Widget())

		local btn = ui.TextButton()
		btn:setTexture ("ui/btn_lower_normal:filtered")
		btn:setTextStyle (ui.text_styles.button_regular.build())
		btn:setText (L "KEY_ITEM_DELIVERED_BUTTON")
		btn:setTag (view.RESULT_GOTO_SHOP)
		top_layout:addWidget (btn)

		events.connect (btn:getClickedSignalProxy(), function()
			self.result = view.RESULT_GOTO_SHOP
			self:finalize()
		end)
		

		top_layout:addWidget (ui.Widget())
		frame_layout:addItem (top_layout)

		v_sep = ui.Widget()
		v_sep:setSize(1.0, 0.002)
		v_sep:setVerticalSizePolicy (ui.SizePolicyFixed)
		frame_layout:addWidget (v_sep)
	end
	
	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()

		local max_char_height = math.min (1.0 - frame_size.y - 0.05, character_size.y)
		
		character_size.x = character_size.x * max_char_height / character_size.y
		character_size.y = max_char_height
		
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, 1.0 - character_size.y - frame_size.y - 0.05)
		character_image:setSize (character_size)
		character_image:setVerticalSizePolicy (ui.SizePolicyFixed)
		character_image:setHorizontalSizePolicy (ui.SizePolicyFixed)
		character_image:setPlacingMode(ui.Picture_PlacingModeStretch)
		
		frame:setOrigin (0.0, 1.0 - frame_size.y - 0.05)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)

	return widget
end
