--require "ui/core/utils.lua"
require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "social/facebook_sharing.lua"
require "ui/modal/rateus.lua"
require "ui/widgets/picture_with_backlight.lua"
require "ui/widgets/item_widget.lua"
require "ui/button_colors.lua"
require "underworld2/equipment_view_state.lua"

local view = setmetatable (
	{
		RESULT_UPGRADE = 0,
		RESULT_CANCEL = 1 
	}, { __index = ui.core.view })
ui.levelup = view

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui (level_id, fb_share_available)
	local equipment_view_state = underworld2.equipment_view_state
	local session = equipment_view_state.session
	session.unlock_slots_for_level(level_id)

	if fb_share_available == nil then
		fb_share_available = true
	end

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_image = ui.Picture()
	character_image:setTexture ("ui/characters/ch_advisor_modal:filtered")
	widget:addChild (character_image)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/levelup/ico_newrank:filtered", "ui/levelup/backlight_newrank:filtered")
		top_layout:addWidget (ico)
		
		do
			local vlayout = ui.VerticalLayout()
			
			local title_label = ui.TextLabel()
			title_label:setTextStyle (ui.text_styles.normal_hl.build())
			title_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
			title_label:setText (L "KEY_LEVELUP")
			vlayout:addWidget (title_label)
			
			local hlayout = ui.HorizontalLayout()
			hlayout:setItemSpacing(ui.config.hspacing)
			
			local badge = ui.NotificationBadge()
			badge:setPendingNotifications (level_id)
			badge:setBadgeTexture ("ui/ico_level_large:filtered")
			badge:setBadgeTextStyle (ui.text_styles.badge.build())
			badge:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			hlayout:addWidget (badge)
			
			local strings = underworld2.StringsDatabase()
			
			local level_label = ui.TextLabel()
			level_label:setTextStyle (ui.text_styles.large_hl.build())
			level_label:setText (strings:get (underworld2.LevelsDatabase():get (level_id):getNameKey()))
			level_label:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)
			hlayout:addWidget (level_label)
			
			vlayout:addItem (hlayout)
			
			top_layout:addItem (vlayout)
		end
		
		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function()
			local lambda = function()
				self.result = view.RESULT_CANCEL
				self:finalize()
			end

			if (level_id > 5) and ui.rateus.available() then
				local dialog = ui.rateus.create()

				ui.core.view.delegate_builder (
				{
					finished = function(modal)
						ui.core.view_controller:close_modal_view (modal)
						lambda()
					end
				}).build (dialog)

				ui.core.view_controller:show_modal_view (dialog)
				get_sound_pool():play ("ui/SFX_Notice.wav")
			else
				lambda()
			end
		end)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do
		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (L "KEY_LEVELUP_GUY")
		text_layout:addWidget (title_label)
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (strings:get(math.random_value(594, 636)))
		text_label:setRowsCount (4)
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do
		local button_layout = ui.HorizontalLayout()
		
		button_layout:addWidget (ui.Widget())

		if fb_share_available then

			local share_button = underworld2.facebook_sharing.create_button( 
				L"KEY_FB_SHARING_SHARE", 
				underworld2.facebook_sharing.levelup_bonus(underworld2.player),
				function()
					if not underworld2.player then return end
					underworld2.facebook_sharing.levelup( underworld2.player, function( success )
							if (level_id > 5) and ui.rateus.available() then
								local dialog = ui.rateus.create()

								ui.core.view.delegate_builder (
								{
									finished = function(modal)
										ui.core.view_controller:close_modal_view (modal)
										self:finalize()
										if success then 
											get_sound_pool():play ("ui/SFX_Premium_Purchase.wav") 
										end
									end
								}).build (dialog)

								ui.core.view_controller:show_modal_view (dialog)

								get_sound_pool():play ("ui/SFX_Notice.wav")
							else
								if success then 
									get_sound_pool():play ("ui/SFX_Premium_Purchase.wav") 
								end
								self:finalize()
							end
						end )
			end )
			button_layout:addWidget(share_button)
		else
			local btn_ok = ui.UniversalButton()
			btn_ok:setTexture ("ui/equipment/btn_64_universal:filtered")
			btn_ok:setStretch (ui.UniversalButton_StretchHorizontal)
			ui.set_button_colors (btn_ok, ui.button_colors.green)

			local layout = ui.VerticalLayout()

			local text_label = ui.TextLabel()
			text_label:setTextStyle (ui.text_styles.button_regular.build())
			text_label:setText (L "KEY_BTN_OK")
			text_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
			layout:addWidget (text_label)

			events.connect (btn_ok:getClickedSignalProxy(), function()
				self:finalize()
			end)

			btn_ok:setLayout (layout)
			button_layout:addWidget (btn_ok)
		end
		button_layout:addWidget (ui.Widget())
		
		frame_layout:addItem (button_layout)		
	end
	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end

function view.test()
	local builder = ui.core.view.builder(view)
	local v = builder.build(5)

	ui.core.view.delegate_builder(
	{
		finished = function (modal)
			ui.core.view_controller:close_modal_view (modal)
		end
	}).build (v)

	ui.core.view_controller:show_modal_view (v)
end