require "ui/core/view.lua"
require "ui/config.lua"
require "ui/animations/factory.lua"
require "ui/text_styles.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"
require "social/facebook_sharing.lua"
require "underworld2/shared.lua"

local view = setmetatable (
	{
		RESULT_CANCEL = 0,
		RESULT_COMMENT = 1
	}, { __index = ui.core.view })

local builder = ui.core.view.builder (view)
ui.news_modal = view

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view.create (title, text, img_url, external_url, exit_cb)
	local obj = builder.build(title, text, img_url)
	obj.external_url = external_url
	obj.exit_cb = exit_cb
	return obj
end

function view.build_abstract_view (self, title, text, img_url, is_arbitrary)
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)

	local animations = {
		ui.TransmissionWidget(),
		ui.TransmissionWidget()
	}

	widget:addChild(animations[1])
	widget:addChild(animations[2])

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)

	local function resize()
		local frame_size = frame:getPreferredSize()
		local frame_origin = math.vec2(0.0, 0.5 - frame_size.y / 2)
		frame:setOrigin (frame_origin)
		frame:setSize (1.0, frame_size.y)

		animations[1]:setHorizontalSizePolicy(ui.SizePolicyFixed)
		animations[1]:setVerticalSizePolicy(ui.SizePolicyFixed)
		animations[1]:setOrigin(0.0, 0.0)
		animations[1]:setSize(1.0, frame_origin.y)

		animations[2]:setHorizontalSizePolicy(ui.SizePolicyFixed)
		animations[2]:setVerticalSizePolicy(ui.SizePolicyFixed)
		animations[2]:setOrigin(0.0, frame_origin.y + frame_size.y)
		animations[2]:setSize(1.0, 1.0 - frame_origin.y - frame_size.y)
		animations[2]:setDirection(1)
	end

	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_news:filtered", "ui/backlight_storagefull:filtered")
		top_layout:addWidget (ico)

		local vertical_layout = ui.VerticalLayout()
		--vertical_layout:setItemSpacing (ui.config.vspacing)

		local decor_label = ui.TextLabel()
		decor_label:setTextStyle (ui.text_styles.modal_window_title_small.build())
		decor_label:setText (L"KEY_NEWS_INCOMING_TRANSMISSION")

		local title_label = ui.RichTextWidget()
		title_label:setDefaultTextStyle (ui.text_styles.modal_window_title_med.build())
		title_label:setText (title)

		vertical_layout:addWidget(decor_label)
		vertical_layout:addWidget(title_label)
		vertical_layout:setHorizontalPolicy(ui.SizePolicyMaximum)

		top_layout:addItem (vertical_layout)

		if not is_arbitrary then
			local close = ui.ImageButton ()
			close:setTexture ("ui/btn_close:filtered")
			close:setSound ("ui/UI_Back.wav")
			close:setOrigin (0, 0)
			top_layout:addWidget (close)

			events.connect (close:getClickedSignalProxy(), function()
				self.result = view.RESULT_CANCEL
				self:finalize()
			end)
		end
		
		frame_layout:addItem (top_layout)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		do
			local image = underworld2.RemotePicture()
			local hhlayout = ui.HorizontalLayout()

			image:setTexture("ui/messaging/modal_news_empty:filtered")

			local width = 1 - frame:getLeftMargin() - frame:getRightMargin()
			local pS = image:getPreferredSize()
			local pW = pS.x
			local pH = pS.y
			local pRatio = width / pW

			image:setPlacingMode(ui.Picture_PlacingModeStretch)
			image:setHorizontalSizePolicy(ui.SizePolicyFixed)
			image:setVerticalSizePolicy(ui.SizePolicyFixed)

			image:setHeight(pH * pRatio)
			image:setWidth(width)

			if img_url and (img_url ~= "") then
				local image_layout = ui.HorizontalLayout()
				image:setLayout(image_layout)

				local waiting_widget = ui.WaitingWidget()
				waiting_widget:setHorizontalSizePolicy (ui.SizePolicyMinimum)

				image_layout:addWidget(ui.Widget())
				image_layout:addWidget(waiting_widget)
				image_layout:addWidget(ui.Widget())

				events.connect(image:getDownloadedSignalProxy(), function(owner)
					waiting_widget:setHidden(true)

					local ss = owner:getTextureSize()
					local ww = ss.x
					local hh = ss.y

					if ww > 0 then -- just in any ugly case
						image:setHeight(hh * (width / ww))
					end

					resize()
				end)

				image:download(img_url)
			end

			hhlayout:addWidget(ui.Widget())
			hhlayout:addWidget(image)
			hhlayout:addWidget(ui.Widget())

			frame_layout:addItem(hhlayout)
		end

		local scroll = ui.ScrollWidget()
		scroll:setScrollType (ui.ScrollTypeVertical)
		scroll:setScrollBarTexture ("ui/scroll:filtered")
		scroll:setFrameTexture ("ui/frame_modal_light:filtered")
		scroll:setHeight(0.25)
		scroll:setVerticalSizePolicy(ui.SizePolicyFixed)

		local text_layout = ui.VerticalLayout()
		scroll:setLayout (text_layout)

		local text_label = ui.RichTextWidget()
		text_label:setDefaultTextStyle (ui.text_styles.tutorial_text.build())
		text_label:setText (text)
		text_label:setMaxWidth (1.0 - frame:getLeftMargin() - frame:getRightMargin() - scroll:getLeftMargin() - scroll:getRightMargin())
		text_layout:addWidget (text_label)

		frame_layout:addWidget (scroll)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)

	frame_layout:addWidget (sep)
	frame:setLayout(frame_layout)

	widget:addChild (frame)

	return {
		root = widget,
		layout = frame_layout,
		resize = resize
	}
end

function view:build_ui(...)
	self.ui = {}
	
	local abstract_view = view.build_abstract_view(self, ...)
	local widget = abstract_view.root
	local layout = abstract_view.layout

	do
		local h_layout = ui.HorizontalLayout()
		local fb_button = underworld2.facebook_sharing.create_button(L"KEY_NEWS_COMMENT", 0, function()
			self.result = view.RESULT_COMMENT
			self:finalize()
		end)

		fb_button:setHorizontalSizePolicy (ui.SizePolicyFixed)
		fb_button:setWidth(0.4)

		h_layout:addWidget(ui.Widget())
		h_layout:addWidget(fb_button)
		h_layout:addWidget(ui.Widget())

		layout:addItem(h_layout)
	end

	abstract_view.resize()

	return widget
end

function view:update (dt)
end

function view.show(...)
	local v = view.create(...)

	ui.core.view.delegate_builder(
	{
		finished = function (view)
			if view.result == view.RESULT_COMMENT then
				if view.external_url and (view.external_url ~= "") then
					os.device:openURL(view.external_url)
				else
					underworld2.shared.show_fb_page()
				end
			end

			if view.exit_cb then
				view.exit_cb()
			end

			ui.core.view_controller:close_modal_view (view)
		end
	}).build (v)

	ui.core.view_controller:show_modal_view (v)
end