require "ui/widgets/picture_with_backlight.lua"
require "ui/widgets/item_widget.lua"
require "ui/button_colors.lua"

local view = setmetatable ({}, { __index = ui.core.view })
ui.not_enough_cash = view

function view:back()
	--self.result = view.RESULT_CANCEL
	self:finalize()
end

function view.show( cash_missing, convertible_to_gold )
	local v = ui.core.view.builder (view).build (cash_missing, 0, convertible_to_gold)
	ui.core.view_controller:show_modal_view (v)
	v.delegate_builder {
		finished = function (self)
			ui.core.view_controller:close_modal_view (self)
		end
	}. build(v)

	if debug then
		print( "not_enough_cash ", cash_missing )
		print(debug.STP.stacktrace())
	end

	get_sound_pool():play ("ui/SFX_Notice.wav")
end

function view:build_ui (cash_missing, gold_price, convertible_to_gold)
	local mode = convertible_to_gold and "contextual" or "fill"

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)
	
	do
		local header_layout = ui.HorizontalLayout()
		header_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/vault/ico_vault:filtered", "ui/vault/backlight_vault:filtered")
		header_layout:addWidget (ico)
		
		local header_label = ui.TextLabel()
		header_label:setTextStyle (ui.text_styles.large_hl.build())
		header_label:setText (L"KEY_NEED_MORE_CASH")
		header_layout:addWidget (header_label)
		
		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function() self:finalize() end)
		header_layout:addWidget (close_button)
		
		layout:addItem (header_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	layout:addWidget (sep)
	
	do
		local center_widget = ui.Widget()
		--center_widget:setTexture ("ui/vault/backlight_vault:filtered")
		--center_widget:setPlacingMode (ui.Picture_PlacingModeStretch)
		center_widget:setHeight (0.2)
		center_widget:setVerticalSizePolicy (ui.SizePolicyFixed)
		
		local center_widget_layout = ui.HorizontalLayout()
		center_widget_layout:setItemSpacing (ui.config.hspacing)
		
		center_widget_layout:addWidget (ui.Widget())
		
		local ditch_label = ui.TextLabel()
		ditch_label:setTextStyle (ui.text_styles.large.build())
		ditch_label:setText (L("KEY_BUY_MISSING_TEXT_1", mode))
		ditch_label:setWordWrap (false)
		ditch_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		ditch_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
		center_widget_layout:addWidget (ditch_label)
		
		local ico = ui.Picture()
		ico:setTexture ("ui/ico_cash_small:filtered")
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		--ico:setAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
		center_widget_layout:addWidget (ico)
		
		local amount_label = ui.TextLabel()
		amount_label:setTextStyle (ui.text_styles.large.build())
		amount_label:setText (L ("KEY_BUY_MISSING_TEXT_2", cash_missing, mode))
		amount_label:setWordWrap (false)
		amount_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		amount_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
		center_widget_layout:addWidget (amount_label)
		
		center_widget_layout:addWidget (ui.Widget())
		
		center_widget:setLayout (center_widget_layout)
		
		layout:addWidget (center_widget)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	layout:addWidget (sep)
	
	do

		local btn_layout = ui.HorizontalLayout()
		
		btn_layout:addWidget (ui.Widget())
		
		if convertible_to_gold then
			local price_button = ui.PriceButton()
			price_button:setStateTexture ("ui/btn_premium:filtered")
			price_button:setTextStyle (ui.text_styles.button_regular.build())
			price_button:setPrice (gold_price)
			price_button:setText (L"KEY_BUY_MISSING_BUTTON")
			price_button:setCurrencyIcon ("ui/ico_pbtn_gold:filtered")
			btn_layout:addWidget (price_button)

			events.connect (price_button:getClickedSignalProxy(), function()
				self:finalize()
			end)
		else
			local open_vault = ui.widgets.item_widget.create_btn (L"KEY_OPEN_VAULT_BUTTON", ui.button_colors.green)
			open_vault:setHorizontalSizePolicy (ui.SizePolicyFixed)
			open_vault:setWidth (0.5)
			events.connect (open_vault:getClickedSignalProxy(), function()
				self:finalize()
				ui.status_panel.show_vault()
			end)

			btn_layout:addWidget (open_vault)
		end
		btn_layout:addWidget (ui.Widget())
		
		layout:addItem (btn_layout)
	
	end
	
	frame:setLayout (layout)
	
	do
		local frame_size = frame:getPreferredSize()
		frame:setSize (1.0, frame_size.y)
		frame:setOrigin (0.0, (1.0 - frame_size.y) * 0.5)
	end
	
	return frame
end
