require "ui/widgets/picture_with_backlight.lua"
require "ui/widgets/item_widget.lua"
require "ui/button_colors.lua"
require "ui/store.lua"

local view = setmetatable ({
	RESULT_CANCEL=0,
	RESULT_STORE=1
}, { __index = ui.core.view } )
ui.not_enough_gold = view

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view.show( gold_missing, restore_callback )
	local v = ui.core.view.builder (view).build (gold_missing, restore_callback)
	ui.core.view_controller:show_modal_view (v)
	v.delegate_builder {
		finished = function (self)
			ui.core.view_controller:close_modal_view (self)
			if self.result == view.RESULT_STORE then
				underworld2.ui.stack:push_window (ui.store_view.create(restore_callback))
			elseif restore_callback then
				restore_callback()
			end
		end
	}. build(v)

	get_sound_pool():play ("ui/SFX_Notice.wav")
end


function view:build_ui (gold_missing, restore_callback)
	local mode = convertible_to_gold and "contextual" or "fill"

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)
	
	do
		local header_layout = ui.HorizontalLayout()
		header_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/store/ico_gold:filtered", "ui/store/backlight_gold:filtered")
		header_layout:addWidget (ico)
		
		local header_label = ui.TextLabel()
		header_label:setTextStyle (ui.text_styles.large_hl.build())
		header_label:setText (L"KEY_MORE_GOLD_TITLE")
		header_layout:addWidget (header_label)
		
		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		close_button:setSound ("ui/UI_Back.wav")
		events.connect (close_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize() 
		end)
		header_layout:addWidget (close_button)
		
		layout:addItem (header_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	layout:addWidget (sep)
	
	do
		local center_widget = ui.Widget()
		--center_widget:setTexture ("ui/vault/backlight_vault:filtered")
		--center_widget:setPlacingMode (ui.Picture_PlacingModeStretch)
		center_widget:setHeight (0.2)
		center_widget:setVerticalSizePolicy (ui.SizePolicyFixed)
		
		local center_widget_layout = ui.HorizontalLayout()
		center_widget_layout:setItemSpacing (ui.config.hspacing)
		
		center_widget_layout:addWidget (ui.Widget())
		
		local ditch_label = ui.TextLabel()
		ditch_label:setTextStyle (ui.text_styles.large.build())
		ditch_label:setText (L("KEY_MORE_GOLD_BODY_1", mode))
		ditch_label:setWordWrap (false)
		ditch_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		ditch_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
		center_widget_layout:addWidget (ditch_label)
		--[[
		local ico = ui.Picture()
		ico:setTexture ("ui/ico_cash_small:filtered")
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		--ico:setAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
		center_widget_layout:addWidget (ico)
		
		local amount_label = ui.TextLabel()
		amount_label:setTextStyle (ui.text_styles.large.build())
		amount_label:setText (L ("KEY_BUY_MISSING_TEXT_2", cash_missing, mode))
		amount_label:setWordWrap (false)
		amount_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		amount_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
		center_widget_layout:addWidget (amount_label)
		]]
		
		center_widget_layout:addWidget (ui.Widget())
		
		center_widget:setLayout (center_widget_layout)
		
		layout:addWidget (center_widget)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	layout:addWidget (sep)
	
	do

		local btn_layout = ui.HorizontalLayout()
		
		btn_layout:addWidget (ui.Widget())
				
		local open_vault = ui.widgets.item_widget.create_btn (L"KEY_MORE_GOLD_BUTTON", ui.button_colors.green)
		open_vault:setHorizontalSizePolicy (ui.SizePolicyFixed)
		open_vault:setWidth (0.5)
		events.connect (open_vault:getClickedSignalProxy(), function()
			self.result = view.RESULT_STORE
			self:finalize()
		end)

		btn_layout:addWidget (open_vault)

		btn_layout:addWidget (ui.Widget())
		
		layout:addItem (btn_layout)
	
	end
	
	frame:setLayout (layout)
	
	do
		local frame_size = frame:getPreferredSize()
		frame:setSize (1.0, frame_size.y)
		frame:setOrigin (0.0, (1.0 - frame_size.y) * 0.5)
	end
	
	return frame
end
