require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"

local view = setmetatable ({ }, { __index = ui.core.view } )
ui.quest_failed_view = view

local lui = ui.core.lib

function view:back()
	self:finalize()
end

function view.show (quest, handler)
	local v = ui.core.view.builder (view).build (quest)
	ui.core.view.delegate_builder (
	{
		finished = function()
			ui.core.view_controller:close_modal_view (v)
			handler()
		end
	}).build (v)
	ui.core.view_controller:show_modal_view (v)
end

function view:build_ui (quest)

	local story = quest:get():getStory():get() --safe

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_image = ui.Picture()
	local image_id = ui.config.quests.characters.get_resource_id ("modal", story:getCharacterId())
	character_image:setTexture (image_id)
	widget:addChild (character_image)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/vault/ico_vault:filtered", "ui/vault/backlight_vault:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_QUEST_FAILED_MODAL_TITILE")
		top_layout:addWidget (title_label)
		
		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function() self:finalize() end)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	local text_frame = lui.frame {
		texture = "ui/frame_modal_light:filtered",
		left_margin 	= ui.config.hspacing * 0.5,
		right_margin 	= ui.config.hspacing * 0.5,
		top_margin 		= ui.config.vspacing * 0.5,
		bottom_margin	= ui.config.vspacing * 0.5,
		layout = lui.vertical_layout {
			items = {
				lui.text_label {
					text_style = ui.text_styles.modal_character_name.build(),
					text = story:getCharacterName(),
				},
				lui.rich_text {
					text = story:getQuestFailedText(),
					text_style = ui.text_styles.small_hl.build(),
					vsize_policy = ui.SizePolicyFixed,
					height = 0.15,
				}
			}
		}
	}
	frame_layout:addWidget (text_frame)
	
	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end

