require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"
require "underworld2/shared.lua"

local view = setmetatable ({ }, { __index = ui.core.view } )
ui.rateus = view

local builder = ui.core.view.builder(view)

local __store_urls = {
	android = "market://details?id=com.asteroids.uw",
	iOS = "https://itunes.apple.com/us/app/underworld-shadow-trade/id912785647"
}

local function _send_feedback_handler()
	underworld2.shared.send_feedback()
end

local function _rate_handler()
	os.device:openURL (__store_urls[ os.device:getDeviceOSString() ] )
end

function view:back()
	--self.result = view.RESULT_CANCEL
	self:finalize()
end

function view.create (...)
	local obj = builder.build(...)
	return obj
end

function view.available()
	return not underworld2.settings:get ("rated", false)
end

function view:build_ui ()
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_image = ui.Picture()
	character_image:setTexture ("ui/characters/ch_runner_modal_1:filtered")
	widget:addChild (character_image)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_news:filtered", "ui/backlight_storagefull:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_RATEUS")
		top_layout:addWidget (title_label)
		
		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function() self:finalize() end)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do
		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (L "KEY_RATEUS_GUY")
		text_layout:addWidget (title_label)
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (L "KEY_RATEUS_TEXT")
		text_label:setRowsCount (4)
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	frame:setLayout (frame_layout)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)

		local contact_button = ui.TextButton()
		contact_button:setTexture ("ui/btn_share:filtered")
		contact_button:setTextStyle (ui.text_styles.button_regular.build())
		contact_button:setText (L "KEY_RATEUS_CONTACT")
		top_layout:addWidget (contact_button)

		events.connect (contact_button:getClickedSignalProxy(), function()
			underworld2.settings:set ("rated", true)
			underworld2.settings:flush()

			_send_feedback_handler()

			self:finalize()
		end)

		local rate_button = ui.TextButton()
		rate_button:setTexture ("ui/btn_lower_normal:filtered")
		rate_button:setTextStyle (ui.text_styles.button_regular.build())
		rate_button:setText (L "KEY_RATEUS_RATE")
		top_layout:addWidget (rate_button)

		events.connect (rate_button:getClickedSignalProxy(), function()
			underworld2.settings:set ("rated", true)
			underworld2.settings:flush()

			_rate_handler()

			self:finalize()
		end)

		frame_layout:addItem (top_layout)
	end
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end

function view.test()
	local v = ui.rateus.create ()
	ui.core.view_controller:show_modal_view (v)
end