require "ui/core/view.lua"
require "ui/config.lua"
require "ui/animations/factory.lua"
require "ui/text_styles.lua"
require "ui/format.lua"
require "ui/widgets/gold_to_cash_widget.lua"
require "ui/widgets/picture_with_backlight.lua"
require "ui/modal/ditch_confirm.lua"

local view = setmetatable (
	{
		RESULT_RENT = 0,
		RESULT_CANCEL = 1 ,
		RESULT_DITCH = 2,
	}, { __index = ui.core.view })

local builder = ui.core.view.builder (view)
ui.rent_modal = view

local texts = {
	577,
	578,
	579,
	580,
	581,
	582,
	583,
	584
}

local characters = {
	"ui/characters/ch_supplier_modal_1_2:filtered",
	"ui/characters/ch_supplier_modal_1_2:filtered",
	"ui/characters/ch_supplier_modal_3:filtered",
	"ui/characters/ch_supplier_modal_4:filtered",
	"ui/characters/ch_supplier_modal_5:filtered",
	"ui/characters/ch_supplier_modal_6:filtered",
	"ui/characters/ch_supplier_modal_7:filtered",
	"ui/characters/ch_supplier_modal_8:filtered"
}

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view.create (...)
	local obj = builder.build(...)
	return obj
end

function view:build_ui (storage, courier, rent_price)
	self.ui = {}

  	local strings = underworld2.StringsDatabase()
	local player = underworld2.player
	local max_money = player:getMaxMoney()
	local current_money = math.min (max_money, player:getMoney())
		
  	local skill = underworld2.SkillsDatabase():getGroup ("SMUGGLERSHIP"):getSkill ("ITEM")
	local supplier_key = math.max(courier:getCommodityId() - 1, 1)
  	local supplier_name = strings:get (skill:getLevel (supplier_key):getTitleKey())
    
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_image = ui.Picture()
	character_image:setTexture (characters[courier:getCommodityId()])
	widget:addChild (character_image)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_warning_modal:filtered", "ui/backlight_busted:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_RENT_FULL")
		title_label:setHorizontalSizePolicy (ui.SizePolicyMaximum)

		local close = ui.ImageButton ()
		close:setTexture ("ui/btn_close:filtered")
		close:setOrigin (0, 0)
		close:setSound ("ui/UI_Back.wav")
		events.connect (close:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)

		top_layout:addWidget (title_label)
		top_layout:addWidget (close)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (L("KEY_CREW_SUPPLIER") .. ", " .. supplier_name)
		text_layout:addWidget (title_label)
		
		local commodity = underworld2.CommoditiesDatabase():get (courier:getCommodityId())
		local commodity_name =  strings:get (commodity:getNameKey())

		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (strings:get(texts[courier:getCommodityId()])) -- TO DO: guess, should not hardcode
		text_label:setRowsCount (3)
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end

	do
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")

		local text_layout = ui.HorizontalLayout()
		text_layout:setItemSpacing (ui.config.hspacing)
		text_frame:setLayout (text_layout)

		local ico = ui.Picture()
		ico:setTexture(ui.config.commodity_icons[courier:getCommodityId()])
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)

		local material_amount = ui.TextLabel()
		material_amount:setTextStyle (ui.text_styles.large.build())
		material_amount:setText (tostring (courier:getItemsAmount()))
		material_amount:setWordWrap (false)
		material_amount:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		material_amount:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)


		local confiscated_lbl = ui.TextLabel()
		confiscated_lbl:setTextStyle (ui.text_styles.large.build())
		confiscated_lbl:setText (L "KEY_RENT_NOT_ENOUGH_SPACE")
		confiscated_lbl:setWordWrap (false)
		confiscated_lbl:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		confiscated_lbl:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)

		text_layout:addWidget (confiscated_lbl)
		text_layout:addWidget (ico)
		text_layout:addWidget (material_amount)

		frame_layout:addWidget (text_frame)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)

		local ditch_button = ui.PriceButton()
		ditch_button:setStateTexture ("ui/btn_ditch:filtered")
		ditch_button:setTextStyle (ui.text_styles.button_regular.build())
		ditch_button:setPrice (courier:getItemsAmount())
		ditch_button:setText (L "KEY_RENT_DITCH")

		ditch_button:setCurrencyIcon (ui.config.commodity_icons[courier:getCommodityId()])
		top_layout:addWidget (ditch_button)

		events.connect (ditch_button:getClickedSignalProxy(), function()
			local owner = self
			ui.ditch_confirm.show (courier:getItemsAmount(), ui.config.commodity_icons[courier:getCommodityId()], function (confirmed)
				if confirmed then
					owner.result = owner.RESULT_DITCH
					owner:finalize()
				end
			end)

			get_sound_pool():play ("ui/SFX_Notice.wav")
		end)

		local price_button = ui.PriceButton()
		price_button:setStateTexture ("ui/btn_premium:filtered")
		price_button:setTextStyle (ui.text_styles.button_regular.build())
		price_button:setPrice (rent_price)
		price_button:setText (L "KEY_RENT_RENT")
		price_button:setCurrencyIcon ("ui/ico_pbtn_gold:filtered")
		top_layout:addWidget (price_button)

		events.connect (price_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_RENT
			self:finalize()
		end)

		frame_layout:addItem (top_layout)
	end
	
	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, 1.0 - character_size.y - frame_size.y - 0.05)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, 1.0 - frame_size.y - 0.05)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)

	return widget
end

