require "ui/core/view.lua"
require "ui/config.lua"
require "ui/text_styles.lua"
require "ui/format.lua"
require "ui/widgets/gold_to_cash_widget.lua"
require "ui/widgets/picture_with_backlight.lua"

require "social/facebook_sharing.lua"

require "underworld2/couriers.lua"

local view = setmetatable (
	{
		RESULT_CANCEL = 1,
	}, { __index = ui.core.view })

local builder = ui.core.view.builder (view)
ui.respect_modal = view

function view.create (...)
	local obj = builder.build (...)
	return obj
end

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end


local function current_timespan(timestamp)
	local timespan = timestamp - underworld2.server_time()
	if timespan < 0 then timespan = 0 end
	return timespan
end

local function build_boosts_widget(boosts)
	local widget = ui.Frame()
	widget:setBordersTexture ("ui/frame_modal_light:filtered")
	widget:setMargins (ui.config.hspacing * 2, ui.config.hspacing * 2, ui.config.vspacing, ui.config.vspacing)

	local vlayout = ui.VerticalLayout()
	vlayout:setItemSpacing(ui.config.vspacing)
	widget:setLayout(vlayout)

	local widgets = {}

	for name, data in pairs(boosts) do
		local boost_widget = ui.Widget()

		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing(ui.config.hspacing)

		local ico_rp = ui.load_ui ("ui/rp_bonus.ui")
		local ico_size = ico_rp:getPreferredSize()
		ico_rp:findChild ("ribbon"):setWidth (ico_size.x)
		ico_rp:findChild ("ribbon_offset_bottom"):setHeight (ico_size.y * 0.05)
		runtime.cast_to (ico_rp:findChild ("modifier"), "ui::TextLabel"):setText (data.value)
		hlayout:addWidget (ico_rp)

		local param = ui.TextLabel()
		param:setTextStyle (ui.text_styles.modal_character_name.build())
		param:setText ( data.text )
		param:setHorizontalSizePolicy(ui.SizePolicyMaximum)
		hlayout:addWidget (param)

		local ico = ui.Picture()
		ico:setTexture ("ui/ico_timer:filtered")
		hlayout:addWidget (ico)

		local param_value = ui.TextLabel()
		param_value:setTextStyle (ui.text_styles.tutorial_text.build())
		param_value:setText ( "" )
		param_value:setWordWrap (false)
		hlayout:addWidget (param_value)

		boost_widget:setLayout(hlayout)

		vlayout:addWidget(boost_widget)

		widgets[name] = {
			name_label = param,
			time_label = param_value,
			widget = boost_widget
		}
	end

	widgets.container = widget

	return widgets
end

function view:build_ui ()
	local strings = underworld2.StringsDatabase()
	local player = underworld2.player

	local now = underworld2.server_time()

	self.ui = {}

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)

	local strings = underworld2.StringsDatabase()

	local character_image = ui.Picture()
	character_image:setTexture ("ui/characters/ch_advisor_modal:filtered")
	widget:addChild (character_image)

	do
		local boosts = { }

		local bonus_ptr = underworld2.player:getBonus ("RespectBoostBonus")
		if bonus_ptr:get() then
			local bonus = runtime.cast_to (bonus_ptr:get(), "underworld2::ModifierBonus")
			for i, modifier in pairs (bonus:getModifiers()) do
				if now > modifier.StartTime and now < modifier.EndTime then
					local value = ui.format.modifier (modifier.Value)
					boosts[value] = { text = L("KEY_RP_BOOST_TITLE", ui.format.modifier (modifier.Value)), value = value, timestamp = modifier.EndTime }
				end
			end
			if next (boosts) then
				local widgets = build_boosts_widget(boosts)
				widget:addChild(widgets.container)

				self.boosts = boosts
				self.ui.boosts = widgets
			end		
		end
	end

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_dropheat:filtered", "ui/backlight_busted:filtered")
		top_layout:addWidget (ico)

		local vlayout = ui.VerticalLayout()

		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
		title_label:setText (L("KEY_RESPECT_TITLE", multiplier))

		vlayout:addWidget (title_label)

		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)

		top_layout:addItem (vlayout)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText ( L("KEY_ADVISOR"))
		text_layout:addWidget (title_label)
		
		local text_label = ui.RichTextWidget()
		text_label:setDefaultTextStyle (ui.text_styles.tutorial_text.build())
		text_label:setText (L"KEY_RESPECT_INFO_HINT")
		text_label:setMaxWidth (1.0 - frame:getLeftMargin() - frame:getRightMargin() - text_frame:getLeftMargin() - text_frame:getRightMargin())
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end

	local level_id = player:getLevel()
	local level = underworld2.LevelsDatabase():get (level_id)
	local level_progress_value = 1.0

	do
		local level_layout = ui.HorizontalLayout()
		level_layout:setItemSpacing (ui.config.hspacing)

		local badge = ui.NotificationBadge()
 		badge:setBadgeTexture ("ui/ico_level:filtered")
 		badge:setPendingNotifications (level_id)
 		badge:setBadgeTextStyle (ui.text_styles.badge.build())
 		badge:setHorizontalSizePolicy (ui.SizePolicyMinimum)
 		level_layout:addWidget (badge)

		local level_label = ui.TextLabel ()
		level_label:setText (underworld2.StringsDatabase():get (level:getNameKey()))
		level_label:setTextStyle (ui.text_styles.modal_param_value.build())
		level_label:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		level_label:setWordWrap(false)
		level_layout:addWidget (level_label)

		
		local player = underworld2.player
		
		local next_level 	= underworld2.LevelsDatabase():get (player:getLevel() + 1)
		if level:get() and next_level:get() then
			local rp  = player:getRespectPoints() - level:getRespectPoints()
			local max = next_level:getRespectPoints() - level:getRespectPoints()

			level_progress_value = math.clamp (rp / max, 0, 1)

			local next_rank = ui.TextLabel ()
			next_rank:setText (L ("KEY_RESPECT_MODAL_RP_TO_NEXT_LEVEL", max - rp))
			next_rank:setTextStyle (ui.text_styles.modal_param_value.build())
			next_rank:setWordWrap(false)
			level_layout:addWidget (next_rank)
		end

		frame_layout:addItem (level_layout)
	end

	do
		local level_progress = ui.ProgressBar ()
		level_progress:setTexture ("ui/main/bar_topbar_green:filtered")
		level_progress:setTextStyle (ui.text_styles.small_hl.build())
		level_progress:setValue (level_progress_value)
		frame_layout:addWidget (level_progress)

		local exp_layout = ui.HorizontalLayout()

		local w = ui.Widget()
		w:setHorizontalSizePolicy(ui.SizePolicyMaximum)
		exp_layout:addWidget(w)

		local ico_rp = ui.Picture()
		ico_rp:setTexture ("ui/main/ico_topbar_rp:filtered")
		ico_rp:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		exp_layout:addWidget (ico_rp)

		level_progress:setLayout (exp_layout)
	end
	
	frame:setLayout (frame_layout)

	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)

		self.ui.character_image = character_image
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)

		self.ui.frame = frame

		-- boosts
		if self.boosts then
			local boosts = self.ui.boosts.container
			boosts:setSize(1.0 - ui.config.hspacing * 4, boosts:getPreferredSize().y)
			boosts:setOrigin(ui.config.hspacing * 2.0, (1.0 - sum_height) * 0.5 + character_size.y - boosts:getHeight() - ui.config.vspacing*2)
		end
	end
	
	widget:addChild (frame)

	return widget
end

function view:update(dt)
	-- boosts
	local boosts = self.boosts
	if boosts then
		local timespan = 0

		for name, data in pairs(boosts) do
			local layout = self.ui.boosts[name]
			local duration = current_timespan(data.timestamp)
			if duration > timespan then timespan = duration end

			if duration > 0 then
				layout.time_label:setText(ui.format.duration_string(duration))
				layout.widget:setHidden(false)
			else
				layout.widget:setHidden(true)
			end
		end

		local container = self.ui.boosts.container
		if timespan > 0 then
			container:setHidden(false)
			container:setHeight(container:getPreferredSize().y)
			container:setOrigin(container:getOrigin().x, self.ui.frame:getOrigin().y - container:getHeight() - ui.config.vspacing*2)
		else
			container:setHidden(true)
		end
	end
end

function view.show(...)
	local builder = ui.core.view.builder(view)
	local v = builder.build(...)

	ui.core.view.delegate_builder(
	{
		finished = function (view)
			ui.core.view_controller:close_modal_view (view)
		end
	}).build (v)

	ui.core.view_controller:show_modal_view (v)
end
