require "ui/core/view.lua"
require "ui/config.lua"
require "ui/animations/factory.lua"
require "ui/text_styles.lua"
require "ui/format.lua"
require "ui/widgets/gold_to_cash_widget.lua"
require "ui/widgets/picture_with_backlight.lua"

require "social/facebook_sharing.lua"

require "underworld2/couriers.lua"

local view = setmetatable (
	{
		RESULT_CANCEL = 1,
		RESULT_BUY = 2
	}, { __index = ui.core.view })

local builder = ui.core.view.builder (view)
ui.rp_boost_duplicate = view

function view.create (...)
	local obj = builder.build (...)
	return obj
end

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

local function current_timespan(timestamp)
	local timespan = timestamp - underworld2.server_time()
	if timespan < 0 then timespan = 0 end
	return timespan
end

function view:build_ui (effect_endtime, effect_duration, text)
	local strings = underworld2.StringsDatabase()

	self.ui = {}

	self.effect_endtime = effect_endtime
	self.effect_duration = effect_duration

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)

	local character_image = ui.Picture()
	character_image:setTexture (string.format ("ui/characters/Modal/ch_advisor_modal:filtered"))
	widget:addChild (character_image)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_dropheat:filtered", "ui/backlight_busted:filtered")
		top_layout:addWidget (ico)

		local vlayout = ui.VerticalLayout()

		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title_big.build())
		title_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
		title_label:setText (L("KEY_MORE_BOOST_TIME"))

		vlayout:addWidget (title_label)

		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)

		top_layout:addItem (vlayout)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText ( L ("KEY_ADVISOR") )
		text_layout:addWidget (title_label)
		
		local text_label = ui.RichTextWidget()
		text_label:setDefaultTextStyle (ui.text_styles.tutorial_text.build())
		text_label:setText (L"KEY_EFFECT_DUPLICATE_RP_BOOST_HINT")
		text_label:setMaxWidth (1.0 - frame:getLeftMargin() - frame:getRightMargin() - text_frame:getLeftMargin() - text_frame:getRightMargin())
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end

	do
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local text_layout = ui.VerticalLayout()

		do
			local hlayout = ui.HorizontalLayout()
			hlayout:setItemSpacing(ui.config.hspacing)

			local param = ui.TextLabel()
			param:setTextStyle (ui.text_styles.modal_param.build())
			param:setText ( L ("KEY_CURRENTLY_ACTIVE_RP_BOOST") )
			param:setHorizontalSizePolicy(ui.SizePolicyMaximum)
			hlayout:addWidget (param)

   			local ico = ui.Picture()
  			ico:setTexture ("ui/ico_timer:filtered")
  			hlayout:addWidget (ico)

			local param_value = ui.TextLabel()
			param_value:setTextStyle (ui.text_styles.modal_param_value.build())
			param_value:setText ( "")
			param_value:setWordWrap (false)
			hlayout:addWidget (param_value)

			self.ui.active_protection = param_value
    
			text_layout:addItem(hlayout)
		end

		do
			local hlayout = ui.HorizontalLayout()
			hlayout:setItemSpacing(ui.config.hspacing)

			local param = ui.TextLabel()
			param:setTextStyle (ui.text_styles.modal_param.build())
			param:setText ( L ("KEY_NEW_RP_BOOST") )
			param:setHorizontalSizePolicy(ui.SizePolicyMaximum)
			hlayout:addWidget (param)

   			local ico = ui.Picture()
  			ico:setTexture ("ui/ico_timer:filtered")
  			hlayout:addWidget (ico)

			local param_value = ui.TextLabel()
			param_value:setTextStyle (ui.text_styles.modal_param_value.build())
			param_value:setText ( "" )
			param_value:setWordWrap (false)
			hlayout:addWidget (param_value)

			self.ui.new_protection = param_value
    
			text_layout:addItem(hlayout)
		end

		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end

	frame_layout:addWidget(ui.Widget())

	do
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)

		hlayout:addWidget (ui.Widget())

		local btn = ui.TextButton()
		btn:setTexture ("ui/settings/btn_green_normal:filtered")
		btn:setTextStyle (ui.text_styles.normal_hl.build())
		btn:setText (L"KEY_BUY_RP_BOOST")

		hlayout:addWidget (btn)
		hlayout:addWidget (ui.Widget())

		events.connect (btn:getClickedSignalProxy(), function()
			self.result = view.RESULT_BUY
			self:finalize()
		end)

		frame_layout:addItem (hlayout)
	end

	
	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)

	return widget
end

function view:update (dt)
	self.ui.active_protection:setText(ui.format.duration_string(current_timespan(self.effect_endtime)))
	self.ui.new_protection:setText(ui.format.duration_string (self.effect_duration))
end

function view.show (effect_endtime, effect_duration, text, on_finished)
	local builder = ui.core.view.builder(view)
	local v = builder.build (effect_endtime, effect_duration, text, on_finished)

	ui.core.view.delegate_builder(
	{
		finished = function (view)
			ui.core.view_controller:close_modal_view (view)
			if on_finished then on_finished (view.result) end
		end
	}).build (v)

	ui.core.view_controller:show_modal_view (v)
end
