require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"
require "underworld2/couriers.lua"

local view = setmetatable ({
	RESULT_CANCEL = 1,
	RESULT_MANAGE = 0
}, { __index = ui.core.view } )
ui.runners_busy = view

local builder = ui.core.view.builder(view)
local courier_name_map = underworld2.couriers.names

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view.create (...)
	local obj = builder.build(...)
	return obj
end

function view:build_ui ()
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)

	local strings = underworld2.StringsDatabase()
	local player = underworld2.player
	local skill = underworld2.SkillsDatabase():getGroup ("SMUGGLERSHIP"):getSkill ("COURIERS")
	local skill_level_id = player:getSkillLevel (skill:getId())
	local level = skill:getLevel (skill_level_id)
	
	local character_image = ui.Picture()
	character_image:setTexture (string.format ("ui/characters/ch_runner_modal_%d:filtered", skill_level_id))
	widget:addChild (character_image)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_runner_modal:filtered", "ui/vault/backlight_vault:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_RUNNERS_BUSY")
		top_layout:addWidget (title_label)
		
		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		close_button:setTag (view.RESULT_CANCEL)
		events.connect (close_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)

		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do
		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (L("KEY_RUNNER") .. ", " .. strings:get (level:getTitleKey()) )
		text_layout:addWidget (title_label)
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (L "KEY_RUNNERS_BUSY_TEXT")
		text_label:setRowsCount (4)
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end

	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	frame:setLayout (frame_layout)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)

		local manage_button = ui.TextButton()
		manage_button:setTexture ("ui/btn_lower_normal:filtered")
		manage_button:setTextStyle (ui.text_styles.button_regular.build())
		manage_button:setText (L "KEY_RUNNERS_BUSY_MANAGE")
		manage_button:setTag (view.RESULT_MANAGE)
		events.connect (manage_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_MANAGE
			self:finalize()
		end)

		top_layout:addWidget (ui.Widget())
		top_layout:addWidget (manage_button)
		top_layout:addWidget (ui.Widget())

		frame_layout:addItem (top_layout)
	end
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end

function view.test()
	local v = ui.runners_busy.create ()
	ui.core.view_controller:show_modal_view (v)
end