require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"

local view = setmetatable (
	{
		RESULT_GOTO_UPGRADES = 0,
		RESULT_CANCEL = 1 
	}, { __index = ui.core.view })
ui.upgrade_delivery_confirmation = view

local builder = ui.core.view.builder(view)

function view.unschedule_all()
	scheduled_views = { }
end

local function set_skill (self, pending_skill)
	local strings = underworld2.StringsDatabase()
	
	local skill = underworld2.SkillsDatabase():getGroup ("SMUGGLERSHIP"):getSkillById (pending_skill:getSkillId())
	local level_id = pending_skill:getSkillLevelId()
	local level = skill:getLevel (level_id)
	
	local frame_layout = ui.VerticalLayout()

	do
		local top_layout = ui.VerticalLayout()
		top_layout:setItemSpacing (ui.config.vspacing)
		
		local title_label = ui.TextLabel ()
		title_label:setTextStyle (ui.text_styles.large_hl.build())
		title_label:setText (L "KEY_UPGRADE_DELIVERY_NOTIFICATION")
		title_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
		title_label:setVerticalSizePolicy (ui.SizePolicyMinimum)
		top_layout:addWidget (title_label)

		do
			local hlayout = ui.HorizontalLayout()
			
			do

				local vlayout = ui.VerticalLayout()
				vlayout:addWidget (ui.Widget())
			
				local narator_image = ui.Picture()
				narator_image:setTexture ("ui/modal_char_tutor:filtered")
				narator_image:setHorizontalSizePolicy (ui.SizePolicyMinimum)
				vlayout:addWidget (narator_image)
				
				hlayout:addItem (vlayout)
			end
			
			local dialog_message = ui.ChatMessageWidget()
			dialog_message:setFrameTexture ("ui/messaging/frame_chat_message_other:filtered")
			dialog_message:setTailTexture ("ui/messaging/frame_chat_message_other_tip:filtered")
			dialog_message:setTextStyle (ui.text_styles.xsmall_hl.build())
			dialog_message:appendText (L "KEY_UPGRADE_DELIVERY_SAMPLE_TEXT")
			dialog_message:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			dialog_message:setEnabled (false)
			dialog_message:setNicknameTextStyle (ui.text_styles.small.build())
			dialog_message:setNickname (L "KEY_UPGRADE_DELIVERY_GUY")
			
			hlayout:addWidget (dialog_message)
			
			top_layout:addItem (hlayout)		
		end
		
		frame_layout:addItem (top_layout)
	end
	
	do
		local upgrade_layout = ui.VerticalLayout()
		upgrade_layout:setItemSpacing (ui.config.vspacing)
	
		local line_sep = ui.Picture()
		line_sep:setTexture ("ui/gold_line_large:filtered")
		line_sep:setVerticalSizePolicy (ui.SizePolicyMinimum)
		upgrade_layout:addWidget (line_sep)

		upgrade_layout:addWidget (ui.Widget())

		local name_label = ui.TextLabel()
		name_label:setText (strings:get (level:getTitleKey()))
		name_label:setTextStyle (ui.text_styles.normal_hl.build())
		name_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
		upgrade_layout:addWidget (name_label)

		local desc = ui.upgrade_widget.skill_desc[skill:getType()]
		
		local image = ui.Picture()
		image:setTexture (desc.get_image_id (level_id))
		upgrade_layout:addWidget (image)
		
		do
			local effects_layout = ui.HorizontalLayout()
			effects_layout:addWidget (ui.Widget())
			desc.add_effect_icons (effects_layout, level_id)
			
			local value_label = ui.TextLabel()
			value_label:setWordWrap (false)
			value_label:setTextStyle (ui.text_styles.small_hl.build())
			value_label:setText (desc.display_value (level:getValue(), true))
			value_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			effects_layout:addWidget (value_label)
			
			effects_layout:addWidget (ui.Widget())
			
			upgrade_layout:addItem (effects_layout)
		end
		
		local line_sep = ui.Picture()
		line_sep:setTexture ("ui/gold_line_large:filtered")
		upgrade_layout:addWidget (line_sep)

		do
			local hlayout = ui.HorizontalLayout()
			hlayout:addWidget (ui.Widget())

			local sell_button = ui.widgets.item_widget.create_btn (L "UPGRADE_DELIVERY_MANAGE_TEAM", ui.widgets.item_widget.btn_colors.buy)
			sell_button:setHorizontalSizePolicy (ui.SizePolicyFixed)
			sell_button:setWidth (math.max (0.5, sell_button:getPreferredSize().x))
			events.connect (sell_button:getClickedSignalProxy(), function()
				self.result = view.RESULT_GOTO_UPGRADES
				self:finalize()
			end)
			hlayout:addWidget (sell_button)

			hlayout:addWidget (ui.Widget())
			upgrade_layout:addItem (hlayout)
		end
		
		frame_layout:addItem (upgrade_layout)
	end

	local frame = self.frame
	local close = self.close

	frame:setLayout (frame_layout)

	local size = frame:getPreferredSize()
	size.x = math.max (0.8, size.x)
	local origin = math.vec2 ((1.0 - size.x) * 0.5, (1.0 - size.y) * 0.5)
	frame:setOrigin (origin)
	frame:setSize (size)

	local close_btn_size = close:getPreferredSize()
	close:setOrigin (origin.x + size.x - close_btn_size.x * 0.75, origin.y - close_btn_size.y * 0.25)
	close:setSize (close_btn_size)
end

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui (pending_skill)
	self.ui = { }

	local root = ui.Widget()
	root:setOrigin (0.0, 0.0)
	root:setSize (1.0, 1.0)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/window_modal_large:filtered")
	local margins = { x = frame:getLeftMargin() * 0.5, y = frame:getTopMargin() * 0.5 }
	frame:setMargins (margins.x, margins.x, margins.y, margins.y)
	
	local close = ui.ImageButton ()
	close:setTexture ("ui/btn_close:filtered")
	close:setSound ("ui/SFX_Menu_Close.wav")
	events.connect (close:getClickedSignalProxy(), function() 
		self.result = view.RESULT_CANCEL
		self:finalize()
	end)

	self.frame = frame
	self.close = close
	root:addChild (frame)
	root:addChild (close)
	
	set_skill (self, pending_skill)
	
	return root
end
