require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"

local view = setmetatable ({ }, { __index = ui.core.view } )
ui.vault_full = view

local texts = {
	585,
	586,
	587,
	588,
	589,
	590,
	591,
	592,
	593
}

function view:back()
	--self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui (skill_level, level_id)
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local character_image = ui.Picture()
	character_image:setTexture (string.format ("ui/characters/ch_accountant_modal_%d:filtered", skill_level:getId()))
	widget:addChild (character_image)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/vault/ico_vault:filtered", "ui/vault/backlight_vault:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_VAULT")
		top_layout:addWidget (title_label)
		
		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function() self:finalize() end)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do
		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (L("KEY_VAULT_ACCOUNTANT") .. ", " .. strings:get (skill_level:getTitleKey()))
		text_layout:addWidget (title_label)
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (strings:get(texts[skill_level:getId()]))
		text_label:setRowsCount (text_label:getRowsForWidth(0.9))
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end
	
	do
		local progress_bar = ui.ProgressBar()
		progress_bar:setTexture ("ui/bar_modal_cash:filtered")
		progress_bar:setValue (1.0)
		progress_bar:setTextStyle (ui.text_styles.normal_hl.build())
		progress_bar:setText (ui.format.digit (skill_level:getValue()))
		
		local progress_layout = ui.HorizontalLayout()
		progress_layout:addWidget (ui.Widget())
		
		local cash_ico = ui.Picture()
		cash_ico:setTexture ("ui/ico_cash:filtered")
		cash_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		progress_layout:addWidget (cash_ico)
		
		progress_bar:setLayout (progress_layout)
		
		frame_layout:addWidget (progress_bar)
	end
	
	local hint_label = ui.TextLabel()
	hint_label:setTextStyle (ui.text_styles.normal_hl.build())
	hint_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
	hint_label:setText (L "KEY_VAULT_FULL")
	frame_layout:addWidget (hint_label)
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end

