require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/widgets/picture_with_backlight.lua"
require "ui/widgets/item_widget.lua"
require "ui/button_colors.lua"
require "ui/modal/ditch_confirm.lua"

local view = setmetatable ({ 
	RESULT_CANCEL = 0,
	RESULT_DITCH = 1,
	RESULT_RENT = 2
}, { __index = ui.core.view } )
ui.vault_warning = view

local texts = {
	585,
	586,
	587,
	588,
	589,
	590,
	591,
	592,
	593
}

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui (skill_level, ditch_amount, collected_amount, rent_price)
	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)
	
	local btn_handler = function (btn)
		self.result = btn:getTag()
		self:finalize()
	end
	
	local character_image = ui.Picture()
	character_image:setTexture (string.format ("ui/characters/ch_accountant_modal_%d:filtered", skill_level:getId()))
	widget:addChild (character_image)
	
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)
	
	do
		local top_layout = ui.HorizontalLayout()
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/ico_warning_modal:filtered", "ui/backlight_busted:filtered")
		top_layout:addWidget (ico)
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title.build())
		title_label:setText (L "KEY_VAULT_FULL")
		top_layout:addWidget (title_label)
		
		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		close_button:setTag (view.RESULT_CANCEL)
		events.connect (close_button:getClickedSignalProxy(), btn_handler)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do
		local strings = underworld2.StringsDatabase()
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local margins = { x = text_frame:getLeftMargin() * 0.5, y = text_frame:getTopMargin() }
		text_frame:setMargins (margins.x, margins.x, margins.y, margins.y)
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText (L("KEY_VAULT_ACCOUNTANT") .. ", " .. strings:get (skill_level:getTitleKey()))
		text_layout:addWidget (title_label)
		
		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.small_hl.build())
		text_label:setText (strings:get(texts[skill_level:getId()]))
		text_label:setRowsCount (text_label:getRowsForWidth(0.9))
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end
	
	do
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local text_layout = ui.HorizontalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.large_hl.build())
		title_label:setText (L("KEY_VAULT_NOT_ENOUGH_SPACE"))
		text_layout:addWidget (title_label)
		
		local ico = ui.Picture()
		ico:setTexture ("ui/ico_cash:filtered")
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		
		local ditch_label = ui.TextLabel()
		ditch_label:setTextStyle (ui.text_styles.large_hl.build())
		ditch_label:setText (ui.format.digit (ditch_amount))
		ditch_label:setWordWrap (false)
		ditch_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		text_layout:addWidget (ditch_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)
	
	do
		local buttons_layout = ui.HorizontalLayout()
		buttons_layout:setItemSpacing (ui.config.hspacing * 2)
		
		local ditch_btn = ui.widgets.item_widget.create_price_btn (L "KEY_VAULT_DITCH", ditch_amount, "ui/ico_cash_small:filtered", ui.button_colors.red)
		ditch_btn:setTag (view.RESULT_DITCH)
		events.connect (ditch_btn:getClickedSignalProxy(), function()
			ui.ditch_confirm.show (ditch_amount, "ui/icons/ico_cash_small:filtered", function (confrmed)
				if confrmed then
					self.result = view.RESULT_DITCH
					self:finalize()
				end
			end)
		end)
		buttons_layout:addWidget (ditch_btn)
		
		local rent_button = ui.PriceButton()
		rent_button:setStateTexture ("ui/btn_premium:filtered")
		rent_button:setTextStyle (ui.text_styles.button_regular.build())
		rent_button:setText (L "KEY_VAULT_RENT")
		rent_button:setPrice (tostring (rent_price))
		rent_button:setCurrencyIcon ("ui/ico_pbtn_gold:filtered")
		rent_button:setTag (view.RESULT_RENT)
		events.connect (rent_button:getClickedSignalProxy(), btn_handler)
		buttons_layout:addWidget (rent_button)
		
		frame_layout:addItem (buttons_layout)
		
	end
	
	frame:setLayout (frame_layout)
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)
	
	return widget
end
