require "ui/core/view.lua"
require "ui/config.lua"
require "ui/animations/factory.lua"
require "ui/text_styles.lua"
require "ui/format.lua"
require "ui/widgets/gold_to_cash_widget.lua"
require "ui/widgets/picture_with_backlight.lua"

require "social/facebook_sharing.lua"

require "underworld2/couriers.lua"

local view = setmetatable (
	{
		RESULT_CANCEL = 1,
	}, { __index = ui.core.view })

local builder = ui.core.view.builder (view)
ui.wore_off_bust_protection = view

function view.create (...)
	local obj = builder.build (...)
	return obj
end

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui (text)
	local strings = underworld2.StringsDatabase()

	self.ui = {}

	local widget = ui.Widget()
	widget:setOrigin (0.0, 0.0)
	widget:setSize (1.0, 1.0)

	local backlights = {
		ui.Picture(),
		ui.Picture()
	}

	backlights[1]:setTexture ("ui/backlight_modal_large:filtered")
	backlights[1]:setColor(render.color(255, 11, 11, 255))
	backlights[1]:setSize(math.vec2(0.9, 0.5))
	backlights[1]:setPlacingMode (ui.Picture_PlacingModeStretch)

	backlights[2]:setTexture ("ui/backlight_modal_large:filtered")
	backlights[2]:setColor(render.color(11, 93, 255, 255))
	backlights[2]:setSize(math.vec2(0.9, 0.5))
	backlights[2]:setPlacingMode (ui.Picture_PlacingModeStretch)

	widget:addChild(backlights[1])
	widget:addChild(backlights[2])
	

	local character_image = ui.Picture()
	character_image:setTexture (string.format ("ui/characters/ch_cop_modal:filtered"))
	widget:addChild (character_image)

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/modal_wide_window:filtered")
	frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing (ui.config.vspacing)

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)
		
		local ico = ui.widgets.picture_with_backlight.create ("ui/icons/ico_premium_police_disabled:filtered", "ui/backlight_busted:filtered")
		top_layout:addWidget (ico)

		local vlayout = ui.VerticalLayout()

		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_window_title_med.build())
		title_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
		title_label:setText (L("KEY_BUST_PROTECTION"))

		local decor_label = ui.TextLabel()
		decor_label:setTextStyle (ui.text_styles.modal_window_title_small.build())
		decor_label:setText (L"KEY_EFFECT_WORE_OFF")
		vlayout:addWidget(decor_label)

		vlayout:addWidget (title_label)

		local close_button = ui.ImageButton()
		close_button:setTexture ("ui/btn_close:filtered")
		close_button:setSound ("ui/UI_Back.wav")
		close_button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (close_button:getClickedSignalProxy(), function()
			self.result = view.RESULT_CANCEL
			self:finalize()
		end)

		top_layout:addItem (vlayout)
		top_layout:addWidget (close_button)
		
		frame_layout:addItem (top_layout)
	end
	
	local sep = ui.Picture()
	sep:setTexture ("ui/gold_line_large:filtered")
	local sep_height = sep:getPreferredSize().y
	sep:setHeight (sep_height)
	sep:setPlacingMode (ui.Picture_PlacingModeStretch)
	sep:setVerticalSizePolicy (ui.SizePolicyFixed)
	frame_layout:addWidget (sep)

	do
		
	
		local text_frame = ui.Frame()
		text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
		
		local text_layout = ui.VerticalLayout()
		
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.modal_character_name.build())
		title_label:setText ( L ("KEY_DETECTIVE") )
		text_layout:addWidget (title_label)
		
		local text_label = ui.RichTextWidget()
		text_label:setDefaultTextStyle (ui.text_styles.tutorial_text.build())
		text_label:setText (text)
		text_label:setMaxWidth (1.0 - frame:getLeftMargin() - frame:getRightMargin() - text_frame:getLeftMargin() - text_frame:getRightMargin())
		text_layout:addWidget (text_label)
		
		text_frame:setLayout (text_layout)
		
		frame_layout:addWidget (text_frame)
	end
	
	frame:setLayout (frame_layout)

	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, 1.0 - character_size.y - frame_size.y - 0.05)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, 1.0 - frame_size.y - 0.05)
		frame:setSize (1.0, frame_size.y)

		local origin = character_image:getOrigin()
		backlights[1]:setOrigin(origin.x - 0.27, origin.y)
		backlights[2]:setOrigin(origin.x + 0.15, origin.y + 0.04)
	end
	
	do
		local character_size = character_image:getPreferredSize()
		local frame_size = frame:getPreferredSize()
		
		local sum_height = character_size.y + frame_size.y
		character_image:setOrigin ((1.0 - character_size.x) * 0.5, (1.0 - sum_height) * 0.5)
		character_image:setSize (character_size)
		
		frame:setOrigin (0.0, (1.0 - sum_height) * 0.5 + character_size.y)
		frame:setSize (1.0, frame_size.y)
	end
	
	widget:addChild (frame)

	local red = render.color(255, 11, 11, 255)
	local blue = render.color(11, 93, 255, 255)

	local blink = ui.animations.build {
		duration = 0.5,
		from = 255,
		to = 0,
		times = -1,
		oscillate = true,

		step = function (self, value)
			local l = render.color(((self.iteration + 0) % 4 > 1) and blue or red)
			local r = render.color(((self.iteration + 1) % 4 > 1) and red or blue)

			backlights[1]:setColor(render.color(l.r, l.g, l.b, value))
			backlights[2]:setColor(render.color(r.r, r.g, r.b, 255-value))
		end
	}

	self.animations = ui.animations.factory()
  	self.animations:bind ("blink", blink)

	self:update(0)

	return widget
end

function view:update (dt)
	self.animations:update(dt)
end

function view.show(handler)
	local builder = ui.core.view.builder(view)
	local v = builder.build(L"KEY_EFFECT_WORE_OFF_BUST_PROTECTION_HINT")

	ui.core.view.delegate_builder(
	{
		finished = function (view)
			ui.core.view_controller:close_modal_view (view)
			if handler then handler() end
		end
	}).build (v)

	ui.core.view_controller:show_modal_view (v)
end
