require "ui/completed_deal_widget.lua"
require "ui/store.lua"
require "ui/deal_card_view.lua"
require "ui/config.lua"
require "ui/modal/levelup.lua"
require "ui/loan_shark.lua"
require "ui/account_settings_view.lua"
require "ui/equipment_view.lua"
require "ui/crew_view.lua"

require "ui/dev_view.lua"
require "ui/sound_check_view.lua"
require "underworld2/shared.lua"
require "ui/modal/top_hustlers.lua"

require "ui/modal/respect_modal.lua"
--require "ui/image_customization_view.lua"

local view = setmetatable (
	{
	},
	{ __index = ui.core.view }
)

ui.profile_bar = view

local builder = ui.core.view.builder (view)

function view.create()
	return builder.build()
end

local function _show_sound_check()
	local view = ui.core.view.builder (ui.sound_check_view).build()
	underworld2.ui.stack:push_window (view)
	underworld2.ui.status_panel:hide_profile_info()
end

local function _show_account_settings()
	local view = ui.core.view.builder (ui.account_settings_view).build()
	underworld2.ui.stack:push_window (view)
	underworld2.ui.status_panel:hide_profile_info()

	--ui.image_customization_view.show()
end

function show_crew_locked()
	local view = ui.core.view.builder (ui.crew_locked).build()
	ui.core.view.delegate_builder ({
		finished = function (self)
			ui.core.view_controller:close_modal_view (self)
		end
	}).build (view)
	ui.core.view_controller:show_modal_view (view)
end

local function _show_fb_page()
	underworld2.shared.show_fb_page()
end

local _create_menu_button = function(icon, text, events_amount, min_level, locked)
	local min_level = min_level or 0
	local locked = locked or false

	local button = ui.ContainerButton()
 	button:setTexture (locked and "ui/side_menu/btn_sm_locked:filtered" or "ui/side_menu/btn_sm_center:filtered")
	
	local margins = { x = button:getLeftMargin() * 0.5, y = button:getTopMargin() * 0.5 }
	button:setMargins (margins.x, margins.x, margins.y, margins.y)

 	local layout = ui.HorizontalLayout()
 	layout:setItemSpacing (ui.config.hspacing)

 	local picture = ui.SideMenuIcon()
	picture:setTexture (icon)
	picture:setLocked (locked)
	picture:setRequiredLevel (min_level)
	picture:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (picture)

 	local label = ui.TextLabel()
 	label:setTextStyle (locked and ui.text_styles.side_menu_locked.build() or ui.text_styles.side_menu_normal.build())
 	label:setText (text)
 	label:setWordWrap (false)
 	layout:addWidget (label)

 	if (events_amount and events_amount > 0) then
 		layout:addWidget(ui.Widget())
 		local badge = ui.NotificationBadge()
 		badge:setBadgeTexture ("ui/ico_notification:filtered")
 		badge:setPendingNotifications (events_amount)
 		badge:setBadgeTextStyle (ui.text_styles.badge.build())
 		badge:setHorizontalSizePolicy (ui.SizePolicyMinimum)
 		badge:setDisplayCount(false)
 		layout:addWidget (badge)
 	end

 	if not events_amount or events_amount <= 0 then
	 	local stretcher = ui.Widget()
	 	stretcher:setHorizontalSizePolicy (ui.SizePolicyMaximum)
	 	layout:addWidget (stretcher)
	end

 	local arrow_ico = ui.Picture()
	arrow_ico:setTexture("ui/arrow_off_ico:filtered")
	arrow_ico:setPlacingMode(ui.Picture_PlacingModeCenter)
	arrow_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (arrow_ico)

 	button:setLayout (layout)

 	return button
end 

function view:build_ui()
	self.completed_deals = { }
	self.ui = { }

	local root_widget = ui.Background()
	root_widget:setOrigin (0.0, 0.0)
	root_widget:setSize (0.8, 1.0)
	root_widget:setTexture ("ui/solid_background:filtered")
	root_widget:setColor (render.color (38, 43, 48, 255))

	local scroll_widget = ui.ScrollWidget()
	scroll_widget:setOrigin (0.0, 0.0)
	scroll_widget:setScrollType (ui.ScrollTypeVertical)
	--scroll_widget:setSurfaceSize (math.vec2 (0.8, 1.5))
	root_widget:addChild (scroll_widget)
	
	local root_layout = ui.VerticalLayout()
	root_layout:setItemSpacing (ui.config.vspacing)
	scroll_widget:setLayout (root_layout)

	local player = underworld2.player

	do
		--[[local wrapper = ui.Picture()
		wrapper:setMargins (ui.config.hspacing * 2, ui.config.hspacing * 2, ui.config.vspacing, ui.config.vspacing)
		wrapper:setTexture ("ui/side_menu_header_large:filtered")
		wrapper:setPlacingMode (ui.Picture_PlacingModeStretch)]]
		local wrapper = ui.StatsBackgroundWidget()
		wrapper:setMargins (ui.config.hspacing * 2, ui.config.hspacing * 2, ui.config.vspacing, ui.config.vspacing)
		wrapper:setOrnamentEnabled (true)

		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)

		local avatar = ui.AvatarWidget ()
		player:retain() -- inc ref count before initializing smart_ptr
		avatar:setProfile (underworld2.ProfilePtr (runtime.cast_to (player:get(), "underworld2::Profile")))
		avatar:setAvatarSize (underworld2.Avatar_SizeLarge)
		avatar:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		hlayout:addWidget (avatar)

		local vlayout = ui.VerticalLayout()

		local nickname = ui.TextLabel ()
		nickname:setText (player:getNickname())
		nickname:setTextStyle (ui.text_styles.nickname_large.build())
		nickname:setElide (true)
		vlayout:addWidget (nickname)

		local current_level = underworld2.LevelsDatabase():get (player:getLevel())

		do
			local level_layout = ui.HorizontalLayout()
			level_layout:setItemSpacing (ui.config.hspacing)

			local badge = ui.NotificationBadge()
	 		badge:setBadgeTexture ("ui/ico_level:filtered")
	 		badge:setPendingNotifications (player:getLevel())
	 		badge:setBadgeTextStyle (ui.text_styles.badge.build())
	 		badge:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	 		self.ui.level_badge = badge
	 		level_layout:addWidget (badge)

			local level = ui.TextLabel ()
			level:setText (underworld2.StringsDatabase():get (current_level:getNameKey()))
			level:setTextStyle (ui.text_styles.small.build())
			level_layout:addWidget (level)

			vlayout:addItem (level_layout)
		end


		local level_progress = ui.ProgressBar ()
		level_progress:setTexture ("ui/bar_experience_large:filtered")
		level_progress:setTextStyle (ui.text_styles.small_hl.build())
		vlayout:addWidget (level_progress)

		local exp_layout = ui.VerticalLayout()
		local ico_rp = ui.Picture()
		ico_rp:setTexture ("ui/ico_rp_sidemenu:filtered")
		ico_rp:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		exp_layout:addWidget (ico_rp)
		level_progress:setLayout (exp_layout)


		self.ui.level_progress = level_progress

		hlayout:addItem (vlayout)
		wrapper:setLayout (hlayout)

		root_layout:addWidget (wrapper)
	end

	do
		local wrapper = ui.Widget()
		wrapper:setMargins (ui.config.hspacing, ui.config.hspacing, 0.0, 0.0)
		--wrapper:setLeftMargin (ui.config.hspacing * 0.5)
		--wrapper:setRightMargin (ui.config.hspacing * 0.5)

		local vlayout = ui.VerticalLayout()
		vlayout:setItemSpacing (ui.config.vspacing)

		self.menu_buttons = { }

		--todo: remove reference
		local equipment_min_level = underworld2.BalanceValuesDatabase():get ("equipmentUnlockLevel", 2)
		local equipment_locked = player:getLevel() < equipment_min_level

		button = _create_menu_button (
			equipment_locked and "ui/side_menu/ico_sm_equipment_locked:filtered" or "ui/side_menu/ico_sm_equipment:filtered", 
			L ("KEY_PROFILE_BTN_EQUIPMENT"),
			underworld2.upgrades_tracker.has_new_items { level = player:getLevel(), category = "items" } and 1 or 0,
			equipment_min_level, equipment_locked
		)
		self.menu_buttons.equipment = button
		events.connect (button:getClickedSignalProxy(), function() 
			if not equipment_locked then
				self:show_equipment()
			else
			end
		end)
		vlayout:addWidget (button)

		local crew_min_level = underworld2.BalanceValuesDatabase():get ("crewUnlockLevel", 2)
		local crew_locked = player:getLevel() < crew_min_level

		local button = _create_menu_button (
			crew_locked and "ui/side_menu/ico_sm_crew_locked:filtered" or "ui/side_menu/ico_sm_crew:filtered", 
			L ("KEY_PROFILE_BTN_CREW"),
			underworld2.upgrades_tracker.has_new_items { level = player:getLevel(), category = "skills" } and 1 or 0,
			crew_min_level, crew_locked
		)
		self.menu_buttons.crew = button
		events.connect (button:getClickedSignalProxy(), function() 
			if not crew_locked then
				self:show_crew()
			else
				show_crew_locked()
			end
		end)
		vlayout:addWidget (button)
				
		--for i = 1, 15 do
		button = _create_menu_button ("ui/side_menu/ico_sm_premiumstore:filtered", L"KEY_PROFILE_BANK")
		events.connect (button:getClickedSignalProxy(), function() self:show_store() end)
 		vlayout:addWidget (button)

 		button = _create_menu_button ("ui/side_menu/ico_sm_tophustler:filtered", L ("KEY_PROFILE_BTN_LEADERBOARDS"))
 		events.connect (button:getClickedSignalProxy(), function() self:show_leaderboard() end)
 		vlayout:addWidget (button)

 		button = _create_menu_button ("ui/ico_sm_news:filtered", "Underworld News")
 		events.connect (button:getClickedSignalProxy(), _show_fb_page)
 		vlayout:addWidget (button)

 		button = _create_menu_button ("ui/side_menu/ico_sm_settings:filtered", L ("KEY_PROFILE_BTN_ACCOUNT_SETTINGS"))
 		vlayout:addWidget (button)
 		events.connect (button:getClickedSignalProxy(), function() _show_account_settings() end)
		--end

 		wrapper:setLayout (vlayout)
 		root_layout:addWidget (wrapper)
	end

	local placeholder = ui.Widget()
	placeholder:setVerticalSizePolicy (ui.SizePolicyMaximum)
	root_layout:addWidget (placeholder)

	scroll_widget:setSize (0.8, 1.0)

	self.ui.root_widget = root_widget
	self.ui.root_layout = root_layout
	self.ui.scroll_widget = scroll_widget

	self.stats_signal_conn = events.connect (player:getStatsChangedSignalProxy(), function()
		self:update_stats()
	end)
	self:update_stats()

	return root_widget
end

function view:show_crew()
	local v = ui.core.view.builder (ui.crew_view).build()
	underworld2.ui.stack:push_window (v)
	underworld2.ui.status_panel:hide_profile_info()
end

function view:show_equipment()
	local v = ui.core.view.builder (ui.equipment_view).build()
	underworld2.ui.stack:push_window (v)
	underworld2.ui.status_panel:hide_profile_info()
end

function view:show_store()
	local store = ui.core.view.builder (ui.store_view).build()
	underworld2.ui.stack:push_window (store)
	underworld2.ui.status_panel:hide_profile_info()
end

function view:show_loan_shark()
	local v = ui.core.view.builder (ui.loan_shark).build()
	underworld2.ui.stack:push_window (v)
	underworld2.ui.status_panel:hide_profile_info()
end

function view:show_leaderboard()
	ui.top_hustlers.show()
end

function view:update_stats()
	local player = underworld2.player
	local level 		= underworld2.LevelsDatabase():get (player:getLevel())
	local next_level 	= underworld2.LevelsDatabase():get (player:getLevel() + 1)
	if level:get() and next_level:get() then
		local rp  = player:getRespectPoints() - level:getRespectPoints()
		local max = next_level:getRespectPoints() - level:getRespectPoints()

		self.ui.level_progress:setValue (rp / max)
		self.ui.level_progress:setText (string.format ("%d/%d", rp, max))
	else
		self.ui.level_progress:setValue (1.0)
		self.ui.level_progress:setText ("")
	end
end

function view:destroy()
	--underworld2.profile.remove_delegate (self)
	events.disconnect (underworld2.player:getStatsChangedSignalProxy(), self.stats_signal_conn)
end


