require "ui/text_styles.lua"
require "ui/format.lua"
require "ui/enter_name_view.lua"

local sound_check_view = setmetatable ({ }, { __index = ui.core.window })

local function _create_group (text)
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/frame_grey")

	local vlayout = ui.VerticalLayout()
	vlayout:setItemSpacing (ui.config.vspacing)

	local title = ui.TextLabel()
	title:setTextStyle (ui.text_styles.large.build())
	title:setText (text)
	title:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalRight)
	vlayout:addWidget (title)

	frame:setVerticalSizePolicy (ui.SizePolicyMinimum)
	frame:setLayout (vlayout)

	return frame, vlayout
end

function sound_check_view:build_ui( )

	local W = ui.Background()
	W:setSize (1, 1)
	W:setOrigin (0, 0)
	W:setTexture("ui/card_bg", false)

	local contents = ui.Widget()
	contents:setOrigin (0.0, underworld2.ui.status_panel:get_size().y)
	contents:setSize (1.0, 1.0 - underworld2.ui.status_panel:get_size().y)
	W:addChild (contents)

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)
	contents:setLayout (layout)
		
	get_sound_pool():load ("ui/SFX_Scroll_01.wav")
	get_sound_pool():load ("ui/SFX_Scroll_02.wav")
	get_sound_pool():load ("ui/SFX_Scroll_03.wav")
	get_sound_pool():load ("ui/SFX_Scroll_04.wav")
	get_sound_pool():load ("ui/SFX_Scroll_05.wav")
	get_sound_pool():load ("ui/SFX_Scroll_06.wav")
	get_sound_pool():load ("ui/SFX_Scroll_07.wav")
	
	get_sound_pool():load ("ui/UI_Click_01.wav")
	get_sound_pool():load ("ui/UI_Click_02.wav")
	get_sound_pool():load ("ui/UI_Click_04.wav")
		
	do
	
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)
	
		local btn = ui.TextButton()
		btn:setTexture ("ui/btn_lower_normal:filtered")
		btn:setSound ("ui/UI_Click_01.wav")
		btn:setTextStyle (ui.text_styles.button_regular.build())
		btn:setText ("UI_Click_01")
		hlayout:addWidget (btn)
	
		local btn = ui.TextButton()
		btn:setTexture ("ui/btn_lower_normal:filtered")
		btn:setSound ("ui/UI_Click_02.wav")
		btn:setTextStyle (ui.text_styles.button_regular.build())
		btn:setText ("UI_Click_02")
		hlayout:addWidget (btn)

		layout:addItem (hlayout)

	end
	
	do
	
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)
	
		local btn = ui.TextButton()
		btn:setTexture ("ui/btn_lower_normal:filtered")
		btn:setSound ("ui/UI_Click_03.wav")
		btn:setTextStyle (ui.text_styles.button_regular.build())
		btn:setText ("UI_Click_03")
		hlayout:addWidget (btn)
	
		local btn = ui.TextButton()
		btn:setTexture ("ui/btn_lower_normal:filtered")
		btn:setSound ("ui/UI_Click_04.wav")
		btn:setTextStyle (ui.text_styles.button_regular.build())
		btn:setText ("UI_Click_04")
		hlayout:addWidget (btn)

		layout:addItem (hlayout)

	end
	
	do
		local range_label = ui.TextLabel()
		range_label:setTextStyle (ui.text_styles.button_regular.build())
		range_label:setText ("SFX_Scroll_[01-04]")
		layout:addWidget (range_label)
	
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)
	
		local reel = ui.Reel()
		reel:setTextStyle (ui.text_styles.reel.build())
		reel:setScrollSound ("ui/SFX_Scroll_01.wav")
		reel:setBackgroundTexture ("ui/reel:filtered")
		reel:setLoop (true)
		hlayout:addWidget (reel)
		
		local reel = ui.Reel()
		reel:setTextStyle (ui.text_styles.reel.build())
		reel:setScrollSound ("ui/SFX_Scroll_02.wav")
		reel:setBackgroundTexture ("ui/reel:filtered")
		reel:setLoop (true)
		hlayout:addWidget (reel)
		
		local reel = ui.Reel()
		reel:setTextStyle (ui.text_styles.reel.build())
		reel:setScrollSound ("ui/SFX_Scroll_03.wav")
		reel:setBackgroundTexture ("ui/reel:filtered")
		reel:setLoop (true)
		hlayout:addWidget (reel)
		
		local reel = ui.Reel()
		reel:setTextStyle (ui.text_styles.reel.build())
		reel:setScrollSound ("ui/SFX_Scroll_04.wav")
		reel:setBackgroundTexture ("ui/reel:filtered")
		reel:setLoop (true)
		hlayout:addWidget (reel)
		
		hlayout:addWidget (ui.Widget())
		
		layout:addItem (hlayout)
		
	end
	
	do
		local range_label = ui.TextLabel()
		range_label:setTextStyle (ui.text_styles.button_regular.build())
		range_label:setText ("SFX_Scroll_[05-08]")
		layout:addWidget (range_label)
	
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)
	
		local reel = ui.Reel()
		reel:setTextStyle (ui.text_styles.reel.build())
		reel:setScrollSound ("ui/SFX_Scroll_05.wav")
		reel:setBackgroundTexture ("ui/reel:filtered")
		reel:setLoop (true)
		hlayout:addWidget (reel)
		
		local reel = ui.Reel()
		reel:setTextStyle (ui.text_styles.reel.build())
		reel:setScrollSound ("ui/SFX_Scroll_06.wav")
		reel:setBackgroundTexture ("ui/reel:filtered")
		reel:setLoop (true)
		hlayout:addWidget (reel)
		
		local reel = ui.Reel()
		reel:setTextStyle (ui.text_styles.reel.build())
		reel:setScrollSound ("ui/SFX_Scroll_07.wav")
		reel:setBackgroundTexture ("ui/reel:filtered")
		reel:setLoop (true)
		hlayout:addWidget (reel)
		
		local reel = ui.Reel()
		reel:setTextStyle (ui.text_styles.reel.build())
		reel:setScrollSound ("ui/SFX_Scroll_08.wav")
		reel:setBackgroundTexture ("ui/reel:filtered")
		reel:setLoop (true)
		hlayout:addWidget (reel)
		
		hlayout:addWidget (ui.Widget())
		
		layout:addItem (hlayout)
		
	end
	
	return W
end

function sound_check_view:destroy()

	get_sound_pool():unload ("ui/SFX_Scroll_01.wav")
	get_sound_pool():unload ("ui/SFX_Scroll_02.wav")
	get_sound_pool():unload ("ui/SFX_Scroll_03.wav")
	get_sound_pool():unload ("ui/SFX_Scroll_04.wav")
	get_sound_pool():unload ("ui/SFX_Scroll_05.wav")
	get_sound_pool():unload ("ui/SFX_Scroll_06.wav")
	get_sound_pool():unload ("ui/SFX_Scroll_07.wav")
	
	get_sound_pool():unload ("ui/UI_Click_01.wav")
	get_sound_pool():unload ("ui/UI_Click_02.wav")
	get_sound_pool():unload ("ui/UI_Click_04.wav")

end

function sound_check_view:title()
	return string.format ("Sound Check")
end

function sound_check_view:icon()
end

ui.sound_check_view = ui.sound_check_view or sound_check_view
