require "ui/core/view.lua"
require "ui/text_styles.lua"
require "ui/profile_bar.lua"

require "ui/dealer_stack.lua"

require "ui/modal/heat_modal.lua"

require "ui/modal/levelup.lua"
require "ui/modal/inbox.lua"
require "ui/modal/vault.lua"
require "ui/modal/vault_full.lua"
require "ui/modal/couriers.lua"

require "ui/activity_log_view.lua"

require "ui/modal/item_delivery_confirmation.lua"

require "ui/modal/not_enough_gold.lua"
require "underworld2/upgrades_tracker.lua"

require "ui/widgets/stat_boost.lua"
require "ui/widgets/animated_stat_bar_controller.lua"
require "ui/widgets/police_pane_controller.lua"

require "ui/modal/respect_modal.lua"

local view = setmetatable ( {
	STACK_CONTROLL_MODE = 1,
	BOOKMARKS_CONTROLL_MODE = 2,
}, { __index = ui.core.view } )


local function _show_vault_full()
	local skill = underworld2.SkillsDatabase():getGroup ("SMUGGLERSHIP"):getSkill ("WALLET")
	local level_id = underworld2.player:getSkillLevel (skill:getId())
	local level = skill:getLevel (level_id)
	local view = ui.core.view.builder (ui.vault_full).build (level, level_id)
	ui.core.view_controller:show_modal_view (view)
	view.delegate_builder (
	{
		finished = function (view)
			ui.core.view_controller:close_modal_view (view)
		end
	}).build (view)
end

local function _show_vault()
	local skill = underworld2.SkillsDatabase():getGroup ("SMUGGLERSHIP"):getSkill ("WALLET")
	local level_id = underworld2.player:getSkillLevel (skill:getId())
	local level = skill:getLevel (level_id)
  ui.vault.show (level)
end

local function _show_couriers()
	local v = ui.core.view.builder (ui.couriers).build()
	ui.core.view_controller:show_modal_view (v)
	v.delegate_builder (
	{
		finished = function (v)
			ui.core.view_controller:close_modal_view (v)
		end
	}).build (v)
end

view.show_vault =_show_vault

function view:build_ui()
	self.ui = { new_pending_items = 0 }
	self.show_background = false
	self.animate_background = false
	self.enabled = true

	self.cash_updates_lock = 0
	self.rp_updates_lock = 0

	local root_widget = ui.Widget()
	root_widget:setOrigin (0.0, 0.0)
	root_widget:setSize (1.0, 1.0)
	self.ui.root_widget = root_widget

	local background = ui.StatsBackgroundWidget ("")
	background:setOrigin (0.0, 0.0)
	background:setSize (1.0, 1.0)
	self.ui.background = background
	root_widget:addChild (background)

	local panel = ui.Widget()
	panel:setOrigin (0, 0)
	panel:setSize (1, 1)
	panel:setMargins (ui.config.hspacing, ui.config.hspacing, ui.config.vspacing, 0)
	root_widget:addChild (panel)

	local panel_layout = ui.VerticalLayout()

	local bar_scale_offset = 0
	local bar_width = 0.23
	local cash_bar_width = bar_width
	
	local cash_hint_label = ui.TextLabel()
	cash_hint_label:setText ("Max: 0")
	cash_hint_label:setTextStyle (ui.text_styles.cash_hint.build())
	cash_hint_label:setTextAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalCenter)
	cash_label_height = cash_hint_label:getPreferredSize().y
	self.ui.cash_hint_label = cash_hint_label

	local rank_name_label = ui.TextLabel()
	rank_name_label:setText ("Looser")
	rank_name_label:setTextStyle (ui.text_styles.cash_hint.build())
	rank_name_label:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)
	self.ui.rank_name_label = rank_name_label
	
	do 
		local hlayout = ui.HorizontalLayout()

		local button = ui.NotificationButton()
		button:setAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalTop)
		button:setTexture ("ui/main/btn_topbar_sidemenu:filtered")
		button:setBadgeTexture ("ui/ico_notification:filtered")
		button:setSound ("ui/SFX_Menu_Woosh_Open.wav")
		button:setBadgeTextStyle (ui.text_styles.badge.build())
		button:setBadgeAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalTop)
		button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (button:getClickedSignalProxy(), function () self:show_profile_info() end)
		hlayout:addWidget (button)
		button:setDisplayCount(false)
		self.ui.profile = button

		local button = ui.NotificationButton()
		button:setAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalTop)
		button:setTexture ("ui/main/btn_topbar_back:filtered")
		button:setBadgeTexture ("ui/ico_notification:filtered")
		button:setBadgeTextStyle (ui.text_styles.badge.build())
		button:setBadgeAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalTop)
		button:setHidden (true)
		button:setSound ("ui/UI_Back.wav")
		button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		events.connect (button:getClickedSignalProxy(), function()
			underworld2.ui.stack:pop_window()
		end)
		hlayout:addWidget (button)
		self.ui.back = button
		
		local hspacing = 0

		-- calc layout spacing, to make '+' to be right above another '+' 
		do
			local btn = ui.ImageButton ()
			btn:setTexture ("ui/main/btn_topbar_plus:filtered")
			
			bar_scale_offset = btn:getSize().x * 0.75
			heat_bar_scale_offset = btn:getSize().x
			hspacing = bar_width - btn:getSize().x - button:getPreferredSize().x

			cash_bar_width = cash_bar_width + btn:getSize().x
			
			ui.destroy (btn)
		end

		local police_exposure_controller = ui.widgets.police_pane_controller.create()
		self.heat_drop_conn_id = events.connect (police_exposure_controller.ui.heat_drop:getClickedSignalProxy(), ui.heat_modal.show)
		events.connect (police_exposure_controller.ui.shield_pane:getClickedSignalProxy(), ui.heat_modal.show)
		hlayout:addWidget (police_exposure_controller.ui.root)
		self.police_exposure_controller = police_exposure_controller

		local button = ui.NotificationButton()
		button:setAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalTop)
		button:setTexture ("ui/main/btn_topbar_inbox:filtered")
		button:setBadgeTexture ("ui/ico_notification:filtered")
		button:setBadgeTextStyle (ui.text_styles.badge.build())
		button:setBadgeAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalTop)
		button:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		self.ui.conversations = button
		events.connect (button:getClickedSignalProxy(), function() self:show_inbox() end)
		hlayout:addWidget (button)

		hlayout:setItemSpacing (hspacing)

		panel_layout:addItem (hlayout)
	end

	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing (ui.config.hspacing)


	do
		local vlayout = ui.VerticalLayout()

		local rp_bar_controller = ui.widgets.animated_stat_bar_controller.create 
		{
			ui = "ui/rp_bar.ui",
			width = cash_bar_width,
			bonus_type = "RespectBoostBonus",
			display_value = function (t, max_value) 
				return string.format ("%d%%", math.floor (t * 100 / max_value))
			end
		}
		self.rp_bar_controller = rp_bar_controller
		events.connect (rp_bar_controller.ui.more:getClickedSignalProxy(), function()
			ui.respect_modal.show()
		end)

		local couriers_bar = ui.load_ui ("ui/couriers_bar.ui")
		couriers_bar:setWidth (bar_width)
		self.ui.couriers = runtime.cast_to (couriers_bar:findChild ("text"), "ui::TextLabel")

		local btn_couriers = runtime.cast_to (couriers_bar:findChild ("more"), "ui::Button")

		events.connect (btn_couriers:getClickedSignalProxy(), _show_couriers)

		vlayout:addWidget (rp_bar_controller.ui.highlight)
		vlayout:addWidget (couriers_bar)

		hlayout:addItem (vlayout)
	end

	do

		local title = ui.AnimatedTitle()
		title:setSpacing (ui.config.hspacing)
		title:setAnimationTime (0.125)
		title:setTitleStyle (ui.text_styles.status_panel_window_title.build())
		title:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		self.ui.title = title

		hlayout:addWidget (title)
	end

	do
		local vlayout = ui.VerticalLayout()

		local cash_bar_controller = ui.widgets.animated_stat_bar_controller.create 
		{
			ui = "ui/cash_bar.ui",
			width = cash_bar_width,
			bonus_type = "CashBoostBonus",
			display_value = function (t, max_value)
				return string.format ("%d", math.floor (t))
			end
		}
		self.cash_bar_controller = cash_bar_controller

		events.connect (cash_bar_controller.ui.more:getClickedSignalProxy(), function()
			local player = underworld2.player
			if player:getMoney() >= player:getMaxMoney() then
				_show_vault_full()
			else
				_show_vault()
			end
		end)

		local gold_bar = ui.load_ui ("ui/gold_bar.ui")
		gold_bar:findChild ("bar"):setWidth (bar_width)
		self.ui.gold = runtime.cast_to (gold_bar:findChild ("text"), "ui::TextLabel")

		local btn_shop = runtime.cast_to (gold_bar:findChild ("more"), "ui::Button")

		events.connect (btn_shop:getClickedSignalProxy(), function() self:show_store() end)

		vlayout:addWidget (cash_bar_controller.ui.highlight)
		vlayout:addWidget (gold_bar)

		hlayout:addItem (vlayout)
	end

	panel_layout:addItem (hlayout)
	panel:setLayout (panel_layout)

	self.size = math.vec2 (1, panel:getPreferredSize().y)

	root_widget:setSize (self.size)
	background:setSize (self.size)
	panel:setSize (self.size)

	panel_layout:addWidget (ui.Widget())
	
	return root_widget
end

function view:init()

	self.player_stats_conn_id = events.connect (underworld2.player:getStatsChangedSignalProxy(), function()
		self:on_player_stats_changed()
	end)

	self.player_courier_state_conn_id = events.connect (underworld2.player:getCourierStateChangedSignalProxy(), function()
		self:on_courier_state_changed()
	end)

	self:on_player_stats_changed()
	self:on_courier_state_changed()
end

function view:get_courier_pointer_pos()
	local label = self.ui.couriers
	local origin = label:mapToRoot (label:getOrigin())
	local size = label:getSize()
	return math.vec2 (origin.x + size.x, origin.y + size.y * 0.5)
end

function view:show_inbox()
	ui.inbox.show_news()
end

function view:show_store()
	local window = underworld2.ui.stack:top()

	if window.show_store then 
		window:show_store() 
	else
		underworld2.ui.stack:push_window (ui.store_view.create())
	end
end

function view:on_window_stack_changed (stack)
	self:set_mode (stack:size() == 0 and view.BOOKMARKS_CONTROLL_MODE or view.STACK_CONTROLL_MODE)
	self.ui.background:setOrnamentEnabled (stack:size() > 0)
	self:update_window_title()
end

function view:set_title (icon, title)
	self.ui.title:pushState (icon or "", title or "")
end

function view:update_window_title()
	local window = underworld2.ui.stack:top()
	self.animate_background = self.show_background ~= window:has_title_background()
	self.show_background = window:has_title_background()
	self:set_title (window:icon(), window:title())
end

function view:start_open_profile_animation()
	local elapsed = 0
	local animation_time = 0.15
	local start_x = underworld2.ui.window._pdata:getWindowOffset().x
	local shift_x = self.profile_bar.ui.root_widget:getSize().x - start_x
	self.animation_closure = function (dt)
		elapsed = elapsed + dt
		if elapsed > animation_time then
			elapsed = animation_time
		end
		
		local dx = shift_x * (elapsed / animation_time)
		underworld2.ui.window._pdata:setWindowOffset (start_x + dx, 0.0)

		if elapsed >= animation_time then
			underworld2.ui.window._pdata:setWindowOffset (start_x + shift_x, 0.0)
			return false
		else 
			return true
		end
	end
end

function view:start_close_profile_animation()
	local elapsed = 0
	local animation_time = 0.15
	local shift_x = underworld2.ui.window._pdata:getWindowOffset().x
	self.animation_closure = function (dt)
		elapsed = elapsed + dt
		if elapsed > animation_time then
			elapsed = animation_time
		end
		
		local dx = shift_x - shift_x * (elapsed / animation_time)
		underworld2.ui.window._pdata:setWindowOffset (dx, 0.0)

		if elapsed >= animation_time then
			underworld2.ui.window:close()
			underworld2.ui.window._pdata:setWindowOffset (0, 0)						
			self.profile_bar:finalize()
			self.profile_bar = nil
			underworld2.ui.window._pdata:removeSubview (self.move_gesture)
			self.move_gesture = nil
			return false
		else 
			return true
		end
	end

	get_sound_pool():play ("ui/SFX_Menu_Close.wav")
end

function view:show_profile_info ()
	if not self.profile_bar then
		self.profile_bar = ui.profile_bar.create()
		underworld2.ui.window:open (self.profile_bar)
		local shift_x = self.profile_bar.ui.root_widget:getSize().x

		self:start_open_profile_animation()

		local move_gesture = ui.MoveGestureView (shift_x, 0, 1.0 - shift_x, 1.0)
		underworld2.ui.window._pdata:addSubview (move_gesture)
		
		local path_length = 0
		local last_touch_pos = nil

		local dir = -1
		events.connect (move_gesture:getMovedSignalProxy(), function(to)
			local x = to.x

			if not last_touch_pos then
				last_touch_pos = x
			end

			dir = (x - last_touch_pos) < 0 and -1 or 1

			path_length = path_length + math.abs (x - last_touch_pos)
			last_touch_pos = x

			x = x > 0 and x or 0
			x = x < shift_x and x or shift_x
			underworld2.ui.window._pdata:setWindowOffset (x, 0)
		end)

		events.connect (move_gesture:getFinishedSignalProxy(), function(at)

			if (path_length < ui.config.hspacing) or dir == -1 then
				self:start_close_profile_animation()
			else
				self:start_open_profile_animation()
			end
			last_touch_pos = nil
			path_length = 0
		end)
		self.move_gesture = move_gesture
	end
end

function view:hide_profile_info (skip_animation)
	if self.profile_bar then
		if skip_animation then
			underworld2.ui.window:close()
			underworld2.ui.window._pdata:setWindowOffset (0, 0)						
			self.profile_bar:finalize()
			self.profile_bar = nil
			underworld2.ui.window._pdata:removeSubview (self.move_gesture)
			self.move_gesture = nil
		else
			self:start_close_profile_animation()
		end
	end
end

function view:set_mode (mode)
	if mode == ui.status_panel.STACK_CONTROLL_MODE then
		self.ui.back:setHidden (false)
		self.ui.profile:setHidden (true)
	elseif mode == ui.status_panel.BOOKMARKS_CONTROLL_MODE then
		self.ui.back:setHidden (true)
		self.ui.profile:setHidden (false)
	end
end

function view:add_pending_notifications(count)
	local new_pending_items = self.ui.new_pending_items + count
	if new_pending_items < 0 then new_pending_items = 0 end
	self.ui.new_pending_items = new_pending_items
end

function view:update(dt)
	local player = underworld2.player
	if not player then
		return
	end
	
	if self.animation_closure then
		if not self.animation_closure (dt) then
			self.animation_closure = nil
		end
	end

	self.rp_bar_controller:update (dt)
	self.cash_bar_controller:update (dt)
	self.police_exposure_controller:update (dt)

	local new_conversations = underworld2.inbox.new_conversations() + self.ui.new_pending_items
	if underworld2.inbox and new_conversations ~= self.new_conversations then
		self.new_conversations = new_conversations
		self.ui.conversations:setPendingNotifications (self.new_conversations)
	end
end

function view:destroy()
	if self.player_stats_conn_id then
		events.disconnect (underworld2.player:getStatsChangedSignalProxy(), self.player_stats_conn_id)
		self.player_stats_conn_id = nil
	end

	if self.player_courier_state_conn_id then
		events.disconnect (underworld2.player:getCourierStateChangedSignalProxy(), self.player_courier_state_conn_id)
		self.player_courier_state_conn_id = nil
	end
end

function view:set_heat_delta (delta)
	self.police_exposure_controller:set_heat_delta (delta)
end

function view:on_courier_state_changed()
	local player = underworld2.player
	
	local total_couriers = player:getCouriersCount()
	local available_couriers = total_couriers - player:getBusyCouriersCount()

	self.ui.couriers:setText (available_couriers.."/"..total_couriers)
end

function view:update_cash()
	local player = underworld2.player
	local skill = underworld2.SkillsDatabase():getGroup ("SMUGGLERSHIP"):getSkill ("WALLET")
	local skill_level_id = player:getSkillLevel (skill:getId())
	local skill_level = skill:getLevel (skill_level_id)
	local max_cash = skill_level:getValue()
	self.cash_bar_controller:on_progress_changed (player:getMoney(), max_cash)
	self.cash_bar_controller:set_hint (L ("KET_STATUS_MAX", ui.format.digit (max_cash)))
end

function view:update_rp()
	local S = underworld2.StringsDatabase()
	local player = underworld2.player

	local level_id = player:getLevel()
	local level 		= underworld2.LevelsDatabase():get (level_id)
	local next_level 	= underworld2.LevelsDatabase():get (level_id + 1)
	local target_value  = 1

	if level:get() and next_level:get() then
		local rp  = player:getRespectPoints() - level:getRespectPoints()
		local max = next_level:getRespectPoints() - level:getRespectPoints()
		target_value = rp / max
	end
	self.rp_bar_controller:on_progress_changed (target_value, 1)
	
	local rank = ""
	if level:get() then
		rank = S:get (level:getNameKey())
	end
	self.rp_bar_controller:set_hint (rank)
	self.rp_bar_controller:set_more_hint (tostring (level_id))
end

function view:lock_rp_updates()
	self.rp_updates_lock = self.rp_updates_lock + 1
end

function view:unlock_rp_updates()
	self.rp_updates_lock = self.rp_updates_lock - 1
	if self.rp_updates_lock == 0 then
		self:update_rp()
	end
end

function view:lock_cash_updates()
	self.cash_updates_lock = self.cash_updates_lock + 1
end

function view:unlock_cash_updates()
	self.cash_updates_lock = self.cash_updates_lock - 1
	if self.cash_updates_lock == 0 then
		self:update_cash()
	end
end

function view:on_player_stats_changed()
	if self.ignore_updates then return end

	local player = underworld2.player

	self.ui.profile:setPendingNotifications (underworld2.upgrades_tracker.has_new_items { level = player:getLevel() } and 1 or 0)
		
	self.ui.gold:setText (player:getGold())
	
	local total_couriers = player:getCouriersCount()
	local available_couriers = total_couriers - player:getBusyCouriersCount()

	self.ui.couriers:setText (available_couriers.."/"..total_couriers)

	if self.cash_updates_lock == 0 then
		self:update_cash()
	end
	if self.rp_updates_lock == 0 then
		self:update_rp()
	end
end

function view:on_new_items_list_updated()
	self.ui.profile:setPendingNotifications (underworld2.upgrades_tracker.has_new_items { level = underworld2.player:getLevel() } and 1 or 0)

	local _, skills_count = underworld2.upgrades_tracker.new_items { level = underworld2.player:getLevel(), category = "skills" }
	local _, items_count = underworld2.upgrades_tracker.new_items { level = underworld2.player:getLevel(), category = "items" }

	underworld2.ui.main_view.ui.btn_crew:setNewItemsCount (skills_count)
	underworld2.ui.main_view.ui.btn_equipment:setNewItemsCount (items_count)
end


ui.status_panel = ui.status_panel or view
