--require "ui/core/utils.lua"
require "ui/core/view.lua"
require "ui/config.lua"
require "ui/format.lua"
require "ui/button_colors.lua"

local view = setmetatable (
	{
		RESULT_RENT = 0,
		RESULT_CANCEL = 1 ,
		RESULT_DITCH = 2,
	}, { __index = ui.core.view })
ui.storage_alert_view = view

local builder = ui.core.view.builder(view)

function view.create (storage, courier, rent_price)
	local v = builder.build()

	local commodity_id = courier:getCommodityId()

	local strings = underworld2.StringsDatabase()
	local commodity = underworld2.CommoditiesDatabase():get (commodity_id)

	local used 		= storage:getUsage()
	local available = storage:getAvailable()
	local capacity 	= storage:getCapacity()
	local total_amount = used + courier:getItemsAmount()

	v.ui.item_icon:setItemTexture (ui.config.commodity_icons[commodity_id])
	v.ui.amount_label:setText (tostring (courier:getItemsAmount()))
	v.ui.storage_label:setText (string.format ("%d/%d", total_amount, capacity))
	v.ui.ditch_amount_label:setText (tostring (math.min (used, capacity) + courier:getItemsAmount() - capacity))
	v.ui.rent_button:setPrice (tostring (rent_price))
	v.ui.item_title:setText (strings:get (commodity:getNameKey()))

	return v
end

function view:back()
	self.result = view.RESULT_CANCEL
	self:finalize()
end

function view:build_ui()
	self.ui = { }

	local root = ui.Widget()
	root:setOrigin (0.0, 0.0)
	root:setSize (1.0, 1.0)

	local frame_size = math.vec2 (0.7, 0.6)
	local frame_origin = math.vec2 (0.5 - frame_size.x * 0.5, 0.5 - frame_size.y * 0.5)
	local image_shift_proportion = 0.4

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/window_modal_large:filtered")
	frame:setSize (frame_size.x, frame_size.y)
	frame:setOrigin (frame_origin.x, frame_origin.y)

	local img = ui.Picture()
	img:setTexture ("ui/header_storage")
	local image_size = img:getPreferredSize()
	img:setOrigin (frame_origin.x + (frame_size.x - image_size.x) * 0.5, frame_origin.y - image_size.y * image_shift_proportion)
	img:setSize (image_size)

	local title = ui.TextLabel()
	title:setTextStyle (ui.text_styles.modal_window_title.build())
	title:setText (L "KEY_STORAGE_FULL")
	title:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
	title:setSize (1.0, img:getSize().y)
	title:setOrigin (0.0, img:getOrigin().y)

	local close = ui.ImageButton ()
	close:setTexture ("ui/btn_close:filtered")
	close:setSound ("ui/UI_Back.wav")
	close:setOrigin (frame_origin.x + frame_size.x - close:getSize().x * 0.75, frame_origin.y - close:getSize().y * 0.25)
	events.connect (close:getClickedSignalProxy(), function()
		self.result = view.RESULT_CANCEL
		self:finalize()

		get_sound_pool():play ("ui/UI_Back.wav")
	end)

	root:addChild (frame)
	root:addChild (img)
	root:addChild (title)
	root:addChild (close)

	local vlayout = ui.VerticalLayout()
	vlayout:setItemSpacing (ui.config.vspacing)

	local spacer = ui.Widget()
	spacer:setSize (frame_size.x - frame:getLeftMargin() - frame:getRightMargin(), image_size.y * (1.0 - image_shift_proportion))
	spacer:setVerticalSizePolicy (ui.SizePolicyFixed)
	vlayout:addWidget (spacer)

	local hint0 = ui.TextLabel()
	hint0:setTextStyle (ui.text_styles.small_hl.build())
	hint0:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalTop)
	hint0:setText (L ("KEY_STORAGE_ALERT_HINT0"))
	vlayout:addWidget (hint0)

	vlayout:addWidget (ui.Widget())

	do
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)

		hlayout:addWidget (ui.Widget())

		local item_icon = ui.ItemIcon()
		item_icon:setBackgroundTexture ("ui/item_background:filtered")
		item_icon:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		hlayout:addWidget (item_icon)

		self.ui.item_icon = item_icon

		do
			local vlayout = ui.VerticalLayout()

			do

				local hlayout = ui.HorizontalLayout()

				local ico_box = ui.Picture()
				ico_box:setTexture ("ui/ico_box_small:filtered")
				hlayout:addWidget (ico_box)

				local amount_label = ui.TextLabel()
				amount_label:setText ("100")
				amount_label:setWordWrap (false)
				amount_label:setTextStyle (ui.text_styles.normal_hl.build())
				hlayout:addWidget (amount_label)

				self.ui.amount_label = amount_label

				vlayout:addItem (hlayout)
			end

			local item_title = ui.TextLabel()
			item_title = ui.TextLabel()
			item_title:setWordWrap (false)
			item_title:setText (L "KEY_STORAGE_FAKE_MEDS")
			item_title:setTextStyle (ui.text_styles.deal_hint.build())
			vlayout:addWidget (item_title)

			self.ui.item_title = item_title

			vlayout:setHorizontalPolicy (ui.SizePolicyMinimum)

			hlayout:addItem (vlayout)
		end

		local arrow_ico = ui.Picture()
		arrow_ico:setTexture ("ui/arrow_result")
		arrow_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		hlayout:addWidget (arrow_ico)

		do
			local vlayout = ui.VerticalLayout()

			do

				local hlayout = ui.HorizontalLayout()

				local ico_box = ui.Picture()
				ico_box:setTexture ("ui/ico_box_small:filtered")
				hlayout:addWidget (ico_box)

				local storage_label = ui.TextLabel()
				storage_label:setText ("255/200")
				storage_label:setWordWrap (false)
				storage_label:setTextStyle (ui.text_styles.normal_hl.build())
				hlayout:addWidget (storage_label)

				self.ui.storage_label = storage_label

				vlayout:addItem (hlayout)
			end

			local storage_hint = ui.TextLabel()
			storage_hint = ui.TextLabel()
			storage_hint:setWordWrap (false)
			storage_hint:setText (L "KEY_STORAGE_STORAGE_SPACE")
			storage_hint:setTextStyle (ui.text_styles.deal_hint.build())
			vlayout:addWidget (storage_hint)

			vlayout:setHorizontalPolicy (ui.SizePolicyMinimum)

			hlayout:addItem (vlayout)
		end

		hlayout:addWidget (ui.Widget())

		vlayout:addItem (hlayout)
	end

	vlayout:addWidget (ui.Widget())

	local hint1 = ui.TextLabel()
	hint1:setTextStyle (ui.text_styles.hint_grey_small.build())
	hint1:setText (L ("KEY_STORAGE_ALERT_HINT1"))
	hint1:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
	vlayout:addWidget (hint1)

	do

		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)

		local colors = ui.button_colors.red
		
		local ditch = ui.UniversalButton()
		ditch:setStretch (ui.UniversalButton_StretchHorizontal)
		ditch:setTexture ("ui/button_universal:filtered")
		ditch:setPrimaryColor (ui.Button_ButtonStateNormal, colors.primary0)
		ditch:setSecondaryColor (ui.Button_ButtonStateNormal, colors.secondary0)
		ditch:setPrimaryColor (ui.Button_ButtonStatePressed, colors.primary1)
		ditch:setSecondaryColor (ui.Button_ButtonStatePressed, colors.secondary1)

		events.connect (ditch:getClickedSignalProxy(), function() 
			self.result = view.RESULT_DITCH
			self:finalize()
		end)

		do
			local ditch_layout = ui.HorizontalLayout()
			ditch_layout:setItemSpacing (ui.config.hspacing)

			ditch_layout:addWidget (ui.Widget())

			local ditch_label = ui.TextLabel()
			ditch_label:setTextStyle (ui.text_styles.button_regular.build())
			ditch_label:setWordWrap (false)
			ditch_label:setText (L "KEY_STORATE_DITCH")
			ditch_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			ditch_layout:addWidget (ditch_label)

			local ico_box = ui.Picture()
			ico_box:setTexture ("ui/ico_box_small:filtered")
			ico_box:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			ditch_layout:addWidget (ico_box)

			local ditch_amount_label = ui.TextLabel()
			ditch_amount_label:setTextStyle (ui.text_styles.button_regular.build())
			ditch_amount_label:setWordWrap (false)
			ditch_amount_label:setText ("25")
			ditch_amount_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			ditch_layout:addWidget (ditch_amount_label)
			self.ui.ditch_amount_label = ditch_amount_label

			ditch_layout:addWidget (ui.Widget())



			ditch:setLayout (ditch_layout)
		end

		--[[ditch:setTextStyle (ui.text_styles.button_regular.build())
		ditch:setStateTexture ("ui/btn_premium_side:filtered")
		ditch:setCurrencyIcon ("ui/ico_box_small:filtered")
		ditch:setText ("Ditch")
		ditch:setPrice ("25")
		)]]
		hlayout:addWidget (ditch)

		self.ui.ditch_button = ditch

		local rent = ui.PriceButton()
		rent:setSound ("")
		rent:setTextStyle (ui.text_styles.button_regular.build())
		rent:setStateTexture ("ui/btn_premium_side:filtered")
		rent:setCurrencyIcon ("ui/ico_gold_small:filtered")
		rent:setText (L "KEY_STORAGE_RENT")
		rent:setPrice ("9999")
		events.connect (rent:getClickedSignalProxy(), function() 
			self.result = view.RESULT_RENT
			self:finalize()
		end)
		hlayout:addWidget (rent)

		self.ui.rent_button = rent

		vlayout:addItem (hlayout)
	end

	frame:setLayout (vlayout)

	return root
end
