require "ui/config.lua"
require "ui/text_styles.lua"
require "ui/modal/notice_window.lua"
require "ui/core/library.lua"
require "ui/widgets/store_inapp_item.lua"

require "ui/widgets/store_premium_item_controller.lua"
require "ui/widgets/store_gold_item_controller.lua"
require "ui/widgets/store_offer_item_controller.lua"

require "ui/modal/cash_boost_duplicate.lua"
require "ui/modal/bust_protection_duplicate.lua"
require "ui/modal/rp_boost_duplicate.lua"
require "ui/modal/confirm_avatar.lua"
require "ui/modal/choose_avatar.lua"
require "ui/modal/avatar_updated.lua"

local lui = ui.core.lib

local store_view = setmetatable ({
	MODE_PREMIUMS = 0,
	MODE_INAPPS = 1,

}, { __index = ui.core.window })

ui.store_view = store_view


local handlers = {
}

local function __process_buy_premium ( context )
	local feature = context.feature:get()

	ui.event_view.show()
	underworld2.network.push_back {
		url = "/api/buyPremiumItem.do",
		body =
		{
			name = "request",
			children =
			{
				{ name = "id", text = feature:getId() },
				{ name = "price", text = feature:getPrice() },
				{ name = "activateGift", text = context.certificate_id },
				context.arg
			}
		},
		on_completed = function() 
			ui.event_view.close()
			if context.handler then context.handler() end 
		end,
	}

end

local function __premium_on_cash_boost_requested (ptr, certificate_id, handler)
	local feature = runtime.cast_to (ptr:get(), ptr:getClass():getScopedName())

	if feature:getRequiredLevel() > underworld2.player:getLevel() then return end

	local context = 
	{
		feature = ptr,
		certificate_id = certificate_id,
		handler = handler
	}

	local bonus = underworld2.player:getBonus ("CashBoostBonus")
	if bonus:get() then
		local modifier_bonus = runtime.cast_to (bonus:get(), "underworld2::ModifierBonus")
		local modifier = modifier_bonus:getModifiers()[modifier_bonus:getModifierNameByValue (feature:getModifier())]
		if underworld2.server_time() < modifier.EndTime then
			ui.cash_boost_duplicate.show (modifier.EndTime, feature:getDuration(), nil, function (result)
				if result == ui.cash_boost_duplicate.RESULT_BUY then
					__process_buy_premium (context)
					
				end
			end)
			return
		end
	end
	__process_buy_premium (context)
end

local function __premium_on_respect_boost_requested (ptr, certificate_id, handler)
	local feature = runtime.cast_to (ptr:get(), ptr:getClass():getScopedName())

	if feature:getRequiredLevel() > underworld2.player:getLevel() then return end

	local context =
	{
		feature = ptr,
		certificate_id = certificate_id,
		handler = handler,
	}

	local bonus = underworld2.player:getBonus ("RespectBoostBonus")
	if bonus:get() then
		local modifier_bonus = runtime.cast_to (bonus:get(), "underworld2::ModifierBonus")
		local modifier = modifier_bonus:getModifiers()[modifier_bonus:getModifierNameByValue (feature:getModifier())]
		if underworld2.server_time() < modifier.EndTime then
			ui.rp_boost_duplicate.show (modifier.EndTime, feature:getDuration(), nil, function (result)
				if result == ui.rp_boost_duplicate.RESULT_BUY then
					__process_buy_premium (context)
				end
			end)
			return
		end
	end
	__process_buy_premium (context)
end

local function __premium_on_police_protection_requested (ptr, certificate_id, handler)
	local feature = runtime.cast_to (ptr:get(), ptr:getClass():getScopedName())

	if feature:getRequiredLevel() > underworld2.player:getLevel() then return end

	local bonus = underworld2.player:getBonus ("PoliceProtectionBonus")
	
	local context = 
	{
		feature = ptr,
		certificate_id = certificate_id,
		handler = handler
	}

	if bonus:get() then
		local police_protection_bonus = runtime.cast_to (bonus:get(), "underworld2::PoliceProtectionBonus")
		local expires_at = police_protection_bonus:getEndTime()
		if underworld2.server_time() < expires_at then
			ui.bust_protection_duplicate.show (expires_at, feature:getDuration(), nil, function (result)
				if result == ui.bust_protection_duplicate.RESULT_BUY then
					__process_buy_premium (context)
				end
			end)
			return
		end
	end
	__process_buy_premium (context)
end

local function __premium_on_change_id_requested (ptr, certificate_id, handler)
	local feature = runtime.cast_to (ptr:get(), ptr:getClass():getScopedName())

	if feature:getRequiredLevel() > underworld2.player:getLevel() then return end

	ui.enter_name_view.show (underworld2.player:getNickname(), function (view)
		ui.event_view.show()
		__process_buy_premium 
		{
			feature = ptr,
			certificate_id = certificate_id,
			arg = { name = "param", text = view.name },
			handler = function ()
				ui.event_view.close()
				view:set_completed()
				if handler then handler() end
			end,
		}
	end)
end

local function __premium_on_change_avatar_requested (ptr, certificate_id, handler)
	local feature = runtime.cast_to (ptr:get(), ptr:getClass():getScopedName())

	if feature:getRequiredLevel() > underworld2.player:getLevel() then return end

	ui.choose_avatar.show (function (generated_id)
		ui.event_view.show()

		__process_buy_premium 
		{
			feature = ptr,
			certificate_id = certificate_id,
			arg = { name = "param", text = generated_id },
			handler = function ()
				ui.event_view.close()
				ui.avatar_updated.show()
			end,
		}
	end)
end


local premiums_request_handlers =
{
	["underworld2::premium::CashBoostFeature"] = __premium_on_cash_boost_requested,
	["underworld2::premium::RespectBoostFeature"] = __premium_on_respect_boost_requested,
	["underworld2::premium::PoliceProtectionFeature"] = __premium_on_police_protection_requested,
	["underworld2::premium::ChangeIDFeature"] = __premium_on_change_id_requested,
	["underworld2::premium::ChangeAvatarFeature"] = __premium_on_change_avatar_requested,
}


local function __premiums_handler (self, feature_ptr, certificate_id)
	local feature = feature_ptr:get()
	if not feature then return end

	if not certificate_id then
		local gold_available = underworld2.player:getGold()
		local price = feature:getPrice()
		if price > gold_available then
			ui.not_enough_gold.show (price - gold_available)
			return
		end
	end
	local name = feature_ptr:getClass():getScopedName()
	if premiums_request_handlers[name] then
		premiums_request_handlers[name] (feature_ptr, certificate_id, function()
			local certificates = underworld2.player:getStoreCertificates()
			for i, item in ipairs (self.products) do
				item:on_certificates_updated (certificates)
			end
		end)
	end
end


local updating_view

billing.setup {
	purchase_state_updated = function( store_item_id, state ) 
		if updating_view then
			ui.core.view_controller:close_modal_view (updating_view)
			updating_view:finalize()
			updating_view = nil
		end
	end
}

function store_view.create( restore_callback, mode )
	local view = ui.core.view.builder (store_view).build (mode)
	view.restore_state = restore_callback
	view.category = category
	return view
end

function store_view.ad_finished()
	underworld2.network.ping()
end

local offer_handlers = {}
local ready = false

local function __load_dependencies()
	local vungle_listener = {
		on_ad_unavailable = function(reason)
			ui.notice_window.show(
				L "IAPP_OFFER_VIDEO_UNAVAILABLE",
				L "KEY_NOTICE_WINDOW_TITLE",
				  "ui/characters/ch_runner_modal_3:filtered",
				L "IAPP_OFFER_VIDEO_GUY")

			get_sound_pool():play ("ui/UI_Error.wav")
		end,

		on_video_viewed = function(completed)
			print("[vungle] on_video_viewed")

			underworld2.network.ping()
		end
	}

	ads.vungle.add_listener(vungle_listener)
	ads.tapjoy.set_earned_tap_points_singal("ui.store_view.ad_finished")

	-- offers
	offer_handlers["default"] = function(product)
		underworld2.ui.stack:push_window (ui.store_view.create( nil, store_view.MODE_INAPPS))
	end

	offer_handlers["tapjoy"] = function(product)
		if product.available() then
			ads.tapjoy.show_offer_wall (bundle.tapjoy.gold_currency_id)
		else
			ui.notice_window.show(
				L"IAPP_OFFER_SPECIAL_UNAVAILABLE",
				L"KEY_NOTICE_WINDOW_TITLE",
				"ui/characters/ch_accountant_modal_4:filtered",
				L"IAPP_OFFER_SPECIAL_GUY")
		end
	end

	offer_handlers["vungle"] = function(product)
		if product.available() then
			ads.vungle.play_ad()
		else
			ui.notice_window.show(
				L "IAPP_OFFER_VIDEO_UNAVAILABLE",
				L "KEY_NOTICE_WINDOW_TITLE",
				  "ui/characters/ch_runner_modal_3:filtered",
				L "IAPP_OFFER_VIDEO_GUY")
		end
	end

	offer_handlers["facebook"] = function(product)
		print("[facebook] unavailable")
	end

	handlers[ui.widgets.store_inapp_item.PRODUCT_CATEGORY_OFFER] = function(product)
		local handler = offer_handlers[product.string_id]
		if handler then handler(product) end
	end

	-- inapps
	handlers[ui.widgets.store_inapp_item.PRODUCT_CATEGORY_PAID] = function(product)
		updating_view = ui.event_view.create (L("KEY_PLEASE_WAIT"))
		ui.core.view_controller:show_modal_view (updating_view)	
		billing.purchase( product.store_id ) 
	end

	return true
end

local offers = {
	["VUNGLE"] = {
		string_id 	= "vungle",
		icon_uri 	= "ui/bank/ill_inapp_free_1:filtered",
		description = L"IAPP_OFFER_WATCH_VIDEO",
		type 		= ui.store_offer_item_controller.PRODUCT_TYPE_VIDEO,
		available 	= ads.vungle.available
	},

	["TAP_JOY"] = {
		string_id 	= "tapjoy",
		icon_uri 	= "ui/bank/ill_inapp_free_2:filtered",
		description = L"IAPP_OFFER_COMPLETE_OFFER_CONDITIONS",
		type 		= ui.store_offer_item_controller.PRODUCT_TYPE_SPECIAL,
		available 	= ads.tapjoy.available
	}
}

local function __load_offers (products)
	local db = underworld2.OffersDatabase()
	local count = db:count()
	
	local tinsert = table.insert

	for i = 1, count do
		local offer_id = db:getOffer(i-1)
		local offer = offers[offer_id]
		if offer then
			tinsert(products, ui.store_offer_item_controller.create (offer, handlers[ui.widgets.store_inapp_item.PRODUCT_CATEGORY_OFFER]))
		end
	end
end

local function __load_purchases (products)
	local purchases = underworld2.databases.in_app_purchases
	local count = purchases.count()

	local tinsert = table.insert

	for i=1,count do
		local purchase = purchases.get(i)

		-- taken from the database
		tinsert(products, ui.store_gold_item_controller.create ({
			store_id 		= purchase.store_id,
			icon_uri		= purchase.icon_uri,
			description 	= purchase.description or "",
			value 			= purchase.value,
			tags 			= purchase.tags or {},
			label 			= purchase.label,

			-- limited and gratis (extra value)
			gratis 			= purchase.gratis or 0,
			lifetime 		= purchase.lifetime or 0,

			-- taken from market
			price_string	= purchase.price_string,

			-- client-side conf.
			category 		= ui.widgets.store_inapp_item.PRODUCT_CATEGORY_PAID,
		}, handlers[ui.widgets.store_inapp_item.PRODUCT_CATEGORY_PAID]))
	end
end

local function __load_premiums (self, products)
	local db = underworld2.premium.FeaturesDatabase()
	local features = db:getFeatures()

	local builder = ui.store_premium_item_controller.create

	local handler = function (feature, cert_id)
		__premiums_handler (self, feature, cert_id)
	end

	for i, feature in pairs (features) do
		table.insert (products, builder (feature, handler))
	end
end

function store_view:build_ui (mode)
	if not ready then
		ready = __load_dependencies()
	end

	self.products = {}

	-- inapps
	local tinsert = table.insert
	
	local top = underworld2.ui.status_panel:get_size().y

	local root = lui.widget
	{
		origin 	= math.vec2 (0, 0),
		size 	= math.vec2 (1, 1),
		children = 
		{
			lui.picture
			{
				mode 	= ui.Picture_PlacingModeStretch,
				origin 	= math.vec2 (0, top),
				size 	= math.vec2 (1, 1 - top),
				texture = "ui/solid_background:filtered",
				color   = render.color (35,39,45,255),
				layout  = lui.vertical_layout
				{
					items =
					{
						lui.widget {
							left_margin = ui.config.hspacing,
							top_margin = ui.config.vspacing,
							bottom_margin = ui.config.vspacing,
							right_margin = ui.config.hspacing,
							layout = lui.horizontal_layout 
							{
								items = 
								{
									lui.text_button
									{
										id = "btn_gold",
										texture = "ui/Buttons/btn_glossy_blue:filtered",
										text_style = ui.text_styles.button_regular.build(),
										text = L "KEY_BANK_INAPPS_TAB_TITLE",
										on_clicked = function() self:set_mode (store_view.MODE_INAPPS) end
									},
									lui.text_button
									{
										id = "btn_premium",
										texture = "ui/Buttons/btn_glossy_disabled:filtered",
										text_style = ui.text_styles.button_regular.build(),
										text = L "KEY_BANK_PREMIUMS_TAB_TITLE",
										on_clicked = function () self:set_mode (store_view.MODE_PREMIUMS) end
									}
								},
								spacing = ui.config.hspacing,
							},
							vsize_policy = ui.SizePolicyMinimum
						},
						lui.scroll_widget
						{
							id 					= "scroll",
							scroll_type 		= ui.ScrollTypeVertical,
							scroll_bar_texture 	= "ui/scroll:filtered",
						}
					}
				}
			}
		}
	}

	self.ui = { 
		scroll 		= runtime.cast_to (root:findChild ("scroll"), "ui::ScrollWidget"),
		btn_premium = runtime.cast_to (root:findChild ("btn_premium"), "ui::TextButton"),
		btn_gold 	= runtime.cast_to (root:findChild ("btn_gold"), "ui::TextButton"),
	}
	self.item_padding = self.ui.scroll:getRightMargin()
	self.ui.scroll:setMargins (0, 0, 0, 0)

	self:set_mode (mode or store_view.MODE_INAPPS)
	
	return root
end

local function _clear_children (widget)
	while widget:getChildAt (0) do
		widget:removeChildAt (0)
	end
end

local function _show_inapps (self, layout)

	self.products = { }
	__load_purchases (self.products)
	__load_offers (self.products)
	
	local spacer = ui.Widget()
	spacer:setHeight (ui.config.vspacing * 0.05)
	spacer:setVerticalSizePolicy (ui.SizePolicyFixed)
	layout:addWidget (spacer)

	local item_height 

	for i, item in ipairs (self.products) do
		local widget = item.ui.root
		local height = widget:getPreferredSize().y
		if not item_height or height > item_height then
			item_height = height
		end
	end

	for i, item in ipairs (self.products) do
		local widget = item.ui.root
		widget:setVerticalSizePolicy (ui.SizePolicyFixed)
		widget:setHeight (item_height)
		layout:addWidget (widget)
	end

	layout:setItemSpacing (ui.config.hspacing)
	layout:addWidget(ui.Widget())
end

local function _show_premiums (self, layout)
	self.products = { }
	__load_premiums (self, self.products)

	local spacer = ui.Widget()
	spacer:setHeight (ui.config.vspacing * 0.05)
	spacer:setVerticalSizePolicy (ui.SizePolicyFixed)
	layout:addWidget (spacer)

	local certificates = underworld2.player:getStoreCertificates()

	for i, item in ipairs (self.products) do
		local widget = item.ui.root
		item:on_certificates_updated (certificates)
		layout:addWidget (widget)
	end

	layout:setItemSpacing (ui.config.hspacing)
	layout:addWidget(ui.Widget())
end

function store_view:set_mode (mode)
	if self.mode == mode then
		return
	end
	_clear_children (self.ui.scroll)
	self.mode = mode

	local scroll = self.ui.scroll

	scroll:breakLayout()
	local layout = ui.VerticalLayout()
	scroll:setLayout (layout)
	scroll:setScroll (math.vec2 (0.5, 0.0))

	if (mode == store_view.MODE_INAPPS) then
		self.ui.btn_premium:setTexture ("ui/Buttons/btn_glossy_disabled:filtered")
		self.ui.btn_gold:setTexture ("ui/Buttons/btn_glossy_blue:filtered")
		_show_inapps (self, layout)
	elseif (mode == store_view.MODE_PREMIUMS) then
		self.ui.btn_premium:setTexture ("ui/Buttons/btn_glossy_blue:filtered")
		self.ui.btn_gold:setTexture ("ui/Buttons/btn_glossy_disabled:filtered")
		_show_premiums (self, layout)
	end
end

function store_view:title()
	return L "IAPP_STORE_TITLE"
end

function store_view:icon()
	--return "ui/ico_store"
end

function store_view:show_store()
end

function store_view:update(dt)
	for i = 1, #self.products do
		self.products[i]:update(dt)
	end
end

function store_view:on_destroy()
	if self.restore_state then
		self.restore_state()
		self.restore_state = nil
	end
end