require "ui/config.lua"
require "ui/text_styles.lua"
require "ui/widgets/picture_with_backlight.lua"

local wrapper = { }
ui.widgets.gold_to_cash_widget = wrapper

function wrapper.create (params)
	local frame = ui.Frame()
	frame:setBordersTexture ("ui/frame_modal_light:filtered")
	
	local frame_layout = ui.HorizontalLayout()
	
	local left_margin
	
	do
	
		local layout = ui.VerticalLayout()
		layout:addWidget (ui.Widget())
	
		local ico = ui.widgets.picture_with_backlight.create (params.ico_id, "ui/vault/backlight_vault:filtered")
		layout:addWidget (ico)
		
		left_margin = ico:getWidth()
		
		layout:addWidget (ui.Widget())
		
		frame_layout:addItem (layout)
	end
	
	do
		local vlayout = ui.VerticalLayout()
		vlayout:setItemSpacing (ui.config.vspacing)
		vlayout:setHorizontalPolicy (ui.SizePolicyMaximum)

		do
			local top_layout = ui.HorizontalLayout()
			
			local title_label = ui.TextLabel()
			title_label:setTextStyle (ui.text_styles.normal_hl.build())
			title_label:setText (L ("KEY_FILL_VAULT", params.value))
			title_label:setWordWrap (false)
			top_layout:addWidget (title_label)
			
			local ico = ui.Picture()
			ico:setTexture ("ui/ico_cash_small:filtered")
			ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			top_layout:addWidget (ico)
			
			local value_label = ui.TextLabel()
			value_label:setTextStyle (ui.text_styles.normal_hl.build())
			value_label:setText (string.format ("+%s", ui.format.digit (params.cash)))
			value_label:setWordWrap (false)
			value_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			top_layout:addWidget (value_label)
			
			vlayout:addItem (top_layout)			
		end
		
		local price_button = ui.PriceButton()
		price_button:setStateTexture ("ui/btn_premium:filtered")
		price_button:setTextStyle (ui.text_styles.button_regular.build())
		price_button:setPrice (tostring (params.gold))
		price_button:setText (L "KEY_BUY_CASH")
		price_button:setCurrencyIcon ("ui/ico_pbtn_gold:filtered")
		
		events.connect (price_button:getClickedSignalProxy(), function()
			params.handler (params.wallet_size, params.cash, params.gold)
		end)
		
		vlayout:addWidget (price_button)

		frame_layout:addItem (vlayout)
	end
	
	local margins = { x = frame:getLeftMargin() * 0.5, y = frame:getTopMargin() * 0.5 }
	frame:setMargins (left_margin, margins.x, margins.y, margins.y)
	frame_layout:setItemSpacing (left_margin)
	
	frame:setLayout (frame_layout)
	
	return frame
end



