require "ui/widgets/recent_contact_button.lua"
require "underworld2/blacklist.lua"

local wrapper = { }

local function build_player_massage_layout (text)
	
	local layout = ui.HorizontalLayout()

	local m = ui.PlayerMessageWidget()
	m:setFrameTexture ("ui/messaging/frame_chat_message_your:filtered")
	m:setTailTexture ("ui/messaging/frame_chat_message_your_tip:filtered")
	m:setTextStyle (ui.text_styles.inbox_chat_player_message.build())
	m:appendText (text)
	m:setHorizontalSizePolicy (ui.SizePolicyMinimum)

	layout:addWidget (ui.Widget())
	layout:addWidget (m)

	return layout, m
end

local function build_chat_massage_layout (profile_id, nickname, text, handler)
	
	local layout = ui.HorizontalLayout()

	local profile = underworld2.ProfileDatabase():hasProfile (profile_id) and underworld2.ProfileDatabase():get (profile_id)

	if not profile then
		local ptr = underworld2.Profile (profile_id, true)
		ptr:setNickname (nickname)
		ptr:setAvatarUrl ("")
		profile = underworld2.ProfilePtr (runtime.disown (ptr))
	end

	local avatar = ui.AvatarWidget()
	avatar:setProfile (profile)
	avatar:setAvatarSize (underworld2.Avatar_SizeSmall)
	avatar:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	avatar:setVerticalSizePolicy (ui.SizePolicyMinimum)
	avatar:setTag (profile_id)
	events.connect (avatar:getClickedSignalProxy(), handler)
	layout:addWidget (avatar)

	local m = ui.ChatMessageWidget()
	m:setFrameTexture ("ui/messaging/frame_chat_message_other:filtered")
	m:setTailTexture ("ui/messaging/frame_chat_message_other_tip:filtered")
	m:setTextStyle (ui.text_styles.inbox_chat_message.build())
	m:appendText (text)
	m:setTag (profile_id)
	m:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	events.connect (m:getClickedSignalProxy(), handler)

	if profile then
		m:setNicknameTextStyle (ui.text_styles.inbox_chat_sender_nickname.build())
		m:setNickname (profile:getNickname())
	end

	layout:addWidget (m)
	layout:addWidget (ui.Widget())

	return layout, m
end

function wrapper.create (parent)
	
	local obj = setmetatable ( { ui = { } }, { __index = wrapper } )

	local root = ui.Widget()

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)
	
	local hmargin = 0

	do	
		local scroll = ui.ScrollWidget()
		scroll:setScrollType (ui.ScrollTypeVertical)
		scroll:setScrollBarTexture ("ui/scroll:filtered")
		hmargin = scroll:getLeftMargin()
		scroll:setFrameTexture ("ui/messaging/frame_chat_bg_main:filtered")
		obj.ui.scroll = scroll

		local messages_layout = ui.VerticalLayout()
		messages_layout:setItemSpacing (ui.config.vspacing * 1.5)
		obj.ui.messages_layout = messages_layout
		
		scroll:setLayout (messages_layout)

		layout:addWidget (scroll)
	end

	do 
	
		local footer = ui.Widget()
		footer:setVerticalSizePolicy (ui.SizePolicyMinimum)
		footer:setLeftMargin (hmargin)
		footer:setRightMargin (hmargin)
		footer:setLayout (ui.VerticalLayout())

		local bg = ui.Frame()
		bg:setBordersTexture ("ui/messaging/bar_inbox_navigation_hl:filtered")
		bg:setLeftMargin (ui.config.hspacing)
		bg:setTopMargin (ui.config.vspacing)
		bg:setBottomMargin (ui.config.vspacing)
		bg:setRelativeSize (false)

		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (bg:getRightMargin())

		local message = ui.LineEdit()
		message:setTextFieldTexture ("ui/messaging/btn_send_dark:filtered")
		message:setHintText ( L "KEY_CHAT_PROMT")
		message:setTextStyle (ui.text_styles.inbox_chat_message_hint.build())
		events.connect (message:getEditingFinishedSignalProxy(), function (editor, text)
			local text = message:getText()
			if obj.room_id and #text > 0 then
				message:setText ("")
				social.jabber.muc.send (obj.room_id, text, 
				{
					{
						name = "nick",
						xmlns = "http://jabber.org/protocol/nick",
						children = { { text = underworld2.player:getNickname() } }
					}
				})
			end
		end)
		hlayout:addWidget (message)

		local hint = ui.TextLabel()
		hint:setText (L "KEY_INBOX_SEND")
		hint:setTextStyle (ui.text_styles.button_regular.build())
		hint:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		hint:setWordWrap (false)
		hlayout:addWidget (hint)

		bg:setLayout (hlayout)

		footer:addChild (bg)

		layout:addWidget (footer)
	end

	root:setTopMargin (hmargin * ui.config.screen_ratio)
	root:setBottomMargin (hmargin * ui.config.screen_ratio)
	root:setLayout (layout)

	obj.initialized = false 

	obj.ui.root = root

	obj.chat_btn_handler = function (btn)
		local profile_id = btn:getTag()
		local profile = underworld2.ProfileDatabase():hasProfile (profile_id) and underworld2.ProfileDatabase():get (profile_id)
		if profile then
			parent:push_submenu (ui.widgets.inbox_private_chat.create (parent, profile, L "KEY_INBOX_NAVIGATION_GLOBAL"), true)
		end
	end

	return obj
end

function wrapper:refresh()
	local layout = self.ui.messages_layout
	while layout:getItemAt(0) do
		layout:removeItemAt(0)
	end

	if self.initialized then
		local j = social.jabber
		local room_id = string.format ("global@muc.%s", j.domain_name)
		social.jabber.muc.leave (self.room_id)
		--j:add_delegate (self)
		j.muc.join (room_id.."/"..tostring (underworld2.player:id()))
	end
end

function wrapper:activated()
	if not self.initialized then
		local j = social.jabber
		j:add_delegate (self)
		local room_id = string.format ("global@muc.%s", j.domain_name)
		self.room_id = room_id
		j.muc.join (room_id.."/"..tostring (underworld2.player:id()))
		self.initialized = true
	end
end

function wrapper:auth_state_changed(state)
end

local function get_nickname (stanza)
	local tag = social.jabber.get_stanza_child (stanza, "nick")
	if tag and tag.children and tag.children[1] then
		return tostring (tag.children[1].text) or " "
	end
	return " "
end

function wrapper:stanza_received (stanza)
	if tostring (stanza.name) == "message" and tostring (stanza.type == "groupchat") and social.jabber.jid_without_resource (tostring (stanza.from)) == self.room_id then
		local profile_id = tonumber (social.jabber.resource_from_jid (stanza.from))
		if not profile_id then return end
		if underworld2.inbox.is_blocked(profile_id) then return end

		local player 	= underworld2.player
		local player_id = player:id()

		-- we do not want to play sounds if message comes from the history
		local delay = social.jabber.get_stanza_child (stanza, "delay")
		if not delay then
			if profile_id == player_id then
				get_sound_pool():play ("ui/SFX_Message_Send.wav")
			else
				get_sound_pool():play ("ui/SFX_Message_Get_Soft.wav")
			end
		end

		local max = self.ui.scroll:getMaxScroll().y
		local current = self.ui.scroll:getScroll().y
		local do_scroll_to_bottom = (profile_id == player_id) or (current >= (max - 0.0001))

		local merge_bubbles = profile_id == self.last_sender_id
		if merge_bubbles then
			local text = social.jabber.get_stanza_child (stanza, "body").children[1].text
			self.last_message_widget:appendText (text)
		else
			self.last_sender_id = profile_id
			local text = social.jabber.get_stanza_child (stanza, "body").children[1].text
			if player_id == profile_id then
				local layout, mw = build_player_massage_layout (text)
				self.last_message_widget = mw
				self.ui.messages_layout:addItem (layout)
				self.ui.scroll:setRelativeScroll (math.vec2 (0.5, 1)) 
			else
				local nickname = get_nickname (stanza)
				local layout, mw = build_chat_massage_layout (profile_id, nickname, text, self.chat_btn_handler)
				self.ui.messages_layout:addItem (layout)
				self.last_message_widget = mw
			end
		end

		if do_scroll_to_bottom then
			self.ui.scroll:setRelativeScroll (math.vec2 (0.5, 1))
		end
	end
end

function wrapper:stanza_sent (stanza)
end

function wrapper:on_close()
	if self.initialized then
		social.jabber:remove_delegate (self)
	end
	if self.room_id then
		social.jabber.muc.leave (self.room_id)
	end
end

ui.widgets = ui.widgets or { }
ui.widgets.inbox_global_tab = ui.widgets.inbox_global_tab or wrapper
