require "ui/widgets/recent_contact_button.lua"
require "ui/widgets/inbox_private_chat.lua"
require "underworld2/inbox.lua"
require "social/facebook_sharing.lua"
require "social_ids.lua"

local wrapper = { }
local handlers = {}

local function build_facebook_row(parent, h_margin, login_function)
	local W = ui.Widget()

	local frame = ui.Frame()
	frame:setBordersTexture("ui/top_hustler/frame_tophustler:filtered")
	frame:setHorizontalSizePolicy(ui.SizePolicyFixed)
	frame:setRelativeSize(true)

	local vlayout = ui.VerticalLayout()
	vlayout:setItemSpacing(ui.config.vspacing*2)
	vlayout:addWidget(ui.Widget())
	--vlayout:addWidget(ui.Widget())

	local texts = {
	}

	local handler = function() end

	if underworld2.social_ids.logged_in() then
		texts.hint = L"TOP_HUSTLERS_INVITE_FRIENDS"
		texts.action = L"TOP_HUSTLERS_FACEBOOK_INVITE"

		handler = function()
			social.facebook.web_dialog("apprequests", {message = L"KEY_FB_SHARING_APP_NAME"},
				function(state, payload)
				end)
		end
	else
		texts.hint = L"TOP_HUSTLERS_CONNECT_WITH_FACEBOOK"
		texts.action = L"TOP_HUSTLERS_FACEBOOK_CONNECT"

		handler = function()
			underworld2.social_ids.login("FACEBOOK", function(successful, err)
				if not successful then
					return true
				else
					if login_function then
						login_function(parent)
					end
				end
			end)
		end
	end

	local vvlayout = ui.VerticalLayout()
	vvlayout:setItemSpacing(ui.config.vspacing)
	vvlayout:addWidget(ui.Widget())

	do
		local label = ui.TextLabel()
		label:setText(texts.hint)
		label:setTextStyle(ui.text_styles.normal_hl.build())
		label:setTextAlignment(ui.AlignmentHorizontalCenter)
		label:setHorizontalSizePolicy(ui.SizePolicyFixed)
		label:setWidth(0.8)
		--label:setVerticalSizePolicy(ui.SizePolicyMinimum)
		label:setRowsCount(label:getRowsForWidth(label:getWidth()))

		local hhlayout = ui.HorizontalLayout()
		hhlayout:addWidget(ui.Widget())
		hhlayout:addWidget(label)
		hhlayout:addWidget(ui.Widget())

		vvlayout:addItem(hhlayout)
	end

	do
		local button = underworld2.facebook_sharing.create_button(texts.action, 0, handler)
		button:setHorizontalSizePolicy(ui.SizePolicyFixed)
		button:setWidth(0.6)

		local hhlayout = ui.HorizontalLayout()
		hhlayout:addWidget(ui.Widget())
		hhlayout:addWidget(button)
		hhlayout:addWidget(ui.Widget())

		vvlayout:addItem(hhlayout)
	end

	vvlayout:addWidget(ui.Widget())

	local vvwidget = ui.Widget()
	vvwidget:setLayout(vvlayout)
	vvwidget:setVerticalSizePolicy(ui.SizePolicyMinimum)

	vlayout:addWidget(vvwidget)
	vlayout:addWidget(ui.Widget())
	--vlayout:addWidget(ui.Widget())

	frame:setLayout(vlayout)
	frame:setOrigin(h_margin,0)
	frame:setSize(1-h_margin*2, vvwidget:getPreferredSize().y + 0.06)

	W:addChild(frame)
	W:setHorizontalSizePolicy(ui.SizePolicyFixed)
	W:setVerticalSizePolicy(ui.SizePolicyFixed)
	W:setSize(frame:getSize())

	return {
		widget = W,
		frame = frame
	}
end

local function build_buttons(parent, buttons)
	local navigation_bar = ui.Picture()
	navigation_bar:setMargins(ui.config.hspacing*2,ui.config.hspacing*2,ui.config.hspacing,ui.config.hspacing)
	navigation_bar:setVerticalSizePolicy(ui.SizePolicyMinimum)
	navigation_bar:setTexture("ui/top_hustler/tophustler_nav_bottom_bg:filtered")
	navigation_bar:setPlacingMode(ui.Picture_PlacingModeStretch)

	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing(ui.config.hspacing*2)
	navigation_bar:setLayout(hlayout)

	local button_width = (1 - ui.config.hspacing*4 - ui.config.hspacing*4)/#buttons

	local list = {}

	for i, v in ipairs(buttons) do
		local btn = ui.TextButton()
		btn:setTexture ("ui/top_hustler/btn_glossy_blue:filtered")
		btn:setInactiveTexture ("ui/top_hustler/btn_glossy_disabled:filtered")
		btn:setTextStyle (ui.text_styles.button_regular.build())
		btn:setText (v.name)
		btn:setTag(v.tag)
		btn:setHorizontalSizePolicy(ui.SizePolicyFixed)
		btn:setSize(button_width, 1)
		btn:setInactive(true)

		events.connect(btn:getClickedSignalProxy(), function(sender)
			parent:set_active_category(sender:getTag())
		end)

		hlayout:addWidget(btn)

		list[v.tag] = btn
	end

	navigation_bar:setSize(navigation_bar:getPreferredSize())

	return navigation_bar, list
end

function wrapper:clear()
	local layout = self.ui.messages_layout
	while layout:getItemAt (0) do
		layout:removeItemAt (0)
	end

	self.ui.scroll:setRelativeScroll(math.vec2(0,0))
end

function wrapper:set_active_category(category)
	local buttons = self.ui.buttons
	local active_category = self.active_category
	if category == active_category then return end
	if active_category then
		buttons[active_category]:setInactive(true)
	end

	self:clear()

	buttons[category]:setInactive(false)
	handlers[category](self)

	self.active_category = category
end

function wrapper:set_loading()
	local layout = self.ui.messages_layout

	local waiting_widget = ui.WaitingWidget()
	waiting_widget:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	waiting_widget:setAlignment(ui.AlignmentHorizontalCenter)

	local hlayout = ui.HorizontalLayout()
	hlayout:addWidget(ui.Widget())
	hlayout:addWidget(waiting_widget)
	hlayout:addWidget(ui.Widget())

	local wrapper_widget = ui.Widget()
	wrapper_widget:setHorizontalSizePolicy(ui.SizePolicyMaximum)
	wrapper_widget:setVerticalSizePolicy(ui.SizePolicyMaximum)
	wrapper_widget:setLayout(hlayout)

	--layout:addWidget(ui.Widget())
	layout:addWidget(wrapper_widget)
	layout:setHorizontalPolicy(ui.SizePolicyMaximum)
	--layout:addWidget(ui.Widget())
end

function wrapper:refresh()
	local active_category = self.active_category
	if not active_category then return end
	self:clear()
	if handlers[active_category] then
		handlers[active_category](self)
	end
end

function wrapper:_add_friends(friends)
	self.ui.messages = {}

	local p_id = underworld2.player:id()
	local timestamp = social.jabber.date_time.time
	local contacts = underworld2.inbox.contacts()

	for i, profile in ipairs (friends) do
		if profile and profile:get() then
			local profile_id = profile:id()
			if (not underworld2.inbox.is_blocked(profile_id)) and (profile_id ~= p_id) then
				local c = ui.widgets.recent_contact_button.create (profile, self.message_handler)
				local contact = contacts[profile_id]

				if contact then
					c:set_message (contact.last_msg, timestamp(contact.when), contact.pending)
				end

				self.ui.messages_layout:addWidget (c.ui.root)
				self.ui.messages[profile_id] = c
			end
		end
	end

	local facebook_row = build_facebook_row(self,0, self.refresh)
	self.ui.messages_layout:addWidget(facebook_row.widget)
end

function wrapper.create (parent)
	local obj = setmetatable ( { ui = { messages = { } }, valid = true, friends = {} }, { __index = wrapper } )

	local root = ui.Widget()

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)

	local navigation_bar, buttons = build_buttons(obj, {
		{
			tag = 0,
			name = L"KEY_INBOX_NAVIGATION_CONTACTED"
		},
		{
			tag = 1,
			name = L"KEY_INBOX_NAVIGATION_FRIENDS"
		},
		{
			tag = 2,
			name = L"KEY_INBOX_NAVIGATION_BLOCKED"
		}
	})

	handlers[0] = function(owner)
		local contacts = underworld2.inbox.contacts()
		local sorted_index_map = { }
		local tinsert = table.insert
		for profile_id, info in pairs (contacts) do
			tinsert (sorted_index_map, profile_id)
		end
		local timestamp = social.jabber.date_time.time
		table.sort (sorted_index_map, function (i, j) 
			local a = contacts[i]
			local b = contacts[j]
			return timestamp (a.when) > timestamp (b.when)
		end)

		local now = os.time()

		for i, profile_id in ipairs (sorted_index_map) do
			local profile = underworld2.ProfileDatabase():hasProfile (profile_id) and underworld2.ProfileDatabase():get (profile_id)
			local info = contacts[profile_id]
			if profile and profile:get() and (not underworld2.inbox.is_blocked(profile_id)) then
				local c = ui.widgets.recent_contact_button.create (profile, owner.message_handler)
				local when = timestamp (info.when)

				c:set_message (info.last_msg, when, info.pending)
				owner.ui.messages_layout:addWidget (c.ui.root)
				owner.ui.messages[profile_id] = c
			end
		end
	end

	handlers[1] = function(owner)
		if #owner.friends > 0 then
			owner:_add_friends(owner.friends)
		elseif underworld2.social_ids.logged_in() then
			owner:set_loading()

			social.facebook.graph {
				method = "GET",
				path = "me/friends",
				read_permissions = { "user_friends" },
				parameters = {
				},
				callback = function(operation)
					if not operation:hasFailed() then
						local list = operation:getLuaResponse().data
						local ids = {}
						for i = 1, #list do
							table.insert(ids, list[i].id)
						end

						underworld2.network.friends (#ids, ids, function (friends)
							if not owner.valid then return end
							if owner.active_category ~= 1 then return end

							local contacts = friends.rows
							for i, info in ipairs (contacts) do
								table.insert(owner.friends, info.player)
							end

							owner:clear()
							owner:_add_friends(owner.friends)
						end)
					end
				end
			}
		else
			local facebook_row = build_facebook_row(owner,0, owner.refresh)
			owner.ui.messages_layout:addWidget(facebook_row.widget)
		end
	end

	handlers[2] = function(owner)
		local contacts = underworld2.inbox.blocked_contacts()
		local sorted_index_map = { }
		local tinsert = table.insert
		for profile_id, info in pairs (contacts) do
			tinsert (sorted_index_map, profile_id)
		end

		for i, profile_id in ipairs (sorted_index_map) do
			local profile = underworld2.ProfileDatabase():hasProfile (profile_id) and underworld2.ProfileDatabase():get (profile_id)
			local info = contacts[profile_id]
			if profile and profile:get() then
				local c = ui.widgets.recent_contact_button.create (profile, owner.message_handler)
				owner.ui.messages_layout:addWidget (c.ui.root)
				owner.ui.messages[profile_id] = c
			end
		end
	end

	obj.ui.navigation_bar = navigation_bar
	obj.ui.buttons = buttons

	local hmargin = 0

	do
		local scroll = ui.ScrollWidget()
		scroll:setScrollType (ui.ScrollTypeVertical)
		scroll:setScrollBarTexture ("ui/scroll:filtered")
		scroll:setVerticalSizePolicy(ui.SizePolicyFixed)
		scroll:setHeight(1-navigation_bar:getHeight()-parent.ui.header_size.y-ui.config.vspacing*3)

		hmargin = scroll:getLeftMargin()

		local messages_layout = ui.VerticalLayout()
		messages_layout:setItemSpacing (ui.config.vspacing * 0.5)
		obj.ui.messages_layout = messages_layout

		obj.message_handler = function(button) 
			local profile = underworld2.ProfileDatabase():get (button:getTag())
			parent:push_submenu (ui.widgets.inbox_private_chat.create (parent, profile, L "KEY_INBOX_NAVIGATION_MESSAGES"), true)
		end
		
		scroll:setLayout (messages_layout)
		obj.ui.scroll = scroll

		layout:addWidget (scroll)
	end

	layout:addWidget(navigation_bar)

	root:setTopMargin (hmargin * ui.config.screen_ratio)
	root:setBottomMargin (hmargin * ui.config.screen_ratio)
	root:setLayout (layout)

	obj.ui.root = root

	underworld2.inbox:add_delegate (obj)

	obj:set_active_category(0)

	return obj
end

function wrapper:activated()
	self.valid = true
end

function wrapper:on_close()
	self.valid = false
	underworld2.inbox:remove_delegate (self)
end

function wrapper:conversation_updated (profile_id, conv_info)
	local item = self.ui.messages[profile_id]
	if not item then
		local profile = underworld2.ProfileDatabase():hasProfile (profile_id) and underworld2.ProfileDatabase():get (profile_id)
		if profile and profile:get() then
			item = ui.widgets.recent_contact_button.create (profile, self.message_handler)
			self.ui.messages_layout:insertWidget (item.ui.root, 1)
			self.ui.messages[profile_id] = item
		end
	else
		local index = self.ui.messages_layout:getWidgetIndex (item.ui.root)
		local layout_item = self.ui.messages_layout:takeItemAt (index)
		self.ui.messages_layout:insertItem (layout_item, 1)
	end
	if item then
		item:set_message (conv_info.last_msg, os.difftime (os.time(), social.jabber.date_time.time (conv_info.when)), conv_info.pending)
	end
end

ui.widgets = ui.widgets or { }
ui.widgets.inbox_messages_tab = ui.widgets.inbox_messages_tab or wrapper
