require "ui/format.lua"
require "ui/widgets/news_report.lua"

local wrapper = { }

local function __build(key, value, handler)
	local badge = nil

	local root = ui.ContainerButton()
	root:setTexture ("ui/messaging/bar_news_message:filtered")
	root:setVerticalSizePolicy (ui.SizePolicyMinimum)
	if handler then
		events.connect(root:getClickedSignalProxy(), function()
			if badge then badge:setHidden(true) end
			handler(key, value)
		end)
	end

	local layout = ui.HorizontalLayout()
	layout:setItemSpacing (ui.config.hspacing)
	root:setLayout(layout)

	do
		local container = ui.VerticalLayout()
		local icon = underworld2.RemotePicture()

		--we can set some placeholder
		icon:setTexture("ui/messaging/avatar_news_empty:filtered")

		if value.icon_url and (value.icon_url ~= "") then
			local icon_layout = ui.HorizontalLayout()
			icon:setLayout(icon_layout)

			local waiting_widget = ui.WaitingWidget()
			waiting_widget:setHorizontalSizePolicy (ui.SizePolicyMinimum)

			icon_layout:addWidget(ui.Widget())
			icon_layout:addWidget(waiting_widget)
			icon_layout:addWidget(ui.Widget())

			events.connect(icon:getDownloadedSignalProxy(), function()
				waiting_widget:setHidden(true)
			end)

			icon:download(value.icon_url)
		end

		container:addWidget(icon)
		layout:addItem(container)
	end

	do
		local container = ui.VerticalLayout()
		layout:addItem(container)

		local title_label = ui.RichTextWidget()
		title_label:setDefaultTextStyle (ui.text_styles.inbox_news_item_title.build())
		title_label:setText(value.title)
		title_label:setHorizontalSizePolicy(ui.SizePolicyMaximum)
		container:addWidget(title_label)

		local timestamp_label = ui.TextLabel()
		timestamp_label:setTextStyle (ui.text_styles.inbox_news_item_timestamp.build())
		timestamp_label:setText( ui.format.time_passed(underworld2.server_time(), value.timestamp) )
		timestamp_label:setHorizontalSizePolicy(ui.SizePolicyMaximum)
		container:addWidget(timestamp_label)

		do

			local hhlayout = ui.HorizontalLayout()
			hhlayout:setItemSpacing(ui.config.hspacing)

			local text_label = ui.TextLabel()
			text_label:setTextStyle (ui.text_styles.inbox_news_item_text.build())

			if value.short_description and (value.short_description ~= "") then
				text_label:setText(value.short_description)
			else
				text_label:setText(value.description)
			end

			text_label:setHorizontalSizePolicy(ui.SizePolicyFixed)
			text_label:setElide(true)
			text_label:setWidth(0.7)

			badge = ui.Picture()
			badge:setTexture("ui/loan_shark/ico_warning:filtered")

			badge:setHidden(value.trace >= value.timestamp)

			hhlayout:addWidget(badge)
			hhlayout:addWidget(text_label)
			
			container:addItem(hhlayout)

		end
	end

	do
		local arrow = ui.Picture()
		arrow:setHorizontalSizePolicy(ui.SizePolicyMaximum)
		arrow:setTexture("ui/messaging/arrow_inbox_news:filtered")

		local right_widget = ui.Widget()
		right_widget:setHorizontalSizePolicy(ui.SizePolicyMaximum)

		layout:addWidget(right_widget)
		layout:addWidget(arrow)
	end

	return root
end

function wrapper.create (parent)
	local obj = setmetatable ( { ui = { } }, { __index = wrapper } )

	local root = ui.Widget()
	obj.ui.root = root
	obj.valid = true

	local scroll = ui.ScrollWidget()
	scroll:setScrollType (ui.ScrollTypeVertical)
	scroll:setScrollBarTexture ("ui/scroll:filtered")

	local hmargin = scroll:getLeftMargin()
	root:setTopMargin (hmargin * ui.config.screen_ratio)
	root:setBottomMargin (hmargin * ui.config.screen_ratio)

	--scroll:setFrameTexture ("ui/messaging/frame_chat_bg_main:filtered")
	obj.ui.scroll = scroll

	local news_layout = ui.VerticalLayout()
	news_layout:setItemSpacing (ui.config.vspacing)
	obj.ui.news_layout = news_layout
	scroll:setLayout (news_layout)


	local function __show(key, value)
		if value.trace < value.timestamp then
			underworld2.ui.status_panel:add_pending_notifications(-1)
			if key then
				underworld2.archives.news.mark(key)
			end
		end

		parent:push_submenu (ui.widgets.news_report.create (parent, L"KEY_INBOX_NAVIGATION_NEWS", value), true)
	end

	local function __reload(mark)
		local count = underworld2.archives.news.count()
		for index = 1, count do
			if mark then
				underworld2.archives.news.mark(index)
			end

			news_layout:addWidget(__build(index, underworld2.archives.news.get(index), __show))
		end
	end

	if underworld2.archives.news.count() > 0 then
		__reload()
	else
		underworld2.network.get_news(function()
			if obj.valid then
				__reload(true)
			end
		end)
	end

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)
	root:setLayout (layout)

	layout:addWidget(scroll)


	return obj
end

function wrapper:activated()
	self.valid = true
end

function wrapper:on_close()
	self.valid = false
end

ui.widgets = ui.widgets or { }
ui.widgets.inbox_news_tab = ui.widgets.inbox_news_tab or wrapper
