require "ui/modal/confirm_window.lua"

local wrapper = { }
local timespan = 30 * 60

local function build_player_massage_layout (text)
	
	local layout = ui.HorizontalLayout()

	local m = ui.PlayerMessageWidget()
	m:setFrameTexture ("ui/messaging/frame_chat_message_your:filtered")
	m:setTailTexture ("ui/messaging/frame_chat_message_your_tip:filtered")
	m:setTextStyle (ui.text_styles.inbox_chat_player_message.build())
	m:appendText (text)
	m:setHorizontalSizePolicy (ui.SizePolicyMinimum)

	layout:addWidget (ui.Widget())
	layout:addWidget (m)

	return layout
end

local function build_chat_massage_layout (profile, text)
	
	local layout = ui.HorizontalLayout()

	local avatar = ui.AvatarWidget()
	avatar:setProfile (profile)
	avatar:setAvatarSize (underworld2.Avatar_SizeSmall)
	avatar:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	avatar:setVerticalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (avatar)

	local m = ui.ChatMessageWidget()
	m:setFrameTexture ("ui/messaging/frame_chat_message_other:filtered")
	m:setTailTexture ("ui/messaging/frame_chat_message_other_tip:filtered")
	m:setTextStyle (ui.text_styles.inbox_chat_message.build())
	m:appendText (text)
	m:setHorizontalSizePolicy (ui.SizePolicyMinimum)

	layout:addWidget (m)
	layout:addWidget (ui.Widget())

	return layout
end

local function build_header_layout (profile_ptr, parent)
	
	local profile = profile_ptr:get()

	local btn_builder = function (bg, text, ico)
		local btn = ui.ContainerButton()
		btn:setTexture (bg)
		btn:setTopMargin (ui.config.vspacing)
		btn:setBottomMargin (ui.config.vspacing)

		local btn_layout = ui.VerticalLayout()

		local btn_ico = ui.Picture()
		btn_ico:setTexture (ico)
		btn_layout:addWidget (btn_ico)

		local action_text = ui.TextLabel()
		action_text:setText (text)
		action_text:setTextStyle (ui.text_styles.xsmall.build())
		action_text:setWordWrap (false)
		action_text:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
		btn_layout:addWidget (action_text)

		btn:setLayout (btn_layout)

		return btn
	end

	local layout = ui.HorizontalLayout()
	layout:setItemSpacing (ui.config.hspacing)

	local avatar = ui.AvatarWidget()
	avatar:setAvatarSize (underworld2.Avatar_SizeLarge)
	avatar:setProfile (profile_ptr)
	avatar:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (avatar)

	do

		local wrapper_layout = ui.VerticalLayout()
		wrapper_layout:setItemSpacing (ui.config.vspacing)

		do
			local hhlayout = ui.HorizontalLayout()

			local title_label = ui.TextLabel()
			title_label:setTextStyle (ui.text_styles.inbox_news_item_title.build())
			title_label:setText(profile:getNickname())
			--title_label:setHorizontalSizePolicy(ui.SizePolicyMaximum)

			hhlayout:addWidget(title_label)
			wrapper_layout:addItem(hhlayout)
		end

		do

			local level_layout = ui.HorizontalLayout()
			level_layout:setItemSpacing (ui.config.hspacing)
			level_layout:setVerticalPolicy (ui.SizePolicyMinimum)

			local badge = ui.NotificationBadge()
	 		badge:setBadgeTexture ("ui/ico_level:filtered")
	 		badge:setPendingNotifications (profile:getLevel())
	 		badge:setBadgeTextStyle (ui.text_styles.badge.build())
	 		badge:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	 		level_layout:addWidget (badge)

	 		local strings = underworld2.StringsDatabase()
	 		local level = underworld2.LevelsDatabase():get (profile:getLevel())

	 		local level_desc = ui.TextLabel()
	 		level_desc:setTextStyle (ui.text_styles.small_hl.build())
			level_desc:setText (level:get() and strings:get (level:getNameKey()) or "")
			level_layout:addWidget (level_desc)

		 	wrapper_layout:addItem (level_layout)
			wrapper_layout:setHorizontalPolicy(ui.SizePolicyMaximum)
	 	end

		local sep = ui.Widget()
		sep:setVerticalSizePolicy(ui.SizePolicyMaximum)
	 	wrapper_layout:addWidget (sep)

	 	layout:addItem (wrapper_layout)
	end

	do
		local wrapper_layout = ui.VerticalLayout()
		wrapper_layout:setItemSpacing (ui.config.vspacing)

		-- block button
		local btn_layout = ui.HorizontalLayout()
		btn_layout:setItemSpacing (ui.config.hspacing)

		local block_btn = ui.ContainerButton()
		block_btn:setTexture ("ui/messaging/btn_red_small:filtered")
		local height = block_btn:getPreferredSize().y
		block_btn:setVerticalSizePolicy (ui.SizePolicyMinimum)

		btn_layout:addWidget(ui.Widget())

		local ico = ui.Picture()
		ico:setTexture ("ui/messaging/ico_block_34:filtered")
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		btn_layout:addWidget (ico)


		local text_label = ui.TextLabel()
		text_label:setTextStyle (ui.text_styles.button_regular.build())
		text_label:setWordWrap (false)
		text_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
		btn_layout:addWidget (text_label)

		btn_layout:addWidget(ui.Widget())
		block_btn:setLayout (btn_layout)

		if underworld2.inbox.is_blocked(profile:id()) then
			text_label:setText( L"KEY_INBOX_UNBLOCK")
		else
			text_label:setText (L "KEY_INBOX_BLOCK")
		end

		wrapper_layout:addWidget (block_btn)

		events.connect (block_btn:getClickedSignalProxy(), function()
			if underworld2.inbox.is_blocked(profile:id()) then
				underworld2.inbox.remove_from_blacklist(profile:id())
				text_label:setText (L "KEY_INBOX_BLOCK")
				parent:refresh()
			else
				local text = L("KEY_BLOCK_TEXT", profile:getNickname())
				local title = L("KEY_BLOCK_TITLE", profile:getNickname())
				local wnd = ui.confirm_window.create(title, text, L"KEY_BLOCK_CONFIRM", L"KEY_BLOCK_CANCEL")

				ui.core.view.delegate_builder(
				{
					finished = function (view)
						if view.result == ui.confirm_window.RESULT_CONFIRM then
							print(profile:id() .. " BLOCKED!")
							underworld2.inbox.add_to_blacklist(profile:id())

							text_label:setText( L"KEY_INBOX_UNBLOCK")

							parent:refresh()
						end

						ui.core.view_controller:close_modal_view (view)
					end
				}).build (wnd)

				ui.core.view_controller:show_modal_view (wnd)
			end
		end)

		local sep = ui.Widget()
		sep:setVerticalSizePolicy(ui.SizePolicyMaximum)
	 	wrapper_layout:addWidget (sep)

	 	layout:addItem (wrapper_layout)
	end

	layout:setVerticalPolicy (ui.SizePolicyMinimum)

	return layout

end

function __build_title_widget (profile)
	local w = ui.Widget()

	local vlayout = ui.VerticalLayout()

	local hlayout = ui.HorizontalLayout()

	hlayout:addWidget (ui.Widget())
	local nickname = ui.TextLabel()
	nickname:setTextStyle (ui.text_styles.small_hl.build())
	nickname:setText (L "KEY_INBOX_PROFILE")
	nickname:setWordWrap (false)
	hlayout:addWidget (nickname)
	hlayout:addWidget (ui.Widget())
	vlayout:addItem (hlayout)

	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing (ui.config.hspacing)

	hlayout:addWidget (ui.Widget())

	local status_ico = ui.Picture()
	status_ico:setTexture ("ui/messaging/ico_offline_inbox:filtered")
	status_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hlayout:addWidget (status_ico)

	local status_label = ui.TextLabel()
	status_label:setTextStyle (ui.text_styles.hint_grey_small.build())
	status_label:setText (L "KEY_INBOX_OFFLINE")
	status_label:setWordWrap (false)
	status_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hlayout:addWidget (status_label)

	status_ico:setHidden (true)
	status_label:setHidden (true)

	hlayout:addWidget (ui.Widget())

	--vlayout:addItem (hlayout)

	w:setLayout (vlayout)

	return w
end

function build_blocked_layout()
	local layout = ui.VerticalLayout()
	layout:setItemSpacing(ui.config.vspacing)

	local hhlayout = ui.HorizontalLayout()

	local ico = ui.Picture()
	ico:setTexture ("ui/messaging/ico_block_256:filtered")
	ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)

	hhlayout:addWidget(ui.Widget())
	hhlayout:addWidget(ico)
	hhlayout:addWidget(ui.Widget())

	local hint = ui.TextLabel()
	hint:setText (L "KEY_INBOX_BLOCKED_HINT_1")
	hint:setTextStyle (ui.text_styles.button_regular.build())
	--hint:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hint:setTextAlignment(ui.AlignmentHorizontalCenter)
	hint:setWordWrap (true)

	local hint2 = ui.TextLabel()
	hint2:setText (L "KEY_INBOX_BLOCKED_HINT_2")
	hint2:setTextStyle (ui.text_styles.normal_hl.build())
	--hint:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hint2:setTextAlignment(ui.AlignmentHorizontalCenter)
	hint2:setWordWrap (true)

	--layout:addWidget(ui.Widget())
	layout:addWidget(ui.Widget())
	layout:addItem(hhlayout)
	layout:addWidget(hint)
	layout:addWidget(hint2)
	layout:addWidget(ui.Widget())

	return layout, ico
end

function wrapper.create (parent, profile_ptr, parent_menu_name)
	local obj = setmetatable ({ ui = { } }, { __index = wrapper })

	local profile = profile_ptr:get()

	obj.profile_ptr = profile_ptr
	obj.profile_id = profile:id()
	obj.jid = string.format ("%s@%s", profile:id(), social.jabber.domain_name)

	local root = ui.Widget()
	obj.widget = root

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)

	local hmargin = 0

	local scroll = ui.ScrollWidget()
	scroll:setScrollType (ui.ScrollTypeVertical)
	scroll:setScrollBarTexture ("ui/scroll:filtered")
	hmargin = scroll:getLeftMargin()
	scroll:setFrameTexture ("ui/messaging/frame_chat_bg_main:filtered")
	obj.ui.scroll = scroll

	do
		local w = ui.Decorator()
		w:setTexture ("ui/messaging/bar_inbox_navigation:filtered", ui.Decorator_StyleHStretcherWithBorders)
		w:setVerticalSizePolicy (ui.SizePolicyMinimum)
		bg_size = math.vec2 (1, w:getPreferredSize().y)
		
		if parent_menu_name then
			local back = ui.TextButton()
			back:setTexture ("ui/messaging/btn_back_name:filtered")
			local back_size = back:getPreferredSize()
			back:setOrigin (hmargin, (bg_size.y - back_size.y) * 0.5)
			back:setTextStyle (ui.text_styles.small_hl.build())
			back:setText (parent_menu_name)
			back:setSize (back_size)
			w:addChild (back)

			events.connect (back:getClickedSignalProxy(), function() 
				parent:pop_submenu()
			end)
		end

		local title = __build_title_widget (profile)
		local title_size = title:getPreferredSize()
		title:setOrigin ((bg_size - title_size) * 0.5)
		title:setSize (title_size)
		w:addChild (title)

		layout:addWidget (w)
	end

	local messages_layout = ui.VerticalLayout()
	messages_layout:setItemSpacing (ui.config.vspacing * 1.5)
	obj.ui.messages_layout = messages_layout

	scroll:setLayout (messages_layout)

	local header_layout = build_header_layout (profile_ptr, obj)
	local header = ui.Widget()
	header:setMargins (hmargin, hmargin, 0, 0)
	header:setLayout (header_layout)
	header:setVerticalSizePolicy (ui.SizePolicyMinimum)

	layout:addWidget (header)
	layout:addWidget (scroll)

	do 
	
		local footer = ui.Widget()
		footer:setVerticalSizePolicy (ui.SizePolicyMinimum)
		footer:setLeftMargin (hmargin)
		footer:setRightMargin (hmargin)
		footer:setLayout (ui.VerticalLayout())

		local bg = ui.Frame()
		bg:setBordersTexture ("ui/messaging/bar_inbox_navigation_hl:filtered")
		bg:setLeftMargin (ui.config.hspacing)
		bg:setTopMargin (ui.config.vspacing)
		bg:setBottomMargin (ui.config.vspacing)
		bg:setRelativeSize (false)

		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (bg:getRightMargin())

		local message = ui.LineEdit()
		message:setTextFieldTexture ("ui/messaging/btn_send_dark:filtered")
		message:setHintText (L "KEY_CHAT_PROMT")
		message:setTextStyle (ui.text_styles.inbox_chat_message_hint.build())
		events.connect (message:getEditingFinishedSignalProxy(), function (editor, text)
			local text = message:getText()
			message:setText ("")
			local player = underworld2.player
			if #text > 0 then
				social.jabber.send_message (obj.jid, text,
				{
					{
						name = "profile",
						xmlns = "uw2:profile_data",
						nickname = player:getNickname(),
						level = tostring (player:getLevel()),
						avatarUrl = player:getAvatarUrl()
					}
				})
			end
		end)
		hlayout:addWidget (message)
		obj.ui.input = message

		local hint = ui.TextLabel()
		hint:setText (L "KEY_INBOX_SEND")
		hint:setTextStyle (ui.text_styles.button_regular.build())
		hint:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		hint:setWordWrap (false)
		hlayout:addWidget (hint)

		bg:setLayout (hlayout)

		footer:addChild (bg)

		layout:addWidget (footer)
	end

	root:setTopMargin (hmargin * ui.config.screen_ratio)
	root:setBottomMargin (hmargin * ui.config.screen_ratio)

	root:setLayout (layout)

	obj:refresh()

	underworld2.inbox.silent = true


	return obj

end

function wrapper:refresh()
	social.jabber:remove_delegate (self)

	local layout = self.ui.messages_layout
	while layout:getItemAt (0) do
		layout:removeItemAt (0)
	end

	self.ui.blocked = nil

	if underworld2.inbox.is_blocked(self.profile_id) then
		local l, ico = build_blocked_layout()

		layout:addItem(l)

		self.ui.blocked = ico
		self.ui.input:setEnabled(false)
		self.ui.scroll:setRelativeScroll (math.vec2 (0.5, 0))
	else
		social.jabber:add_delegate (self)

		local conv = social.jabber.archive.get (self.jid)

		if conv then
			local last = #conv
			local first = math.max (1, #conv - 50 + 1)
			

			for i=first, last do
				local msg 	= conv[i]
				local text 	= msg.text
				local when 	= msg.ts or "1970-01-01T00:00:00Z"
				if msg.sid then
					self:add_chat_message (text, when)
				else
					self:add_player_message (text, when)
				end
			end
		end

		self.ui.input:setEnabled(true)
		self.ui.scroll:setRelativeScroll (math.vec2 (0.5, 1))
	end
end

function wrapper:format_time(timestamp)
	-- looks like TZ is shifted, not sure though if it's a good idea to hardcode
	local current_time = os.time() - 3600
	local diff = current_time - timestamp
	local format = ""

	if diff > 365 * 24 * 60 * 60 then
		format = "%Y %b %d"
	elseif diff > 7 * 24 * 60 * 60 then
		format = "%a, %b %d, %H:%M"
	elseif diff > 2 * 24 * 60 * 60 then
		format = "%a %H:%M"
	elseif diff > 24 * 60 * 60 then
		format = L"KEY_DATE_FORMAT_YESTERDAY"
	else
		format = L"KEY_DATE_FORMAT_TODAY"
	end

	return format
end

function wrapper:add_timestamp (ts)
	local label = ui.TextLabel()

	local timestamp = social.jabber.date_time.time (ts)
	local display_value = os.date (self:format_time(timestamp), timestamp)

	label:setTextStyle (ui.text_styles.inbox_chat_timestamp.build())
	label:setText (display_value)
	label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
	label:setVerticalSizePolicy (ui.SizePolicyMinimum)
	self.ui.messages_layout:addWidget (label)
end

function wrapper:timestamp_available(current_timestamp)
	if not self.last_timestamp then return true end
	return (current_timestamp - self.last_timestamp) > timespan
end

function wrapper:add_player_message (text, ts)
	local current_timestamp = social.jabber.date_time.time (ts)

	if self:timestamp_available(current_timestamp) then
		self:add_timestamp (ts)
	end
	self.last_message_from_player = true
	self.last_timestamp = current_timestamp

	self.ui.messages_layout:addItem (build_player_massage_layout (text))
	self.ui.scroll:setRelativeScroll (math.vec2 (0.5, 1))
end

function wrapper:add_chat_message (text, ts)
	local current_timestamp = social.jabber.date_time.time (ts)
	if self:timestamp_available(current_timestamp) then
		self:add_timestamp (ts)
	end

	self.last_timestamp = social.jabber.date_time.time (ts)

	self.last_message_from_player = false
	local max = self.ui.scroll:getMaxScroll().y
	local current = self.ui.scroll:getScroll().y

	self.ui.messages_layout:addItem (build_chat_massage_layout (self.profile_ptr, text))

	if (current >= (max - 0.0001)) then
		self.ui.scroll:setRelativeScroll (math.vec2 (0.5, 1))
	end
end

function wrapper:auth_state_changed()
end

function get_profile (stanza)
	local tag = social.jabber.get_stanza_child (stanza, "profile")
end

function wrapper:stanza_received (stanza)
	if tostring (stanza.name) == "message" and tostring (stanza.type == "chat") and string.find (tostring (stanza.from), self.jid, 1, true) then

		local profile_id = tonumber (social.jabber.resource_from_jid (stanza.from))
		if profile_id and underworld2.inbox.is_blocked(profile_id) then return end

		-- we do not want to play sounds if message comes from the history
		local delay = social.jabber.get_stanza_child (stanza, "delay")
		if not delay then
			get_sound_pool():play ("ui/SFX_Message_Get_Soft.wav")
		end

		local text = social.jabber.get_stanza_child (stanza, "body").children[1].text
		self:add_chat_message (text, social.jabber.date_time.delay_stamp())
	end
end

function wrapper:stanza_sent (stanza)
	if tostring (stanza.name) == "message" and tostring (stanza.type == "chat") and string.find (tostring (stanza.to), self.jid, 1, true) then
		local text = social.jabber.get_stanza_child (stanza, "body").children[1].text
		self:add_player_message (text, social.jabber.date_time.delay_stamp())
		get_sound_pool():play ("ui/SFX_Message_Send.wav")
	end
end

function  wrapper:on_close()
	underworld2.inbox.mark_as_read (self.profile_id)
	underworld2.inbox.silent = false
	social.jabber:remove_delegate (self)
end

ui.widgets = ui.widgets or { }
ui.widgets.inbox_private_chat = ui.widgets.inbox_private_chat or wrapper
