require "ui/config.lua"
require "ui/text_styles.lua"

require "underworld2/equipment_view_state.lua"

local item_widget = { 
	
	STATE_NOT_PURCHASED = 0,
	STATE_IN_DELIVERY = 1,
	STATE_OWNED = 2,
	STATE_EQUIPED = 3,
}

local effect_icons_map = 
{
	[underworld2.StatTypeStorage] 		= { "ui/equipment/ico_32_storage:filtered" },
	[underworld2.StatTypeHeatScale] 	= { "ui/equipment/ico_32_heat:filtered" },
	[underworld2.StatTypeHeatDrop] 		= { "ui/equipment/ico_32_cooldown:filtered" },
	[underworld2.StatTypeDeliveryTime] 	= { "ui/equipment/ico_32_time:filtered" },
	[underworld2.StatTypeLabSales] 		= { 
		ui.config.commodity_small_icons[1],
		ui.config.commodity_small_icons[2],
		ui.config.commodity_small_icons[3],
		ui.config.commodity_small_icons[4],
		ui.config.commodity_small_icons[5],
		ui.config.commodity_small_icons[6],
		ui.config.commodity_small_icons[7],
		ui.config.commodity_small_icons[8],
	},
	[underworld2.StatTypeSafetyBonus] 	= {
		ui.config.commodity_small_icons[1],
		ui.config.commodity_small_icons[2],
		ui.config.commodity_small_icons[3],
		ui.config.commodity_small_icons[4],
		ui.config.commodity_small_icons[5],
		ui.config.commodity_small_icons[6],
		ui.config.commodity_small_icons[7],
		ui.config.commodity_small_icons[8],
	}
}

local function level_value(effect_index) return effect_index:getLevel() end

local effect_db

local function percent_value(effect_index, commodity)
	if not effect_db then effect_db = underworld2.EffectsDatabase() end
	local effect       = effect_db:get (effect_index:getStatId ())
	local effect_level = effect:getLevel (effect_index:getLevel ())
	return math.round (100 * effect_level:getValue (1))
end

local function make_percent_value(commodity) 
	return function(effect_index) return percent_value(effect_index, commodity) end
end

local get_effect_value = {
	[underworld2.StatTypeStorage] 		= {level_value},
	[underworld2.StatTypeHeatScale] 	= {level_value},
	[underworld2.StatTypeHeatDrop] 		= {level_value},
	[underworld2.StatTypeDeliveryTime] 	= {level_value},
	[underworld2.StatTypeLabSales] 		= {
		make_percent_value(1),
		make_percent_value(2),
		make_percent_value(3),
		make_percent_value(4),
		make_percent_value(5),
		make_percent_value(6),
		make_percent_value(7),
		make_percent_value(8),
	},
	[underworld2.StatTypeSafetyBonus] 	= {
		make_percent_value(1),
		make_percent_value(2),
		make_percent_value(3),
		make_percent_value(4),
		make_percent_value(5),
		make_percent_value(6),
		make_percent_value(7),
		make_percent_value(8),
	}
}

local get_effect_format = {
	[underworld2.StatTypeStorage] 		= {"KEY_EQUIPEMENT_LEVEL"},
	[underworld2.StatTypeHeatScale] 	= {"KEY_EQUIPEMENT_LEVEL"},
	[underworld2.StatTypeHeatDrop] 		= {"KEY_EQUIPEMENT_LEVEL"},
	[underworld2.StatTypeDeliveryTime] 	= {"KEY_EQUIPEMENT_LEVEL"},
	[underworld2.StatTypeLabSales] 		= {
		"KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT",
		"KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT"
	},
	[underworld2.StatTypeSafetyBonus] 	= {
		"KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT",
		"KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT", "KEY_EQUIPEMENT_PERCENT"
	}
}

local get_effect_diff_format = {
	[underworld2.StatTypeStorage] 		= {"KEY_EQUIPEMENT_DIFF_LEVEL"},
	[underworld2.StatTypeHeatScale] 	= {"KEY_EQUIPEMENT_DIFF_LEVEL"},
	[underworld2.StatTypeHeatDrop] 		= {"KEY_EQUIPEMENT_DIFF_LEVEL"},
	[underworld2.StatTypeDeliveryTime] 	= {"KEY_EQUIPEMENT_DIFF_LEVEL"},
	[underworld2.StatTypeLabSales] 		= {
		"KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT",
		"KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT"
	},
	[underworld2.StatTypeSafetyBonus] 	= {
		"KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT",
		"KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT", "KEY_EQUIPEMENT_DIFF_PERCENT"
	}
}


item_widget.btn_colors =
{
	sell = {
		primary0 = render.color (255, 190, 56, 255),
		secondary0 = render.color (209, 97, 10, 255),
		primary1 = render.color (255, 190, 56, 255),
		secondary1 = render.color (209, 97, 10, 255),
	},
	buy_locked = {
		primary0    = render.color (145, 138, 133),
		secondary0  = render.color (102, 97, 94),
		primary1    = render.color (145, 138, 133),
		secondary1  = render.color (102, 97, 94),
	},
	buy = {
		primary0   = render.color (38, 228, 96, 255),
		secondary0 = render.color (15, 168, 23, 255),
		primary1   = render.color (38, 228, 96, 255),
		secondary1 = render.color (15, 168, 23, 255),
	},
	equip = {
		primary0 = render.color (24, 159, 223, 255),
		secondary0 = render.color (29, 127, 189, 255),
		primary1 = render.color (24, 159, 223, 255),
		secondary1 = render.color (29, 127, 189, 255),
	},
	unequip = {
		primary0 = render.color (24, 159, 223, 255),
		secondary0 = render.color (29, 127, 189, 255),
		primary1 = render.color (24, 159, 223, 255),
		secondary1 = render.color (29, 127, 189, 255),
	},
}

function item_widget.set_button_colors( btn, colors )
	btn:setPrimaryColor (ui.Button_ButtonStateNormal, colors.primary0)
	btn:setSecondaryColor (ui.Button_ButtonStateNormal, colors.secondary0)
	btn:setPrimaryColor (ui.Button_ButtonStatePressed, colors.primary1)
	btn:setSecondaryColor (ui.Button_ButtonStatePressed, colors.secondary1)
end

function item_widget.create_btn (text, colors)
	local btn = ui.UniversalButton()
	btn:setTexture ("ui/equipment/btn_64_universal:filtered")
	btn:setStretch (ui.UniversalButton_StretchHorizontal)
	item_widget.set_button_colors(btn, colors)

	local layout = ui.VerticalLayout()

	local text_label = ui.TextLabel()
	text_label:setTextStyle (ui.text_styles.button_regular.build())
	text_label:setText (text)
	text_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
	layout:addWidget (text_label)

	btn:setLayout (layout)

	return btn
end

function item_widget.create_price_btn (text, price, ico_id, colors)
	local btn = ui.UniversalButton()
	btn:setTexture ("ui/equipment/btn_64_universal:filtered")
	btn:setStretch (ui.UniversalButton_StretchHorizontal)
	item_widget.set_button_colors(btn, colors)

	local layout = ui.HorizontalLayout()
	layout:setItemSpacing (ui.config.hspacing)

	layout:addWidget (ui.Widget())

	local text_label = ui.TextLabel()
	text_label:setTextStyle (ui.text_styles.button_regular.build())
	text_label:setText (text)
	text_label:setWordWrap (false)
	text_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (text_label)

	local ico = ui.Picture()
	ico:setTexture (ico_id)
	ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (ico)

	local price_label = ui.TextLabel()
	price_label:setTextStyle (ui.text_styles.button_regular.build())
	price_label:setText (ui.format.digit (price))
	price_label:setWordWrap (false)
	price_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (price_label)

	layout:addWidget (ui.Widget())

	btn:setLayout (layout)

	return btn, price_label
end

function item_widget.create (player, slot, item, handlers, tag, width)
	local obj = setmetatable ({ item = item, slot = slot, widgets = { stats = {} } }, { __index = item_widget } )

	local frame = ui.Frame()
	frame:setVerticalSizePolicy (ui.SizePolicyMinimum)
	frame:setBordersTexture ("ui/equipment/frame_28dark:filtered")
	
	local frame_margins = { x = frame:getLeftMargin() * 0.5, y = frame:getTopMargin() * 0.5 }
	frame:setMargins (frame_margins.x, frame_margins.x, frame_margins.y, frame_margins.y)
	
	local frame_layout = ui.HorizontalLayout()
	frame_layout:setItemSpacing (ui.config.hspacing)

	local item_ico = ui.ItemWidget()
	obj.widgets.item_ico = item_ico
	
	item_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	item_ico:setWidth ((width - frame:getLeftMargin() - frame:getRightMargin() - ui.config.hspacing) / 3.0)
	item_ico:setHorizontalSizePolicy (ui.SizePolicyFixed)
	item_ico:setLockedTextStyle (ui.text_styles.locked.build())
	frame_layout:addWidget (item_ico)
	
	local info_layout = ui.VerticalLayout()
	info_layout:setItemSpacing (ui.config.vspacing)

	do
		local hhlayout = ui.HorizontalLayout()
		item_ico:setLayout(hhlayout)

		local waiting_widget = ui.WaitingWidget()
		waiting_widget:setHorizontalSizePolicy (ui.SizePolicyMinimum)

		obj.widgets.waiting_widget = waiting_widget

		hhlayout:addWidget(ui.Widget())
		hhlayout:addWidget(waiting_widget)
		hhlayout:addWidget(ui.Widget())

		events.connect(item_ico:getDownloadedSignalProxy(), function()
			waiting_widget:setHidden(true)
			obj:update_widgets()
		end)
	end

	do
		local top_layout = ui.HorizontalLayout()
		top_layout:setItemSpacing (ui.config.hspacing)

		local level_badge = ui.NotificationBadge()
		obj.widgets.level_badge = level_badge

 		level_badge:setBadgeTexture ("ui/equipment/frame_item_level:filtered")
 		level_badge:setBadgeTextStyle (ui.text_styles.badge.build())
 		level_badge:setHorizontalSizePolicy (ui.SizePolicyMinimum)
 		top_layout:addWidget (level_badge)

 		local name_label = ui.TextLabel()
 		obj.widgets.name_label = name_label

 		name_label:setTextStyle (ui.text_styles.normal_hl.build())
 		top_layout:addWidget (name_label)

 		local state_label = ui.TextLabel()
 		state_label:setTextStyle (ui.text_styles.small.build())
 		state_label:setText (L "KEY_ITEM_IF_EQUIPPED")
 		state_label:setWordWrap (false)
 		state_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
 		top_layout:addWidget (state_label)
 		obj.state_label = state_label
 		obj.widgets.state_label = state_label

 		info_layout:addItem (top_layout)
	end

	do
		local details_layout = ui.HorizontalLayout()
		local container = ui.Widget()
		local stats_layout = ui.HorizontalLayout()
		stats_layout:setItemSpacing (ui.config.hspacing)
		--obj:add_item_stats (stats_layout, item)

		function obj:update_stats( item )
			self:add_item_stats(stats_layout, item)
		end

		container:setLayout (stats_layout)
		container:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		details_layout:addWidget (container)

		details_layout:addWidget (ui.Widget())

		details_layout:addWidget (ui.Widget())

		info_layout:addItem (details_layout)
		obj.details_layout = details_layout
	end

	do

		local btns_layout = ui.HorizontalLayout()
		btns_layout:setItemSpacing (ui.config.hspacing)

		local delivery_widget = ui.Widget()

		local delivery_layout = ui.VerticalLayout()
		delivery_layout:setItemSpacing (ui.config.vspacing)

		local delivery_label = ui.TextLabel()
 		delivery_label:setTextStyle (ui.text_styles.small.build())
 		delivery_label:setText (L "KEY_ITEM_DELIVERY_TIME")
 		delivery_label:setTextAlignment (ui.AlignmentHorizontalCenter + ui.AlignmentVerticalCenter)
 		delivery_layout:addWidget (delivery_label)

 		local delivery_bar = ui.ProgressBar()
 		delivery_bar:setTexture ("ui/bar_delivery_side:filtered")
 		delivery_bar:setTextStyle (ui.text_styles.small.build())
 		delivery_layout:addWidget (delivery_bar)
 		obj.delivery_bar = delivery_bar

 		delivery_widget:setLayout (delivery_layout)

 		btns_layout:addWidget (delivery_widget)
		
 		obj.handlers = handlers

		local sell_btn = item_widget.create_btn (L"KEY_ITEM_WIDGET_SELL", item_widget.btn_colors.sell)
		sell_btn:setTag (tag)
		events.connect (sell_btn:getClickedSignalProxy(), function(...) obj.handlers.sell(...) end)
		btns_layout:addWidget (sell_btn)

		local equip_btn = item_widget.create_btn (L"KEY_ITEM_WIDGET_EQUIP", item_widget.btn_colors.equip)
		equip_btn:setTag (tag)
		events.connect (equip_btn:getClickedSignalProxy(), function(...) obj.handlers.equip(...) end)
		btns_layout:addWidget (equip_btn)

		local unequip_btn = item_widget.create_btn (L"KEY_ITEM_WIDGET_UNEQUIP", item_widget.btn_colors.unequip)
		unequip_btn:setTag (tag)
		events.connect (unequip_btn:getClickedSignalProxy(), function(...) obj.handlers.unequip(...) end)
		btns_layout:addWidget (unequip_btn)

		local buy_btn, price_label = item_widget.create_price_btn (L"KEY_ITEM_WIDGET_BUY", item:getPrice(), "ui/ico_cash_small:filtered", item_widget.btn_colors.buy)
		buy_btn:setTag (tag)
		events.connect (buy_btn:getClickedSignalProxy(), function(...) obj.handlers.buy(...) end)
		btns_layout:addWidget (buy_btn)

		local boost_btn = ui.PriceButton()
		boost_btn:setText (L "KEY_ITEM_BOOST")
		boost_btn:setPrice (item:getPrice())
		boost_btn:setTextStyle (ui.text_styles.button_regular.build())
		boost_btn:setStateTexture ("ui/btn_premium:filtered")
		boost_btn:setCurrencyIcon ("ui/ico_gold_small:filtered")
		boost_btn:setTag (tag)
		events.connect (boost_btn:getClickedSignalProxy(), function(...) obj.handlers.equip_now(...) end)
		btns_layout:addWidget (boost_btn)

		obj.widgets.buy_price_label = price_label
		obj.btns = { sell = sell_btn, equip = equip_btn, buy = buy_btn, boost = boost_btn, unequip = unequip_btn, delivery_widget = delivery_widget }

		btns_layout:addWidget (buy_btn)

		info_layout:addItem (btns_layout)
	end

	frame_layout:addItem (info_layout)

	frame:setLayout (frame_layout)

	function obj:set_visible( visible )
		frame:setHidden (not visible)
	end

	function obj:update_widgets()
		local locked = self.item:getLevelRequired() > player:getLevel()
		self.widgets.item_ico:setLocked (locked)

		if locked then
			self.widgets.item_ico:setLockedText( L("KEY_ITEM_LOCK_LEVEL", self.item:getLevelRequired() ) )
		end
	end

	obj.widget = frame

	return obj
end

function item_widget:harlem_shake()
	self.widgets.item_ico:startShakeAnimation()
end

function item_widget:set_item( player, slot, item, tag, is_new )
	local widgets = self.widgets
	local locked = item:getLevelRequired() > player:getLevel()
	
	self.item = item
	self.slot = slot

	self.widgets.item_ico:setLocked(false)

	if locked then
		--widgets.item_ico:setLockedText( L("KEY_ITEM_LOCK_LEVEL", item:getLevelRequired() ) )
		item_widget.set_button_colors( self.btns.buy, item_widget.btn_colors.buy_locked )
		widgets.level_badge:setBadgeTextStyle (ui.text_styles.badge.build())
	else
		item_widget.set_button_colors( self.btns.buy, item_widget.btn_colors.buy )
		widgets.level_badge:setBadgeTextStyle (ui.text_styles.badge_gold.build())
	end

	widgets.waiting_widget:setHidden(false)

	widgets.item_ico:setIsNewItem(is_new)
	
	widgets.item_ico:setItem (item)
	widgets.level_badge:setPendingNotifications (item:getLevelRequired())
 	widgets.name_label:setText (underworld2.StringsDatabase():get (item:getNameKey()))

 	self.btns.sell:setTag (tag)
 	self.btns.equip:setTag (tag)
 	self.btns.unequip:setTag (tag)
 	self.btns.buy:setTag (tag)
 	widgets.buy_price_label:setText (ui.format.digit (item:getPrice()))
 	self.btns.boost:setTag (tag)

 	self.slot = slot
 	self.item = item

 	self:update_stats( item )
end



function item_widget:set_state (state)
	self.state = state
	local btns = self.btns

	btns.buy:setVisible (state == item_widget.STATE_NOT_PURCHASED)
	btns.sell:setVisible (state == item_widget.STATE_OWNED or state == item_widget.STATE_EQUIPED)
	btns.equip:setVisible (state == item_widget.STATE_OWNED)
	btns.boost:setVisible (state == item_widget.STATE_IN_DELIVERY)
	btns.unequip:setVisible (state == item_widget.STATE_EQUIPED)
	btns.delivery_widget:setVisible (state == item_widget.STATE_IN_DELIVERY)
	if state == item_widget.STATE_EQUIPED then
		self.state_label:setText (L "KEY_ITEM_EQUIPPED")
		self.widget:setBordersTexture ("ui/equipment/frame_28green:filtered")
		self.state_label:setTextStyle (ui.text_styles.positive_value.build())
	else
		self.state_label:setTextStyle (ui.text_styles.small.build())
		self.state_label:setText ("")
		self.widget:setBordersTexture ("ui/equipment/frame_28dark:filtered")
	end
end

function item_widget:set_delivery_time (time, duration)
	self.delivery_time = time
	self.delivery_duration = duration
end

function item_widget:update (server_time)
	if self.state == item_widget.STATE_IN_DELIVERY then
		local time_left = math.max (0, self.delivery_time - server_time)
		if time_left == 0 then
			self:set_state (item_widget.STATE_OWNED)
		else
			self.delivery_bar:setValue (1 - (time_left / self.delivery_duration))
			self.delivery_bar:setText (ui.format.duration_string (time_left))
			self.btns.boost:setPrice (underworld2.player:getItemDeliveryPrice (self.item:getId(), server_time))
		end
	end
end

function item_widget:add_stat_value (index, layout, icon_id, value, value_style)
	self.widgets.stats[layout] = self.widgets.stats[layout] or {}
	local stats = self.widgets.stats[layout][index]

	if not stats then
		local icon = ui.Picture()
		
		icon:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		layout:addWidget (icon)

		local value_label = ui.TextLabel()
		value_label:setText (value)
		value_label:setTextStyle (value_style)
		value_label:setWordWrap (false)
		value_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		layout:addWidget (value_label)

		stats = { icon = icon, value_label = value_label }

		function stats.set_visible( visible )
			local hidden = not visible
			icon:setHidden(hidden)
			value_label:setHidden(hidden)
		end

		self.widgets.stats[layout][index] = stats
	end

	stats.set_visible( true )
	stats.icon:setTexture (icon_id)
	stats.value_label:setText (value)
end

function item_widget:add_item_stats (layout, item)
	local effects_db 	= underworld2.EffectsDatabase()
	local default_stats = underworld2.player:getDefaultStats()

	for k,v in pairs(self.widgets.stats[layout] or {}) do
		v.set_visible(false)
	end

	local effects_count = item:getEffectsCount()
	local sorted_stats_table = {}

	for i = 0, effects_count - 1 do
		local effect_index = item:getEffect (i)
		local stat = default_stats:getStat (effect_index:getStatId())	
		if stat:get() then
			local icon_id = effect_icons_map[stat:getType()][stat:getKey()]
			table.insert(sorted_stats_table, { 
				icon_id = icon_id, 
				index = effect_index:getStatId(), 
				value = L(get_effect_format[stat:getType()][stat:getKey()], get_effect_value[stat:getType()][stat:getKey()](effect_index) )
			} )
		end
	end

	table.sort( sorted_stats_table, function(a,b) return a.index < b.index end )

	for i,v in ipairs(sorted_stats_table) do
		self:add_stat_value (v.index, layout, v.icon_id, v.value, ui.text_styles.small_hl.build())
	end
end

function item_widget:compare (item)
	if item:get() then
		local item_is_better = self.item:getPrice() > item:getPrice()
		local locked = item:getLevelRequired() > underworld2.player:getLevel()
		self.widgets.item_ico:markAsBetterItem(item_is_better and not locked)
	end
	-- For now - do nothing more
	do return end
	self.details_layout:removeItemAt (2)
	if item:get() and self.item:getId() == item:getId() then
		self.details_layout:addWidget (ui.Widget())
		return
	end

	local effects_db 	= underworld2.EffectsDatabase()
	local default_stats = underworld2.player:getDefaultStats()

	local diff = { }

	if item:get() then

		local effects_count = item:getEffectsCount()
		for i = 0, effects_count - 1 do
			local effect_index = item:getEffect (i)
			local stat = default_stats:getStat (effect_index:getStatId())
			diff[effect_index:getStatId()] = -get_effect_value[stat:getType()][stat:getKey()](effect_index)
		end

	end

	effects_count = self.item:getEffectsCount()
	for i = 0, effects_count - 1 do
		local effect_index = self.item:getEffect (i)
		local stat = default_stats:getStat (effect_index:getStatId())
		diff[effect_index:getStatId()] = 
					(diff[effect_index:getStatId()] or 0) + 
					get_effect_value[stat:getType()][stat:getKey()](effect_index)
	end

	local stats_diff_widget = ui.Widget()
	stats_diff_widget:setHorizontalSizePolicy (ui.SizePolicyMinimum)

	local layout = ui.HorizontalLayout()
	layout:setItemSpacing (ui.config.hspacing)
	local idx = 1

	local tinsert = table.insert
	local function sorted_diff()
		local p = {}
		local n = {}
		for i,v in pairs(diff) do 
			if v > 0 then
				tinsert( p, { stat_id = i, value = v, text_style = ui.text_styles.positive_value.build() } ) 
			elseif v < 0 then 
				tinsert( n, { stat_id = i, value = v, text_style = ui.text_styles.negative_value.build() } ) 
			end
		end
		
		table.sort( p, function(a,b) return a.stat_id < b.stat_id end )	
		table.sort( n, function(a,b) return a.stat_id < b.stat_id end )

		local ret = {}
		for i,v in ipairs(p) do tinsert( ret, v ) end
		for i,v in ipairs(n) do tinsert( ret, v ) end

		return ret
	end

	for k,v in pairs(self.widgets.stats[layout] or {}) do
		v.set_visible(false)
	end

	for i,v in ipairs (sorted_diff()) do
		local stat_id = v.stat_id
		local level_diff = v.value
		local text_style = v.text_style

		local stat = default_stats:getStat (stat_id)
		if stat:get() then
			local icon_id = effect_icons_map[stat:getType()][stat:getKey()]
			self:add_stat_value (idx, layout, icon_id, L(get_effect_diff_format[stat:getType()][stat:getKey()], level_diff), text_style)
			idx = idx + 1
		end
	end
	stats_diff_widget:setLayout (layout)
	self.details_layout:addWidget (stats_diff_widget)
end

ui.widgets.item_widget = item_widget
