require "ui/config.lua"

local widget = 
{
	STATE_UP = 0,
	STATE_DOWN = 1,
	STATE_SAME = 2
}

local rank_widget_width

function widget.create (profile, place, rp, left_margin)
	local profile_id = profile:id()
	if not left_margin then left_margin = 0 end

	local strings = underworld2.StringsDatabase()
	local level = underworld2.LevelsDatabase():get (profile:getLevel())
	
	if not level:get() then
		ErrorPrint (string.format ("can't find level %d description(player id=%d)", profile:getLevel(), profile:id()))
	end

	local root_wrapper = ui.Widget()
	local container_button = ui.ContainerButton()
	container_button:setTexture (profile_id == underworld2.player:id() and "ui/top_hustler/tab_you:filtered" or "ui/top_hustler/tab_other:filtered")
	container_button:setVerticalSizePolicy(ui.SizePolicyFixed)
	container_button:setHorizontalSizePolicy(ui.SizePolicyFixed)
	root_wrapper:addChild(container_button)

	local arrow = ui.Decorator()
	arrow:setTexture("ui/top_hustler/player_pointer:filtered", ui.Decorator_StyleHStretcherWithBorders)
	local arrow_size = arrow:getPreferredSize()
	local arrow_width = arrow_size.x*1.8

	if profile_id == underworld2.player:id() then
		arrow:setHorizontalSizePolicy(ui.SizePolicyFixed)
		arrow:setVerticalSizePolicy(ui.SizePolicyFixed)
		arrow:setOrigin(0,0)
		arrow:setSize(math.vec2(arrow_width,arrow_size.y))
		root_wrapper:addChild(arrow)
	end

	local root = ui.Widget()
	root:setLeftMargin (ui.config.hspacing)
	root:setRightMargin (ui.config.hspacing)
	root:setVerticalSizePolicy(ui.SizePolicyMinimum)
	root_wrapper:addChild(root)

	local root_layout = ui.HorizontalLayout()
	root_layout:setItemSpacing (ui.config.hspacing)

	--[[local state_ico = ui.Picture()
	state_ico:setTexture (state == widget.STATE_DOWN and "ui/ico_arrow_orange" or "ui/ico_arrow_green")
	state_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	if state == widget.STATE_SAME then
		state_ico:setColor (render.color (0, 0, 0, 0))
	end
	root_layout:addWidget (state_ico)]]

	local rank_widget = ui.Widget()
	local rank_ico_id = ""

	if not rank_widget_width then
		rank_widget_width = arrow_width*0.93-left_margin
	end

	rank_widget:setHorizontalSizePolicy (ui.SizePolicyFixed)
	rank_widget:setWidth (rank_widget_width)

	do

		local layout = ui.HorizontalLayout()
		--layout:setItemSpacing(ui.config.hspacing)

		local place_label = ui.TextLabel()
		place_label:setTextStyle (ui.text_styles.small_hl.build())
		place_label:setText (tostring (place))
		place_label:setWordWrap (false)
		place_label:setTextAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalCenter)
		layout:addWidget (place_label)

		layout:addWidget (ui.Widget())

		rank_widget:setLayout (layout)
	end

	root_layout:addWidget (rank_widget)


	local avatar = ui.AvatarWidget()
	avatar:setAvatarSize (underworld2.Avatar_SizeLarge)
	avatar:setScale (0.5)
	avatar:setProfile (profile)
	avatar:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	root_layout:addWidget (avatar)

	do
		local vlayout = ui.VerticalLayout()

		local nickname_label = ui.TextLabel()
		nickname_label:setTextStyle (ui.text_styles.normal_hl.build())
		nickname_label:setText (profile:getNickname())
		nickname_label:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalCenter)
		nickname_label:setElide (true)
		vlayout:addWidget (nickname_label)
		do
			local hlayout = ui.HorizontalLayout()
			hlayout:setItemSpacing (ui.config.hspacing)

			local badge = ui.NotificationBadge()
	 		badge:setBadgeTexture ("ui/ico_level:filtered")
	 		badge:setPendingNotifications (profile:getLevel())
	 		badge:setBadgeTextStyle (ui.text_styles.badge.build())
	 		badge:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	 		hlayout:addWidget (badge)

			local level_label = ui.TextLabel ()
			level_label:setText (strings:get (level:getNameKey()))
			level_label:setTextStyle (ui.text_styles.hint_grey_small.build())
			--level_label:setWordWrap (false)
			level_label:setElide (true)
			hlayout:addWidget (level_label)

			vlayout:addItem (hlayout)
		end

		vlayout:setHorizontalPolicy(ui.SizePolicyMaximum)

		root_layout:addItem (vlayout)
	end

	local rp_level = ui.TextLabel()
	rp_level:setTextStyle (ui.text_styles.leaderboard_fame.build())
	rp_level:setText (ui.format.digit (rp))
	rp_level:setTextAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalCenter)
	rp_level:setWordWrap (false)
	root_layout:addWidget (rp_level)

	local ico = ui.Picture()
	ico:setTexture("ui/icons/ico_fame_hustler_list:filtered")
	ico:setSize(ico:getPreferredSize())
	root_layout:addWidget(ico)

	local more_ico = ui.Picture()
	more_ico:setTexture ("ui/top_hustler/arrow_tab_other:filtered")
	more_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)

	root:setLayout (root_layout)
	if profile_id ~= underworld2.player:id() then
		root_layout:addWidget (more_ico)

		events.connect (container_button:getClickedSignalProxy(), function()
			ui.inbox.show_chat (profile_id)
		end)
	else
		local empty_widget = ui.Widget()
		empty_widget:setHorizontalSizePolicy(ui.SizePolicyFixed)
		empty_widget:setSize(more_ico:getPreferredSize())
		root_layout:addWidget(empty_widget)
	end

	local preferred_size = root:getPreferredSize()
	local width = 1.0 - left_margin * 2
	local height = preferred_size.y

	root:setSize(width, height)

	local preferred_height = container_button:getPreferredSize().y
	if preferred_height > height then height = preferred_height end
	if arrow:getHeight() > height then height = arrow:getHeight() end

	container_button:setOrigin(left_margin, 0)
	container_button:setSize(width, height)

	root:setOrigin(left_margin, height / 2 - preferred_size.y / 2)

	root_wrapper:setHorizontalSizePolicy(ui.SizePolicyFixed)
	root_wrapper:setVerticalSizePolicy(ui.SizePolicyFixed)
	root_wrapper:setSize(width, height)

	return root_wrapper
end

ui.widgets = ui.widgets or { }
ui.widgets.leaderboard_row = widget
