require "ui/format.lua"

local wrapper = { }

local function __build(title, description, time, icon_url, img_url, external_url, h_max)
	local root = ui.Widget()
	root:setVerticalSizePolicy (ui.SizePolicyMinimum)

	local vertical_layout = ui.VerticalLayout()
	vertical_layout:setItemSpacing (ui.config.vspacing)
	root:setLayout(vertical_layout)

	do
		local layout = ui.HorizontalLayout()
		layout:setItemSpacing (ui.config.hspacing)
		vertical_layout:addItem(layout)

		do
			local container = ui.VerticalLayout()
			local icon = underworld2.RemotePicture()

			--we can set some placeholder
			icon:setTexture("ui/messaging/avatar_news_empty:filtered")

			if icon_url and (icon_url ~= "") then
				local icon_layout = ui.HorizontalLayout()
				icon:setLayout(icon_layout)

				local waiting_widget = ui.WaitingWidget()
				waiting_widget:setHorizontalSizePolicy (ui.SizePolicyMinimum)

				icon_layout:addWidget(ui.Widget())
				icon_layout:addWidget(waiting_widget)
				icon_layout:addWidget(ui.Widget())

				events.connect(icon:getDownloadedSignalProxy(), function()
					waiting_widget:setHidden(true)
				end)

				icon:download(icon_url)
			end

			container:addWidget(icon)
			layout:addItem(container)
		end

		do
			local container = ui.VerticalLayout()
			layout:addItem(container)

			local title_label = ui.RichTextWidget()
			title_label:setDefaultTextStyle (ui.text_styles.inbox_news_item_title.build())
			title_label:setText(title)
			title_label:setHorizontalSizePolicy(ui.SizePolicyMaximum)
			container:addWidget(title_label)

			local timestamp_label = ui.TextLabel()
			timestamp_label:setTextStyle (ui.text_styles.inbox_news_item_text.build())
			timestamp_label:setTextAlignment (ui.AlignmentHorizontalLeft)
			timestamp_label:setText( ui.format.time_passed(underworld2.server_time(), time) )
			timestamp_label:setHorizontalSizePolicy(ui.SizePolicyMaximum)
			container:addWidget(timestamp_label)
		end

		do
			local right_widget = ui.Widget()
			right_widget:setHorizontalSizePolicy(ui.SizePolicyMaximum)
			layout:addWidget(right_widget)
		end
	end

	local scroll = nil
	local max_scroll_height = 0

	local function update_scroll()
		local hh_max = h_max - root:getPreferredSize().y + scroll:getHeight()
		local preferred_height = max_scroll_height
		if preferred_height > hh_max then preferred_height = hh_max end
		scroll:setHeight(preferred_height)
	end

	do
		scroll = ui.ScrollWidget()
		scroll:setScrollType (ui.ScrollTypeVertical)
		scroll:setScrollBarTexture ("ui/scroll:filtered")
		--scroll:setFrameTexture ("ui/frame_modal_light:filtered")
		scroll:setMargins(0,0,0,0)
		scroll:setVerticalSizePolicy(ui.SizePolicyFixed)

		local text_layout = ui.VerticalLayout()
		scroll:setLayout (text_layout)

		local text_label = ui.RichTextWidget()
		text_label:setDefaultTextStyle (ui.text_styles.tutorial_text.build())
		text_label:setText (description)
		text_label:setMaxWidth (0.93)
		text_label:setHorizontalSizePolicy(ui.SizePolicyMinimum)
		text_layout:addWidget (text_label)

		max_scroll_height = text_label:getPreferredSize().y

		vertical_layout:addWidget (scroll)

		do
			local hhlayout = ui.HorizontalLayout()

			local image = underworld2.RemotePicture()
			image:setTexture("ui/messaging/modal_news_empty:filtered")

			local width = 0.93
			local pS = image:getPreferredSize()
			local pW = pS.x
			local pH = pS.y
			local pRatio = width / pW

			image:setPlacingMode(ui.Picture_PlacingModeStretch)
			image:setHorizontalSizePolicy(ui.SizePolicyFixed)
			image:setVerticalSizePolicy(ui.SizePolicyFixed)

			image:setHeight(pH * pRatio)
			image:setWidth(width)

			if img_url and (img_url ~= "") then
				local image_layout = ui.HorizontalLayout()
				image:setLayout(image_layout)

				local waiting_widget = ui.WaitingWidget()
				waiting_widget:setHorizontalSizePolicy (ui.SizePolicyMinimum)

				image_layout:addWidget(ui.Widget())
				image_layout:addWidget(waiting_widget)
				image_layout:addWidget(ui.Widget())

				events.connect(image:getDownloadedSignalProxy(), function(owner)
					waiting_widget:setHidden(true)

					local ss = owner:getTextureSize()
					local ww = ss.x
					local hh = ss.y

					if ww > 0 then -- just in any ugly case
						image:setHeight(hh * (width / ww))
					end

					update_scroll()
				end)

				image:download(img_url)
			end

			hhlayout:addWidget(ui.Widget())
			hhlayout:addWidget(image)
			hhlayout:addWidget(ui.Widget())

			vertical_layout:addItem(hhlayout)
		end
	end

	do
		local h_layout = ui.HorizontalLayout()
		local fb_button = underworld2.facebook_sharing.create_button(L"KEY_NEWS_COMMENT", 0, function()
			if external_url and (external_url ~= "") then
				os.device:openURL(external_url)
			else
				underworld2.shared.show_fb_page()
			end
		end)

		fb_button:setHorizontalSizePolicy (ui.SizePolicyFixed)
		fb_button:setWidth(0.4)

		h_layout:addWidget(ui.Widget())
		h_layout:addWidget(fb_button)
		h_layout:addWidget(ui.Widget())

		vertical_layout:addItem(h_layout)
	end

	update_scroll()

	return root
end

local function __build_title_widget ()
	local w = ui.Widget()

	local vlayout = ui.VerticalLayout()

	local hlayout = ui.HorizontalLayout()

	hlayout:addWidget (ui.Widget())
	local nickname = ui.TextLabel()
	nickname:setTextStyle (ui.text_styles.small_hl.build())
	nickname:setText (L"KEY_INBOX_NAVIGATION_MESSAGE")
	nickname:setWordWrap (false)
	hlayout:addWidget (nickname)
	hlayout:addWidget (ui.Widget())
	vlayout:addItem (hlayout)

	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing (ui.config.hspacing)

	hlayout:addWidget (ui.Widget())

	local status_ico = ui.Picture()
	status_ico:setTexture ("ui/messaging/ico_offline_inbox:filtered")
	status_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hlayout:addWidget (status_ico)

	local status_label = ui.TextLabel()
	status_label:setTextStyle (ui.text_styles.hint_grey_small.build())
	status_label:setText (L "KEY_INBOX_OFFLINE")
	status_label:setWordWrap (false)
	status_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	hlayout:addWidget (status_label)

	status_ico:setHidden (true)
	status_label:setHidden (true)

	hlayout:addWidget (ui.Widget())

	--vlayout:addItem (hlayout)

	w:setLayout (vlayout)

	return w
end

function wrapper.create (parent, parent_menu_name, details)
	local obj = setmetatable ( { ui = { } }, { __index = wrapper } )

	local root = ui.Widget()
	obj.widget = root

	local layout = ui.VerticalLayout()
	layout:setItemSpacing (ui.config.vspacing)
	obj.ui.layout = layout

	local w = nil

	do
		w = ui.Decorator()
		w:setTexture ("ui/messaging/bar_inbox_navigation:filtered", ui.Decorator_StyleHStretcherWithBorders)
		w:setVerticalSizePolicy (ui.SizePolicyMinimum)
		local bg_size = math.vec2 (1, w:getPreferredSize().y)
		
		if parent_menu_name then
			local back = ui.TextButton()
			back:setTexture ("ui/messaging/btn_back_name:filtered")
			local back_size = back:getPreferredSize()
			back:setOrigin (0.02, (bg_size.y - back_size.y) * 0.5)
			back:setTextStyle (ui.text_styles.small_hl.build())
			back:setText (parent_menu_name)
			back:setSize (back_size)
			w:addChild (back)

			events.connect (back:getClickedSignalProxy(), function() 
				parent:pop_submenu()
			end)
		end

		local title = __build_title_widget ()
		local title_size = title:getPreferredSize()
		title:setOrigin ((bg_size - title_size) * 0.5)
		title:setSize (title_size)
		w:addChild (title)

		layout:addWidget (w)
	end

	do
		local hlayout = ui.HorizontalLayout()

		local frame = ui.Frame()
		frame:setBordersTexture("ui/messaging/frame_chat_bg_main:filtered")
		frame:setVerticalSizePolicy(ui.SizePolicyMinimum)
		frame:setMargins(ui.config.hspacing, ui.config.hspacing,ui.config.vspacing, ui.config.vspacing)
		frame:setHorizontalSizePolicy(ui.SizePolicyFixed)
		frame:setWidth(0.96)

		local frame_layout = ui.VerticalLayout()
		frame:setLayout(frame_layout)

		local h_max = 1.0 - frame:getPreferredSize().y - frame:getBottomMargin() - frame:getTopMargin() - parent.ui.header:getHeight() - w:getPreferredSize().y - ui.config.vspacing * 2
		local report = __build(details.title, details.description, details.timestamp, details.icon_url, details.image_url, details.url, h_max)
		frame_layout:addWidget(report)

		hlayout:addWidget(ui.Widget())
		hlayout:addWidget(frame)
		hlayout:addWidget(ui.Widget())
		layout:addItem(hlayout)
	end

	root:setLayout (layout)

	return obj
end

function wrapper:activated()
	
end

function wrapper:on_close()
end

ui.widgets = ui.widgets or { }
ui.widgets.news_report = ui.widgets.news_report or wrapper
