local controller = { }

local pane_anim_keyframes =
{
	t = { 0.0, 0.2, 0.4, 1.0 },
	offset = { 0.0, 0.1, 0.4, 1.0 }
}

local rays_anim_keyframes= 
{
	t =      { 0.0, 0.2, 0.5, 0.7, 1.0 },
	scale =  { 0.4, 1.6, 2.8, 2.6, 0.4 },
}

local shield_anim_keyframes= 
{
	t =      { 0.0, 0.2, 0.5, 1.0 },
	scale =  { 1.0, 1.3, 1.1, 1.0 },
}

local function __animate_shield_activated (self)
	local pane_fn = math.cspline (pane_anim_keyframes.t, pane_anim_keyframes.offset)
	
	local max_offset = self.ui.root:getHeight()
	local anim = anim.sequence {
		--hide heat
		anim.lerp 
		{
			from = 0.0, to = 1.0, time = 0.5,
			object = self,
			property = function (self, t)
				local offset = math.vec2 (0, pane_fn (t) * -max_offset)
				self.ui.root:setOffset (offset)
			end,
			on_finished = function()
				self.ui.heat_pane:disableRendering()
				self.ui.heat_pane:setEnabled (false)

				self.ui.shield_pane:enableRendering()
				self.ui.shield_pane:setEnabled (true)
			end,
		},
		--show shield
		anim.lerp
		{
			from = 1.0, to = 0.0, time = 0.5,
			object = self,
			property = function (self, t)
				local offset = math.vec2 (0, pane_fn (t) * -max_offset)
				self.ui.root:setOffset (offset)
			end
		},
	}
	self.anim_queue:push (anim)
end

local function __animate_highlight (self)
	local rays_fn = math.cspline (rays_anim_keyframes.t, rays_anim_keyframes.scale)
	local shield_fn = math.cspline (shield_anim_keyframes.t, shield_anim_keyframes.scale)

	local anim = anim.sequence
	{
		anim.delay { time = 0.0, on_finished = function () self.ui.rays:setHidden (false) end },
		anim.lerp
		{
			from = 0.0, to = 1.0, time = 0.7,
			object = self,
			property = function (self, t)
				self.ui.rays:setScale (rays_fn (t))
				self.ui.shield_pane:setScale (shield_fn (t))
			end,
			on_finished = function()
				self.ui.rays:setHidden (true)
			end
		}
	}
	self.anim_queue:push (anim)
end

local function __animate_shield_expired (self)
	local pane_fn = math.cspline (pane_anim_keyframes.t, pane_anim_keyframes.offset)	
	local shield_fn = math.cspline (shield_anim_keyframes.t, shield_anim_keyframes.scale)

	local max_offset = self.ui.root:getHeight()
	local anim = anim.sequence {
		--show shield
		anim.lerp
		{
			from = 0.0, to = 1.0, time = 0.5,
			object = self,
			property = function (self, t)
				local offset = math.vec2 (0, pane_fn (t) * -max_offset)
				self.ui.root:setOffset (offset)
			end,
			on_finished = function()
				self.ui.heat_pane:enableRendering()
				self.ui.heat_pane:setEnabled (true)

				self.ui.shield_pane:disableRendering()
				self.ui.shield_pane:setEnabled (false)
			end
		},
		--hide heat
		anim.lerp 
		{
			from = 1.0, to = 0.0, time = 0.5,
			object = self,
			property = function (self, t)
				local offset = math.vec2 (0, pane_fn (t) * -max_offset)
				self.ui.root:setOffset (offset)
			end,
		},
	}
	self.anim_queue:push (anim)
end

function controller.create()
	local self = setmetatable ({ }, {__index = controller})

	local root = ui.load_ui ("ui/police_exposure_pane.ui")
	self.ui = ui.build_children_map (root)

	local rays = root:findChild ("rays")
	local shield_pane = runtime.cast_to (root:findChild ("shield_pane"), "ui::Button")
	local shield_icon = runtime.cast_to (shield_pane:findChild ("shield_ico"), "ui::Picture")
	local shield_timer = runtime.cast_to (shield_pane:findChild ("shield_timer"), "ui::TextLabel")
	local heat_pane = root:findChild ("heat_pane")
	local exposure = runtime.cast_to (heat_pane:findChild ("exposure"), "ui::PoliceExposureBar")
	local heat_drop = runtime.cast_to (exposure:findChild ("heat_drop"), "ui::Button")
	local bust_hint = runtime.cast_to (exposure:findChild ("bust_hint"), "ui::TextLabel")
	local heat_icon = runtime.cast_to (exposure:findChild ("heat_icon"), "ui::Picture")
	local cooldown_pane = heat_pane:findChild ("cooldown_pane")
	local cooldown_timer = runtime.cast_to (cooldown_pane:findChild ("cooldown_timer"), "ui::TextLabel")

	--[[self.ui = 
	{
		root = root,

		rays = rays,

		shield_pane = shield_pane,
		shield_timer = shield_timer,

		heat_pane = heat_pane,
		exposure = exposure,
		bust_hint = bust_hint,
		heat_drop = heat_drop,

		cooldown_pane = cooldown_pane,
		cooldown_timer = cooldown_timer,
	}]]

	self.ui.cooldown_pane:disableRendering()
	self.ui.shield_pane:disableRendering()
	self.ui.shield_pane:setEnabled (false)

	self.anim_queue = anim.queue()

	local heat_icon_size = heat_icon:getImageSize()
	self.exposure_padding = heat_drop:getPreferredSize().x  * 0.95
	heat_icon:setOffset (math.vec2 (self.exposure_padding - heat_icon_size.x, -heat_icon_size.y * 0.1))

	--local shield_icon_size = shield_icon:getImageSize()
	--shield_timer:setOffset (math.vec2 (0, shield_icon_size.y * 22.0 / 118.0))

	self.heat_level = nil
	self.shield_active = false
	self.shield_expires_at = nil

	return self
end

local heat_level_map = 
{ 
	[1] = L "KEY_STATUS_SAFE", 
	[2] = L "KEY_STATUS_SUSPICIOUS", 
	[3] = L "KEY_STATUS_ALERT", 
	[4] = L "KEY_STATUS_DANGER"
}

function __eval_exposure_bar_display_value (heat_value, padding, bar_width)
	return (padding + (bar_width - padding * 2) * heat_value) / bar_width
end

function __eval_exposure_bar_display_delta_value (delta, padding, bar_width)
	return (bar_width - padding * 2) * delta / bar_width
end

local function __set_shield_active (self, active, end_time)
	if active then
		__animate_shield_activated (self)
		__animate_highlight (self)
	else
		__animate_shield_expired (self)
	end
	self.shield_active = active
end

function controller:update (dt)
	self.anim_queue:update (dt)
	local player = underworld2.player

	if not player then
		return
	end

	local now = underworld2.server_time()

	local pe = player:getPoliceExposure()
	local cooldown_time = pe:getCooldownTime (now)
	local heat_value = pe:getHeatValue (now)
	local heat_level = pe:getHeatLevel (heat_value)

	if cooldown_time > 0 then
		self.ui.cooldown_pane:enableRendering()
		self.ui.cooldown_timer:setText (ui.format.duration_string (cooldown_time))
	else
		self.ui.cooldown_pane:disableRendering()
	end

	if heat_level ~= self.heat_level then
		self.ui.bust_hint:setText (heat_level_map[heat_level] or "")
		self.heat_level = heat_level
	end
	local bar_width = self.ui.exposure:getSize().x
	self.ui.exposure:setHeat (__eval_exposure_bar_display_value (heat_value, self.exposure_padding, bar_width))

	local bonus = player:getBonus ("PoliceProtectionBonus")
	local shield_active = false
	local end_time = nil
	if bonus:get() then
		local police_protection_bonus = runtime.cast_to (bonus:get(), "underworld2::PoliceProtectionBonus")
		start_time = police_protection_bonus:getStartTime()
		end_time = police_protection_bonus:getEndTime()
		shield_active = (now >= start_time) and (now <= end_time)
	end
	if self.shield_active ~= shield_active then
		__set_shield_active (self, shield_active, end_time)
	elseif self.shield_active and (self.shield_expires_at ~= end_time) then
		__animate_highlight (self)
	end
	self.shield_expires_at = end_time

	if shield_active then
		self.ui.shield_timer:setText (ui.format.duration_string (self.shield_expires_at - now))
	end
end


function controller:set_heat_delta (delta)
	local player = underworld2.player
	if player then
		local bar_width = self.ui.exposure:getSize().x
		local display_value = __eval_exposure_bar_display_delta_value (delta / player:getPoliceExposure():getMaxPoints(), self.exposure_padding, bar_width)
		self.ui.exposure:setHeatDelta (display_value)
	end
end

ui.widgets.police_pane_controller = controller
