require "lib/signal.lua"
require "ui/core/library.lua"
local lui = ui.core.lib

local controller = { }

local __slot_icons_name_map =
{
	[1] = "slot0_ico",
	[2] = "slot1_ico",
	[3] = "slot2_ico",
	[4] = "slot3_ico",
}

local __slot_label_name_map =
{
	[1] = "slot0_value",
	[2] = "slot1_value",
	[3] = "slot2_value",
	[4] = "slot3_value",	
}

function __set_reward_slot (self, slot_id, value, icon, hidden)
	local icon_widget = self.ui[__slot_icons_name_map[slot_id]]
	local text_widget = self.ui[__slot_label_name_map[slot_id]]
	icon_widget:setHidden (hidden)
	text_widget:setHidden (hidden)

	if hidden then return end

	icon_widget:setTexture (icon)
	text_widget:setText (value)
end

function controller.create ()
	local self = setmetatable ({ }, { __index = controller } )
	self.geometry_update_required = signal()

	local widget = ui.load_ui ("ui/quests/reward.ui")
	self.ui = ui.build_children_map (widget)

	return self
end

function controller:set_quest (quest)
	self.quest = quest
	self.is_timed = quest:get():getClass():getScopedName() == "underworld2::quests::TimedQuest"

	local reward = quest:get():getReward():get()
	
	local rp 	= reward:getRespectPoints()
	local cash 	= reward:getCash()
	local gold 	= reward:getGold()
	local commodity_id = reward:getCommodityId()
	local commodites = reward:getCommoditiesAmount()
	
	__set_reward_slot (self, 1, tostring (rp), "ui/jobs/ico_rp_34:filtered", rp == 0)
	__set_reward_slot (self, 2, tostring (commodites), bundle.resources.commodity_small_icons[commodity_id], commodites == 0)
	__set_reward_slot (self, 3, tostring (gold), "ui/ico_pbtn_gold:filtered", gold == 0)
	__set_reward_slot (self, 4, tostring (cash), "ui/jobs/ico_cash_34:filtered", cash == 0)

	local hide_timer = quest:get():isCompleted() or not self.is_timed
	self.ui.timer_pane:setHidden (hide_timer)
	
	self:update()

	self.geometry_update_required:notify (self)
end

function controller:update (dt)
	if not self.is_timed then return end
	
	local quest = runtime.cast_to (self.quest:get(), "underworld2::quests::TimedQuest")
	local now = underworld2.server_time()
	local time_left = math.max (quest:getExpiresAt() - now, 0)

	self.ui.timer :setText (ui.format.duration_string (time_left, true))
end

ui.reward_controller = controller
