require "lib/signal.lua"
require "ui/core/library.lua"
local lui = ui.core.lib

local controller = { }

function controller.create (task)
	local self = setmetatable ({ }, { __index = controller } )
	self.geometry_update_required = signal()

	local widget = ui.load_ui ("ui/quests/task_bring_commodities.ui")
	self.ui = ui.build_children_map (widget)

	self.ptr = task
	self.task = runtime.cast_to (task:get(), "underworld2::quests::BringCommoditiesTask")

	local S = underworld2.StringsDatabase()
	local commodity_id = self.task:getCommodityId()
	local commodity = underworld2.CommoditiesDatabase():get (commodity_id)
	self.ui.icon:setTexture (bundle.resources.commodity_icons[self.task:getCommodityId()])
	self.ui.title:setText (string.format ("Give %d %s", self.task:getAmount(), S:get (commodity:getNameKey())))

	self.amount = 0

	self.skip_task = signal()
	self.give_items = signal()
	events.connect (self.ui.skip:getClickedSignalProxy(), function() self.skip_task:notify(self) end)
	events.connect (self.ui.give:getClickedSignalProxy(), function() self.give_items:notify (self, self.amount) end)

	self:on_task_state_updated()

	return self
end

function controller:on_task_state_updated()
	local task = self.task
	local storage = underworld2.player:getStorage (task:getCommodityId())

	local is_completed 		= task:isCompleted()
	local required_amount	= task:getAmount()
	local brought_amount	= is_completed and required_amount or task:getBroughtAmount()
	local items_on_hands 	= storage:get() and storage:getUsage() or 0
	local amount 			= math.min (items_on_hands, required_amount - brought_amount)
	self.amount = amount
	
	
	self.ui.spacer:setHidden (not is_completed)
	self.ui.tick:setHidden (not is_completed)
	self.ui.give:setHidden (is_completed or items_on_hands == 0)
	self.ui.skip:setHidden (is_completed)
	self.ui.no_items:setHidden (is_completed or items_on_hands > 0)

	self.ui.amount:setText (tostring (amount))
	self.ui.skip_price:setText (tostring (task:getSkipPrice()))

	local prev_value = self.value
	local new_value = brought_amount / required_amount
	
	self.ui.progress:setHeatDelta (amount / required_amount)

	self.value = new_value

	if prev_value and new_value and prev_value ~= new_value then
		return anim.lerp
		{
			from = prev_value, to = new_value, time = 0.2,
			object = self,
			property = function (self, t)
				self.ui.progress:setHeat (t)
				self.ui.progress:setText (string.format ("%s/%s", math.floor (t * required_amount), required_amount))
			end
		}
	else
		self.ui.progress:setHeat (new_value)
		self.ui.progress:setText (string.format ("%s/%s", brought_amount, required_amount))
	end
end

ui.task_bring_commodities_controller = controller
