require "lib/signal.lua"
require "ui/core/library.lua"
local lui = ui.core.lib

local controller = { }

function controller.create (task)
	local self = setmetatable ({ }, { __index = controller } )
	self.geometry_update_required = signal()

	local widget = ui.load_ui ("ui/quests/task_trading.ui")
	self.ui = ui.build_children_map (widget)

	self.ptr = task
	self.task = runtime.cast_to (task:get(), "underworld2::quests::EarnCashTask")

	self.ui.icon:setTexture ("ui/map/items/item_cash:filtered")
	self.ui.title:setText (string.format ("Earn %d Cash", self.task:getAmount()))

	self.skip_task = signal()
	events.connect (self.ui.skip:getClickedSignalProxy(), function() self.skip_task:notify(self) end)

	self:on_task_state_updated()

	return self
end

function controller:on_task_state_updated()
	local task = self.task
	local storage = underworld2.player:getStorage (task:getCommodityId())

	local is_completed 		= task:isCompleted()
	local earned, amount 	= task:getEarnedAmount(), task:getAmount()

	self.ui.tick:setHidden (not is_completed)
	self.ui.skip:setHidden (is_completed)
	self.ui.skip_price:setText (tostring (task:getSkipPrice()))
	
	local prev_value = self.value
	local new_value = earned / amount

	self.value = new_value

	if prev_value and new_value and prev_value ~= new_value then
		return anim.lerp
		{
			from = prev_value, to = new_value, time = 0.2,
			object = self,
			property = function (self, t)
				self.ui.progress:setValue (t)
				self.ui.progress:setText (string.format ("%s/%s", math.floor (t * earned), amount))
			end
		}
	else
		self.ui.progress:setValue (new_value)
		self.ui.progress:setText (string.format ("%s/%s", earned, amount))
	end
end

ui.task_earn_cash_controller = controller
