local wrapper = { }

function wrapper.create (profile_ptr, handler)
	local obj = setmetatable ( { ui = { } }, { __index = wrapper } )

	local profile = profile_ptr:get()

	local root = ui.ContainerButton()
	root:setTexture ("ui/messaging/bar_player_message:filtered")
	root:setVerticalSizePolicy (ui.SizePolicyMinimum)
	root:setTag (profile:id())
	events.connect (root:getClickedSignalProxy(), handler)

	local layout = ui.HorizontalLayout()
	layout:setItemSpacing (ui.config.hspacing)

	local avatar = ui.AvatarWidget()
	avatar:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	avatar:setProfile (profile_ptr)
	obj.ui.avatar = avatar
	
	layout:addWidget (avatar)

	do
		local vlayout = ui.VerticalLayout()

		local nickname = ui.TextLabel()
		nickname:setTextStyle (ui.text_styles.dealer_nickname.build())
		nickname:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
		obj.ui.nickname = nickname

		vlayout:addWidget (nickname)

		do
			local hlayout = ui.HorizontalLayout()
			hlayout:setItemSpacing (ui.config.hspacing)

			local message_counter = ui.NotificationBadge()
			message_counter:setBadgeTexture("ui/ico_notification")
			message_counter:setBadgeTextStyle (ui.text_styles.badge.build())
			message_counter:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			obj.ui.message_counter = message_counter

			hlayout:addWidget (message_counter)

			local short_message = ui.TextLabel()
			short_message:setTextStyle (ui.text_styles.message_short.build())
			short_message:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalLeft)
			obj.ui.short_message = short_message

			hlayout:addWidget (short_message)

			vlayout:addItem (hlayout)
		end

		layout:addItem (vlayout)
	end

	local when = ui.TextLabel()
	when:setTextStyle (ui.text_styles.hint_grey_small.build())
	when:setTextAlignment (ui.AlignmentHorizontalRight + ui.AlignmentVerticalCenter)
	when:setWordWrap (false)
	when:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (when)
	obj.ui.when = when

	local network_status = ui.Picture()
	network_status:setTexture ("ui/messaging/ico_offline_inbox:filtered")
	network_status:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (network_status)

	obj.ui.root = root

	root:setLayout (layout)

	obj.ui.avatar:setProfile (profile_ptr)
	obj.ui.nickname:setText (profile:getNickname())

	return obj
end

function wrapper:set_message (msg, when, pending)
	self.ui.message_counter:setHidden (pending == 0)
	
	self.ui.short_message:setElide (true)
	self.ui.short_message:setText (msg)
	self.ui.message_counter:setPendingNotifications (pending)
	self.ui.when:setText (ui.format.time_passed (os.time(), when))
end

ui.widgets = ui.widgets or { }
ui.widgets.recent_contact_button = wrapper

