require "ui/config.lua"
require "ui/text_styles.lua"

local widget = 
{
}

local image_map = 
{
	[underworld2.SkillTypeCouriers] = "ui/upgrades/icons/smug_couriers:filtered",
	[underworld2.SkillTypeItem] 	= "ui/upgrades/icons/commodities_item1:filtered",
	[underworld2.SkillTypeStorage] 	= "ui/upgrades/icons/smug_storage:filtered",
}

local courier_skill = { }

function courier_skill.create (player, skill)
	local image = ui.Picture()
	
	local layout = ui.VerticalLayout()
	layout:addWidget (ui.Widget())
	
	local params_layout = ui.HorizontalLayout()
	params_layout:setVerticalPolicy (ui.SizePolicyMinimum)
	params_layout:setItemSpacing (ui.config.hspacing)
	
	local courier_icon = ui.Picture()
	courier_icon:setTexture ("ui/ico_courier:filtered")
	courier_icon:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	params_layout:addWidget (courier_icon)
	
	local amount_label = ui.TextLabel()
	amount_label:setTextStyle (ui.text_styles.small_hl.build())
	params_layout:addWidget (amount_label)
	
	layout:addItem (params_layout)
	
	image:setLayout (layout)
	
	local obj = setmetatable ({ widget = image, skill = skill, amount_label = amount_label }, {__index = courier_skill})
	obj:refresh (player)
	return obj
end

function courier_skill:refresh (player)
	local skill = self.skill
	local level_id = player:getSkillLevel (skill:getId())
	local ico_id = string.format ("ui/crew/nav/ch_runner_nav_%d:filtered", level_id)
	local level = skill:getLevel (level_id)
	self.widget:setTexture (ico_id)
	self.amount_label:setText (level:get() and tostring (skill:getLevel (level_id):getValue()) or "")
end

local commodity_skill = { }

function commodity_skill.create (player, skill, tag)
	local image = ui.Picture()
	
	local layout = ui.VerticalLayout()
	layout:addWidget (ui.Widget())
	
	local params_layout = ui.HorizontalLayout()
	params_layout:setVerticalPolicy (ui.SizePolicyMinimum)
	layout:addItem (params_layout)
	
	local create_level_widget = function (ico_id)
		local ico = ui.Picture()
		ico:setTexture (ico_id)
		ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		return ico
	end
	
	local obj = setmetatable ({widget = image, skill = skill, levels = { }, }, {__index = commodity_skill})
	
	for i, icon_id in ipairs (ui.config.commodity_small_icons) do
		local w = create_level_widget (ui.config.commodity_small_icons[i])
		params_layout:addWidget (w)
		obj.levels[i] = w
	end
	
	params_layout:addWidget (ui.Widget())
	
	image:setLayout (layout)
	
	obj:refresh (player)	
	return obj
end

function commodity_skill:refresh (player)
	local skill = self.skill
	local level_id = player:getSkillLevel (skill:getId())
	local ico_id = string.format ("ui/crew/nav/ch_supplier_nav_%d:filtered", level_id)
	self.widget:setTexture (ico_id)
	
	for i = 1, 8 do
		self.levels[i]:setVisible ((i - 1) <= level_id)
	end
end

local accountant_skill = { }

function accountant_skill.create (player, skill)
	local image = ui.Picture()
	
	local layout = ui.VerticalLayout()
	layout:addWidget (ui.Widget())
	
	local params_layout = ui.HorizontalLayout()
	params_layout:setItemSpacing (ui.config.hspacing)
	params_layout:setVerticalPolicy (ui.SizePolicyMinimum)
	layout:addItem (params_layout)
	
	local cash_ico = ui.Picture()
	cash_ico:setTexture ("ui/ico_cash_small:filtered")
	cash_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	params_layout:addWidget (cash_ico)
		
	local value_label = ui.TextLabel()
	value_label:setTextStyle (ui.text_styles.small_hl.build())
	params_layout:addWidget (value_label)
	
	image:setLayout (layout)
	
	local obj = setmetatable ({widget = image, skill = skill, value_label = value_label}, {__index = accountant_skill})
	obj:refresh (player)
	return obj
end

function accountant_skill:refresh (player)
	local skill = self.skill
	local level_id = player:getSkillLevel (skill:getId())
	local ico_id = string.format ("ui/crew/nav/ch_accountant_nav_%d:filtered", level_id)
	local level = skill:getLevel (level_id)
	self.value_label:setText (level:get() and ui.format.currency_string (skill:getLevel (level_id):getValue()) or "")
	self.widget:setTexture (ico_id)
end

local skill_map = { 
	["COURIERS"] = courier_skill,
	["ITEM"] 	= commodity_skill,
	["WALLET"] 	= accountant_skill,
}

function widget.create (player, skill, tag)
	local strings = underworld2.StringsDatabase()
	
	local frame = ui.FrameButton()
	frame:setTag (tag)
	frame:setTexture (ui.Button_ButtonStateNormal, "ui/equipment/frame_item_slot:filtered")
	frame:setTexture (ui.Button_ButtonStatePressed, "ui/equipment/frame_item_selected:filtered")
	frame:setSound ("ui/UI_Click_05_Light.wav")

	local margins = { x = frame:getLeftMargin() * 0.5, y = frame:getTopMargin() * 0.5 }
	frame:setMargins (margins.x, margins.x, margins.y, margins.y)
	
	local frame_layout = ui.VerticalLayout()
	frame_layout:setItemSpacing(ui.config.vspacing)
	
	local hlayout = ui.HorizontalLayout()
	hlayout:setVerticalPolicy(ui.SizePolicyMinimum)
	local notification_badge
	do
		local title_label = ui.TextLabel()
		title_label:setTextStyle (ui.text_styles.xsmall_hl.build())
		title_label:setText (strings:get (skill:getTitleKey()))
		title_label:setVerticalSizePolicy (ui.SizePolicyMinimum)
		title_label:setWordWrap (false)
		title_label:setTextAlignment(ui.AlignmentVerticalTop)
		hlayout:addWidget (title_label)

		local spacer = ui.Widget()
		spacer:setVerticalSizePolicy(ui.SizePolicyMinimum)
		hlayout:addWidget(spacer)

		local badge = ui.NotificationBadge()
 		badge:setBadgeTexture ("ui/equipment/ico_item_notification:filtered")

 		badge:setBadgeTextStyle (ui.text_styles.badge_large.build())
 		badge:setHorizontalSizePolicy (ui.SizePolicyMinimum)
 		badge:setVerticalSizePolicy (ui.SizePolicyMinimum)
 		badge:setDisplayCount(false)
 		hlayout:addWidget (badge)
 		notification_badge = badge

 		badge:setPendingNotifications( not underworld2.upgrades_tracker.has_new_items { 
			level = underworld2.player:getLevel(), 
			category = "skills",
			subcategory = skill:getId() 
		} and 0 or 1)
	end

	frame_layout:addItem(hlayout)
	
	local state = skill_map[skill:getType()].create (player, skill)
	frame_layout:addWidget (state.widget)
	
	frame:setLayout (frame_layout)

	local w = ui.Widget()
	w:addChild(frame)

	local overlay = ui.Frame()
	overlay:setBordersTexture ("ui/equipment/frame_nav_darken:filtered")

	w:addChild(overlay)
	
	local obj = setmetatable ({ state = state, widget = w, button = frame, overlay = overlay, skill = skill, notification_badge = notification_badge }, { __index = widget })
		
	return obj
end

function widget:refresh (player)
	self.state:refresh (player)
end

ui.skill_widget = widget