require "ui/config.lua"

local controller = 
{
	PRODUCT_TAG_DEFAULT		= 0,
	PRODUCT_TAG_POPULAR		= 1,
	PRODUCT_TAG_LIMITED		= 2,

	PRODUCT_CATEGORY_PAID  	= 0,
	PRODUCT_CATEGORY_OFFER	= 1,
}

local ribbon_offset

local tag_string_keys = {
	[controller.PRODUCT_TAG_POPULAR] = "IAPP_STORE_POPULAR",
	[controller.PRODUCT_TAG_LIMITED] = "IAPP_STORE_LIMITED"
}

local tag_textures = {
	[controller.PRODUCT_TAG_DEFAULT] = "ui/bank/ribbon_bank_purple:filtered",
	[controller.PRODUCT_TAG_POPULAR] = "ui/bank/ribbon_bank_purple:filtered",
	[controller.PRODUCT_TAG_LIMITED] = "ui/bank/ribbon_bank_red:filtered"
}

function controller:update (dt)
	if self.limited then
		local now = underworld2.server_time()
		local time_left = self.product.lifetime - now
		self.ui.limited_timer:setText (ui.format.duration_string (time_left))
		if time_left <= 0 then
			self.ui.limited:setHidden (true)
			self.limited = false
		end
	end
end

local function __get_default_tag_text (tag)
	local text = tag_string_keys[tag]
	return text and L (text)
end

function controller.create (product, handler)

	local self = setmetatable ({ product = product }, { __index = controller })

	local widget = ui.load_ui ("ui/store/gold.ui")
	
	local now = underworld2.server_time()

	self.ui = ui.build_children_map (widget)
	self.ui.frame:setMargins (0, 0, 0, 0)
	self.ui.title:setText (product.description)
	self.ui.product_icon:setTexture (product.icon_uri..":filtered")
	self.ui.amount:setText (tostring (product.value or L"IAPP_OFFER_FREE"))
	
	local limited = (product.lifetime and product.lifetime > now) or false
	local gratis = product.gratis or 0

	self.limited = true

	if limited then
		local gratis = product.gratis or 0
		self.ui.limited_gratis:setText (gratis > 0 and L ("IAPP_STORE_GRATIS", product.gratis) or " ")
		
		local time_left = product.lifetime - now
		self.ui.limited_timer:setText (ui.format.duration_string (time_left))


		self.ui.gratis:setHidden (true)
		self.ui.limited:setHidden (false)
	elseif gratis > 0 then
		self.ui.gratis:setText (L ("IAPP_STORE_GRATIS", product.gratis))
		self.ui.gratis:setHidden (false)
		self.ui.limited:setHidden (true)
	else
		self.ui.gratis:setHidden (true)
		self.ui.limited:setHidden (true)
	end

	local price = L "IAPP_STORE_DEFAULT_BUY"
	if product.price_string and #product.price_string > 0 then
		price = product.price_string
	end

	self.ui.buy:setText (price)

	local __, tag = next (product.tags)
	if tag then
		self.ui.ribbon:setHidden (false)
	
		local text = product.label or __get_default_tag_text (tag) or ""
		self.ui.tag:setText (text)
	
		local ribbon = tag_textures[tag]
		if ribbon then self.ui.ribbon:setTexture (ribbon) end
	end

	events.connect (self.ui.buy:getClickedSignalProxy(), function() handler (product) end)

	return self
end

ui.store_gold_item_controller = controller
