require "ui/config.lua"

local controller = 
{
	PRODUCT_TYPE_DEFAULT	= 0,
	PRODUCT_TYPE_VIDEO		= 1,
	PRODUCT_TYPE_SPECIAL	= 2,
	PRODUCT_TYPE_INVITE		= 3,
}

local ribbon_offset

local offer_button_keys = {
	[controller.PRODUCT_TYPE_DEFAULT] 	= "IAPP_OFFER_GET_FREE_GOLD",
	[controller.PRODUCT_TYPE_VIDEO] 	= "IAPP_OFFER_WATCH",
	[controller.PRODUCT_TYPE_SPECIAL] 	= "IAPP_OFFER_COMPLETE_OFFER",
	[controller.PRODUCT_TYPE_INVITE] 	= "IAPP_OFFER_INVITE"
}


function controller:update (dt)
	if self.limited then
		local now = underworld2.server_time()
		local time_left = self.product.lifetime - now
		self.ui.limited_timer:setText (ui.format.duration_string (time_left))
		if time_left <= 0 then
			self.ui.limited:setHidden (true)
			self.limited = false
		end
	end
end

local function __get_default_tag_text (tag)
	local text = tag_string_keys[tag]
	return text and L (text)
end

function controller.create (product, handler)

	local self = setmetatable ({ product = product }, { __index = controller })

	local widget = ui.load_ui ("ui/store/free_offer.ui")
	
	local now = underworld2.server_time()

	self.ui = ui.build_children_map (widget)
	self.ui.frame:setMargins (0, 0, 0, 0)
	self.ui.product_icon:setTexture (product.icon_uri..":filtered")
	self.ui.amount:setText (tostring (product.value or L"IAPP_OFFER_FREE"))
	self.ui.description:setText (product.description)
	self.ui.film:setHidden (product.type ~= controller.PRODUCT_TYPE_VIDEO)

	self.ui.complete:setText (L (offer_button_keys [product.type]))

	events.connect (self.ui.complete:getClickedSignalProxy(), function() handler (product) end)

	return self
end

ui.store_offer_item_controller = controller
