require "ui/config.lua"

local controller = { }

local ribbon_offset

local tag_string_keys = {
	["POPULAR"] = "IAPP_STORE_POPULAR",
	["LIMITED"] = "IAPP_STORE_LIMITED"
}

local tag_textures = {
	["DEFAULT"] = "ui/bank/ribbon_bank_purple:filtered",
	["POPULAR"] = "ui/bank/ribbon_bank_purple:filtered",
	["LIMITED"] = "ui/bank/ribbon_bank_red:filtered"
}

local lock_anim_keyframes = 
{
	t = { 0.0, 0.3, 0.6, 0.8, 1.0 },
	offset = { 0.0, 0.4, -0.2, 0.2, 0.0 } 
}

function controller:play_lock_animation()
	if self.lock_anim then return end

	local lock = math.cspline (lock_anim_keyframes.t, lock_anim_keyframes.offset)
	local max_offset = self.ui.lock:getWidth() * 0.05
	local anim = anim.lerp
	{
		from = 0.0, to = 1.0, time = 0.2,
		object = self,
		property = function (self, t)
			self.ui.lock:setOffset (math.vec2 (max_offset * lock (t), 0))
		end,
		on_finished = function() self.lock_anim = nil end
	}
	self.lock_anim = anim
end

function controller:update (dt)
	if self.lock_anim then self.lock_anim:update (dt) end
end

function controller.create (product, handler)

	local self = setmetatable ({ id = product:getId(), product = product }, { __index = controller })

	local widget = ui.load_ui ("ui/store/premium.ui")
	
	self.ui = ui.build_children_map (widget)
	self.ui.frame:setMargins (0, 0, 0, 0)
	self.ui.title:setText (product:getTitle())
	self.ui.description:setText (product:getDescription())
	self.ui.product_icon:setTexture ("ui/icons/premium_items/"..product:getSplash()..":filtered")
	self.ui.buy:setPrice (tostring (product:getPrice()))

	local tag = product:getTag()
	if tag_string_keys[tag] then
		self.ui.tag:setText (L (tag_string_keys[tag]))
		self.ui.ribbon:setTexture (tag_textures[tag])
	else
		self.ui.ribbon:setHidden (true)
	end

	local player_level_id = underworld2.player:getLevel()
	local level_required = product:getRequiredLevel()
	if player_level_id < level_required then
		self.ui.product_icon:setCustomEffect ("grayscale")
		self.ui.rays:setCustomEffect ("grayscale")
		self.ui.level_required:setText ("LVL "..tostring (level_required))
		
		self.ui.buy:setHidden (true)
		self.ui.free:setHidden (true)
		self.ui.buy_locked:setHidden (false)
		
		self.locked = true
	else
		self.ui.lock:setHidden (true)
	end

	events.connect (self.ui.buy_locked:getClickedSignalProxy(), function()
		get_sound_pool():play ("ui/UI_Error.wav")
		self:play_lock_animation() 
	end)

	events.connect (self.ui.buy:getClickedSignalProxy(), function()
		handler (product, self.certificate_id)
	end)

	events.connect (self.ui.free:getClickedSignalProxy(), function()
		handler (product, self.certificate_id)
	end)

	return self
end

function controller:on_certificates_updated (certificates)
	if self.locked then return end

	local is_free = false
	local text = ""
	self.certificate_id = nil
	for i, v in pairs (certificates) do
		if v.store_id == self.id then
			is_free = true
			text = v.reason or ""
			self.certificate_id = v.id
		end
		break
	end
	self.ui.free:setText (text)
	self.ui.free:setHidden (not is_free)
	self.ui.buy:setHidden (is_free)
end

ui.store_premium_item_controller = controller
