require "ui/config.lua"
require "ui/text_styles.lua"

ui.widgets = ui.widgets or { }
ui.widgets.upgrade_description =
{

}

local skill_style =
{
	["STORAGE"] = 
	{
		effect_icon = "ui/ico_box_small",
	},
	["NETWORK_SIZE"] = 
	{
		effect_icon = "ui/ico_world_view",
	},
	["COURIERS"] = 
	{
		effect_icon = "ui/ico_courier",
	},
}

local upgrade_description = ui.widgets.upgrade_description

local get_skill_by_id = function (id)
	local groups = underworld2.skill_groups
	for k, group in ipairs (groups) do
		for i, skill in ipairs (group.skills) do
			if skill.id == id then
				return skill
			end
		end
	end
end

function upgrade_description.create (skill_id, level)
	local obj = setmetatable ( { ui = { }, tm = 0 }, { __index = upgrade_description } )

	local skill = get_skill_by_id (skill_id)
	local level_desc = skill.levels[level]

	local frame = ui.Frame()
	frame:setBordersTexture ("ui/upgrades/details_background")

	local vlayout = ui.VerticalLayout()
	vlayout:setItemSpacing (ui.config.vspacing)

	local title = ui.TextLabel()
	title:setTextStyle (ui.text_styles.normal_hl.build())
	title:setText (level_desc.title)
	vlayout:addWidget (title)

	do
		local hlayout = ui.HorizontalLayout()
		hlayout:setItemSpacing (ui.config.hspacing)
		local max_level = skill.levels.max_level

		obj.ui.levels = { }

		for i = 1, max_level do
			local lvl = ui.Picture()
			lvl:setTexture (i <= level and "ui/upgrades/indicator_tier_large_on" or "ui/upgrades/indicator_tier_large")
			hlayout:addWidget (lvl)

			obj.ui.levels[i] = lvl
		end

		local spacer = ui.Widget()
		spacer:setHorizontalSizePolicy (ui.SizePolicyMaximum)
		hlayout:addWidget (spacer)
		vlayout:addItem (hlayout)

		if skill_style[skill.type] then
			local rid = skill_style[skill.type].effect_icon

			local old_capacity = skill.levels[level - 1].value or 0
			local new_capacity = skill.levels[level].value

			local effect_ico = ui.Picture()
			effect_ico:setTexture (rid)
			hlayout:addWidget (effect_ico)

			local label = ui.TextLabel()
			label:setTextStyle (ui.text_styles.small_hl.build())
			label:setText (tostring (old_capacity))
			label:setWordWrap (false)
			hlayout:addWidget (label)

			local ico_arrow = ui.Picture()
			ico_arrow:setTexture ("ui/arrow_result")
			hlayout:addWidget (ico_arrow)

			local effect_ico = ui.Picture()
			effect_ico:setTexture (rid)
			hlayout:addWidget (effect_ico)

			local label = ui.TextLabel()
			label:setTextStyle (ui.text_styles.small_hl.build())
			label:setText (tostring (new_capacity))
			label:setWordWrap (false)
			hlayout:addWidget (label)
		end
	end

	local desc = ui.TextLabel()
	desc:setTextStyle (ui.text_styles.xsmall.build())
	desc:setText (level_desc.description)
	desc:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalTop)
	desc:setVerticalSizePolicy (ui.SizePolicyMaximum)
	vlayout:addWidget (desc)

	do

		local price_layout = ui.HorizontalLayout()
		price_layout:addWidget (ui.Widget())

		local hint_label = ui.TextLabel()
		hint_label:setTextStyle (ui.text_styles.small_hl.build())
		hint_label:setText (L "KEY_UPGRADE_PRICE")
		hint_label:setWordWrap (false)
		hint_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		price_layout:addWidget (hint_label)
		obj.ui.hint_label = hint_label

		local hint_ico = ui.Picture()
		hint_ico:setTexture ("ui/ico_cash_small")
		hint_ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		price_layout:addWidget (hint_ico)
		obj.ui.hint_ico = hint_ico

		local hint_value_label = ui.TextLabel()
		hint_value_label:setTextStyle (ui.text_styles.small_hl.build())
		hint_value_label:setText ("")
		hint_value_label:setWordWrap (false)
		hint_value_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
		price_layout:addWidget (hint_value_label)
		obj.ui.hint_value_label = hint_value_label

		price_layout:addWidget (ui.Widget())

		vlayout:addItem (price_layout)
	end

	local button = ui.TextButton()
	button:setTexture ("ui/btn_lower_normal:filtered")
	button:setTextStyle (ui.text_styles.button_regular.build())
	button:setText (L "KEY_UPGRADE_BUY_UPGRADE")
	vlayout:addWidget (button)

	frame:setLayout (vlayout)

	obj.ui.widget = frame

	obj.ui.conditions = conditions
	obj.ui.confirm = button

	obj.skill_id = skill_id
	obj.level = level


	underworld2.profile.add_delegate (obj)

	obj:profile_updated(underworld2.profile)

	return obj
end

function upgrade_description:profile_stats_updated() end

function upgrade_description:profile_updated (profile)
	local skill = get_skill_by_id (self.skill_id)
	local level_desc = skill.levels[self.level]

	local required_player_level = level_desc.player_level_requirement
	local current_level = profile.get_skill_level (skill.id)

	self.current_level = current_level

	if current_level >= self.level then
		self.ui.hint_label:setHidden (false)
		self.ui.hint_ico:setHidden (true)
		self.ui.hint_value_label:setHidden (true)
		self.ui.confirm:setHidden (true)

		self.ui.hint_label:setTextStyle (ui.text_styles.upgrade_state_owned.build())
		self.ui.hint_label:setText (L "KEY_UPGRADE_OWNED")

	elseif (underworld2.profile.level >= required_player_level) and (current_level + 1) == self.level then
		self.ui.hint_label:setHidden (false)
		self.ui.hint_ico:setHidden (false)
		self.ui.hint_value_label:setHidden (false)
		self.ui.confirm:setHidden (false)

		self.ui.hint_label:setTextStyle (ui.text_styles.upgrade_state_available.build())
		self.ui.hint_value_label:setTextStyle (ui.text_styles.upgrade_state_available.build())

		self.ui.hint_label:setText (L "KEY_UPGRADE_PRICE")
		self.ui.hint_value_label:setText (tostring (underworld2.levels_description[profile.level + 1].cost))

	else
		self.ui.hint_label:setHidden (false)
		self.ui.hint_ico:setHidden (true)
		self.ui.hint_value_label:setHidden (true)
		self.ui.confirm:setHidden (true)

		self.ui.hint_label:setTextStyle (ui.text_styles.upgrade_state_not_available.build())

		self.ui.hint_label:setText (L ("KEY_UPGRADE_LEVEL_REQUIRED", required_player_level))

	end
end


function upgrade_description:update (dt)
	self.tm = self.tm + dt
	local min_alpha = 0.2
	local alpha = math.abs (self.tm % 1 - 0.5) * 2.0 * (1.0 - min_alpha) + min_alpha
	local color = render.color (255, 255, 255, 255 * alpha)
	for i, v in pairs (self.ui.levels) do
		if i > self.current_level then
			v:setColor (color)
		else
			v:setColor (render.color_white)
		end
	end
end

function upgrade_description:deal_completed() end

function upgrade_description:destroy()
	underworld2.profile.remove_delegate (self)
end

