
local upgrade_widget = { 
	STATE_NOT_AVAILABLE = 0,
	STATE_AVAILABLE = 1,
	STATE_IN_DELIVERY = 2,
	STATE_OWNED = 3,
	STATE_CONFIRM = 4,
}

local btn_colors =
{
	buy = {
		primary0 = render.color (38, 228, 96, 255),
		secondary0 = render.color (15, 168, 23, 255),
		primary1 = render.color (38, 228, 96, 255),
		secondary1 = render.color (15, 168, 23, 255),
	},
	buy_inactive = 
	{
		--[[primary0 = render.color (255, 190, 56, 255),
		secondary0 = render.color (209, 97, 10, 255),
		primary1 = render.color (255, 190, 56, 255),
		secondary1 = render.color (209, 97, 10, 255),]]
		primary0    = render.color (145, 138, 133),
		secondary0  = render.color (102, 97, 94),
		primary1    = render.color (145, 138, 133),
		secondary1  = render.color (102, 97, 94),
	}
}

upgrade_widget.skill_desc = 
{
	["COURIERS"] = {
		buy = L"KEY_CREW_HIRE",
		confirm = "Add To Team",
		delivery = "New Recruit In",
		owned = L"KEY_CREW_HIRED",
		get_image_id = function (level_id) return string.format ("ui/crew/icons/ch_runner_%d:filtered", level_id) end,
		add_effect_icons = function (layout, level_id)
			local ico = ui.Picture()
			ico:setTexture ("ui/ico_courier:filtered")
			ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			layout:addWidget (ico)
		end,
		display_value = function (value, owned) return owned and tostring (value) or string.format ("+%d", value) end
	},
	["ITEM"] = {
		buy = L"KEY_CREW_HIRE",
		confirm = "Add To Team",
		delivery = "New Recruit In",
		owned = L"KEY_CREW_HIRED",
		get_image_id = function (level_id) return string.format ("ui/crew/icons/ch_supplier_%d:filtered", level_id) end,
		add_effect_icons = function (layout, level_id)
			if level_id == 1 then
				local ico1 = ui.Picture()
				ico1:setTexture (ui.config.commodity_small_icons[1])
				ico1:setHorizontalSizePolicy (ui.SizePolicyMinimum)
				layout:addWidget (ico1)
				
				local ico2 = ui.Picture()
				ico2:setTexture (ui.config.commodity_small_icons[2])
				ico2:setHorizontalSizePolicy (ui.SizePolicyMinimum)
				layout:addWidget (ico2)
			else			
				local ico = ui.Picture()
				--level ids start from 1, also first level gives player two commodities at once
				ico:setTexture (ui.config.commodity_small_icons[level_id + 1])
				ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
				layout:addWidget (ico)
			end
		end,
		display_value = function (value, owned) return "" end
	},
	["WALLET"] = {
		buy = L"KEY_CREW_HIRE",
		confirm = "Add To Team",
		delivery = "New Recruit In",
		owned = L"KEY_CREW_HIRED",
		get_image_id = function (level_id) return string.format ("ui/crew/icons/ch_accountant_%d:filtered", level_id) end,
		add_effect_icons = function (layout, level_id)
			local ico = ui.Picture()
			ico:setTexture ("ui/ico_cash_small:filtered")
			ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			layout:addWidget (ico)
		end,
		display_value = function (value, owned) return (owned and "" or "+")..ui.format.digit (value) end
	},
}


local function create_btn (text, colors)
	local btn = ui.UniversalButton()
	btn:setTexture ("ui/equipment/btn_64_universal:filtered")
	btn:setStretch (ui.UniversalButton_StretchHorizontal)
	btn:setPrimaryColor (ui.Button_ButtonStateNormal, colors.primary0)
	btn:setSecondaryColor (ui.Button_ButtonStateNormal, colors.secondary0)
	btn:setPrimaryColor (ui.Button_ButtonStatePressed, colors.primary1)
	btn:setSecondaryColor (ui.Button_ButtonStatePressed, colors.secondary1)

	local layout = ui.VerticalLayout()

	local text_label = ui.TextLabel()
	text_label:setTextStyle (ui.text_styles.button_regular.build())
	text_label:setText (text)
	text_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
	layout:addWidget (text_label)

	btn:setLayout (layout)

	return btn
end

local function create_price_btn (text, price, ico_id, colors)
	local btn = ui.UniversalButton()
	btn:setTexture ("ui/equipment/btn_64_universal:filtered")
	btn:setStretch (ui.UniversalButton_StretchHorizontal)
	btn:setPrimaryColor (ui.Button_ButtonStateNormal, colors.primary0)
	btn:setSecondaryColor (ui.Button_ButtonStateNormal, colors.secondary0)
	btn:setPrimaryColor (ui.Button_ButtonStatePressed, colors.primary1)
	btn:setSecondaryColor (ui.Button_ButtonStatePressed, colors.secondary1)

	local layout = ui.HorizontalLayout()
	layout:setItemSpacing (ui.config.hspacing)

	layout:addWidget (ui.Widget())

	local text_label = ui.TextLabel()
	text_label:setTextStyle (ui.text_styles.button_regular.build())
	text_label:setText (text)
	text_label:setWordWrap (false)
	text_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (text_label)

	local ico = ui.Picture()
	ico:setTexture (ico_id)
	ico:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (ico)

	local price_label = ui.TextLabel()
	price_label:setTextStyle (ui.text_styles.button_regular.build())
	price_label:setText (tostring (price))
	price_label:setWordWrap (false)
	price_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	layout:addWidget (price_label)

	layout:addWidget (ui.Widget())

	btn:setLayout (layout)

	return btn
end

function upgrade_widget.create (skill, level_id, handlers)
	local strings = underworld2.StringsDatabase()
	
	local desc = upgrade_widget.skill_desc[skill:getType()]
	
	local skill_level = skill:getLevel (level_id)
	
	local obj = setmetatable ({ skill = skill, level = skill_level, delivery_time = 0 }, { __index = upgrade_widget } )
	
	local frame = ui.Frame()
	frame:setVerticalSizePolicy (ui.SizePolicyMinimum)
	frame:setBordersTexture ("ui/equipment/frame_28dark:filtered")
	
	local margins = { x = frame:getLeftMargin() * 0.5, y = frame:getTopMargin() * 0.5 }
	frame:setMargins (margins.x, margins.x, margins.y, margins.y)
	
	local hlayout = ui.HorizontalLayout()
	hlayout:setItemSpacing (ui.config.hspacing)
	
	local upgrade = ui.UpgradeWidget()
	upgrade:setImage (desc.get_image_id (level_id))
	upgrade:setLevelRequired (skill_level:getLevelRequired())
	upgrade:setHorizontalSizePolicy (ui.SizePolicyMinimum)
	upgrade:setLocked (false)
	obj.upgrade = upgrade
	
	hlayout:addWidget (upgrade)
	
	do
		local vlayout = ui.VerticalLayout()
		vlayout:setItemSpacing (ui.config.vspacing)
		
		do
			local title_layout = ui.HorizontalLayout()
			title_layout:setVerticalPolicy (ui.SizePolicyMinimum)
			title_layout:setItemSpacing(ui.config.hspacing)
			
			local title_label = ui.TextLabel()
			title_label:setTextStyle (ui.text_styles.small_hl.build())
			title_label:setText (strings:get (skill_level:getTitleKey()))
			title_layout:addWidget (title_label)
			
			desc.add_effect_icons (title_layout, level_id)
			
			local value_label = ui.TextLabel()
			value_label:setWordWrap (false)
			value_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
			value_label:setTextStyle (ui.text_styles.small_hl.build())
			title_layout:addWidget (value_label)
			obj.value_label = value_label
			
			vlayout:addItem (title_layout)			
		end
		
		local desc_label = ui.TextLabel()
		desc_label:setTextStyle (ui.text_styles.xsmall.build())
		desc_label:setText (strings:get (skill_level:getDescriptionKey()))
		desc_label:setTextAlignment (ui.AlignmentHorizontalLeft + ui.AlignmentVerticalTop)
		desc_label:setVerticalSizePolicy (ui.SizePolicyMaximum)
		vlayout:addWidget (desc_label)
		
		do
			local actions_layout = ui.HorizontalLayout()
			actions_layout:setItemSpacing (ui.config.hspacing)
			
			local buy_button = create_price_btn (desc.buy, ui.format.digit (skill_level:getCost()),  "ui/ico_cash_small:filtered", btn_colors.buy)
			buy_button:setTag (level_id)
			actions_layout:addWidget (buy_button)
			
			local confirm_button = create_btn (desc.confirm, btn_colors.buy)
			confirm_button:setTag (level_id)
			actions_layout:addWidget (confirm_button)
			
			do
				local delivery_widget = ui.Widget()
			
				local delivery_layout = ui.VerticalLayout()
				delivery_layout:setItemSpacing (ui.config.vspacing)
				
				local delivery_hint_label = ui.TextLabel()
				delivery_hint_label:setTextStyle (ui.text_styles.xsmall.build())
				delivery_hint_label:setText (desc.delivery)
				delivery_hint_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
				delivery_layout:addWidget (delivery_hint_label)
				
				local delivery_bar = ui.ProgressBar()
				delivery_bar:setTexture ("ui/bar_delivery_side:filtered")
				delivery_bar:setTextStyle (ui.text_styles.xsmall.build())
				delivery_layout:addWidget (delivery_bar)
				delivery_layout:addWidget (delivery_bar)
				
				obj.delivery_bar = delivery_bar
				obj.delivery_widget = delivery_widget
				
				delivery_widget:setLayout (delivery_layout)
				
				actions_layout:addWidget (delivery_widget)
			end
			
			do
				local owned_frame = ui.Widget()
				--owned_frame:setBordersTexture ("ui/window_popup:filtered")
				
				local owned_layout = ui.HorizontalLayout()
				owned_layout:setItemSpacing (ui.config.hspacing)
				
				owned_layout:addWidget (ui.Widget())
				
				local owned_label = ui.TextLabel()
				owned_label:setTextStyle (ui.text_styles.normal_hl.build())
				owned_label:setWordWrap (false)
				owned_label:setText (desc.owned)
				owned_label:setHorizontalSizePolicy (ui.SizePolicyMinimum)
				
				local text_height = owned_label:getPreferredSize().y

				local tick_pic = ui.Picture()
				tick_pic:setTexture("ui/crew/ico_tick_large:filtered")
				local pic_size = tick_pic:getPreferredSize()

				tick_pic:setVerticalSizePolicy(ui.SizePolicyFixed)
				tick_pic:setHorizontalSizePolicy(ui.SizePolicyFixed)
				tick_pic:setPlacingMode(ui.Picture_PlacingModeFit)

				tick_pic:setSize(math.vec2(pic_size.x * text_height / pic_size.y,text_height))
				tick_pic:setTexture("ui/crew/ico_tick_large:filtered")

				owned_layout:addWidget (tick_pic)
				owned_layout:addWidget (owned_label)
				
				owned_layout:addWidget (ui.Widget())
				
				owned_frame:setLayout (owned_layout)
				
				actions_layout:addWidget (owned_frame)
				
				obj.owned_frame = owned_frame
			end
			
			local boost_button = ui.PriceButton()
			boost_button:setText (L "KEY_UPGRADE_BOOST")
			boost_button:setPrice ("123")
			boost_button:setTextStyle (ui.text_styles.button_regular.build())
			boost_button:setStateTexture ("ui/btn_premium:filtered")
			boost_button:setCurrencyIcon ("ui/ico_gold_small:filtered")
			boost_button:setTag (level_id)
			actions_layout:addWidget (boost_button)
			
			vlayout:addItem (actions_layout)
			
			obj.buttons =
			{
				buy 	= buy_button,
				confirm = confirm_button,
				boost	= boost_button
			}
			
			events.connect (obj.buttons.buy:getClickedSignalProxy(), handlers.buy)
			events.connect (obj.buttons.confirm:getClickedSignalProxy(), handlers.confirm)
			events.connect (obj.buttons.boost:getClickedSignalProxy(), handlers.boost)
			
		end
		hlayout:addItem (vlayout)
	end
	
	frame:setLayout (hlayout)
	
	local prev_level = skill:getLevel (level_id - 1)
	obj.level_value = skill:getLevel (level_id):getValue() - (prev_level:get() and prev_level:getValue() or 0)
	
	obj.widget = frame
	
	return obj
end

function upgrade_widget:set_delivery_time (time, total_time)
	self.delivery_time = time
end

function upgrade_widget:set_state (state, is_new_item)
	self.state = state
	local buttons = self.buttons
	
	if state == upgrade_widget.STATE_AVAILABLE then
		local btn, colors = buttons.buy, btn_colors.buy
		
		btn:setVisible (true)
		btn:setEnabled (true)
		
		btn:setPrimaryColor (ui.Button_ButtonStateNormal, colors.primary0)
		btn:setSecondaryColor (ui.Button_ButtonStateNormal, colors.secondary0)
		btn:setPrimaryColor (ui.Button_ButtonStatePressed, colors.primary1)
		btn:setSecondaryColor (ui.Button_ButtonStatePressed, colors.secondary1)
		
	elseif state == upgrade_widget.STATE_NOT_AVAILABLE then
		local btn, colors = buttons.buy, btn_colors.buy_inactive
		
		btn:setVisible (true)
		btn:setEnabled (true)
		
		
		btn:setPrimaryColor (ui.Button_ButtonStateNormal, colors.primary0)
		btn:setSecondaryColor (ui.Button_ButtonStateNormal, colors.secondary0)
		btn:setPrimaryColor (ui.Button_ButtonStatePressed, colors.primary1)
		btn:setSecondaryColor (ui.Button_ButtonStatePressed, colors.secondary1)
	else
		buttons.buy:setVisible (false)
	end
	
	self.delivery_widget:setVisible (state == upgrade_widget.STATE_IN_DELIVERY)
	buttons.boost:setVisible (state == upgrade_widget.STATE_IN_DELIVERY)
	
	self.owned_frame:setVisible (state == upgrade_widget.STATE_OWNED)
		
	self.upgrade:setLocked (state == upgrade_widget.STATE_NOT_AVAILABLE)
	self.upgrade:setIsNewItem(is_new_item or false)
	
	buttons.confirm:setVisible (state == upgrade_widget.STATE_CONFIRM)
	
	local desc = upgrade_widget.skill_desc[self.skill:getType()]
	
	self.value_label:setText (desc.display_value (self.level_value, state == upgrade_widget.STATE_OWNED))

	if state == upgrade_widget.STATE_OWNED then
		self.value_label:setTextStyle (ui.text_styles.small_hl.build())
	else
		self.value_label:setTextStyle (ui.text_styles.positive_value.build())
	end
end

function upgrade_widget:harlem_shake()
	self.upgrade:startShakeAnimation()
end

function upgrade_widget:set_pending_skill (pending_skill)
	self.pending_skill = pending_skill
end

function upgrade_widget:update (server_time)
	if self.state == upgrade_widget.STATE_IN_DELIVERY then
		local pending_skill = self.pending_skill
		local start_at = pending_skill:getDeliveryStartTime()
		local finish_at = pending_skill:getDeliveryEndTime()
		local time_left = finish_at - server_time 
		if (time_left <= 0) then
			self.pending_skill = nil
			self:set_state (upgrade_widget.STATE_CONFIRM)
		else
			local bar = self.delivery_bar
			
			bar:setValue ((server_time - start_at) / (finish_at - start_at))
			bar:setText (ui.format.duration_string (time_left))			
			self.buttons.boost:setPrice (tostring (pending_skill:getDeliveryPrice (server_time)))
		end
	end
end

ui.upgrade_widget = upgrade_widget

