local game_events = 
{
	events = { }
}

function game_events:build (id, priority, routine, allow_multiple)
	return {
				owner = self,

				id 			= id, 
				priority 	= priority,
				start 		= routine,
				finalize 	= function (self) self.owner.current_event = nil end,
				allow_multiple = allow_multiple 
			}
end

local function _erase_event_with_id (events, id)
	for i, v in ipairs (events) do
		if v.id == id then
			table.remove (events, i)
			return
		end
	end
end

function game_events:queue (event)
	local id = event.id
	local priority = event.priority

	if not event.allow_multiple then
		_erase_event_with_id (self.events, id)
	end

	print (string.format ("new game event: %s with priority=%d", id, priority))

	for i, v in ipairs (self.events) do
		if priority > v.priority then
			table.insert (self.events, i, event)
			return
		end
	end

	table.insert (self.events, event)
end

function game_events:reset()
	self.events = { }
	self.current_event = nil
end

events.connect (events.global_update(), function()
	local self = game_events
	if self.current_event then
		return
	end

	for i, v in ipairs (self.events) do
		if v:start() then
			self.current_event = v
			table.remove (self.events, i)
			return
		end	
	end
end)

underworld2.game_events = game_events

underworld2.game_update_event = 
{
	create = function (routine)
		return underworld2.game_events:build ("GAME_UPDATE", 100, routine)
	end
}

underworld2.fame_drop_event = 
{
	create = function (routine)
		return underworld2.game_events:build ("FAME_DROP", 85, routine)
	end
}

underworld2.crew_delivery_event = 
{
	create = function (routine)
		return underworld2.game_events:build ("CREW_MEMBER_DELIVERED", 83, routine)
	end
}

underworld2.activity_report_event =
{
	create = function (routine)
		return underworld2.game_events:build ("ACTIVITY_REPORT", 80, routine)
	end
}

underworld2.bonus_expired_event =
{
	create = function (routine)
		return underworld2.game_events:build ("BONUS_EXPIRED", 75, routine, true)
	end
}

underworld2.quest_expired_event = 
{
	create = function (routine)
		return underworld2.game_events:build ("QUEST_EXPIRED", 70, routine)
	end
}

underworld2.tutorial_event = 
{
	create = function (routine)
		return underworld2.game_events:build ("TUTORIAL", 60, routine)
	end
}

underworld2.news_event = 
{
	create = function (routine)
		return underworld2.game_events:build ("NEWS", 40, routine)
	end
}


