-- timed_events can be used to schedule routines, and uses server_time as a timer base

underworld2.timed_events = { 
	queues = { },
	locks = 0,
}

-- simple unique id generation
local routine_id = 0
local function generate_routine_id()
	local retval = routine_id
	routine_id = routine_id + 1
	return retval 
end

function underworld2.timed_events:create_queue (name, condition_lock_filter)
	self.queues[name] = { events = { }, locks = 0, condition_lock_filter = condition_lock_filter or function() return true end }
end

local function __search_in_queue (queue, id)
	for i, e in pairs (queue.events) do
		if i == id then return e end
	end
end

function underworld2.timed_events:get (...)
	local queue_name, id
	if select (2, ...) then
		queue_name, id = ...
	else
		id = ...
	end

	if not queue_name then
		local event
		for i, q in pairs (self.queues) do
			event = __search_in_queue (q, id)
			if event then break end
		end
		return event
	end
	return __search_in_queue (self.queues[queue_name], id)
end

function underworld2.timed_events:queue (...)
	local queue_name, when, routine
	if select (3, ...) then
		queue_name, when, routine = ...
	else
		queue_name, when, routine = "default", ...
	end
	local queue = self.queues[queue_name]
	local id = generate_routine_id()
	
	queue.events[id] = { when = when, routine = routine }
	return id
end

function underworld2.timed_events:queue_exists (queue_name)
	return self.queues[queue_name] ~= nil
end


function underworld2.timed_events:exec (id)
	for i, queue in pairs (self.queues) do
		if queue.events[id] then
			local event = queue.events[id]
			if queue.condition_lock_filter() then
				queue.events[id] = nil
				event.routine()
			else
				event.when = 0
			end
			return
		end
	end
end

function underworld2.timed_events:unqueue (id)
	for i, queue in pairs (self.queues) do
		if queue.events[id] then
			queue.events[id] = nil
		end
	end
end

function underworld2.timed_events:clear_queue (queue_name)
	if self.queues[queue_name] then
		self.queues[queue_name].events = { }
	end
end

function underworld2.timed_events:unqueue_all (queue_name)
	for i, q in pairs (self.queues) do
		q.events = { }
	end
end

function underworld2.timed_events:lock (queue_name)
	if not queue_name then
		self.locks = self.locks + 1
	elseif self.queues[queue_name] then
		local queue = self.queues[queue_name]
		queue.locks = queue.locks + 1 
	end
end

function underworld2.timed_events:unlock (queue_name)
	if not queue_name then
		self.locks = self.locks - 1
	elseif self.queues[queue_name] then
		local queue = self.queues[queue_name]
		queue.locks = queue.locks - 1 
	end
end

function underworld2.timed_events:reset_lock (queue_name)
	if not queue_name then
		self.locks = 0
		for i, q in pairs (self.queues) do
			q.locks = 0
		end
	elseif self.queues[queue_name] then
		local queue = self.queues[queue_name]
		queue.locks = 0
	end
end

function underworld2.timed_events:reset()
	self:unqueue_all()
	self:reset_lock()
end

underworld2.timed_events:create_queue ("default")

events.connect (events.global_update(), function()
	if underworld2.timed_events.locks > 0 then return end
	
	local queues = underworld2.timed_events.queues
	local now = underworld2.server_time()
	for qi, q in pairs (queues) do
		if q.locks == 0 and q.condition_lock_filter() then
			local events = q.events
			for ei, e in pairs (events) do
				if now >= e.when then
					events[ei] = nil
					e.routine()
					return
				end
			end -- for i, ei
		end --if q.locks
	end -- for qi, q
end)
