local tutorial = { }

function tutorial.completed()
	return false
end

function tutorial:create_arrow_view (point, angle)
	return ui.core.view.builder (
		setmetatable ({

			build_ui = function (self)
				local arrow = ui.StyledSprite()
				arrow:setTexture ("ui/tutorial/tutorial_arrow:filtered")
				arrow:setShadowEnabled (true)
				arrow:setShadowOffset (math.vec2 (4, 4))

				local pivot = math.vec2 (1.0, 0.5)
				local arrow_size = arrow:getSpriteSize()

				local center_shift = math.vec2 (arrow_size.x * pivot.x, arrow_size.y * pivot.y)
				
				local p = point - center_shift

				arrow:setSize (arrow_size)
				arrow:setOrigin (p)
				arrow:rotate (angle)
				arrow:setPivot (pivot)
				
				self.arrow = arrow
				self.spoint = p
				self.max_offset = math.vec2 (arrow_size.x * 0.25 * math.cos (math.pi + angle), -arrow_size.x * 0.25 * math.sin (math.pi + angle) * ui.config.screen_ratio)
				self.animation_time = 0.3
				self.elapsed = 0.0
				return arrow
			end,

			update = function (self, dt)
				self.elapsed = self.elapsed + dt
				local t = (math.cos (self.elapsed * math.pi / self.animation_time) + 1.0) * 0.5
				local p = self.spoint + self.max_offset * t
				self.arrow:setOrigin (p)
			end

		}, {__index = ui.core.view })).build()
end

function tutorial:create_swipe_view (spoint, epoint, angle)
	return ui.core.view.builder (
		setmetatable ({

			build_ui = function (self)
				local root = ui.Widget()
				root:setOrigin (0, 0)
				root:setSize (1, 1)

				self.elapsed = 0.0
				self.states = { 0.3, 1.0, 1.0, 0.5 }
				self.state = 1

				self.particles = { 0.0, 0.4, 0.8 }
				self.current_particle = 1

				do
					local emmiter = ui.FocusRingEmmiter()
					emmiter:setAnimationDuration (0.6)
					emmiter:setParticleColor (render.color_white)

					local emmiter_size = math.vec2 (0.25, 0.25 * ui.config.screen_ratio)
					emmiter:setSize (emmiter_size)
					emmiter:setOrigin (spoint - emmiter_size * 0.5)

					self.emmiter = emmiter

					root:addChild (emmiter)
				end

				do
					local arrow = ui.StyledSprite()
					arrow:setTexture ("ui/tutorial/tutorial_arrow:filtered")
					arrow:setShadowEnabled (true)
					arrow:setShadowOffset (math.vec2 (4, 4))

					local pivot = math.vec2 (1.0, 0.5)
					local arrow_size = arrow:getSpriteSize()
					
					local rvec = epoint - spoint
					local norm_inv = 1.0 / math.sqrt (rvec.x * rvec.x + rvec.y * rvec.y)
					rvec = rvec *norm_inv
					
					self.arrow_spoint = spoint + math.vec2 (-arrow_size.x * pivot.x + rvec.x * arrow_size.x, -arrow_size.y * pivot.y + rvec.y * arrow_size.x * ui.config.screen_ratio)
					self.arrow_epoint = epoint + math.vec2 (-arrow_size.x * pivot.x + rvec.x * arrow_size.x  * 0.5, -arrow_size.y * pivot.y + rvec.y * arrow_size.x * ui.config.screen_ratio * 0.5)

					local cosx = rvec.x
					local arrow_angle
					if rvec.y <= 0.0 then
						arrow_angle = math.acos (cosx)
					else
						arrow_angle = 2.0 * math.pi - math.acos (cosx)
					end

					arrow:setSize (arrow_size)
					arrow:setOrigin (self.arrow_spoint)
					arrow:rotate (arrow_angle)
					arrow:setPivot (pivot)
					
					self.arrow = arrow
					
					root:addChild (arrow)
				end

				do
					self.finger = ui.StyledSprite()
					self.finger:setTexture ("ui/tutorial/tutorial_finger_ill:filtered")
					self.finger:setShadowEnabled (true)
					self.finger:setShadowOffset (math.vec2 (4, 4))

					local pivot = math.vec2 (0.288, 0.064)
					local finger_size = self.finger:getSpriteSize()
					local center_shift = math.vec2 (finger_size.x * pivot.x, finger_size.y * pivot.y)
					self.finger_spoint = spoint - center_shift
					self.finger_epoint = epoint - center_shift

					self.finger:setOrigin (self.finger_spoint)
					self.finger:rotate (angle)
					self.finger:setPivot (pivot)
					self.finger:setSize (finger_size)

					root:addChild (self.finger)
				end
				
				self:update (0.0)
				return root
			end,

			update = function (self, dt)
				self.elapsed = self.elapsed + dt
				local animation_time = self.states[self.state]
				if self.elapsed > animation_time then
					self.state = self.state % #self.states + 1
					self.elapsed = 0.0
					return
				end

				if self.state == 1 then
					self.arrow:setHidden (true)
					self.finger:setHidden (false)
					self.emmiter:setHidden (true)

					self.finger:setOpacity (self.elapsed / animation_time)
					self.finger:setOrigin (self.finger_spoint)
					self.finger:setScale (1.0)

					self.current_particle = 1

				elseif self.state == 2 then
					self.arrow:setHidden (false)
					self.emmiter:setHidden (false)
					local t = self.elapsed / animation_time
					if self.current_particle <= #self.particles and self.elapsed >= self.particles[self.current_particle] then
						self.emmiter:emit()
						self.current_particle = self.current_particle + 1
					end

					local at = self.elapsed / (self.states[2] + self.states[3])
					self.arrow:setOpacity (self.elapsed / animation_time)
					self.arrow:setOrigin (self.arrow_spoint + (self.arrow_epoint - self.arrow_spoint) * at)

				elseif self.state == 3 then
					self.arrow:setHidden (false)
					self.finger:setHidden (false)
					self.emmiter:setHidden (false)

					local t = self.elapsed / animation_time
					local root = animation_time * 0.75
					local min_scale = 0.8
					local x = self.elapsed - root
					local scale = ((1.0 - min_scale) / (root * root)) * x * x + min_scale
					self.finger:setScale (scale)
					self.finger:setOpacity (1.0)
					self.finger:setOrigin (self.finger_spoint + (self.finger_epoint - self.finger_spoint) * t)

					
					local at = (self.elapsed + self.states[2]) / (self.states[2] + self.states[3])
					self.arrow:setOrigin (self.arrow_spoint + (self.arrow_epoint - self.arrow_spoint) * at)
					self.arrow:setOpacity (1.0 - self.elapsed / animation_time)
					
				elseif self.state == 4 then
					self.arrow:setHidden (true)
					self.finger:setHidden (true)
				end
			end

		}, {__index = ui.core.view })).build()
end

function tutorial:create_tap_view (point, angle)
	return ui.core.view.builder (
		setmetatable ({

			build_ui = function (self)
				local root = ui.Widget()
				root:setOrigin (0, 0)
				root:setSize (1, 1)

				self.elapsed = 0.0
				self.states = { 1.0, 1.0 }
				self.state = 1

				self.particles = { 0.0, 0.4, 0.8 }
				self.current_particle = 1

				do
					local emmiter = ui.FocusRingEmmiter()
					emmiter:setAnimationDuration (0.6)
					emmiter:setParticleColor (render.color_white)

					local emmiter_size = math.vec2 (0.25, 0.25 * ui.config.screen_ratio)
					emmiter:setOrigin (point - emmiter_size * 0.5)
					emmiter:setSize (emmiter_size)

					self.emmiter = emmiter

					root:addChild (emmiter)
				end

				do
					self.finger = ui.StyledSprite()
					self.finger:setTexture ("ui/tutorial/tutorial_finger_ill:filtered")
					self.finger:setShadowEnabled (true)
					self.finger:setShadowOffset (math.vec2 (4, 4))

					local pivot = math.vec2 (0.288, 0.064)
					local finger_size = self.finger:getSpriteSize()
					local center_shift = math.vec2 (finger_size.x * pivot.x, finger_size.y * pivot.y)

					self.finger:setSize (finger_size)
					self.finger:setOrigin (point - center_shift)
					self.finger:rotate (angle)
					self.finger:setPivot (pivot)
					root:addChild (self.finger)
				end
				return root
			end,

			set_pos = function (self, point)
				local pivot = math.vec2 (0.288, 0.064)
				local finger_size = self.finger:getSpriteSize()
				local center_shift = math.vec2 (finger_size.x * pivot.x, finger_size.y * pivot.y)

				self.emmiter:setOrigin (point - self.emmiter:getSize() * 0.5)
				self.finger:setOrigin (point - center_shift)
			end,

			update = function (self, dt)
				self.elapsed = self.elapsed + dt
				local animation_time = self.states[self.state]
				if self.elapsed > animation_time then
					self.state = self.state % #self.states + 1
					self.elapsed = 0.0
					return
				end

				if self.state == 1 then
					if self.current_particle <= #self.particles and self.elapsed >= self.particles[self.current_particle] then
						self.emmiter:emit()
						self.current_particle = self.current_particle + 1
					end
					self.finger:setScale (1.0)
				elseif self.state == 2 then
					local root = animation_time / 2
					local x = self.elapsed - root
					local scale = x * x + (1 - root * root)
					self.finger:setScale (scale)

					self.current_particle = 1					
				end
			end
		}, {__index = ui.core.view })).build()
end

function tutorial:create_brief_view (name, text, image_id)
	return ui.core.view.builder (
		setmetatable ({
			t = 0,

			build_ui = function (self)
				local btn = ui.ImageButton()
				btn:setOrigin (0, 0)
				btn:setSize (1, 1)
				btn:setEnabled (false)
				self.btn = btn

				local character = ui.Picture()
				character:setTexture (image_id)
				btn:addChild (character)

				local frame = ui.Frame()
				frame:setBordersTexture ("ui/modal_wide_window:filtered")
				frame:setMargins (ui.config.hspacing * 2, ui.config.hspacing * 2, ui.config.vspacing * 2, ui.config.vspacing * 2)
				btn:addChild (frame)
				local frame_layout = ui.VerticalLayout()
				frame:setLayout (frame_layout)

				local inner_frame = ui.Frame()
				inner_frame:setBordersTexture ("ui/frame_modal_light:filtered")
				local margins = { x = inner_frame:getLeftMargin() * 0.5, y = inner_frame:getTopMargin() * 0.5 }
				inner_frame:setMargins (margins.x, margins.x, margins.y, margins.y)

				local inner_frame_layout = ui.VerticalLayout()
				inner_frame_layout:setItemSpacing (ui.config.vspacing)

				inner_frame:setLayout (inner_frame_layout)

				local name_label = ui.TextLabel()
				name_label:setTextStyle (ui.text_styles.modal_character_name.build())
				name_label:setText (name)
				inner_frame_layout:addWidget (name_label)

				local brief_label = ui.RichTextWidget()
				brief_label:setDefaultTextStyle (ui.text_styles.tutorial_text.build())
				brief_label:setText (text)
				brief_label:setMaxWidth (1.0 - frame:getLeftMargin() - frame:getRightMargin() - inner_frame:getLeftMargin() - inner_frame:getRightMargin())
				inner_frame_layout:addWidget (brief_label)

				local hint_label = ui.TextLabel()
				hint_label:setTextStyle (ui.text_styles.modal_character_name.build())
				hint_label:setText (L "KEY_TUTORIAL_TAP_TO_CONTINUE")
				hint_label:setTextAlignment (ui.AlignmentVerticalCenter + ui.AlignmentHorizontalCenter)
				inner_frame_layout:addWidget (hint_label)
				self.hint_label = hint_label

				frame_layout:addWidget (inner_frame)

				local character_size = character:getPreferredSize()
				local frame_size = frame:getPreferredSize()

				local sep_y = math.max (0.6, character_size.y)

				character:setOrigin ((1.0 - character_size.x) * 0.5, sep_y - character_size.y)
				character:setSize (character_size)

				frame:setSize (1.0, frame_size.y)
				frame:setOrigin (0.0, sep_y)

				events.connect (btn:getClickedSignalProxy(), function() self:finalize() end)

				self:update (0)

				return btn
			end,

			update = function (self, dt)
				local t = self.t + dt * 1.5
				local text_alpha = 0
				if (t >= 1) then
					text_alpha = 1.0 + math.cos ((t % 1) * math.pi * 2 + math.pi) * 0.25 - 0.25
					self.btn:setEnabled (true)
				end
				self.t = t
				self.hint_label:setColor (render.color (255, 255, 255, 255 * text_alpha))
				self.hint_label:setZoom (1 + (math.max (0.5, text_alpha) - 0.5) * 2 * 0.1)
			end
		}, {__index = ui.core.view })).build()
end

function tutorial:create_brief_police_view (name, text, image_id)
	return ui.core.view.builder (
		setmetatable ({
			build_ui = function (self)
				local widget = ui.ImageButton()
				widget:setOrigin (0, 0)
				widget:setSize (1, 1)

				events.connect (widget:getClickedSignalProxy(), function() self:finalize() end)

				self.ui = {}

				local backlights = {
					ui.Picture(),
					ui.Picture()
				}

				backlights[1]:setTexture ("ui/backlight_modal_large:filtered")
				backlights[1]:setColor(render.color(255, 11, 11, 255))
				backlights[1]:setSize(math.vec2(0.9, 0.5))
				backlights[1]:setPlacingMode (ui.Picture_PlacingModeStretch)

				backlights[2]:setTexture ("ui/backlight_modal_large:filtered")
				backlights[2]:setColor(render.color(11, 93, 255, 255))
				backlights[2]:setSize(math.vec2(0.9, 0.5))
				backlights[2]:setPlacingMode (ui.Picture_PlacingModeStretch)

				widget:addChild(backlights[1])
				widget:addChild(backlights[2])
				
				local character_image = ui.Picture()
				character_image:setTexture (image_id)
				widget:addChild (character_image)

				local frame = ui.Frame()
				frame:setBordersTexture ("ui/modal_wide_window:filtered")
				frame:setMargins (ui.config.hspacing * 2.0, ui.config.hspacing * 2.0, ui.config.vspacing * 2.0, ui.config.vspacing * 2.0)
				
				local frame_layout = ui.VerticalLayout()
				frame_layout:setItemSpacing (ui.config.vspacing)

				do
					local top_layout = ui.HorizontalLayout()
					top_layout:setItemSpacing (ui.config.hspacing)
					
					local ico = ui.widgets.picture_with_backlight.create ("ui/ico_busted:filtered", "ui/backlight_busted:filtered")
					top_layout:addWidget (ico)
					
					local title_label = ui.TextLabel()
					title_label:setTextStyle (ui.text_styles.modal_window_title.build())
					title_label:setText ("Busted!")
					top_layout:addWidget (title_label)
					
					frame_layout:addItem (top_layout)
				end
				
				local sep = ui.Picture()
				sep:setTexture ("ui/gold_line_large:filtered")
				local sep_height = sep:getPreferredSize().y
				sep:setHeight (sep_height)
				sep:setPlacingMode (ui.Picture_PlacingModeStretch)
				sep:setVerticalSizePolicy (ui.SizePolicyFixed)
				frame_layout:addWidget (sep)

				do
					local strings = underworld2.StringsDatabase()
				
					local text_frame = ui.Frame()
					text_frame:setBordersTexture ("ui/frame_modal_light:filtered")
					
					local text_layout = ui.VerticalLayout()
					
					local title_label = ui.TextLabel()
					title_label:setTextStyle (ui.text_styles.modal_character_name.build())
					title_label:setText (name)
					text_layout:addWidget (title_label)
					
					local text_label = ui.RichTextWidget()
					text_label:setDefaultTextStyle (ui.text_styles.tutorial_text.build())
					text_label:setText (text)
					text_label:setMaxWidth (1.0 - frame:getLeftMargin() - frame:getRightMargin() - text_frame:getLeftMargin() - text_frame:getRightMargin())
					text_layout:addWidget (text_label)
					
					text_frame:setLayout (text_layout)
					
					frame_layout:addWidget (text_frame)
				end

				local sep = ui.Picture()
				sep:setTexture ("ui/gold_line_large:filtered")
				sep:setHeight (sep_height)
				sep:setPlacingMode (ui.Picture_PlacingModeStretch)
				sep:setVerticalSizePolicy (ui.SizePolicyFixed)
				frame_layout:addWidget (sep)

				frame:setLayout (frame_layout)
				
				do
					local character_size = character_image:getPreferredSize()
					local frame_size = frame:getPreferredSize()

					local border_gap = 0.05
					local frame_pos_y = math.min (0.6, 1.0 - frame_size.y - border_gap)
					local max_char_height = math.min (frame_pos_y - border_gap, character_size.y)

					character_size.x = character_size.x * max_char_height / character_size.y
					character_size.y = max_char_height
					
					character_image:setOrigin ((1.0 - character_size.x) * 0.5, frame_pos_y - character_size.y)
					character_image:setSize (character_size)
					character_image:setVerticalSizePolicy (ui.SizePolicyFixed)
					character_image:setHorizontalSizePolicy (ui.SizePolicyFixed)
					character_image:setPlacingMode(ui.Picture_PlacingModeStretch)
					
					frame:setOrigin (0.0, frame_pos_y)
					frame:setSize (1.0, frame_size.y)

					local origin = character_image:getOrigin()
					backlights[1]:setOrigin(origin.x - 0.27, origin.y)
					backlights[2]:setOrigin(origin.x + 0.15, origin.y + 0.04)
				end
				
				widget:addChild (frame)

				local red = render.color(255, 11, 11, 255)
				local blue = render.color(11, 93, 255, 255)

				local blink = ui.animations.build {
					duration = 0.5,
					from = 255,
					to = 0,
					times = -1,
					oscillate = true,

					step = function (self, value)
						local l = render.color(((self.iteration + 0) % 4 > 1) and blue or red)
						local r = render.color(((self.iteration + 1) % 4 > 1) and red or blue)

						backlights[1]:setColor(render.color(l.r, l.g, l.b, value))
						backlights[2]:setColor(render.color(r.r, r.g, r.b, 255-value))
					end
				}

				self.animations = ui.animations.factory()
			  	self.animations:bind ("blink", blink)
				self:update(0)

				return widget
			end,

			update = function (self, dt)
				self.animations:update(dt)
			end
		}, {__index = ui.core.view })).build()
end

function tutorial:start (stages, name, completion_handler, stage_id)
	self.stages = stages
	self.name = name
	self.completion_handler = completion_handler
	self.stage_key = stage_id and (stage_id - 1) or 0
	self:next_stage()
end

function tutorial:reset()
	if self.stage_cleanup then
		self.stage_cleanup (self)
	end
	self.stages = nil
	self.stage_key = nil
	self.stage_cleanup = nil
	self.stage_name = nil
	self.name = nil
	self.completion_handler = nil
	self.back_handler = nil
end

function tutorial:set_back_handler (handler)
	self.back_handler = handler
end

function tutorial:on_back()
	if not self.back_handler then
		runtime.terminate_runloop()
		return false
	end
	return self.back_handler()
end

function tutorial:skip_to_stage (key)
	self.next_stage_key = key
end

function tutorial:next_stage()
	if not self.stages then
		return
	end

	local stage_name = self.stage_name
	print (string.format ("stage %s finished(%s)", tostring (self.stage_key), tostring (stage_name)))
	if self.stage_cleanup then
		self.stage_cleanup (self)
	end

	local current_stage_key = self.stage_key
	self.stage_key = self.next_stage_key or (self.stage_key + 1)
	print (string.format ("next stage=%d", self.stage_key))
	self.next_stage_key = nil
	local stage = self.stages[self.stage_key]
	
	if current_stage_key > 0 and stage_name then
		underworld2.event_logger.write ({ type = "tutorial", tutorialCode = self.name, stepId = string.format ("%d_%s", current_stage_key, stage_name) })
	end

	if not stage then
		underworld2.network.record_tutorial_step ({ 
			tutorial_name = self.name,
			tutorial_step_name = string.format ("%d_%s", current_stage_key, stage_name), 
			finalize = true}
		)

		self.stages = nil
		self.stage_key = nil
		self.stage_cleanup = nil
		self.stage_name = nil
		self.name = nil
		self.completion_handler()
		return
	end

	self.stage_cleanup, self.stage_name = stage (self)
	self.stage_name = self.stage_name or string.format ("unnamed_stage_%d", self.stage_key)
end

function tutorial:current_stage()
	return self.stage_key and self.stages[self.stage_key]
end

underworld2.tutorial = tutorial
