require "underworld2/tutorial.lua"

local tutorial_deal = setmetatable ({ }, { __index = underworld2.deal })

function tutorial_deal.create (item, owner, courier, bc, type)
	local obj = setmetatable ( 
	{
		item 		= item,			-- item data: id, count, price per unit
		owner 		= owner,		-- profile info: id, is_player
		courier		= courier,		-- id, start_time, end_time, geo_point, respect_points
		type  		= type,			-- deal type, one of SALE, PURCHASE
		bc 			= bc,
		completed 	= false,
	}, { __index = tutorial_deal })
	
	return obj
end

function tutorial_deal:complete (profile, allow_rent, handler)

	local type = self.type
	local item = self.item
	local courier = self.courier

	local rent_price = self:get_rent_price (profile)

	if allow_rent and rent_price > 0 then
		assert (profile.gold >= rent_price)
		profile.gold = math.floor (profile.gold - rent_price)
	end

	if self.type == underworld2.deal.SALE then
		profile.money = math.floor (profile.money + item.price_per_unit * item.count)
	elseif self.type == underworld2.deal.PURCHASE then
		profile.add_items (item.id, item.count)
	end

	local next_level = underworld2.levels_description[underworld2.profile.level + 1] or underworld2.levels_description[underworld2.profile.level]
	local max_rp = next_level.respect_points
	local respect_earned = self.courier.respect_points
	profile.respect_points = math.min (profile.respect_points + respect_earned, max_rp)

	profile.release_courier (self.courier.id)

	self.ts 		= underworld2.server_time()
	self.courier	= nil
	self.completed  = true
	self.bc 		= nil
	handler()

	underworld2.tutorial:next_stage()
end

function tutorial_deal:get_courier_boost_cost ()
	local progress = clamp ((underworld2.server_time() - self.courier.start_time) / (self.courier.end_time - self.courier.start_time), 0, 1)
	return progress == 1 and 0 or 7
end

function tutorial_deal:courier_boost (profile, cost, handler)
	assert (not self.completed)

	local type = self.type
	local item = self.item

	profile.gold = profile.gold - cost
	self.courier.end_time = self.courier.start_time + 1
	self.completed = true
	
	handler()
end

underworld2.tutorial_deal = tutorial_deal