
local lua_updater = {
	text = function() return "" end,
	step = function() return false end,
	progress = function() return 1.0 end,
}

function lua_updater.build (impl)
	local updater = LuaVersionUpdate (setmetatable (impl, { __index = lua_updater }))
	updater:retain()
	return VersionUpdatePtr (updater)
end

---------------------------------------------------------------------------------------------

local function version_795()
	--map tiles provider has changed...
	return lua_updater.build ({
		step = function()
			file.remove ("/home/cache/tiles")
			return false
		end
	}) 
end

local function version_1368_a()
	-- db profile links changes from now 
	return lua_updater.build ({
		step = function()
			file.remove ("/home/inbox.db")
			return false
		end
	})
end

local function version_1368_b()
	return underworld2.TileDatabase():moveCache ("/home/cache/tiles", "/public/cache/tiles")
end

local function version_1368_c()
	-- db profile links changes from now 
	return lua_updater.build ({
		step = function()
			file.remove ("/home/db/profiles.db")
			file.remove ("/home/profiles/")
			return false
		end
	})
end

local function version_1368_d()
	return lua_updater.build ({
		step = function()
			file.remove ("/home/db/profiles.db")
			file.remove ("/home/cache/avatars/")
			return false
		end
	})
end

local function version_2246()
	return lua_updater.build ({
		step = function()
			file.remove ("/public/db/commodities.db")
			return false
		end
	})
end

-----------------------------------------------------------------------------------------------

local known_updaters = 
{
	[795] = version_795,
	[1368] = { version_1368_a, version_1368_b, version_1368_c, version_1368_d },
	[2246] = { version_2246 }
}

function run_updater (handler)
	local version = VERSION
	local previous_version = underworld2.settings:get ("version", 0)

	if previous_version == version then return false end

	print (string.format("Running updater cur: %i; prev: %i", version, previous_version ))

	controller = ui.VersionUpdateController_Init()
	events.connect (controller:getFinishedSignalProxy(), function()
		ui.VersionUpdateController_Terminate()--safe
		
		-- not the best idea, it would be better to flush settings after each update has finished
		underworld2.settings.version = version
		underworld2.settings:flush()

		handler()
	end)

	for ver, val in pairs(known_updaters) do
		if previous_version < ver then
			if type (val) == "table" then
				for __, step in ipairs (val) do
					controller:addUpdate (step())
				end
			else
				controller:addUpdate (val())
			end
		end
	end

	return true
end

